// ============================================================================
// Copyright (c) 2020 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// ============================================================================
//           
//  Terasic Technologies Inc
//  9F., No.176, Sec.2, Gongdao 5th Rd, East Dist, Hsinchu City, 30070. Taiwan
//
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Thu Oct  8 17:19:01 2020
// ============================================================================

//`define ENABLE_DDR4A
//`define ENABLE_DDR4B
//`define ENABLE_DDR4C
//`define ENABLE_DDR4D
//`define ENABLE_PCIE
//`define ENABLE_QSFPDDA
//`define ENABLE_QSFPDDB
`define ENABLE_QSFPRSV_REFCLK
//`define ENABLE_DDR4_DUAL_RANK

module DE10_Agilex_golden_top
   #(
`ifdef ENABLE_DDR4_DUAL_RANK
      parameter RANK = 1
`else
      parameter RANK = 0
`endif
   )
   (

      ///////// CLOCK /////////
      input              CLK_100_B2A,
      input              CLK_50_B2C,
      input              CLK_50_B3A,
      input              CLK_50_B3C,
      input              UFL_CLKIN_p,
      input              CLK_30M72, //reserved for future use

      ///////// Buttons /////////
      input              CPU_RESET_n,
      input    [ 1: 0]   BUTTON, //BUTTON is Low-Active

      ///////// Swtiches /////////
      input    [ 1: 0]   SW,

      ///////// LED /////////
      output   [ 3: 0]   LED_BRACKET, //LED is Low-Active
      output   [ 3: 0]   LED, //LED is Low-Active

`ifdef ENABLE_QSFPRSV_REFCLK
      ///////// QSFPRSV_REFCLK /////////
      input              QSFPDDRSV_REFCLK_p,
`endif /*ENABLE_QSFPRSV_REFCLK*/

`ifdef ENABLE_DDR4A
      ///////// DDR4A /////////
      input              DDR4A_REFCLK_p,
      output   [16: 0]   DDR4A_A,
      output   [ 1: 0]   DDR4A_BA,
      output   [ 1: 0]   DDR4A_BG,
      output   [RANK: 0] DDR4A_CK,
      output   [RANK: 0] DDR4A_CK_n,
      output   [RANK: 0] DDR4A_CKE,
      inout    [ 8: 0]   DDR4A_DQS,
      inout    [ 8: 0]   DDR4A_DQS_n,
      inout    [71: 0]   DDR4A_DQ,
      inout    [ 8: 0]   DDR4A_DBI_n,
      output   [RANK: 0] DDR4A_CS_n,
      output             DDR4A_RESET_n,
      output   [RANK: 0] DDR4A_ODT,
      output             DDR4A_PAR,
      input              DDR4A_ALERT_n,
      output             DDR4A_ACT_n,
      input              DDR4A_EVENT_n,
      inout              DDR4A_SCL,
      inout              DDR4A_SDA,
      input              DDR4A_RZQ,
`endif /*ENABLE_DDR4A*/

`ifdef ENABLE_DDR4B
      ///////// DDR4B /////////
      input              DDR4B_REFCLK_p,
      output   [16: 0]   DDR4B_A,
      output   [ 1: 0]   DDR4B_BA,
      output   [ 1: 0]   DDR4B_BG,
      output   [RANK: 0] DDR4B_CK,
      output   [RANK: 0] DDR4B_CK_n,
      output   [RANK: 0] DDR4B_CKE,
      inout    [ 8: 0]   DDR4B_DQS,
      inout    [ 8: 0]   DDR4B_DQS_n,
      inout    [71: 0]   DDR4B_DQ,
      inout    [ 8: 0]   DDR4B_DBI_n,
      output   [RANK: 0] DDR4B_CS_n,
      output             DDR4B_RESET_n,
      output   [RANK: 0] DDR4B_ODT,
      output             DDR4B_PAR,
      input              DDR4B_ALERT_n,
      output             DDR4B_ACT_n,
      input              DDR4B_EVENT_n,
      inout              DDR4B_SCL,
      inout              DDR4B_SDA,
      input              DDR4B_RZQ,
`endif /*ENABLE_DDR4B*/

`ifdef ENABLE_DDR4C
      ///////// DDR4C /////////
      input              DDR4C_REFCLK_p,
      output   [16: 0]   DDR4C_A,
      output   [ 1: 0]   DDR4C_BA,
      output   [ 1: 0]   DDR4C_BG,
      output   [RANK: 0] DDR4C_CK,
      output   [RANK: 0] DDR4C_CK_n,
      output   [RANK: 0] DDR4C_CKE,
      inout    [ 8: 0]   DDR4C_DQS,
      inout    [ 8: 0]   DDR4C_DQS_n,
      inout    [71: 0]   DDR4C_DQ,
      inout    [ 8: 0]   DDR4C_DBI_n,
      output   [RANK: 0] DDR4C_CS_n,
      output             DDR4C_RESET_n,
      output   [RANK: 0] DDR4C_ODT,
      output             DDR4C_PAR,
      input              DDR4C_ALERT_n,
      output             DDR4C_ACT_n,
      input              DDR4C_EVENT_n,
      inout              DDR4C_SCL,
      inout              DDR4C_SDA,
      input              DDR4C_RZQ,
`endif /*ENABLE_DDR4C*/

`ifdef ENABLE_DDR4D
      ///////// DDR4D /////////
      input              DDR4D_REFCLK_p,
      output   [16: 0]   DDR4D_A,
      output   [ 1: 0]   DDR4D_BA,
      output   [ 1: 0]   DDR4D_BG,
      output   [RANK: 0] DDR4D_CK,
      output   [RANK: 0] DDR4D_CK_n,
      output   [RANK: 0] DDR4D_CKE,
      inout    [ 8: 0]   DDR4D_DQS,
      inout    [ 8: 0]   DDR4D_DQS_n,
      inout    [71: 0]   DDR4D_DQ,
      inout    [ 8: 0]   DDR4D_DBI_n,
      output   [RANK: 0] DDR4D_CS_n,
      output             DDR4D_RESET_n,
      output   [RANK: 0] DDR4D_ODT,
      output             DDR4D_PAR,
      input              DDR4D_ALERT_n,
      output             DDR4D_ACT_n,
      input              DDR4D_EVENT_n,
      inout              DDR4D_SCL,
      inout              DDR4D_SDA,
      input              DDR4D_RZQ,
`endif /*ENABLE_DDR4D*/

      ///////// SI5340A /////////
      inout              SI5340A_I2C_SCL,
      inout              SI5340A_I2C_SDA,
      input              SI5340A_LOL,
      input              SI5340A_LOS_XAXB,
      output             SI5340A_OE_n,
      output             SI5340A_RST_n,

      ///////// GPIO /////////
      input    [ 1: 0]   GPIO_CLK,
      inout    [ 3: 0]   GPIO_p,

`ifdef ENABLE_PCIE
      ///////// PCIE /////////
      inout              PCIE_SMBCLK,
      inout              PCIE_SMBDAT,
      output   [15: 0]   PCIE_TX_p,
      output   [15: 0]   PCIE_TX_n,
      input    [15: 0]   PCIE_RX_p,
      input    [15: 0]   PCIE_RX_n,
      input    [ 1: 0]   PCIE_REFCLK_p,
      input              PCIE_PERST_n,
`endif /*ENABLE_PCIE*/

`ifdef ENABLE_QSFPDDA
      ///////// QSFPDDA /////////
      output   [ 7: 0]   QSFPDDA_TX_p,
      output   [ 7: 0]   QSFPDDA_TX_n,
      input    [ 7: 0]   QSFPDDA_RX_p,
      input    [ 7: 0]   QSFPDDA_RX_n,
      input              QSFPDDA_REFCLK_p,
      output             QSFPDDA_INITMODE,
      input              QSFPDDA_INTERRUPT_n,
      input              QSFPDDA_MOD_PRS_n,
      output             QSFPDDA_MOD_SEL_n,
      output             QSFPDDA_RST_n,
      inout              QSFPDDA_SCL,
      inout              QSFPDDA_SDA,
`else
      input              QSFPDDA_REFCLK_p,
`endif /*ENABLE_QSFPDDA*/

`ifdef ENABLE_QSFPDDB
      ///////// QSFPDDB /////////
      output   [ 7: 0]   QSFPDDB_TX_p,
      output   [ 7: 0]   QSFPDDB_TX_n,
      input    [ 7: 0]   QSFPDDB_RX_p,
      input    [ 7: 0]   QSFPDDB_RX_n,
      input              QSFPDDB_REFCLK_p,
      output             QSFPDDB_INITMODE,
      input              QSFPDDB_INTERRUPT_n,
      input              QSFPDDB_MOD_PRS_n,
      output             QSFPDDB_MOD_SEL_n,
      output             QSFPDDB_RST_n,
      inout              QSFPDDB_SCL,
      inout              QSFPDDB_SDA,


`endif /*ENABLE_QSFPDDB*/

      ///////// AG /////////
      output             AG_UART_TX,
      input              AG_UART_RX,
      input              AG_UART_CTS,
      output             AG_UART_RTS,

      ///////// EXP /////////
      input              EXP_EN,

      ///////// INFO /////////
      output             INFO_SPI_SCLK,
      input              INFO_SPI_MISO,
      output             INFO_SPI_MOSI,
      output             INFO_SPI_CS_n

);


//=======================================================
//  REG/WIRE declarations
//=======================================================

wire				ninit_done;
wire				system_reset_n;
wire				pb_reset_n;
wire	[3:0]		led_hb;


wire				qsfpdda_refclk;
wire				qsfpddb_refclk;
wire				qsfpddrsv_refclk;

//IP interface
wire				Info_Valid;
wire	[15: 0]	Board_Version;		//A~F.ex BOARD_VERSION = 16'h000b ,express to revB
wire	[15: 0]	MaxCode_Version;	//0~F.ex MAX10_VERSION = 16'h0004 ,express to rev4
wire	[15: 0]	PowerIn_Voltage;	//unit : Celsius degree
wire	[15: 0]	PowerIn_Current;	//unit : Celsius degree
wire	[15: 0]	CORE_Voltage;		//unit : mv
wire	[15: 0]	CORE_Current;		//unit : mA
wire	[15: 0]	Fan_Speed;			//unit : rpm
wire	[15: 0]	Fan_Speed2;			//unit : rpm
wire	[15: 0]	Temp_FPGA;			//unit : Celsius degree
wire	[15: 0]	Temp_Board;			//unit : Celsius degree
wire	[15: 0]	PIN_STATUS;			//
wire	[15: 0]	Temp_SDM;			//unit : Celsius degree
wire	[15: 0]	Temp_ETILE;			//unit : Celsius degree
wire	[15: 0]	Temp_PTILE;			//unit : Celsius degree
wire	[15: 0]	CORE_Voltage2;		//unit : mv
wire	[15: 0]	CORE_Current2;		//unit : mA
wire	[15: 0]	Temp_Board2;		//unit : Celsius degree


//----reg
reg	[15: 0]	rBoard_Version;
reg	[15: 0]	rMaxCode_Version;
reg	[15: 0]	rPowerIn_Voltage;
reg	[15: 0]	rPowerIn_Current;
reg	[15: 0]	rCORE_Voltage;
reg	[15: 0]	rCORE_Current;
reg	[15: 0]	rFan_Speed;
reg	[15: 0]	rFan_Speed2;
reg	[15: 0]	rTemp_FPGA;
reg	[15: 0]	rTemp_Board;
reg	[15: 0]	rPIN_STATUS;
reg	[15: 0]	rTemp_SDM;
reg	[15: 0]	rTemp_ETILE;
reg	[15: 0]	rTemp_PTILE;
reg	[15: 0]	rCORE_Voltage2;
reg	[15: 0]	rCORE_Current2;
reg	[15: 0]	rTemp_Board2;


//=======================================================
//  Structural coding
//=======================================================

//assign SI5340A_OE_n = 1'b0;
//assign SI5340A_RST_n = 1'b1;

reset_release reset_release (
    .ninit_done (ninit_done)    //  output,  width = 1, ninit_done.ninit_done
);

gen_reset_n	system_gen_reset_n (
	.tx_clk(CLK_50_B2C),
	.reset_n_in(CPU_RESET_n),
	.reset_n_out(pb_reset_n)
);

assign system_reset_n = pb_reset_n & !ninit_done;
//assign system_reset_n = pb_reset_n;

heart_beat heart_beat(
	.clk(CLK_50_B2C),
	.led(led_hb[0])
);

assign LED[3:0] = system_reset_n ? led_hb : 4'h00;
assign LED_BRACKET = LED;
  

DE10AG_QSYS u0 (
	.clk_clk											(CLK_100_B2A),			//   input,	width = 1,            clk.clk
	.reset_n_reset_n								(system_reset_n),		//   input,	width = 1,          reset.reset

	.power_in_voltage_export               (rPowerIn_Voltage),	//   input,  width = 16,               power_in_voltage.export
	.power_in_current_export               (rPowerIn_Current),	//   input,  width = 16,               power_in_current.export
	.core_voltage1_export                  (rCORE_Voltage),		//   input,  width = 16,                  core_voltage1.export
	.core_current1_export                  (rCORE_Current),		//   input,  width = 16,                  core_current1.export
	.fan_speed1_export                     (rFan_Speed),			//   input,  width = 16,                     fan_speed1.export
	.fan_speed2_export                     (rFan_Speed2),			//   input,  width = 16,                     fan_speed2.export
	.temperature_fpga_export               (rTemp_FPGA),			//   input,  width = 16,               temperature_fpga.export
	.temperature_board1_export             (rTemp_Board),			//   input,  width = 16,             temperature_board1.export
	.pin_status_export                     (rPIN_STATUS),			//   input,  width = 16,                     pin_status.export
	.temperature_sdm_export                (rTemp_SDM),			//   input,  width = 16,                temperature_sdm.export
	.temperature_e_tile_export             (rTemp_ETILE),			//   input,  width = 16,             temperature_e_tile.export
	.temperature_p_tile_export             (rTemp_PTILE),			//   input,  width = 16,             temperature_p_tile.export
	.core_voltage2_export                  (rCORE_Voltage2),		//   input,  width = 16,                  core_voltage2.export
	.core_current2_export                  (rCORE_Current2),		//   input,  width = 16,                  core_current2.export
	.temperature_board2_export             (rTemp_Board2),		//   input,  width = 16,             temperature_board2.export
	.si5340a_i2c_export_scl_pad_io         (SI5340A_I2C_SCL),	//   inout,   width = 1,             si5340a_i2c_export.scl_pad_io
	.si5340a_i2c_export_sda_pad_io         (SI5340A_I2C_SDA),	//   inout,   width = 1,                               .sda_pad_io
	.si5340a_oe_n_export                   (SI5340A_OE_n),		//  output,   width = 1,                   si5340a_oe_n.export
	.si5340a_rst_n_export                  (SI5340A_RST_n),		//  output,   width = 1,                  si5340a_rst_n.export	

	.pio_led_export								(led_hb[3:1]),			//  output,	width = 4,    pio_led.export
	.pio_button_export							(BUTTON),				//   input,	width = 2, pio_button.export
	.pio_sw_export									(SW)						//   input,	width = 2,     pio_sw.export
	
);


//System Monitor IP
BOARD_INFO BOARD_INFO_i(
    .RESET_N          (system_reset_n),
                                            // input
    .CLK_50           (CLK_50_B2C),         // input

    // 4-pin SPI master interface
    .SCLK             (INFO_SPI_SCLK),      // output
    .MISO             (INFO_SPI_MISO),      // input
    .MOSI             (INFO_SPI_MOSI),      // output
    .CS_n             (INFO_SPI_CS_n),      // output

    // status report (output)
    .Info_Valid       (Info_Valid),         //1: following data is valid
    .Board_Version    (Board_Version),      //A~F.ex BOARD_VERSION = 16'h000b ,express to revB
    .MaxCode_Version  (MaxCode_Version),    //0~F.ex MAX10_VERSION = 16'h0004 ,express to rev4
    .PowerIn_Voltage  (PowerIn_Voltage),    //unit : Celsius degree
    .PowerIn_Current  (PowerIn_Current),    //unit : Celsius degree
    .CORE_Voltage     (CORE_Voltage),       //unit : mv
    .CORE_Current     (CORE_Current),       //unit : mA
    .Fan_Speed        (Fan_Speed),          //unit : rpm
    .Fan_Speed2       (Fan_Speed2),         //unit : rpm
    .Temp_FPGA        (Temp_FPGA),          //unit : Celsius degree
    .Temp_Board       (Temp_Board),         //unit : Celsius degree
    .PIN_STATUS       (PIN_STATUS),
    .Temp_SDM         (Temp_SDM),           //unit : Celsius degree
    .Temp_ETILE       (Temp_ETILE),         //unit : Celsius degree
    .Temp_PTILE       (Temp_PTILE),         //unit : Celsius degree
    .CORE_Voltage2    (CORE_Voltage2),      //unit : mv
    .CORE_Current2    (CORE_Current2),      //unit : mA
    .Temp_Board2      (Temp_Board2)         //unit : Celsius degree
);

always @(posedge CLK_50_B2C ) begin
    if (Info_Valid) begin
        rBoard_Version   <= Board_Version;
        rMaxCode_Version <= MaxCode_Version;
        rPowerIn_Voltage <= PowerIn_Voltage;
        rPowerIn_Current <= PowerIn_Current;
        rCORE_Voltage    <= CORE_Voltage;
        rCORE_Current    <= CORE_Current;
        rFan_Speed       <= Fan_Speed;
        rFan_Speed2      <= Fan_Speed2;
        rTemp_FPGA       <= Temp_FPGA;
        rTemp_Board      <= Temp_Board;
        rPIN_STATUS      <= PIN_STATUS;
        rTemp_SDM        <= Temp_SDM;
        rTemp_ETILE      <= Temp_ETILE;
        rTemp_PTILE      <= Temp_PTILE;
        rCORE_Voltage2   <= CORE_Voltage2;
        rCORE_Current2   <= CORE_Current2;
        rTemp_Board2     <= Temp_Board2;
    end
end

endmodule
