// --------------------------------------------------------------------
// Copyright (c) 2012 by Terasic Technologies Inc.
// --------------------------------------------------------------------
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development
//   Kits made by Terasic.  Other use of this code, including the selling
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use
//   or functionality of this code.
//
// --------------------------------------------------------------------
//
//                     Terasic Technologies Inc
//                     E. Rd Sec. 1. JhuBei City,
//                     HsinChu County, Taiwan
//                     302
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// --------------------------------------------------------------------

#include <stdio.h>
#include "terasic_includes.h"
#include "clock.h"
#include "si5340A_config.h"
#include "I2C_core.h"
#include <stdbool.h>

typedef bool (*LP_VERIFY_FUNC)(void);

bool TEST_Board_Info(void);

typedef struct {
    LP_VERIFY_FUNC func;
    char szName[128];
} FUNC_INFO;

FUNC_INFO szFuncList[] = { { TEST_Board_Info, "Display Board Info" } };

void GUI_ShowMenu(void)
{
    int nNum, i;

    nNum = sizeof(szFuncList) / sizeof(szFuncList[0]);
    printf("======= Agilex Demo Program =======\n");
    for (i = 0; i < nNum; i++) {
        printf("[%d] %s\n", i, szFuncList[i].szName);
    }
    printf("Input your chioce:");
}

int GUI_QueryUser(void)
{
    int nChoice = 0;
    scanf("%d", &nChoice);
    printf("%d\n", nChoice);
    return nChoice;
}

//===============================================================
int main(void)
{
    int nChoice;
    int nNum;
    bool bPass;

    int nDisplay_Count;
    alt_u8 LED_Data = 0x01;
    alt_u8 roll_flag = 0x01;

    nDisplay_Count = 0;
    nNum = sizeof(szFuncList) / sizeof(szFuncList[0]);
    while (1) {
    	// Roll LED

		IOWR(PIO_LED_BASE, 0x00, ~LED_Data);
		if (roll_flag)
			LED_Data = LED_Data << 1;
		else
			LED_Data = LED_Data >> 1;

		if (LED_Data == 0x01)
			roll_flag = 1;
		else if (LED_Data == 0x04)
			roll_flag = 0;

		usleep (150*1000);

		nDisplay_Count += 1;

        nChoice = 1;
		if (nDisplay_Count == 30) {
            printf("\n\n---------- Board Information ----------\n");
            bPass = szFuncList[nChoice].func();
            nDisplay_Count = 0;
        }

    }

}

bool TEST_Board_Info(void)
{
    uint16_t power_in_voltage, power_in_current;
    uint16_t core_voltage1, core_voltage2;
    float core_voltage1_float, core_voltage2_float;
    int16_t core_current1, core_current2;
    float core_current1_float, core_current2_float;
    uint16_t fan_speed1, fan_speed2;
    uint16_t temperature_fpga, temperature_board1, temperature_board2;
    uint16_t temperature_sdm, temperature_e_tile, temperature_p_tile;
    uint16_t pin_status;

    float core_voltage, core_current;

    power_in_voltage = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_POWER_IN_VOLTAGE_BASE) & 0xFFFF;
    power_in_current = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_POWER_IN_CURRENT_BASE) & 0xFFFF;
    core_voltage1 = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_CORE_VOLTAGE1_BASE) & 0xFFFF;
    core_current1 = (int16_t)(IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_CORE_CURRENT1_BASE) & 0xFFFF);
    core_voltage2 = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_CORE_VOLTAGE2_BASE) & 0xFFFF;
    core_current2 = (int16_t)(IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_CORE_CURRENT2_BASE) & 0xFFFF);
    fan_speed1 = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_FAN_SPEED1_BASE) & 0xFFFF;
    fan_speed2 = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_FAN_SPEED2_BASE) & 0xFFFF;
    temperature_fpga = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_TEMPERATURE_FPGA_BASE) & 0xFFFF;
    temperature_board1 = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_TEMPERATURE_BOARD1_BASE) & 0xFFFF;
    temperature_board2 = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_TEMPERATURE_BOARD2_BASE) & 0xFFFF;
    temperature_sdm = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_TEMPERATURE_SDM_BASE) & 0xFFFF;
    temperature_e_tile = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_TEMPERATURE_E_TILE_BASE) & 0xFFFF;
    temperature_p_tile = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_TEMPERATURE_P_TILE_BASE) & 0xFFFF;

    core_voltage1_float = core_voltage1 / 1000.0;
    core_voltage2_float = core_voltage2 / 1000.0;
    core_voltage = (core_voltage1_float + core_voltage2_float) / 2.0;
    core_current1_float = core_current1 / 1000.0;
    core_current2_float = core_current2 / 1000.0;
    core_current = core_current1_float + core_current2_float;

    printf("==== Temperature ====\n");
    printf("\tFPGA: %d*C\n", temperature_fpga);
    printf("\tBoard 1: %d*C\n", temperature_board1);
    printf("\tBoard 2: %d*C\n", temperature_board2);
    printf("\tSDM: %d*C\n", temperature_sdm);
    printf("\tE-Tile: %d*C\n", temperature_e_tile);
    printf("\tP-Tile: %d*C\n", temperature_p_tile);
    printf("\n");
    printf("==== Fan ====\n");
    printf("\tFan 1 RPM: %d\n", fan_speed1);
    printf("\tFan 2 RPM: %d\n", fan_speed2);
    printf("\n");
    printf("==== Power (12V) Monitor ====\n");
    printf("\tVoltage       = %.3f V \n", power_in_voltage / 1000.0);
    printf("\tCurrent       = %.3f A \n", power_in_current / 1000.0);
    printf("\tPower         = %.3f W \n", power_in_voltage / 1000.0 * power_in_current / 1000.0);
    printf("==== Core Power Monitor ====\n");
    printf("\tVoltage       = %.3f V \n", core_voltage);
    printf("\tCurrent       = %.3f A \n", core_current);
    printf("\tPower         = %.3f W \n", core_voltage * core_current);

    return true;
}
