// ============================================================================
// Copyright (c) 2020 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development
//   Kits made by Terasic.  Other use of this code, including the selling
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use
//   or functionality of this code.
//
// ============================================================================
//
//  Terasic Technologies Inc
//  9F., No.176, Sec.2, Gongdao 5th Rd, East Dist, Hsinchu City, 30070. Taiwan
//
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Thu Oct  8 17:19:01 2020
// ============================================================================

`define ENABLE_DDR4A
`define ENABLE_DDR4B
`define ENABLE_DDR4C
`define ENABLE_DDR4D
`define ENABLE_PCIE
//`define ENABLE_QSFPDDA
//`define ENABLE_QSFPDDB
//`define ENABLE_QSFPRSV_REFCLK
//`define ENABLE_DDR4_DUAL_RANK

module DE10_Agilex
   #(
`ifdef ENABLE_DDR4_DUAL_RANK
      parameter RANK = 1
`else
      parameter RANK = 0
`endif
   )
   (

      ///////// CLOCK /////////
      input              CLK_100_B2A,
      input              CLK_50_B2C,
      input              CLK_50_B3A,
      input              CLK_50_B3C,
      input              UFL_CLKIN_p,
      input              CLK_30M72, //reserved for future use

      ///////// Buttons /////////
      input              CPU_RESET_n,
      input    [ 1: 0]   BUTTON, //BUTTON is Low-Active

      ///////// Swtiches /////////
      input    [ 1: 0]   SW,

      ///////// LED /////////
      output   [ 3: 0]   LED_BRACKET, //LED is Low-Active
      output   [ 3: 0]   LED, //LED is Low-Active

`ifdef ENABLE_QSFPRSV_REFCLK
      ///////// QSFPRSV_REFCLK /////////
      input              QSFPDDRSV_REFCLK_p,
`endif /*ENABLE_QSFPRSV_REFCLK*/

`ifdef ENABLE_DDR4A
      ///////// DDR4A /////////
      input              DDR4A_REFCLK_p,
      output   [16: 0]   DDR4A_A,
      output   [ 1: 0]   DDR4A_BA,
      output   [ 1: 0]   DDR4A_BG,
      output   [RANK: 0] DDR4A_CK,
      output   [RANK: 0] DDR4A_CK_n,
      output   [RANK: 0] DDR4A_CKE,
      inout    [ 8: 0]   DDR4A_DQS,
      inout    [ 8: 0]   DDR4A_DQS_n,
      inout    [71: 0]   DDR4A_DQ,
      inout    [ 8: 0]   DDR4A_DBI_n,
      output   [RANK: 0] DDR4A_CS_n,
      output             DDR4A_RESET_n,
      output   [RANK: 0] DDR4A_ODT,
      output             DDR4A_PAR,
      input              DDR4A_ALERT_n,
      output             DDR4A_ACT_n,
      input              DDR4A_EVENT_n,
      inout              DDR4A_SCL,
      inout              DDR4A_SDA,
      input              DDR4A_RZQ,
`endif /*ENABLE_DDR4A*/

`ifdef ENABLE_DDR4B
      ///////// DDR4B /////////
      input              DDR4B_REFCLK_p,
      output   [16: 0]   DDR4B_A,
      output   [ 1: 0]   DDR4B_BA,
      output   [ 1: 0]   DDR4B_BG,
      output   [RANK: 0] DDR4B_CK,
      output   [RANK: 0] DDR4B_CK_n,
      output   [RANK: 0] DDR4B_CKE,
      inout    [ 8: 0]   DDR4B_DQS,
      inout    [ 8: 0]   DDR4B_DQS_n,
      inout    [71: 0]   DDR4B_DQ,
      inout    [ 8: 0]   DDR4B_DBI_n,
      output   [RANK: 0] DDR4B_CS_n,
      output             DDR4B_RESET_n,
      output   [RANK: 0] DDR4B_ODT,
      output             DDR4B_PAR,
      input              DDR4B_ALERT_n,
      output             DDR4B_ACT_n,
      input              DDR4B_EVENT_n,
      inout              DDR4B_SCL,
      inout              DDR4B_SDA,
      input              DDR4B_RZQ,
`endif /*ENABLE_DDR4B*/

`ifdef ENABLE_DDR4C
      ///////// DDR4C /////////
      input              DDR4C_REFCLK_p,
      output   [16: 0]   DDR4C_A,
      output   [ 1: 0]   DDR4C_BA,
      output   [ 1: 0]   DDR4C_BG,
      output   [RANK: 0] DDR4C_CK,
      output   [RANK: 0] DDR4C_CK_n,
      output   [RANK: 0] DDR4C_CKE,
      inout    [ 8: 0]   DDR4C_DQS,
      inout    [ 8: 0]   DDR4C_DQS_n,
      inout    [71: 0]   DDR4C_DQ,
      inout    [ 8: 0]   DDR4C_DBI_n,
      output   [RANK: 0] DDR4C_CS_n,
      output             DDR4C_RESET_n,
      output   [RANK: 0] DDR4C_ODT,
      output             DDR4C_PAR,
      input              DDR4C_ALERT_n,
      output             DDR4C_ACT_n,
      input              DDR4C_EVENT_n,
      inout              DDR4C_SCL,
      inout              DDR4C_SDA,
      input              DDR4C_RZQ,
`endif /*ENABLE_DDR4C*/

`ifdef ENABLE_DDR4D
      ///////// DDR4D /////////
      input              DDR4D_REFCLK_p,
      output   [16: 0]   DDR4D_A,
      output   [ 1: 0]   DDR4D_BA,
      output   [ 1: 0]   DDR4D_BG,
      output   [RANK: 0] DDR4D_CK,
      output   [RANK: 0] DDR4D_CK_n,
      output   [RANK: 0] DDR4D_CKE,
      inout    [ 8: 0]   DDR4D_DQS,
      inout    [ 8: 0]   DDR4D_DQS_n,
      inout    [71: 0]   DDR4D_DQ,
      inout    [ 8: 0]   DDR4D_DBI_n,
      output   [RANK: 0] DDR4D_CS_n,
      output             DDR4D_RESET_n,
      output   [RANK: 0] DDR4D_ODT,
      output             DDR4D_PAR,
      input              DDR4D_ALERT_n,
      output             DDR4D_ACT_n,
      input              DDR4D_EVENT_n,
      inout              DDR4D_SCL,
      inout              DDR4D_SDA,
      input              DDR4D_RZQ,
`endif /*ENABLE_DDR4D*/

      ///////// SI5340A /////////
      inout              SI5340A_I2C_SCL,
      inout              SI5340A_I2C_SDA,
      input              SI5340A_LOL,
      input              SI5340A_LOS_XAXB,
      output             SI5340A_OE_n,
      output             SI5340A_RST_n,

      ///////// GPIO /////////
      input    [ 1: 0]   GPIO_CLK,
      inout    [ 3: 0]   GPIO_p,

`ifdef ENABLE_PCIE
      ///////// PCIE /////////
      inout              PCIE_SMBCLK,
      inout              PCIE_SMBDAT,
      output   [15: 0]   PCIE_TX_p,
      output   [15: 0]   PCIE_TX_n,
      input    [15: 0]   PCIE_RX_p,
      input    [15: 0]   PCIE_RX_n,
      input    [ 1: 0]   PCIE_REFCLK_p,
      input              PCIE_PERST_n,
`endif /*ENABLE_PCIE*/

`ifdef ENABLE_QSFPDDA
      ///////// QSFPDDA /////////
      output   [ 7: 0]   QSFPDDA_TX_p,
      output   [ 7: 0]   QSFPDDA_TX_n,
      input    [ 7: 0]   QSFPDDA_RX_p,
      input    [ 7: 0]   QSFPDDA_RX_n,
      input              QSFPDDA_REFCLK_p,
      output             QSFPDDA_INITMODE,
      input              QSFPDDA_INTERRUPT_n,
      input              QSFPDDA_MOD_PRS_n,
      output             QSFPDDA_MOD_SEL_n,
      output             QSFPDDA_RST_n,
      inout              QSFPDDA_SCL,
      inout              QSFPDDA_SDA,
`else
      input              QSFPDDA_REFCLK_p,
`endif /*ENABLE_QSFPDDA*/

`ifdef ENABLE_QSFPDDB
      ///////// QSFPDDB /////////
      output   [ 7: 0]   QSFPDDB_TX_p,
      output   [ 7: 0]   QSFPDDB_TX_n,
      input    [ 7: 0]   QSFPDDB_RX_p,
      input    [ 7: 0]   QSFPDDB_RX_n,
      input              QSFPDDB_REFCLK_p,
      output             QSFPDDB_INITMODE,
      input              QSFPDDB_INTERRUPT_n,
      input              QSFPDDB_MOD_PRS_n,
      output             QSFPDDB_MOD_SEL_n,
      output             QSFPDDB_RST_n,
      inout              QSFPDDB_SCL,
      inout              QSFPDDB_SDA,
`endif /*ENABLE_QSFPDDB*/

      ///////// AG /////////
      output             AG_UART_TX,
      input              AG_UART_RX,
      input              AG_UART_CTS,
      output             AG_UART_RTS,

      ///////// EXP /////////
      input              EXP_EN,

      ///////// INFO /////////
      output             INFO_SPI_SCLK,
      input              INFO_SPI_MISO,
      output             INFO_SPI_MOSI,
      output             INFO_SPI_CS_n

);


//=======================================================
//  REG/WIRE declarations
//=======================================================

wire [3: 0] pio_led;
assign LED = ~pio_led;
wire ddr4a_local_cal_success;
wire ddr4b_local_cal_success;
wire ddr4c_local_cal_success;
wire ddr4d_local_cal_success;
assign LED_BRACKET = ~{ddr4d_local_cal_success, ddr4c_local_cal_success, ddr4b_local_cal_success, ddr4a_local_cal_success};

//=======================================================
//  Structural coding
//=======================================================

pcie_ed u0 (
    .hip_serial_rx_n_in0   (PCIE_RX_n[0]),      //   input,  width = 1, hip_serial.rx_n_in0
    .hip_serial_rx_n_in1   (PCIE_RX_n[1]),      //   input,  width = 1,           .rx_n_in1
    .hip_serial_rx_n_in2   (PCIE_RX_n[2]),      //   input,  width = 1,           .rx_n_in2
    .hip_serial_rx_n_in3   (PCIE_RX_n[3]),      //   input,  width = 1,           .rx_n_in3
    .hip_serial_rx_n_in4   (PCIE_RX_n[4]),      //   input,  width = 1,           .rx_n_in4
    .hip_serial_rx_n_in5   (PCIE_RX_n[5]),      //   input,  width = 1,           .rx_n_in5
    .hip_serial_rx_n_in6   (PCIE_RX_n[6]),      //   input,  width = 1,           .rx_n_in6
    .hip_serial_rx_n_in7   (PCIE_RX_n[7]),      //   input,  width = 1,           .rx_n_in7
    .hip_serial_rx_n_in8   (PCIE_RX_n[8]),      //   input,  width = 1,           .rx_n_in8
    .hip_serial_rx_n_in9   (PCIE_RX_n[9]),      //   input,  width = 1,           .rx_n_in9
    .hip_serial_rx_n_in10  (PCIE_RX_n[10]),     //   input,  width = 1,           .rx_n_in10
    .hip_serial_rx_n_in11  (PCIE_RX_n[11]),     //   input,  width = 1,           .rx_n_in11
    .hip_serial_rx_n_in12  (PCIE_RX_n[12]),     //   input,  width = 1,           .rx_n_in12
    .hip_serial_rx_n_in13  (PCIE_RX_n[13]),     //   input,  width = 1,           .rx_n_in13
    .hip_serial_rx_n_in14  (PCIE_RX_n[14]),     //   input,  width = 1,           .rx_n_in14
    .hip_serial_rx_n_in15  (PCIE_RX_n[15]),     //   input,  width = 1,           .rx_n_in15
    .hip_serial_rx_p_in0   (PCIE_RX_p[0]),      //   input,  width = 1,           .rx_p_in0
    .hip_serial_rx_p_in1   (PCIE_RX_p[1]),      //   input,  width = 1,           .rx_p_in1
    .hip_serial_rx_p_in2   (PCIE_RX_p[2]),      //   input,  width = 1,           .rx_p_in2
    .hip_serial_rx_p_in3   (PCIE_RX_p[3]),      //   input,  width = 1,           .rx_p_in3
    .hip_serial_rx_p_in4   (PCIE_RX_p[4]),      //   input,  width = 1,           .rx_p_in4
    .hip_serial_rx_p_in5   (PCIE_RX_p[5]),      //   input,  width = 1,           .rx_p_in5
    .hip_serial_rx_p_in6   (PCIE_RX_p[6]),      //   input,  width = 1,           .rx_p_in6
    .hip_serial_rx_p_in7   (PCIE_RX_p[7]),      //   input,  width = 1,           .rx_p_in7
    .hip_serial_rx_p_in8   (PCIE_RX_p[8]),      //   input,  width = 1,           .rx_p_in8
    .hip_serial_rx_p_in9   (PCIE_RX_p[9]),      //   input,  width = 1,           .rx_p_in9
    .hip_serial_rx_p_in10  (PCIE_RX_p[10]),     //   input,  width = 1,           .rx_p_in10
    .hip_serial_rx_p_in11  (PCIE_RX_p[11]),     //   input,  width = 1,           .rx_p_in11
    .hip_serial_rx_p_in12  (PCIE_RX_p[12]),     //   input,  width = 1,           .rx_p_in12
    .hip_serial_rx_p_in13  (PCIE_RX_p[13]),     //   input,  width = 1,           .rx_p_in13
    .hip_serial_rx_p_in14  (PCIE_RX_p[14]),     //   input,  width = 1,           .rx_p_in14
    .hip_serial_rx_p_in15  (PCIE_RX_p[15]),     //   input,  width = 1,           .rx_p_in15
    .hip_serial_tx_n_out0  (PCIE_TX_n[0]),      //  output,  width = 1,           .tx_n_out0
    .hip_serial_tx_n_out1  (PCIE_TX_n[1]),      //  output,  width = 1,           .tx_n_out1
    .hip_serial_tx_n_out2  (PCIE_TX_n[2]),      //  output,  width = 1,           .tx_n_out2
    .hip_serial_tx_n_out3  (PCIE_TX_n[3]),      //  output,  width = 1,           .tx_n_out3
    .hip_serial_tx_n_out4  (PCIE_TX_n[4]),      //  output,  width = 1,           .tx_n_out4
    .hip_serial_tx_n_out5  (PCIE_TX_n[5]),      //  output,  width = 1,           .tx_n_out5
    .hip_serial_tx_n_out6  (PCIE_TX_n[6]),      //  output,  width = 1,           .tx_n_out6
    .hip_serial_tx_n_out7  (PCIE_TX_n[7]),      //  output,  width = 1,           .tx_n_out7
    .hip_serial_tx_n_out8  (PCIE_TX_n[8]),      //  output,  width = 1,           .tx_n_out8
    .hip_serial_tx_n_out9  (PCIE_TX_n[9]),      //  output,  width = 1,           .tx_n_out9
    .hip_serial_tx_n_out10 (PCIE_TX_n[10]),     //  output,  width = 1,           .tx_n_out10
    .hip_serial_tx_n_out11 (PCIE_TX_n[11]),     //  output,  width = 1,           .tx_n_out11
    .hip_serial_tx_n_out12 (PCIE_TX_n[12]),     //  output,  width = 1,           .tx_n_out12
    .hip_serial_tx_n_out13 (PCIE_TX_n[13]),     //  output,  width = 1,           .tx_n_out13
    .hip_serial_tx_n_out14 (PCIE_TX_n[14]),     //  output,  width = 1,           .tx_n_out14
    .hip_serial_tx_n_out15 (PCIE_TX_n[15]),     //  output,  width = 1,           .tx_n_out15
    .hip_serial_tx_p_out0  (PCIE_TX_p[0]),      //  output,  width = 1,           .tx_p_out0
    .hip_serial_tx_p_out1  (PCIE_TX_p[1]),      //  output,  width = 1,           .tx_p_out1
    .hip_serial_tx_p_out2  (PCIE_TX_p[2]),      //  output,  width = 1,           .tx_p_out2
    .hip_serial_tx_p_out3  (PCIE_TX_p[3]),      //  output,  width = 1,           .tx_p_out3
    .hip_serial_tx_p_out4  (PCIE_TX_p[4]),      //  output,  width = 1,           .tx_p_out4
    .hip_serial_tx_p_out5  (PCIE_TX_p[5]),      //  output,  width = 1,           .tx_p_out5
    .hip_serial_tx_p_out6  (PCIE_TX_p[6]),      //  output,  width = 1,           .tx_p_out6
    .hip_serial_tx_p_out7  (PCIE_TX_p[7]),      //  output,  width = 1,           .tx_p_out7
    .hip_serial_tx_p_out8  (PCIE_TX_p[8]),      //  output,  width = 1,           .tx_p_out8
    .hip_serial_tx_p_out9  (PCIE_TX_p[9]),      //  output,  width = 1,           .tx_p_out9
    .hip_serial_tx_p_out10 (PCIE_TX_p[10]),     //  output,  width = 1,           .tx_p_out10
    .hip_serial_tx_p_out11 (PCIE_TX_p[11]),     //  output,  width = 1,           .tx_p_out11
    .hip_serial_tx_p_out12 (PCIE_TX_p[12]),     //  output,  width = 1,           .tx_p_out12
    .hip_serial_tx_p_out13 (PCIE_TX_p[13]),     //  output,  width = 1,           .tx_p_out13
    .hip_serial_tx_p_out14 (PCIE_TX_p[14]),     //  output,  width = 1,           .tx_p_out14
    .hip_serial_tx_p_out15 (PCIE_TX_p[15]),     //  output,  width = 1,           .tx_p_out15
    .refclk0_clk           (PCIE_REFCLK_p[0]),  //   input,  width = 1,    refclk0.clk
    .refclk1_clk           (PCIE_REFCLK_p[1]),  //   input,  width = 1,    refclk1.clk
    .pin_perst_pin_perst   (PCIE_PERST_n),      //   input,  width = 1,  pin_perst.pin_perst
    .pio_button_export     (~BUTTON),           //   input,  width = 4, pio_button.export
    .pio_led_export        (pio_led),           //  output,  width = 4,    pio_led.export

    //DDR4A
    .emif_fm_ddr4a_local_reset_req_local_reset_req     (1'b1),
    //   input,   width = 1,    emif_fm_ddr4a_local_reset_req.local_reset_req
    .emif_fm_ddr4a_local_reset_status_local_reset_done (),
    //  output,   width = 1, emif_fm_ddr4a_local_reset_status.local_reset_done
    .emif_fm_ddr4a_pll_ref_clk_clk                     (DDR4A_REFCLK_p),
    //   input,   width = 1,        emif_fm_ddr4a_pll_ref_clk.clk
    .emif_fm_ddr4a_oct_oct_rzqin                       (DDR4A_RZQ),
    //   input,   width = 1,                emif_fm_ddr4a_oct.oct_rzqin
    .emif_fm_ddr4a_mem_mem_ck                          (DDR4A_CK),
    //  output,   width = 1,                emif_fm_ddr4a_mem.mem_ck
    .emif_fm_ddr4a_mem_mem_ck_n                        (DDR4A_CK_n),
    //  output,   width = 1,                                 .mem_ck_n
    .emif_fm_ddr4a_mem_mem_a                           (DDR4A_A),
    //  output,  width = 17,                                 .mem_a
    .emif_fm_ddr4a_mem_mem_act_n                       (DDR4A_ACT_n),
    //  output,   width = 1,                                 .mem_act_n
    .emif_fm_ddr4a_mem_mem_ba                          (DDR4A_BA),
    //  output,   width = 2,                                 .mem_ba
    .emif_fm_ddr4a_mem_mem_bg                          (DDR4A_BG),
    //  output,   width = 2,                                 .mem_bg
    .emif_fm_ddr4a_mem_mem_cke                         (DDR4A_CKE),
    //  output,   width = 1,                                 .mem_cke
    .emif_fm_ddr4a_mem_mem_cs_n                        (DDR4A_CS_n),
    //  output,   width = 1,                                 .mem_cs_n
    .emif_fm_ddr4a_mem_mem_odt                         (DDR4A_ODT),
    //  output,   width = 1,                                 .mem_odt
    .emif_fm_ddr4a_mem_mem_reset_n                     (DDR4A_RESET_n),
    //  output,   width = 1,                                 .mem_reset_n
    .emif_fm_ddr4a_mem_mem_par                         (DDR4A_PAR),
    //  output,   width = 1,                                 .mem_par
    .emif_fm_ddr4a_mem_mem_alert_n                     (DDR4A_ALERT_n),
    //   input,   width = 1,                                 .mem_alert_n
    .emif_fm_ddr4a_mem_mem_dqs                         (DDR4A_DQS),
    //   inout,   width = 9,                                 .mem_dqs
    .emif_fm_ddr4a_mem_mem_dqs_n                       (DDR4A_DQS_n),
    //   inout,   width = 9,                                 .mem_dqs_n
    .emif_fm_ddr4a_mem_mem_dq                          (DDR4A_DQ),
    //   inout,  width = 72,                                 .mem_dq
    .emif_fm_ddr4a_mem_mem_dbi_n                       (DDR4A_DBI_n),
    //   inout,   width = 9,                                 .mem_dbi_n
    .emif_fm_ddr4a_status_local_cal_success            (ddr4a_local_cal_success),
    //  output,   width = 1,             emif_fm_ddr4a_status.local_cal_success
    .emif_fm_ddr4a_status_local_cal_fail               (),
    //  output,   width = 1,                                 .local_cal_fail

    //DDR4B
    .emif_fm_ddr4b_local_reset_req_local_reset_req     (1'b1),
    //   input,   width = 1,    emif_fm_ddr4b_local_reset_req.local_reset_req
    .emif_fm_ddr4b_local_reset_status_local_reset_done (),
    //  output,   width = 1, emif_fm_ddr4b_local_reset_status.local_reset_done
    .emif_fm_ddr4b_pll_ref_clk_clk                     (DDR4B_REFCLK_p),
    //   input,   width = 1,        emif_fm_ddr4b_pll_ref_clk.clk
    .emif_fm_ddr4b_oct_oct_rzqin                       (DDR4B_RZQ),
    //   input,   width = 1,                emif_fm_ddr4b_oct.oct_rzqin
    .emif_fm_ddr4b_mem_mem_ck                          (DDR4B_CK),
    //  output,   width = 1,                emif_fm_ddr4b_mem.mem_ck
    .emif_fm_ddr4b_mem_mem_ck_n                        (DDR4B_CK_n),
    //  output,   width = 1,                                 .mem_ck_n
    .emif_fm_ddr4b_mem_mem_a                           (DDR4B_A),
    //  output,  width = 17,                                 .mem_a
    .emif_fm_ddr4b_mem_mem_act_n                       (DDR4B_ACT_n),
    //  output,   width = 1,                                 .mem_act_n
    .emif_fm_ddr4b_mem_mem_ba                          (DDR4B_BA),
    //  output,   width = 2,                                 .mem_ba
    .emif_fm_ddr4b_mem_mem_bg                          (DDR4B_BG),
    //  output,   width = 2,                                 .mem_bg
    .emif_fm_ddr4b_mem_mem_cke                         (DDR4B_CKE),
    //  output,   width = 1,                                 .mem_cke
    .emif_fm_ddr4b_mem_mem_cs_n                        (DDR4B_CS_n),
    //  output,   width = 1,                                 .mem_cs_n
    .emif_fm_ddr4b_mem_mem_odt                         (DDR4B_ODT),
    //  output,   width = 1,                                 .mem_odt
    .emif_fm_ddr4b_mem_mem_reset_n                     (DDR4B_RESET_n),
    //  output,   width = 1,                                 .mem_reset_n
    .emif_fm_ddr4b_mem_mem_par                         (DDR4B_PAR),
    //  output,   width = 1,                                 .mem_par
    .emif_fm_ddr4b_mem_mem_alert_n                     (DDR4B_ALERT_n),
    //   input,   width = 1,                                 .mem_alert_n
    .emif_fm_ddr4b_mem_mem_dqs                         (DDR4B_DQS),
    //   inout,   width = 9,                                 .mem_dqs
    .emif_fm_ddr4b_mem_mem_dqs_n                       (DDR4B_DQS_n),
    //   inout,   width = 9,                                 .mem_dqs_n
    .emif_fm_ddr4b_mem_mem_dq                          (DDR4B_DQ),
    //   inout,  width = 72,                                 .mem_dq
    .emif_fm_ddr4b_mem_mem_dbi_n                       (DDR4B_DBI_n),
    //   inout,   width = 9,                                 .mem_dbi_n
    .emif_fm_ddr4b_status_local_cal_success            (ddr4b_local_cal_success),
    //  output,   width = 1,             emif_fm_ddr4b_status.local_cal_success
    .emif_fm_ddr4b_status_local_cal_fail               (),
    //  output,   width = 1,                                 .local_cal_fail

    //DDR4C
    .emif_fm_ddr4c_local_reset_req_local_reset_req     (1'b1),
    //   input,   width = 1,    emif_fm_ddr4c_local_reset_req.local_reset_req
    .emif_fm_ddr4c_local_reset_status_local_reset_done (),
    //  output,   width = 1, emif_fm_ddr4c_local_reset_status.local_reset_done
    .emif_fm_ddr4c_pll_ref_clk_clk                     (DDR4C_REFCLK_p),
    //   input,   width = 1,        emif_fm_ddr4c_pll_ref_clk.clk
    .emif_fm_ddr4c_oct_oct_rzqin                       (DDR4C_RZQ),
    //   input,   width = 1,                emif_fm_ddr4c_oct.oct_rzqin
    .emif_fm_ddr4c_mem_mem_ck                          (DDR4C_CK),
    //  output,   width = 1,                emif_fm_ddr4c_mem.mem_ck
    .emif_fm_ddr4c_mem_mem_ck_n                        (DDR4C_CK_n),
    //  output,   width = 1,                                 .mem_ck_n
    .emif_fm_ddr4c_mem_mem_a                           (DDR4C_A),
    //  output,  width = 17,                                 .mem_a
    .emif_fm_ddr4c_mem_mem_act_n                       (DDR4C_ACT_n),
    //  output,   width = 1,                                 .mem_act_n
    .emif_fm_ddr4c_mem_mem_ba                          (DDR4C_BA),
    //  output,   width = 2,                                 .mem_ba
    .emif_fm_ddr4c_mem_mem_bg                          (DDR4C_BG),
    //  output,   width = 2,                                 .mem_bg
    .emif_fm_ddr4c_mem_mem_cke                         (DDR4C_CKE),
    //  output,   width = 1,                                 .mem_cke
    .emif_fm_ddr4c_mem_mem_cs_n                        (DDR4C_CS_n),
    //  output,   width = 1,                                 .mem_cs_n
    .emif_fm_ddr4c_mem_mem_odt                         (DDR4C_ODT),
    //  output,   width = 1,                                 .mem_odt
    .emif_fm_ddr4c_mem_mem_reset_n                     (DDR4C_RESET_n),
    //  output,   width = 1,                                 .mem_reset_n
    .emif_fm_ddr4c_mem_mem_par                         (DDR4C_PAR),
    //  output,   width = 1,                                 .mem_par
    .emif_fm_ddr4c_mem_mem_alert_n                     (DDR4C_ALERT_n),
    //   input,   width = 1,                                 .mem_alert_n
    .emif_fm_ddr4c_mem_mem_dqs                         (DDR4C_DQS),
    //   inout,   width = 9,                                 .mem_dqs
    .emif_fm_ddr4c_mem_mem_dqs_n                       (DDR4C_DQS_n),
    //   inout,   width = 9,                                 .mem_dqs_n
    .emif_fm_ddr4c_mem_mem_dq                          (DDR4C_DQ),
    //   inout,  width = 72,                                 .mem_dq
    .emif_fm_ddr4c_mem_mem_dbi_n                       (DDR4C_DBI_n),
    //   inout,   width = 9,                                 .mem_dbi_n
    .emif_fm_ddr4c_status_local_cal_success            (ddr4c_local_cal_success),
    //  output,   width = 1,             emif_fm_ddr4c_status.local_cal_success
    .emif_fm_ddr4c_status_local_cal_fail               (),
    //  output,   width = 1,                                 .local_cal_fail

    //DDR4D
    .emif_fm_ddr4d_local_reset_req_local_reset_req     (1'b1),
    //   input,   width = 1,    emif_fm_ddr4d_local_reset_req.local_reset_req
    .emif_fm_ddr4d_local_reset_status_local_reset_done (),
    //  output,   width = 1, emif_fm_ddr4d_local_reset_status.local_reset_done
    .emif_fm_ddr4d_pll_ref_clk_clk                     (DDR4D_REFCLK_p),
    //   input,   width = 1,        emif_fm_ddr4d_pll_ref_clk.clk
    .emif_fm_ddr4d_oct_oct_rzqin                       (DDR4D_RZQ),
    //   input,   width = 1,                emif_fm_ddr4d_oct.oct_rzqin
    .emif_fm_ddr4d_mem_mem_ck                          (DDR4D_CK),
    //  output,   width = 1,                emif_fm_ddr4d_mem.mem_ck
    .emif_fm_ddr4d_mem_mem_ck_n                        (DDR4D_CK_n),
    //  output,   width = 1,                                 .mem_ck_n
    .emif_fm_ddr4d_mem_mem_a                           (DDR4D_A),
    //  output,  width = 17,                                 .mem_a
    .emif_fm_ddr4d_mem_mem_act_n                       (DDR4D_ACT_n),
    //  output,   width = 1,                                 .mem_act_n
    .emif_fm_ddr4d_mem_mem_ba                          (DDR4D_BA),
    //  output,   width = 2,                                 .mem_ba
    .emif_fm_ddr4d_mem_mem_bg                          (DDR4D_BG),
    //  output,   width = 2,                                 .mem_bg
    .emif_fm_ddr4d_mem_mem_cke                         (DDR4D_CKE),
    //  output,   width = 1,                                 .mem_cke
    .emif_fm_ddr4d_mem_mem_cs_n                        (DDR4D_CS_n),
    //  output,   width = 1,                                 .mem_cs_n
    .emif_fm_ddr4d_mem_mem_odt                         (DDR4D_ODT),
    //  output,   width = 1,                                 .mem_odt
    .emif_fm_ddr4d_mem_mem_reset_n                     (DDR4D_RESET_n),
    //  output,   width = 1,                                 .mem_reset_n
    .emif_fm_ddr4d_mem_mem_par                         (DDR4D_PAR),
    //  output,   width = 1,                                 .mem_par
    .emif_fm_ddr4d_mem_mem_alert_n                     (DDR4D_ALERT_n),
    //   input,   width = 1,                                 .mem_alert_n
    .emif_fm_ddr4d_mem_mem_dqs                         (DDR4D_DQS),
    //   inout,   width = 9,                                 .mem_dqs
    .emif_fm_ddr4d_mem_mem_dqs_n                       (DDR4D_DQS_n),
    //   inout,   width = 9,                                 .mem_dqs_n
    .emif_fm_ddr4d_mem_mem_dq                          (DDR4D_DQ),
    //   inout,  width = 72,                                 .mem_dq
    .emif_fm_ddr4d_mem_mem_dbi_n                       (DDR4D_DBI_n),
    //   inout,   width = 9,                                 .mem_dbi_n
    .emif_fm_ddr4d_status_local_cal_success            (ddr4d_local_cal_success),
    //  output,   width = 1,             emif_fm_ddr4d_status.local_cal_success
    .emif_fm_ddr4d_status_local_cal_fail               ()
    //  output,   width = 1,                                 .local_cal_fail
);



endmodule
