// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//-----------------------------------------------------------------------------------
// This confidential and proprietary software may be used only as authorized by
// a licensing agreement from Intel
// copyright notice must be reproduced on all authorized copies.
//-----------------------------------------------------------------------------------
// Copyright (C) 2017 Intel Corporation. All rights reserved.  Intel products are
// protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
// other intellectual property laws.
//-----------------------------------------------------------------------------------
//  Project Name:  wharf_rock gen4x8 256 bits @ 500MHZ to 512 bits @ 250 MHz adapter
//  Module Name :  intel_pcie_whr_gen4x8_cfg_if
//  Author      :  sdsawant
//  Date        :  Wed Sep 28 16:52:43 2019
//  Description :
//-----------------------------------------------------------------------------------

module  intel_pcie_whr_gen4x8_cfg_if
 #(
     parameter intended_device_family = "Stratix 10",
     parameter core4_0 = 0,
     parameter core4_1 = 0
  )
  (
   input               clk500,
   input               rst_clk500_n,
   input               clk250,
   input               rst_clk250_n,
   input  logic [15:0] tl_cfg_ctl_i,
   input  logic [4:0]  tl_cfg_add_i,
   input  logic [2:0]  tl_cfg_func_i,
   output logic [15:0] tl_cfg_ctl_o,
   output logic [4:0]  tl_cfg_add_o,
   output logic [2:0]  tl_cfg_func_o
  );

  //******************************************************************************
  // DEFINE VARIABLES
  //******************************************************************************
  logic        cycle_complete;
  logic        wr_en;
  logic [15:0] tl_cfg_ctl_q;
  logic [4:0]  tl_cfg_add_q;
  logic [2:0]  tl_cfg_func_q;
  logic        tl_cfg_fifo_empty;
  logic        tl_cfg_fifo_rdreq, tl_cfg_fifo_rdreq_q;
  logic [4:0]  addr_to_capture;
  logic        capture_even_addr;
  logic        tl_cfg_fifo_wrreq;
  logic        start;

  typedef struct packed {
    logic [15:0] tl_cfg_ctl;
    logic [4:0]  tl_cfg_add;
    logic [2:0]  tl_cfg_func;
  } tl_cfg_data_t;

  tl_cfg_data_t tl_cfg_data_in, tl_cfg_data_out;
  logic [4:0] tl_cfg_rd_numdata;

  generate
    if(core4_0 || core4_1) begin
      assign cycle_complete = (tl_cfg_add_i == '0);
      always @(posedge clk500) begin
        tl_cfg_func_q  <= '0;
      end
    end
    else begin
      assign cycle_complete = (tl_cfg_func_i == '0 & tl_cfg_add_i == '0);
      always @(posedge clk500) begin
        tl_cfg_func_q  <= tl_cfg_func_i;
      end
    end
  endgenerate


//  assign tl_cfg_fifo_wrreq = start ? (capture_even_addr ? !tl_cfg_add_q[0]: tl_cfg_add_q[0]) : ;

//  assign tl_cfg_data_in = {tl_cfg_ctl_q, tl_cfg_add_q, tl_cfg_func_q};

  always @(posedge clk500) begin
    tl_cfg_ctl_q   <= tl_cfg_ctl_i;
    tl_cfg_data_in <= {tl_cfg_ctl_q, tl_cfg_add_q, tl_cfg_func_q};
  end

  always @(posedge clk500) begin // or negedge rst_clk500_n
   if(rst_clk500_n == 0) begin
     capture_even_addr <= !tl_cfg_add_i[0];
     tl_cfg_add_q      <= '0;
     start             <= '0;
     tl_cfg_fifo_wrreq <= '0;
   end
   else begin
     start         <= 1'b1;
     tl_cfg_add_q  <= tl_cfg_add_i;
     if (cycle_complete)
       capture_even_addr <= ~capture_even_addr;

     if (start) begin
       if (capture_even_addr)
         tl_cfg_fifo_wrreq <= !tl_cfg_add_q[0];
       else
         tl_cfg_fifo_wrreq <= tl_cfg_add_q[0];
     end
     else begin
         tl_cfg_fifo_wrreq <= '0;
     end
   end
  end

    intel_pcie_whr_gen4x8_dcfifo
    #(
      .DWIDTH ($bits(tl_cfg_data_in)),
      .AWIDTH (4),
      .intended_device_family(intended_device_family)
     ) tl_cfg_data_fifo
    (
     .wr_rst_n(rst_clk500_n),                         // Write Domain Active low Reset
     .wr_clk(clk500),                                 // Write Domain Clock
     .wr_en(tl_cfg_fifo_wrreq),                       // Write Data Enable
     .wr_data(tl_cfg_data_in),                        // Write Data In
     .rd_rst_n(rst_clk250_n),                         // Read Domain Active low Reset
     .rd_clk(clk250),                                 // Read Domain Clock
     .rd_en(tl_cfg_fifo_rdreq),                       // Read Data Enable
     .r_pempty(4'd2),                                 // FIFO partially empty threshold
     .r_pfull(4'd14),                                 // FIFO partially full threshold
     .rd_data(tl_cfg_data_out),                       // Read Data Out
     .rd_numdata(tl_cfg_rd_numdata),                  // Number of Data available in Read clock
     .wr_numdata(),                                   // Number of Data available in Write clock
     .wr_empty(),                                     // FIFO Empty
     .wr_pempty(),                                    // FIFO Partial Empty
     .wr_full(),                                      // FIFO Full
     .wr_pfull(),                                     // FIFO Parial Full
     .rd_empty(tl_cfg_fifo_empty),                    // FIFO Empty
     .rd_pempty(),                                    // FIFO Partial Empty
     .rd_full(),                                      // FIFO Full
     .rd_pfull()                                      // FIFO Partial Full
     );

 
  always @ (posedge clk250) begin //negedge rst_clk250_n or 
    if (rst_clk250_n == 0) begin
      tl_cfg_ctl_o        <= '0;
      tl_cfg_add_o        <= '0;
      tl_cfg_func_o       <= '0;
      tl_cfg_fifo_rdreq   <= '0;
      tl_cfg_fifo_rdreq_q <= '0;
    end
    else begin
     tl_cfg_fifo_rdreq <= ~(tl_cfg_fifo_empty | (tl_cfg_rd_numdata == 5'd1 & ~tl_cfg_fifo_empty & tl_cfg_fifo_rdreq_q));
      tl_cfg_fifo_rdreq_q <= tl_cfg_fifo_rdreq;
      if (tl_cfg_fifo_rdreq_q) begin
        tl_cfg_ctl_o  <= tl_cfg_data_out.tl_cfg_ctl;
        tl_cfg_add_o  <= tl_cfg_data_out.tl_cfg_add;
        tl_cfg_func_o <= tl_cfg_data_out.tl_cfg_func;
      end
    end
  end
endmodule
