// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//-----------------------------------------------------------------------------------
// This confidential and proprietary software may be used only as authorized by
// a licensing agreement from Intel
// copyright notice must be reproduced on all authorized copies.
//-----------------------------------------------------------------------------------
// Copyright (C) 2017 Intel Corporation. All rights reserved.  Intel products are
// protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
// other intellectual property laws.
//-----------------------------------------------------------------------------------
//  Project Name:  wharf_rock gen4x8 256 bits @ 500MHZ to 512 bits @ 250 MHz adapter
//  Module Name :  intel_pcie_whr_gen4x8_int_if
//  Author      :  sdsawant
//  Date        :  Wed Sep 28 16:52:43 2019
//  Description :
//-----------------------------------------------------------------------------------

module  intel_pcie_whr_gen4x8_int_if
 #(
     parameter intended_device_family = "Stratix 10"
  )
  (
   input               clk500,
   input               rst_clk500_n,
   input               clk250,
   input               rst_clk250_n,
   input   logic [7:0] app_int_i,
   input   logic       int_status_i,
   input   logic [2:0] msi_pnd_func_i,
   input   logic [1:0] msi_pnd_addr_i,
   input   logic [7:0] msi_pnd_byte_i,
   output  logic [7:0] app_int_o,
   output  logic       int_status_o,
   output  logic [2:0] msi_pnd_func_o,
   output  logic [1:0] msi_pnd_addr_o,
   output  logic [7:0] msi_pnd_byte_o
  );

  logic [7:0] app_int_q;
  logic       int_status_q;

  always @ (posedge clk250) begin
    if (rst_clk250_n == 0)
      app_int_q <= '0;
    else 
      app_int_q <= app_int_i;
  end

  always @ (posedge clk500) begin
    int_status_q <= int_status_i;
  end
  
  genvar i;
  generate
     for (i=0; i<8; i=i+1) begin : app_int_loop
       intel_pcie_whr_gen4x8_sync_bit #(.DWIDTH(1), .RESET_VAL(0))
              app_int_sync (.clk(clk500), .rst_n(rst_clk500_n), .din(app_int_q[i]), .dout(app_int_o[i]));
     end
  endgenerate

       intel_pcie_whr_gen4x8_sync_pulse int_status_sync (.wr_clk(clk500), .rd_clk(clk250), .wr_rst_n(rst_clk500_n),
                                                  .rd_rst_n(rst_clk250_n), .din(int_status_q), .dout(int_status_o));
 
//********************************************************************
// DEFINE VARIABLES
//********************************************************************
  typedef struct packed {
    logic [2:0] msi_pnd_func;
    logic [1:0] msi_pnd_addr;
    logic [7:0] msi_pnd_byte;
  } msi_data_t;

  logic msi_fifo_rdreq, msi_fifo_rdreq_q, msi_fifo_empty;
  msi_data_t msi_data_in, msi_data_out; 
  logic [2:0] msi_rd_numdata;
  
  logic [2:0] msi_pnd_func_q;  /* synthesis preserve_syn_only */
  logic [1:0] msi_pnd_addr_q;  /* synthesis preserve_syn_only */
  logic [7:0] msi_pnd_byte_q;  /* synthesis preserve_syn_only */
//********************************************************************
// WRITE CLOCK DOMAIN
//********************************************************************
    
  assign msi_data_in = {msi_pnd_func_q, msi_pnd_addr_q, msi_pnd_byte_q};

  always @ (posedge clk250) begin //negedge rst_clk500_n or 
    if (rst_clk250_n == 0) begin
      msi_pnd_func_q   <= '0;
      msi_pnd_addr_q   <= '0;
      msi_pnd_byte_q   <= '0;
    end
    else begin
      msi_pnd_func_q   <= msi_pnd_func_i;
      msi_pnd_addr_q   <= msi_pnd_addr_i;
      msi_pnd_byte_q   <= msi_pnd_byte_i;
    end
  end

//********************************************************************
// STORAGE
//********************************************************************
    intel_pcie_whr_gen4x8_dcfifo
    #(
      .DWIDTH ($bits(msi_data_in)),
      .AWIDTH (2),
      .intended_device_family(intended_device_family)
     ) msififo
    (
     .wr_rst_n(rst_clk250_n),                         // Write Domain Active low Reset
     .wr_clk(clk250),                                 // Write Domain Clock
     .wr_en(1'b1),                                    // Write Data Enable
     .wr_data(msi_data_in),                           // Write Data In
     .rd_rst_n(rst_clk500_n),                         // Read Domain Active low Reset
     .rd_clk(clk500),                                 // Read Domain Clock
     .rd_en(msi_fifo_rdreq),                          // Read Data Enable
     .r_pempty(2'd2),                                 // FIFO partially empty threshold
     .r_pfull(2'd3),                                  // FIFO partially full threshold
     .rd_data(msi_data_out),                          // Read Data Out
     .rd_numdata(msi_rd_numdata),                     // Number of Data available in Read clock
     .wr_numdata(),                                   // Number of Data available in Write clock
     .wr_empty(),                                     // FIFO Empty
     .wr_pempty(),                                    // FIFO Partial Empty
     .wr_full(),                                      // FIFO Full
     .wr_pfull(),                                     // FIFO Parial Full
     .rd_empty(msi_fifo_empty),                       // FIFO Empty
     .rd_pempty(),                                    // FIFO Partial Empty
     .rd_full(),                                      // FIFO Full
     .rd_pfull()                                      // FIFO Partial Full
     );

//********************************************************************
// READ CLOCK DOMAIN
//********************************************************************
   assign msi_fifo_rdreq = ~(msi_fifo_empty | (msi_rd_numdata == 3'd1 & ~msi_fifo_empty & msi_fifo_rdreq_q));

  always @ (posedge clk500) begin //negedge rst_clk500_n or 
    if (rst_clk500_n == 0) begin
      msi_pnd_func_o   <= '0;
      msi_pnd_addr_o   <= '0;
      msi_pnd_byte_o   <= '0;
      msi_fifo_rdreq_q <= '0;
    end
    else begin
      msi_fifo_rdreq_q <= msi_fifo_rdreq;
      if (msi_fifo_rdreq_q) begin
        msi_pnd_func_o   <= msi_data_out.msi_pnd_func;
        msi_pnd_addr_o   <= msi_data_out.msi_pnd_addr;
        msi_pnd_byte_o   <= msi_data_out.msi_pnd_byte;
      end
    end
  end
endmodule