// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//-----------------------------------------------------------------------------------
// This confidential and proprietary software may be used only as authorized by
// a licensing agreement from Intel
// copyright notice must be reproduced on all authorized copies.
//-----------------------------------------------------------------------------------
// Copyright (C) 2017 Intel Corporation. All rights reserved.  Intel products are
// protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
// other intellectual property laws.
//-----------------------------------------------------------------------------------
//  Project Name:  wharf_rock gen4x8 256 bits @ 500MHZ to 512 bits @ 250 MHz adapter
//  Module Name :  intel_pcie_whr_gen4x8_sync_pulse
//  Author      :  sdsawant
//  Date        :  Wed Sep 28 16:52:43 2019
//  Description :
//-----------------------------------------------------------------------------------

module intel_pcie_whr_gen4x8_sync_pulse
  (
   input  logic wr_clk,
   input  logic rd_clk,
   input  logic wr_rst_n,
   input  logic rd_rst_n,
   input  logic din,
   output logic dout
   );

//******************************************************************************
// Definitions
//******************************************************************************
  logic req_wr_clk;
  logic req_rd_clk;
  logic ack_wr_clk;
  logic ack_rd_clk;
  logic req_rd_clk_d0;

//******************************************************************************
// WRITE CLOCK DOMAIN: Generate req & Store data when synchroniztion is not
// already in progress 
//******************************************************************************
  always @(posedge wr_clk) begin //negedge wr_rst_n or 
     if (wr_rst_n == 1'b0) begin
        req_wr_clk <= 1'b0; 
     end
     else begin
        // Generate a Req when there is change in data 
        if ((req_wr_clk == ack_wr_clk) & din == 1) begin
           req_wr_clk <= ~req_wr_clk;
        end
     end
  end

//******************************************************************************
// WRITE CLOCK DOMAIN:  
//******************************************************************************
  intel_pcie_whr_gen4x8_sync_bit
  #(
  .DWIDTH      (1),         // Sync Data input
  .RESET_VAL   (0)          // Reset value
  )
  ack_wr_clk_sync
     (
     .clk      (wr_clk),
     .rst_n    (wr_rst_n),
     .din      (ack_rd_clk),
     .dout     (ack_wr_clk)
     );
  assign ack_rd_clk = req_rd_clk_d0;

//******************************************************************************
// READ CLOCK DOMAIN:  
//******************************************************************************
  intel_pcie_whr_gen4x8_sync_bit
  #(
  .DWIDTH      (1),         // Sync Data input
  .RESET_VAL   (0)          // Reset value
  )
  req_rd_clk_sync
  (
     .clk      (rd_clk),
     .rst_n    (rd_rst_n),
     .din      (req_wr_clk),
     .dout     (req_rd_clk)
  );

//******************************************************************************
// READ CLOCK DOMAIN:  
//******************************************************************************
  always @(posedge rd_clk) begin //negedge rd_rst_n or 
     if (rd_rst_n == 1'b0) begin
        dout      <= 1'b0;
        req_rd_clk_d0 <= 1'b0;
     end
     else begin 
        req_rd_clk_d0 <= req_rd_clk;
        if (req_rd_clk_d0 != req_rd_clk)
           dout <= 1'b1;
        else
           dout <= 1'b0;
     end
  end
endmodule