// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//-----------------------------------------------------------------------------------
// This confidential and proprietary software may be used only as authorized by
// a licensing agreement from Intel
// copyright notice must be reproduced on all authorized copies.
//-----------------------------------------------------------------------------------
// Copyright (C) 2017 Intel Corporation. All rights reserved.  Intel products are
// protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
// other intellectual property laws.
//-----------------------------------------------------------------------------------
//  Project Name:  wharf_rock gen4x8 256 bits @ 500MHZ to 512 bits @ 250 MHz adapter
//  Module Name :  intel_pcie_whr_gen4x8_avmm_rx_st_if
//  Author      :  sdsawant
//  Date        :  Wed Sep 28 16:52:43 2019
//  Description :
//-----------------------------------------------------------------------------------

module intel_pcie_whr_gen4x8_avmm_adapter
  #(
    parameter pld_tx_parity_ena = "enable",
    parameter pld_rx_parity_ena = "enable",
    parameter enable_sriov_hwtcl = 0,
    parameter intended_device_family  = "Stratix 10"
  )
  (
   /*AUTOINPUT*/
   input logic      clk250,
   input logic      clk500,

   // RESET
   input logic               rst_n,
   input logic               rst_n_clk250,

   // TX INTERFACE
   input  logic [256*2-1:0]  tx_st_data_i,
   input  logic [      1:0]  tx_st_sop_i,
   input  logic [      1:0]  tx_st_eop_i,
   input  logic [      1:0]  tx_st_valid_i,
   input  logic              tx_st_ready_i,
   input  logic [      1:0]  tx_st_err_i,
   input  logic [128*2-1:0]  tx_st_hdr_i,
   input  logic [ 32*2-1:0]  tx_st_tlp_prfx_i,
   input  logic [ 32*2-1:0]  tx_st_data_par_i,
   input  logic [ 16*2-1:0]  tx_st_hdr_par_i,
   input  logic [  4*2-1:0]  tx_st_tlp_prfx_par_i,
   input  logic              tx_par_err_i,
   output logic [255:0]      tx_st_data_o,
   output logic              tx_st_sop_o,
   output logic              tx_st_eop_o,
   output logic              tx_st_valid_o,
   output logic              tx_st_ready_o,
   output logic              tx_st_err_o,
   output logic [127:0]      tx_st_hdr_o,
   output logic [ 31:0]      tx_st_tlp_prfx_o,
   output logic [ 31:0]      tx_st_data_par_o,
   output logic [ 15:0]      tx_st_hdr_par_o,
   output logic [  3:0]      tx_st_tlp_prfx_par_o,
   output logic              tx_par_err_o,

   // RX INTERFACE
   input  logic [255:0]      rx_st_data_i,
   input  logic [ 2:0]       rx_st_empty_i,
   input  logic              rx_st_ready_i,
   input  logic              rx_st_sop_i,
   input  logic              rx_st_eop_i,
   input  logic              rx_st_valid_i,
   input  logic [127:0]      rx_st_hdr_i,
   input  logic [ 31:0]      rx_st_tlp_prfx_i,
   input  logic              rx_st_vf_active_i,
   input  logic [  2:0]      rx_st_func_num_i,
   input  logic [ 10:0]      rx_st_vf_num_i,
   input  logic [  2:0]      rx_st_bar_range_i,
   input  logic [ 31:0]      rx_st_data_par_i,
   input  logic [ 15:0]      rx_st_hdr_par_i,
   input  logic [  3:0]      rx_st_tlp_prfx_par_i,
   input  logic              rx_par_err_i,
   output logic [256*2-1:0]  rx_st_data_o,
   output logic [  3*2-1:0]  rx_st_empty_o,
   output logic              rx_st_ready_o,
   output logic [      1:0]  rx_st_sop_o,
   output logic [      1:0]  rx_st_eop_o,
   output logic [      1:0]  rx_st_valid_o,
   output logic [128*2-1:0]  rx_st_hdr_o,
   output logic [ 32*2-1:0]  rx_st_tlp_prfx_o,
   output logic [      1:0]  rx_st_vf_active_o,
   output logic [  3*2-1:0]  rx_st_func_num_o,
   output logic [ 11*2-1:0]  rx_st_vf_num_o,
   output logic [  3*2-1:0]  rx_st_bar_range_o,
   output logic [ 32*2-1:0]  rx_st_data_par_o,
   output logic [ 16*2-1:0]  rx_st_hdr_par_o,
   output logic [  4*2-1:0]  rx_st_tlp_prfx_par_o,
   output logic              rx_par_err_o,

   // FLR INTERFACE
   input  logic [7:0]  flr_rcvd_pf_i,
   input               flr_rcvd_vf_i,
   input  logic [2:0]  flr_rcvd_pf_num_i,
   input  logic [10:0] flr_rcvd_vf_num_i,
   input  logic [7:0]  flr_completed_pf_i,
   input               flr_completed_vf_i,
   input  logic [2:0]  flr_completed_pf_num_i,
   input  logic [10:0] flr_completed_vf_num_i,
   output logic [7:0]  flr_rcvd_pf_o,
   output logic        flr_rcvd_vf_o,
   output logic [2:0]  flr_rcvd_pf_num_o,
   output logic [10:0] flr_rcvd_vf_num_o,
   output logic [7:0]  flr_completed_pf_o,
   output logic        flr_completed_vf_o,
   output logic [2:0]  flr_completed_pf_num_o,
   output logic [10:0] flr_completed_vf_num_o,

   // ERROR INTERFACE
   input               serr_out_i,
   input               hip_enter_err_mode_i,
   input               app_err_valid_i,
   input  logic [31:0] app_err_hdr_i,
   input  logic [12:0] app_err_info_i,
   input  logic [2:0]  app_err_func_num_i,
   output logic        serr_out_o,
   output logic        hip_enter_err_mode_o,
   output logic        app_err_valid_o,
   output logic [31:0] app_err_hdr_o,
   output logic [12:0] app_err_info_o,
   output logic [2:0]  app_err_func_num_o,

   // CREDIT INTERFACE
   input  logic [15:0]  tx_cdts_limit_i,
   input  logic [2:0]   tx_cdts_limit_tdm_idx_i,
   input  logic [11:0]  rx_buffer_limit_i,
   input  logic [1:0]   rx_buffer_limit_tdm_idx_i,
   output logic [15:0]  tx_cdts_limit_o,
   output logic [2:0]   tx_cdts_limit_tdm_idx_o,
   output logic [11:0]  rx_buffer_limit_o,
   output logic [1:0]   rx_buffer_limit_tdm_idx_o,

   // CII INTERFACE
   input               cii_req_i,
   input               cii_hdr_poisoned_i,
   input  logic [3:0]  cii_hdr_first_be_i,
   input  logic [2:0]  cii_func_num_i,
   input  logic        cii_wr_i,
   input  logic [9:0]  cii_addr_i,
   input  logic [31:0] cii_dout_i,
   input  logic        cii_wr_vf_active_i,
   input  logic [10:0] cii_vf_num_i,
   input  logic        cii_override_en_i,
   input  logic [31:0] cii_override_din_i,
   input  logic        cii_halt_i,
   output logic        cii_req_o,
   output logic        cii_hdr_poisoned_o,
   output logic [3:0]  cii_hdr_first_be_o,
   output logic [2:0]  cii_func_num_o,
   output logic        cii_wr_o,
   output logic [9:0]  cii_addr_o,
   output logic [31:0] cii_dout_o,
   output logic        cii_wr_vf_active_o,
   output logic [10:0] cii_vf_num_o,
   output logic        cii_override_en_o,
   output logic [31:0] cii_override_din_o,
   output logic        cii_halt_o,

   // TL CFG INTERFACE
   input  logic [15:0] tl_cfg_ctl_i,
   input  logic [4:0]  tl_cfg_add_i,
   input  logic [2:0]  tl_cfg_func_i,
   output logic [15:0] tl_cfg_ctl_o,
   output logic [4:0]  tl_cfg_add_o,
   output logic [2:0]  tl_cfg_func_o,

   // HARD IP STATUS INTERFACE
   input  logic       link_up_i,
   input  logic       dl_up_i,
   input  logic [5:0] ltssm_state_i,
   output logic       link_up_o,
   output logic       dl_up_o,
   output logic [5:0] ltssm_state_o,

   // PRS INTERFACE
   input              prs_event_valid_i,
   input  logic [2:0] prs_event_func_i,
   input  logic [1:0] prs_event_i,
   output logic       prs_event_valid_o,
   output logic [2:0] prs_event_func_o,
   output logic [1:0] prs_event_o,

   // PM INTERFACE
   input  logic [2:0] pm_state_i,
   input  logic [7:0] apps_pm_xmt_pme_i,
   input              app_ready_entr_l23_i,
   input              apps_pm_xmt_turnoff_i,
   input              app_init_rst_i,
   input              app_xfer_pending_i,
   output logic [2:0] pm_state_o,
   output logic [7:0] apps_pm_xmt_pme_o,
   output logic       app_ready_entr_l23_o,
   output logic       apps_pm_xmt_turnoff_o,
   output logic       app_init_rst_o,
   output logic       app_xfer_pending_o,

   // HOT PLUG INTERFACE
   input  sys_atten_button_pressed_i,
   input  sys_pwr_fault_det_i,
   input  sys_mrl_sensor_chged_i,
   input  sys_pre_det_chged_i,
   input  sys_cmd_cpled_int_i,
   input  sys_pre_det_state_i,
   input  sys_mrl_sensor_state_i,
   input  sys_eml_interlock_engaged_i,
   input  sys_aux_pwr_det_i,
   output sys_atten_button_pressed_o,
   output sys_pwr_fault_det_o,
   output sys_mrl_sensor_chged_o,
   output sys_pre_det_chged_o,
   output sys_cmd_cpled_int_o,
   output sys_pre_det_state_o,
   output sys_mrl_sensor_state_o,
   output sys_eml_interlock_engaged_o,
   output sys_aux_pwr_det_o,

   //INTERRUPT INTERFACE
   input   logic [7:0] app_int_i,
   input   logic [7:0] int_status_i,
   input   logic [2:0] msi_pnd_func_i,
   input   logic [1:0] msi_pnd_addr_i,
   input   logic [7:0] msi_pnd_byte_i,
   output  logic [7:0] app_int_o,
   output  logic [7:0] int_status_o,
   output  logic [2:0] msi_pnd_func_o,
   output  logic [1:0] msi_pnd_addr_o,
   output  logic [7:0] msi_pnd_byte_o
   );

    reg                                                      rst_n_clk250_reg;
    reg    [3:0]                                             rst_n_tree_clk250_reg_s1,rst_n_tree_clk250_reg_s2;

// rst_n_clk250 is iopll locked. This needs to be registered on ref clock then synchronized on 250MHz.
  always @ (posedge clk500 or negedge rst_n_clk250) begin
     if (!rst_n_clk250) begin
        rst_n_clk250_reg <='0;
     end
     else begin
        rst_n_clk250_reg <= rst_n_clk250; /*synthesis preserve*/
     end
  end

  always @ (posedge clk250 or negedge rst_n_clk250_reg) begin
     if (!rst_n_clk250_reg) begin
        rst_n_tree_clk250_reg_s1 <='0;
        rst_n_tree_clk250_reg_s2 <='0;
     end
     else begin
        rst_n_tree_clk250_reg_s1 <= {4{rst_n_clk250_reg}}; /*synthesis preserve*/
        rst_n_tree_clk250_reg_s2 <= rst_n_tree_clk250_reg_s1; /*synthesis preserve*/
     end
  end


  reg                                                      rst_n_reg;
  reg    [3:0]                                             rst_n_tree_reg_s1,rst_n_tree_reg_s2;

// registering pin_perst.
  always @ (posedge clk500 or negedge rst_n) begin
     if (!rst_n) begin
        rst_n_reg <='0;
     end
     else begin
        rst_n_reg <= rst_n; /*synthesis preserve*/
     end
  end

  always @ (posedge clk500 or negedge rst_n) begin
     if (!rst_n) begin
        rst_n_tree_reg_s1 <='0;
        rst_n_tree_reg_s2 <='0;
     end
     else begin
        rst_n_tree_reg_s1 <= {4{rst_n_reg}}; /*synthesis preserve*/
        rst_n_tree_reg_s2 <= rst_n_tree_reg_s1; /*synthesis preserve*/
     end
  end

  intel_pcie_whr_gen4x8_avmm_tx_st_if #(
    .intended_device_family(intended_device_family)
   ) u_tx_st_if
   (
    .clk500               (clk500),
    .rst_clk500_n         (rst_n),
    .clk250               (clk250),
    .rst_clk250_n         (rst_n_clk250),
    .tx_st_data_i         (tx_st_data_i        ),
    .tx_st_sop_i          (tx_st_sop_i         ),
    .tx_st_eop_i          (tx_st_eop_i         ),
    .tx_st_valid_i        (tx_st_valid_i       ),
    .tx_st_ready_i        (tx_st_ready_i       ),
    .tx_st_err_i          (tx_st_err_i         ),
    .tx_st_hdr_i          (tx_st_hdr_i         ),
    .tx_st_tlp_prfx_i     (tx_st_tlp_prfx_i    ),
    .tx_st_data_par_i     (tx_st_data_par_i    ),
    .tx_st_hdr_par_i      (tx_st_hdr_par_i     ),
    .tx_st_tlp_prfx_par_i (tx_st_tlp_prfx_par_i),
    .tx_par_err_i         (tx_par_err_i        ),
    .tx_st_data_o         (tx_st_data_o        ),
    .tx_st_sop_o          (tx_st_sop_o         ),
    .tx_st_eop_o          (tx_st_eop_o         ),
    .tx_st_valid_o        (tx_st_valid_o       ),
    .tx_st_ready_o        (tx_st_ready_o       ),
    .tx_st_err_o          (tx_st_err_o         ),
    .tx_st_hdr_o          (tx_st_hdr_o         ),
    .tx_st_tlp_prfx_o     (tx_st_tlp_prfx_o    ),
    .tx_st_data_par_o     (tx_st_data_par_o    ),
    .tx_st_hdr_par_o      (tx_st_hdr_par_o     ),
    .tx_st_tlp_prfx_par_o (tx_st_tlp_prfx_par_o),
    .tx_par_err_o         (tx_par_err_o        )
    );

  intel_pcie_whr_gen4x8_avmm_rx_st_if #(
    .intended_device_family(intended_device_family)
   ) u_rx_st_if
   (
    .clk500               (clk500),
    .rst_clk500_n         (rst_n),
    .clk250               (clk250),
    .rst_clk250_n         (rst_n_clk250),
    .rx_st_data_i         (rx_st_data_i        ),
    .rx_st_empty_i        (rx_st_empty_i       ),
    .rx_st_ready_i        (rx_st_ready_i       ),
    .rx_st_sop_i          (rx_st_sop_i         ),
    .rx_st_eop_i          (rx_st_eop_i         ),
    .rx_st_valid_i        (rx_st_valid_i       ),
    .rx_st_hdr_i          (rx_st_hdr_i         ),
    .rx_st_tlp_prfx_i     (rx_st_tlp_prfx_i    ),
    .rx_st_vf_active_i    (rx_st_vf_active_i   ),
    .rx_st_func_num_i     (rx_st_func_num_i    ),
    .rx_st_vf_num_i       (rx_st_vf_num_i      ),
    .rx_st_bar_range_i    (rx_st_bar_range_i   ),
    .rx_st_data_par_i     (rx_st_data_par_i    ),
    .rx_st_hdr_par_i      (rx_st_hdr_par_i     ),
    .rx_st_tlp_prfx_par_i (rx_st_tlp_prfx_par_i),
    .rx_par_err_i         (rx_par_err_i        ),
    .rx_st_data_o         (rx_st_data_o        ),
    .rx_st_empty_o        (rx_st_empty_o       ),
    .rx_st_ready_o        (rx_st_ready_o       ),
    .rx_st_sop_o          (rx_st_sop_o         ),
    .rx_st_eop_o          (rx_st_eop_o         ),
    .rx_st_valid_o        (rx_st_valid_o       ),
    .rx_st_hdr_o          (rx_st_hdr_o         ),
    .rx_st_tlp_prfx_o     (rx_st_tlp_prfx_o    ),
    .rx_st_vf_active_o    (rx_st_vf_active_o   ),
    .rx_st_func_num_o     (rx_st_func_num_o    ),
    .rx_st_vf_num_o       (rx_st_vf_num_o      ),
    .rx_st_bar_range_o    (rx_st_bar_range_o   ),
    .rx_st_data_par_o     (rx_st_data_par_o    ),
    .rx_st_hdr_par_o      (rx_st_hdr_par_o     ),
    .rx_st_tlp_prfx_par_o (rx_st_tlp_prfx_par_o),
    .rx_par_err_o         (rx_par_err_o        )
    );

  intel_pcie_whr_gen4x8_avmm_credit_if #(
    .intended_device_family(intended_device_family)
  ) u_credit_if
  (
   .clk500                     (clk500),
   .rst_clk500_n               (rst_n_tree_reg_s2[2]),
   .clk250                     (clk250),
   .rst_clk250_n               (rst_n_tree_clk250_reg_s2[2]),
   .tx_cdts_limit_i            (tx_cdts_limit_i           ),
   .tx_cdts_limit_tdm_idx_i    (tx_cdts_limit_tdm_idx_i  ),
   .rx_buffer_limit_i          (rx_buffer_limit_i         ),
   .rx_buffer_limit_tdm_idx_i  (rx_buffer_limit_tdm_idx_i),
   .tx_cdts_limit_o            (tx_cdts_limit_o           ),
   .tx_cdts_limit_tdm_idx_o    (tx_cdts_limit_tdm_idx_o  ),
   .rx_buffer_limit_o          (rx_buffer_limit_o         ),
   .rx_buffer_limit_tdm_idx_o  (rx_buffer_limit_tdm_idx_o)
  );

  intel_pcie_whr_gen4x8_avmm_cfg_if #(
    .intended_device_family(intended_device_family)
  ) u_cfg_if
  (
   .clk500        (clk500),
   .rst_clk500_n  (rst_n_tree_reg_s2[3]),
   .clk250        (clk250),
   .rst_clk250_n  (rst_n_tree_clk250_reg_s2[3]),
   .tl_cfg_ctl_i  (tl_cfg_ctl_i ),
   .tl_cfg_add_i  (tl_cfg_add_i ),
   .tl_cfg_func_i (tl_cfg_func_i),
   .tl_cfg_ctl_o  (tl_cfg_ctl_o ),
   .tl_cfg_add_o  (tl_cfg_add_o ),
   .tl_cfg_func_o (tl_cfg_func_o)
  );

  intel_pcie_whr_gen4x8_avmm_error_if #(
    .intended_device_family(intended_device_family)
  ) u_err_if
  (
   .clk500               (clk500),
   .rst_clk500_n         (rst_n),
   .clk250               (clk250),
   .rst_clk250_n         (rst_n_clk250),
   .serr_out_i           (serr_out_i          ),
   .hip_enter_err_mode_i (hip_enter_err_mode_i),
   .app_err_valid_i      (app_err_valid_i     ),
   .app_err_hdr_i        (app_err_hdr_i       ),
   .app_err_info_i       (app_err_info_i      ),
   .app_err_func_num_i   (app_err_func_num_i  ),
   .serr_out_o           (serr_out_o          ),
   .hip_enter_err_mode_o (hip_enter_err_mode_o),
   .app_err_valid_o      (app_err_valid_o     ),
   .app_err_hdr_o        (app_err_hdr_o       ),
   .app_err_info_o       (app_err_info_o      ),
   .app_err_func_num_o   (app_err_func_num_o  )
  );

  intel_pcie_whr_gen4x8_avmm_flr_if #(
    .intended_device_family(intended_device_family)
  ) u_flr_if
  (
   .clk500                 (clk500),
   .rst_clk500_n           (rst_n),
   .clk250                 (clk250),
   .rst_clk250_n           (rst_n_clk250),
   .flr_rcvd_pf_i          (flr_rcvd_pf_i         ),
   .flr_rcvd_vf_i          (flr_rcvd_vf_i         ),
   .flr_rcvd_pf_num_i      (flr_rcvd_pf_num_i     ),
   .flr_rcvd_vf_num_i      (flr_rcvd_vf_num_i     ),
   .flr_completed_pf_i     (flr_completed_pf_i    ),
   .flr_completed_vf_i     (flr_completed_vf_i    ),
   .flr_completed_pf_num_i (flr_completed_pf_num_i),
   .flr_completed_vf_num_i (flr_completed_vf_num_i),
   .flr_rcvd_pf_o          (flr_rcvd_pf_o         ),
   .flr_rcvd_vf_o          (flr_rcvd_vf_o         ),
   .flr_rcvd_pf_num_o      (flr_rcvd_pf_num_o     ),
   .flr_rcvd_vf_num_o      (flr_rcvd_vf_num_o     ),
   .flr_completed_pf_o     (flr_completed_pf_o    ),
   .flr_completed_vf_o     (flr_completed_vf_o    ),
   .flr_completed_pf_num_o (flr_completed_pf_num_o),
   .flr_completed_vf_num_o (flr_completed_vf_num_o)
  );

  intel_pcie_whr_gen4x8_avmm_cii_if #(
    .intended_device_family(intended_device_family)
  ) u_cii_if
  (
   .clk500             (clk500),
   .rst_clk500_n       (rst_n),
   .clk250             (clk250),
   .rst_clk250_n       (rst_n_clk250),
   .cii_req_i          (cii_req_i         ),
   .cii_hdr_poisoned_i (cii_hdr_poisoned_i),
   .cii_hdr_first_be_i (cii_hdr_first_be_i),
   .cii_func_num_i     (cii_func_num_i    ),
   .cii_wr_i           (cii_wr_i          ),
   .cii_addr_i         (cii_addr_i        ),
   .cii_dout_i         (cii_dout_i        ),
   .cii_wr_vf_active_i (cii_wr_vf_active_i),
   .cii_vf_num_i       (cii_vf_num_i      ),
   .cii_override_en_i  (cii_override_en_i ),
   .cii_override_din_i (cii_override_din_i),
   .cii_halt_i         (cii_halt_i        ),
   .cii_req_o          (cii_req_o         ),
   .cii_hdr_poisoned_o (cii_hdr_poisoned_o),
   .cii_hdr_first_be_o (cii_hdr_first_be_o),
   .cii_func_num_o     (cii_func_num_o    ),
   .cii_wr_o           (cii_wr_o          ),
   .cii_addr_o         (cii_addr_o        ),
   .cii_dout_o         (cii_dout_o        ),
   .cii_wr_vf_active_o (cii_wr_vf_active_o),
   .cii_vf_num_o       (cii_vf_num_o      ),
   .cii_override_en_o  (cii_override_en_o ),
   .cii_override_din_o (cii_override_din_o),
   .cii_halt_o         (cii_halt_o        )
  );

  intel_pcie_whr_gen4x8_avmm_hard_ip_status_if #(
    .intended_device_family(intended_device_family)
  ) u_hard_ip_status_if
  (
   .clk500        (clk500),
   .rst_clk500_n  (rst_n_tree_reg_s2[0]),
   .clk250        (clk250),
   .rst_clk250_n  (rst_n_tree_clk250_reg_s2[0]),
   .link_up_i     (link_up_i    ),
   .dl_up_i       (dl_up_i      ),
   .ltssm_state_i (ltssm_state_i),
   .link_up_o     (link_up_o    ),
   .dl_up_o       (dl_up_o      ),
   .ltssm_state_o (ltssm_state_o)
  );

  intel_pcie_whr_gen4x8_avmm_prs_if #(
    .intended_device_family(intended_device_family)
  ) u_prs_if
  (
   .clk500            (clk500),
   .rst_clk500_n      (rst_n_tree_reg_s2[0]),
   .clk250            (clk250),
   .rst_clk250_n      (rst_n_tree_clk250_reg_s2[0]),
   .prs_event_valid_i (prs_event_valid_i),
   .prs_event_func_i  (prs_event_func_i ),
   .prs_event_i       (prs_event_i      ),
   .prs_event_valid_o (prs_event_valid_o),
   .prs_event_func_o  (prs_event_func_o ),
   .prs_event_o       (prs_event_o      )
  );

  intel_pcie_whr_gen4x8_avmm_pm_if #(
    .intended_device_family(intended_device_family)
  ) u_pm_if
  (
   .clk500                (clk500),
   .rst_clk500_n          (rst_n_tree_reg_s2[1]),
   .clk250                (clk250),
   .rst_clk250_n          (rst_n_tree_clk250_reg_s2[1]),
   .pm_state_i            (pm_state_i           ),
   .apps_pm_xmt_pme_i     (apps_pm_xmt_pme_i    ),
   .app_ready_entr_l23_i  (app_ready_entr_l23_i ),
   .apps_pm_xmt_turnoff_i (apps_pm_xmt_turnoff_i),
   .app_init_rst_i        (app_init_rst_i       ),
   .app_xfer_pending_i    (app_xfer_pending_i   ),
   .pm_state_o            (pm_state_o           ),
   .apps_pm_xmt_pme_o     (apps_pm_xmt_pme_o    ),
   .app_ready_entr_l23_o  (app_ready_entr_l23_o ),
   .apps_pm_xmt_turnoff_o (apps_pm_xmt_turnoff_o),
   .app_init_rst_o        (app_init_rst_o       ),
   .app_xfer_pending_o    (app_xfer_pending_o   )
  );

  intel_pcie_whr_gen4x8_avmm_hot_plug_if #(
    .intended_device_family(intended_device_family)
  ) u_hot_plug_if
  (
   .clk500                      (clk500),
   .rst_clk500_n                (rst_n_tree_reg_s2[1]),
   .clk250                      (clk250),
   .rst_clk250_n                (rst_n_tree_clk250_reg_s2[1]),
   .sys_atten_button_pressed_i  (sys_atten_button_pressed_i ),
   .sys_pwr_fault_det_i         (sys_pwr_fault_det_i        ),
   .sys_mrl_sensor_chged_i      (sys_mrl_sensor_chged_i     ),
   .sys_pre_det_chged_i         (sys_pre_det_chged_i        ),
   .sys_cmd_cpled_int_i         (sys_cmd_cpled_int_i        ),
   .sys_pre_det_state_i         (sys_pre_det_state_i        ),
   .sys_mrl_sensor_state_i      (sys_mrl_sensor_state_i     ),
   .sys_eml_interlock_engaged_i (sys_eml_interlock_engaged_i),
   .sys_aux_pwr_det_i           (sys_aux_pwr_det_i          ),
   .sys_atten_button_pressed_o  (sys_atten_button_pressed_o ),
   .sys_pwr_fault_det_o         (sys_pwr_fault_det_o        ),
   .sys_mrl_sensor_chged_o      (sys_mrl_sensor_chged_o     ),
   .sys_pre_det_chged_o         (sys_pre_det_chged_o        ),
   .sys_cmd_cpled_int_o         (sys_cmd_cpled_int_o        ),
   .sys_pre_det_state_o         (sys_pre_det_state_o        ),
   .sys_mrl_sensor_state_o      (sys_mrl_sensor_state_o     ),
   .sys_eml_interlock_engaged_o (sys_eml_interlock_engaged_o),
   .sys_aux_pwr_det_o           (sys_aux_pwr_det_o          )
  );

  intel_pcie_whr_gen4x8_avmm_int_if #(
    .intended_device_family(intended_device_family)
  ) u_int_if
  (
   .clk500         (clk500),
   .rst_clk500_n   (rst_n_tree_reg_s2[1]),
   .clk250         (clk250),
   .rst_clk250_n   (rst_n_tree_clk250_reg_s2[1]),
   .app_int_i      (app_int_i     ),
   .int_status_i   (int_status_i  ),
   .msi_pnd_func_i (msi_pnd_func_i),
   .msi_pnd_addr_i (msi_pnd_addr_i),
   .msi_pnd_byte_i (msi_pnd_byte_i),
   .app_int_o      (app_int_o     ),
   .int_status_o   (int_status_o  ),
   .msi_pnd_func_o (msi_pnd_func_o),
   .msi_pnd_addr_o (msi_pnd_addr_o),
   .msi_pnd_byte_o (msi_pnd_byte_o)
  );
endmodule // altera_pcie_s10_gen3x16_adaptor

