// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//-----------------------------------------------------------------------------------
// This confidential and proprietary software may be used only as authorized by
// a licensing agreement from Intel
// copyright notice must be reproduced on all authorized copies.
//-----------------------------------------------------------------------------------
// Copyright (C) 2017 Intel Corporation. All rights reserved.  Intel products are
// protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
// other intellectual property laws.
//-----------------------------------------------------------------------------------
//  Project Name:  wharf_rock gen4x8 256 bits @ 500MHZ to 512 bits @ 250 MHz adapter
//  Module Name :  intel_pcie_whr_gen4x8_avmm_error_if
//  Author      :  sdsawant
//  Date        :  Wed Sep 28 16:52:43 2019
//  Description :
//-----------------------------------------------------------------------------------

module  intel_pcie_whr_gen4x8_avmm_error_if
 #(
     parameter intended_device_family = "Stratix 10"
  )
  (
   input               clk500,
   input               rst_clk500_n,
   input               clk250,
   input               rst_clk250_n,
   input               serr_out_i,
   input               hip_enter_err_mode_i,
   input               app_err_valid_i,
   input  logic [31:0] app_err_hdr_i,
   input  logic [12:0] app_err_info_i,
   input  logic [2:0]  app_err_func_num_i,
   output logic        serr_out_o,
   output logic        hip_enter_err_mode_o,
   output logic        app_err_valid_o,
   output logic [31:0] app_err_hdr_o,
   output logic [12:0] app_err_info_o,
   output logic [2:0]  app_err_func_num_o
  );

//******************************************************************************
// DEFINE VARIABLES
//******************************************************************************
  logic [31:0] app_err_hdr_write_q [4:0];
  logic        hdr_valid_250;
  logic        app_err_valid;
  logic [2:0]  hdr_cycle_count_250;
  logic        hdr_valid_500;
  logic [2:0]  hdr_cycle_count_500;
  logic [12:0] app_err_info_q;
  logic [2:0]  app_err_func_num_q;
  logic        app_err_valid_q;

  intel_pcie_whr_gen4x8_avmm_sync_pulse serr_out_sync (.wr_clk(clk500), .rd_clk(clk250), .wr_rst_n(rst_clk500_n),
                                                       .rd_rst_n(rst_clk250_n), .din(serr_out_i), .dout(serr_out_o));

  intel_pcie_whr_gen4x8_avmm_sync_pulse hip_enter_err_mode_sync (.wr_clk(clk500), .rd_clk(clk250), .wr_rst_n(rst_clk500_n),
                                                            .rd_rst_n(rst_clk250_n), .din(hip_enter_err_mode_i), .dout(hip_enter_err_mode_o));

//**********************************************************************************
// CDC LOGIC FOR app_err_valid, app_err_hdr, app_err_info, app_err_func_num SIGNALS
//**********************************************************************************

  //******************************************************************************
  // WRITE CLOCK DOMAIN 250MHz
  //******************************************************************************
  assign app_err_valid = (hdr_cycle_count_250 == 3'd4);

  always @(negedge rst_clk250_n or posedge clk250) begin
    if (rst_clk250_n == 0) begin
      hdr_cycle_count_250 <= '0;
      hdr_valid_250       <= '0;
      app_err_valid_q     <= '0;
    end
    else begin
      app_err_valid_q <= app_err_valid;
      if (app_err_valid_i == 1) begin
        app_err_hdr_write_q[0]    <= app_err_hdr_i;
        app_err_info_q            <= app_err_info_i;
        app_err_func_num_q        <= app_err_func_num_i;
        hdr_valid_250             <= 1'b1;
        hdr_cycle_count_250       <= 1'b1;
      end
      else if (hdr_valid_250) begin
        app_err_hdr_write_q[hdr_cycle_count_250] <= app_err_hdr_i;
        if (hdr_cycle_count_250 == 3'd4) begin
          hdr_cycle_count_250 <= '0;
          hdr_valid_250       <= '0;
        end
        else begin
          hdr_cycle_count_250 <= hdr_cycle_count_250 + 1'b1;
        end
      end
    end
  end

  //******************************************************************************
  // READ CLOCK DOMAIN 500MHz
  //******************************************************************************

  typedef struct packed {
   logic [31:0] app_err_hdr0;
   logic [31:0] app_err_hdr1;
   logic [31:0] app_err_hdr2;
   logic [31:0] app_err_hdr3;
   logic [31:0] app_err_hdr4;
   logic [12:0] app_err_info;
   logic [2:0]  app_err_func_num;
  } app_err_data_t;

  logic app_err_fifo_rdreq, app_err_fifo_rdreq_q, app_err_fifo_empty;
  app_err_data_t app_err_data_in, app_err_data_out, app_err_data_out_q;
  logic [2:0] app_err_fifo_rd_numdata;

    assign app_err_data_in = {app_err_hdr_write_q[0], app_err_hdr_write_q[1], app_err_hdr_write_q[2], app_err_hdr_write_q[3], app_err_hdr_write_q[4], app_err_info_q, app_err_func_num_q};
    
    intel_pcie_whr_gen4x8_avmm_dcfifo
    #(
      .DWIDTH ($bits(app_err_data_in)),
      .AWIDTH (2),
      .intended_device_family(intended_device_family)
     ) app_err_data_fifo
    (
     .wr_rst_n(rst_clk250_n),                         // Write Domain Active low Reset
     .wr_clk(clk250),                                 // Write Domain Clock
     .wr_en(app_err_valid_q),                         // Write Data Enable
     .wr_data(app_err_data_in),                       // Write Data In
     .rd_rst_n(rst_clk500_n),                         // Read Domain Active low Reset
     .rd_clk(clk500),                                 // Read Domain Clock
     .rd_en(app_err_fifo_rdreq),                      // Read Data Enable
     .r_pempty(2'd1),                                 // FIFO partially empty threshold
     .r_pfull(2'd2),                                  // FIFO partially full threshold
     .rd_data(app_err_data_out),                      // Read Data Out
     .rd_numdata(app_err_fifo_rd_numdata),            // Number of Data available in Read clock
     .wr_numdata(),                                   // Number of Data available in Write clock
     .wr_empty(),                                     // FIFO Empty
     .wr_pempty(),                                    // FIFO Partial Empty
     .wr_full(),                                      // FIFO Full
     .wr_pfull(),                                     // FIFO Parial Full
     .rd_empty(app_err_fifo_empty),                   // FIFO Empty
     .rd_pempty(),                                    // FIFO Partial Empty
     .rd_full(),                                      // FIFO Full
     .rd_pfull()                                      // FIFO Partial Full
     );

  assign app_err_fifo_rdreq = (~(app_err_fifo_empty | (app_err_fifo_rd_numdata == 5'd1 & ~app_err_fifo_empty & app_err_fifo_rdreq_q))) & (hdr_cycle_count_500 == 3'd0);

  always @(negedge rst_clk500_n or posedge clk500) begin
   if (rst_clk500_n == 1'b0) begin
      hdr_cycle_count_500   <= '0;
      hdr_valid_500         <= '0;
      app_err_valid_o       <= '0;
      app_err_fifo_rdreq_q  <= '0;
   end
   else begin
     app_err_valid_o        <= '0;
     app_err_fifo_rdreq_q   <= app_err_fifo_rdreq;
     if(app_err_fifo_rdreq)
       app_err_data_out_q   <= app_err_data_out;
     if (app_err_fifo_rdreq_q) begin
        app_err_valid_o        <= 1'b1;
        app_err_hdr_o          <= app_err_data_out_q.app_err_hdr0;
        app_err_info_o         <= app_err_data_out_q.app_err_info;
        app_err_func_num_o     <= app_err_data_out_q.app_err_func_num;
        hdr_valid_500          <= 1'b1;
        hdr_cycle_count_500    <= 3'd1;
     end
     else if (hdr_valid_500) begin
       case(hdr_cycle_count_500)
         3'b001: begin
             app_err_hdr_o <= app_err_data_out_q.app_err_hdr1;
         end
         3'b010: begin
             app_err_hdr_o <= app_err_data_out_q.app_err_hdr2;
         end
         3'b011: begin
             app_err_hdr_o <= app_err_data_out_q.app_err_hdr3;
         end
         3'b100: begin
             app_err_hdr_o <= app_err_data_out_q.app_err_hdr4;
         end
       endcase

       if (hdr_cycle_count_500 == 3'd4) begin
         hdr_cycle_count_500 <= '0;
         hdr_valid_500       <= '0;
       end
       else begin
         hdr_cycle_count_500 <= hdr_cycle_count_500 + 1'd1;
       end
     end
   end
  end

endmodule
