// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//-----------------------------------------------------------------------------------
// This confidential and proprietary software may be used only as authorized by
// a licensing agreement from Intel
// copyright notice must be reproduced on all authorized copies.
//-----------------------------------------------------------------------------------
// Copyright (C) 2017 Intel Corporation. All rights reserved.  Intel products are
// protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
// other intellectual property laws.
//-----------------------------------------------------------------------------------
//  Project Name:  wharf_rock gen4x8 256 bits @ 500MHZ to 512 bits @ 250 MHz adapter
//  Module Name :  intel_pcie_whr_gen4x8_avmm_gpx3
//  Author      :  sdsawant
//  Date        :  Wed Sep 28 16:52:43 2019
//  Description :
//-----------------------------------------------------------------------------------

module intel_pcie_whr_gen4x8_avmm_gpx3
  #(
    parameter     DWIDTH = 4
  )(
   input  logic              din_clk,
   input  logic              din_rst_n,
   input  logic [DWIDTH-1:0] din,
   input  logic              dout_clk,
   input  logic              dout_rst_n,
   output logic [DWIDTH-1:0] dout
   );

//********************************************************************
// Binary to gray conversion
//********************************************************************

  //******************************************************************************
  // DEFINE VARIABLES
  //******************************************************************************
  logic [DWIDTH-1:0] din_gry;
  logic [DWIDTH-1:0] din_gry_sync;
  logic [DWIDTH-1:0] din_bin;

  always @ (negedge din_rst_n or posedge din_clk) begin
    if (din_rst_n == 1'b0) begin
      din_gry <= '0;
    end
    else begin
      din_gry[DWIDTH - 1] <= din [DWIDTH - 1];
      for (int i = 0; i < (DWIDTH - 1); i=i+1) begin: gry_convert
        din_gry[i] <= din[i] ^ din[i+1];
      end
    end
  end

//********************************************************************
// Two stage synchronizer
//********************************************************************

  intel_pcie_whr_gen4x8_avmm_sync_bit #(.DWIDTH(DWIDTH)) u_din_gry_sync (.clk(dout_clk), .rst_n(dout_rst_n), .din(din_gry), .dout(din_gry_sync));

//********************************************************************
// Gray to binary conversion
//********************************************************************

  assign din_bin[DWIDTH-1] = din_gry_sync[DWIDTH-1];

  genvar i;
  generate
  for (i = 0; i < (DWIDTH - 1); i=i+1) begin: bin_convert
    assign din_bin[i] = din_gry_sync[i] ^ din_bin[i+1];
  end

  endgenerate

  always @ (negedge dout_rst_n or posedge dout_clk) begin
    if (dout_rst_n == 1'b0) begin
      dout <= '0;
    end
    else begin
      dout <= din_bin;
    end
  end

endmodule
