// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1ns/1ps

module intel_pcie_adapter_tx_if #(
  parameter          UI_WIDTH              = 128,
  parameter          HIP_WIDTH             = 256,
  parameter          PF_ENABLE             = 1,
  parameter          VF_ENABLE             = 1,
  parameter          HEADER_PREFIX_ENABLE  = 1,
  parameter          ROOT_PORT_MODE        = 0,
  parameter          MPS                   = 256,
  parameter          SIM_EMULATE           = 0,
  parameter          ECC_ENABLE            = 0,
  parameter          FAMILY                = "Stratix 10"
) (

  input logic                       clk,
  input logic                       rst,

  //TX Interface
  output logic                       tx_st_ready,
  input  logic [UI_WIDTH-1:0]        tx_st_data,
  input  logic [UI_WIDTH/8-1:0]      tx_st_data_par,
  input  logic [127:0]               tx_st_hdr,
  input  logic [15:0]                tx_st_hdr_par,
  input  logic [31:0]                tx_st_tlp_prfx,
  input  logic [3:0]                 tx_st_tlp_prfx_par,
  input  logic                       tx_st_sop,
  input  logic                       tx_st_eop,
  input  logic                       tx_st_valid,
  input  logic                       tx_st_err,

  input  logic                       tx_st_ready_hip,
  output logic [HIP_WIDTH-1:0]       tx_st_data_hip,
  output logic [HIP_WIDTH/8-1:0]     tx_st_data_par_hip,
  output logic [127:0]               tx_st_hdr_hip,
  output logic [15:0]                tx_st_hdr_par_hip,
  output logic [31:0]                tx_st_tlp_prfx_hip,
  output logic [3:0]                 tx_st_tlp_prfx_par_hip,
  output logic                       tx_st_sop_hip,
  output logic                       tx_st_eop_hip,
  output logic                       tx_st_valid_hip,
  output logic                       tx_st_err_hip
);
  localparam  NUM_DATA_MEM         = HIP_WIDTH/32;
  localparam  NUM_HEADER_MEM       = HEADER_PREFIX_ENABLE ? 160/32 : 128/32;
  localparam  HEADER_PAR_MEM_WIDTH = HEADER_PREFIX_ENABLE ? 20 : 16;
  localparam  MEM_SIZE_INIT        = ((MPS*8)/HIP_WIDTH) * 2;
  localparam  MEM_SIZE             = (MEM_SIZE_INIT <= 16) ? 32 : MEM_SIZE_INIT;
  localparam  MEM_TYPE             = (MPS > 512) ? "M20K" : "MLAB";
  localparam  WIDTH_RATIO          = HIP_WIDTH/UI_WIDTH;
  localparam  MEM_SIZE_WIDTH       = $clog2(MEM_SIZE);



  logic  [WIDTH_RATIO-1:0]      select_line,select_line_r;
  logic  [MEM_SIZE_WIDTH-1:0]   eop_count;
  logic  [MEM_SIZE_WIDTH-1:0]   wr_ptr,rd_ptr;
  logic  [HIP_WIDTH-1:0]        wdata;
  logic  [HIP_WIDTH/8-1:0]      rdata_par_wire,wdata_par;
  logic  [HIP_WIDTH-1:0]        rdata_wire;
  logic  [WIDTH_RATIO-1:0]      data_write;
  logic  [MEM_SIZE_WIDTH-1:0]   fillcount;
  logic                         incr_wr_ptr,incr_wr_ptr_r,incr_wr_ptr_rr;
  logic                         header_write;
  logic                         empty,eop_empty;
  logic                         read_req,read_req_r;
  logic                         fillcount_lessthan_4,eop_count_lessthan_4;
  
  logic  [HEADER_PREFIX_ENABLE ? 159:127:0] header_side_band,wheader,rheader_wire;
  
  logic  [HEADER_PAR_MEM_WIDTH-1:0]   wheader_par,rheader_par_wire;

  logic                         tx_st_ready_hip_r,tx_st_ready_hip_rr;
  logic [127:0]                 tx_st_hdr_r;
  logic [15:0]                  tx_st_hdr_par_r;
  logic [3:0]                   tx_st_tlp_prfx_par_r;
  logic [31:0]                  tx_st_tlp_prfx_r;
  logic                         tx_st_valid_r,tx_st_valid_rr,tx_st_valid_rrr;
  logic                         eop_received;
  logic                         tx_st_valid_hip_pre;
  logic                         tx_st_err_hip_wire,tx_st_err_r,tx_st_err_rr;
  logic                         tx_st_eop_hip_wire,tx_st_eop_r,tx_st_eop_rr,tx_st_eop_rrr;
  logic                         tx_st_sop_hip_wire,tx_st_sop_r,tx_st_sop_rr;
  logic                         tx_st_sop_hip_pre;
  logic                         tx_st_eop_hip_pre;
  logic                         tx_st_err_hip_pre;
  logic [UI_WIDTH-1:0]          tx_st_data_r;
  logic [UI_WIDTH/8-1:0]        tx_st_data_par_r;


  assign header_side_band = HEADER_PREFIX_ENABLE ? {tx_st_tlp_prfx_r,tx_st_hdr_r} : tx_st_hdr_r;
  assign incr_wr_ptr = tx_st_valid_rr & (select_line_r[WIDTH_RATIO-1] | tx_st_eop_rr);
  //assign read_req = !empty & |eop_count & tx_st_ready_hip_r;
  //assign read_req = !empty & !eop_empty & !(eop_count_lessthan_4 & 
                   // (tx_st_valid_hip_pre & tx_st_eop_hip_wire & eop_count[0] & !eop_count[1])) & tx_st_ready_hip_r;
  
  assign read_req = !empty & !eop_empty & !(eop_count_lessthan_4 & 
                    (tx_st_valid_hip_pre & tx_st_eop_hip_pre & eop_count[0] & !eop_count[1])) & tx_st_ready_hip_r;
  

  always_ff @(posedge clk) begin
    tx_st_data_r         <= tx_st_data;
    tx_st_data_par_r     <= tx_st_data_par;
    tx_st_hdr_r          <= tx_st_hdr;
    tx_st_hdr_par_r      <= tx_st_hdr_par;
    tx_st_tlp_prfx_r     <= tx_st_tlp_prfx;
    tx_st_tlp_prfx_par_r <= tx_st_tlp_prfx_par;
    if (rst) begin
      tx_st_ready_hip_r  <= '0;
      tx_st_ready_hip_rr <= '0;
      tx_st_valid_r      <= '0;
      tx_st_valid_rr     <= '0;
      tx_st_valid_rrr    <= '0;
      tx_st_eop_r        <= '0;
      tx_st_eop_rr       <= '0;
      tx_st_eop_rrr      <= '0;
      tx_st_sop_r        <= '0;
      tx_st_sop_rr       <= '0;
      tx_st_err_r        <= '0;
      tx_st_err_rr       <= '0;
      eop_received       <= '0;
    end
    else begin
      tx_st_ready_hip_rr <= tx_st_ready_hip_r;
      tx_st_ready_hip_r <= tx_st_ready_hip;
      tx_st_valid_r     <= tx_st_valid;
      tx_st_valid_rr    <= tx_st_valid_r;
      eop_received      <= tx_st_valid_rr & tx_st_eop_rr;
      tx_st_valid_rrr    <= tx_st_valid_rr;
      tx_st_eop_r       <= tx_st_eop;
      tx_st_eop_rr      <= tx_st_eop_r;
      tx_st_eop_rrr      <= tx_st_eop_rr;
      //tx_st_err_r       <= tx_st_err;
      tx_st_err_rr      <= tx_st_err_r;
      tx_st_sop_r       <= tx_st_sop;
      if (incr_wr_ptr)
        tx_st_sop_rr      <= '0;
      if (tx_st_sop_r)
        tx_st_sop_rr      <= tx_st_sop_r;
      if (!tx_st_err_r | (tx_st_sop & tx_st_valid))
        tx_st_err_r <= tx_st_err & tx_st_valid;
    end
  end
  
  generate
    assign tx_st_data_hip = rdata_wire;
    assign tx_st_hdr_hip  = rheader_wire[127:0];
    if (HEADER_PREFIX_ENABLE)
      assign tx_st_tlp_prfx_hip = rheader_wire[159:128];
    if (!ECC_ENABLE) begin
      assign tx_st_data_par_hip  = rdata_par_wire;
      assign tx_st_hdr_par_hip = rheader_par_wire[15:0];
      if (HEADER_PREFIX_ENABLE)
        assign tx_st_tlp_prfx_par_hip = rheader_par_wire[19:16];
    end
      
   /*  assign tx_st_sop_hip = tx_st_sop_hip_wire;
    assign tx_st_eop_hip = tx_st_eop_hip_wire;
    assign tx_st_err_hip = tx_st_err_hip_wire; */
  endgenerate
  
  always_ff @(posedge clk) begin
  //  tx_st_data_hip     <= rdata_wire;
  //  tx_st_hdr_hip      <= rheader_wire[127:0];
  //  if (HEADER_PREFIX_ENABLE)
  //    tx_st_tlp_prfx_hip <= rheader_wire[159:128];
    if (rst) begin
      tx_st_sop_hip       <= '0;
      tx_st_eop_hip       <= '0;
      tx_st_err_hip       <= '0;
      tx_st_sop_hip_pre       <= '0;
      tx_st_eop_hip_pre       <= '0;
      tx_st_err_hip_pre       <= '0;
      tx_st_valid_hip_pre <= '0;
      tx_st_valid_hip     <= '0;
    end
    else begin
      tx_st_sop_hip       <= tx_st_sop_hip_pre;
      tx_st_eop_hip       <= tx_st_eop_hip_pre;
      tx_st_err_hip       <= tx_st_err_hip_pre;
      tx_st_sop_hip_pre   <= tx_st_sop_hip_wire;
      tx_st_eop_hip_pre   <= tx_st_eop_hip_wire;
      tx_st_err_hip_pre   <= tx_st_err_hip_wire;
      tx_st_valid_hip_pre    <= read_req;
      tx_st_valid_hip        <= tx_st_valid_hip_pre;
      
    end
  end

  always_ff @(posedge clk) begin
    wdata       <= {WIDTH_RATIO{tx_st_data_r}};
    wdata_par   <= {WIDTH_RATIO{tx_st_data_par_r}};
    wheader     <= header_side_band;
    wheader_par <= HEADER_PAR_MEM_WIDTH ? {tx_st_tlp_prfx_par_r,tx_st_hdr_par_r} : tx_st_hdr_par_r;
  end



  always_ff @(posedge clk) begin
    if (rst) begin
      select_line <= {{{WIDTH_RATIO-2}{1'b0}},1'b1};
      select_line_r <= {{{WIDTH_RATIO-2}{1'b0}},1'b1};
    end
    else begin
      select_line_r <= select_line;
      if (tx_st_valid_r)
        select_line <= {select_line[WIDTH_RATIO-2:0],select_line[WIDTH_RATIO-1]};
      if (tx_st_valid_r & tx_st_eop_r)
        select_line <= {{(WIDTH_RATIO-2){1'b0}},1'b1};
    end
  end

  always_ff @(posedge clk) begin
    if (rst) begin
      eop_count <= '0;
      incr_wr_ptr_r <= '0;
      incr_wr_ptr_rr <= '0;
      read_req_r <= '0;
    end
    else begin
      incr_wr_ptr_r <= incr_wr_ptr;
      incr_wr_ptr_rr <= incr_wr_ptr_r;
      read_req_r <= read_req;
      eop_count_lessthan_4 <=  eop_count < 4;
      eop_empty <= eop_count_lessthan_4 & ((tx_st_valid_hip_pre & tx_st_eop_hip_pre & !eop_received & eop_count[0] & !eop_count[1]) 
                                          | !(eop_count[0] | eop_count[1] | eop_count[2]));     
      eop_count <= eop_count;
      if (eop_received)
        eop_count <= eop_count + 1'b1;
      if (tx_st_valid_hip_pre & tx_st_eop_hip_pre)
        eop_count <= eop_count - 1'b1;
      if (tx_st_valid_hip_pre & tx_st_eop_hip_pre & eop_received)
        eop_count <= eop_count;
    end
  end

  always_ff @(posedge clk) begin
    if (rst) begin
      wr_ptr <= '0;
    end
    else begin
      if (incr_wr_ptr) begin
        wr_ptr <= wr_ptr + 1'b1;
      end
      else begin
        wr_ptr <= wr_ptr;
      end
    end
  end

  always_ff @(posedge clk) begin
    if (rst) begin
      rd_ptr <= '0;
    end
    else begin
      if (read_req) begin
        rd_ptr <=  rd_ptr +  1'b1;
      end
      else begin
        rd_ptr <= rd_ptr;
      end
    end
  end

  always_ff @(posedge clk) begin
    if (rst) begin
      fillcount            <= '0;
      tx_st_ready          <= '0;
      empty                <= 1'b1;
      fillcount_lessthan_4 <= 1'b1;
    end
    else begin
      empty <= fillcount_lessthan_4 & ((read_req & !incr_wr_ptr_rr & (!fillcount[1] & fillcount[0])) | !(fillcount[0] | fillcount[1] | fillcount[2]));
      fillcount_lessthan_4 <= fillcount < 4;
      tx_st_ready <= (fillcount < (MEM_SIZE-10));
      if (read_req & !incr_wr_ptr_rr)
        fillcount <= fillcount - 1'b1;
      if (!read_req & incr_wr_ptr_rr)
        fillcount <= fillcount + 1'b1;
    end
  end

  always_ff @(posedge clk) begin
    if (rst) begin
      header_write <= '0;
    end
    else begin
      header_write <= tx_st_sop_r & tx_st_valid_r;
    end
  end

  genvar i,j;
  generate
    for (i=0; i < NUM_DATA_MEM; i=i+1 ) begin : data_fifo_memory
      altera_syncram  data_memory
      (
        .clock0         ( clk                              ),
        .wren_a         ( data_write[(32*i+1)/UI_WIDTH]    ),
        .address_a      ( wr_ptr                           ),
        .data_a         ( wdata[32*i+31:32*i]              ),
        .rden_b         ( 1'b1                             ),
        .address_b      ( rd_ptr                           ),
        .q_b            ( rdata_wire[32*i+31:32*i]         ),
        .aclr0          ( 1'b0                 ),
        .aclr1          ( 1'b0                 ),
        .sclr           ( 1'b0                 ), // Not used.
        .addressstall_a ( 1'b0                 ),
        .addressstall_b ( 1'b0                 ),
        .byteena_a      ( 1'b1                 ),
        .byteena_b      ( 1'b1                 ),
        .clock1         ( 1'b1                 ),
        .clocken0       ( 1'b1                 ),
        .clocken1       ( 1'b1                 ),
        .clocken2       ( 1'b1                 ),
        .clocken3       ( 1'b1                 ),
        .data_b         ( {32 {1'b1}}          ), // Not used.
        .eccencbypass   ( 1'b0                 ), // Not used.
        .eccencparity   ( 8'b0                 ), // Not used.
        .q_a            (                      ), // Not used.
        .rden_a         ( 1'b1                 ), // Not used.
        .wren_b         ( 1'b0                 ), // Not used.
        .eccstatus      (                      )  // Not used.
        );
      defparam
       data_memory.address_aclr_b              = "NONE",
       data_memory.address_reg_b               = "CLOCK0",
       data_memory.clock_enable_input_a        = "BYPASS",
       data_memory.clock_enable_input_b        = "BYPASS",
       data_memory.clock_enable_output_b       = "BYPASS",
       data_memory.enable_ecc                  = "FALSE",
       data_memory.ecc_pipeline_stage_enabled  = "FALSE",
       data_memory.intended_device_family      = FAMILY,
       data_memory.lpm_type                    = "altera_syncram",
       data_memory.maximum_depth               = 512,
       data_memory.numwords_a                  = MEM_SIZE,
       data_memory.numwords_b                  = MEM_SIZE,
       data_memory.operation_mode              = "DUAL_PORT",
       data_memory.outdata_aclr_b              = "NONE",
       data_memory.outdata_reg_b               = "CLOCK0",
       data_memory.power_up_uninitialized      = "FALSE",
       data_memory.ram_block_type              = MEM_TYPE,
       data_memory.widthad_a                   = MEM_SIZE_WIDTH,
       data_memory.widthad_b                   = MEM_SIZE_WIDTH,
       data_memory.width_a                     = 32,
       data_memory.width_b                     = 32,
       //data_memory.enable_ecc                  = "TRUE",
       //data_memory.ecc_pipeline_stage_enabled  = "TRUE", // this really is required for respectable Q speed with ECC
       data_memory.width_byteena_a             = 1;
   end

    if (ECC_ENABLE == 0) begin
      for (i=0; i < WIDTH_RATIO; i=i+1 ) begin : data_par_fifo_memory
        altera_syncram  data_par_memory
        (
          .clock0         ( clk                                                   ),
          .wren_a         ( data_write[i]                                         ),
          .address_a      ( wr_ptr                                                ),
          .data_a         ( wdata_par[(UI_WIDTH/8)*(i+1)-1:(UI_WIDTH/8)*i]        ),
          .rden_b         ( 1'b1                                                  ),
          .address_b      ( rd_ptr                                                ),
          .q_b            ( rdata_par_wire[(UI_WIDTH/8)*(i+1)-1:(UI_WIDTH/8)*i]   ),
          .aclr0          ( 1'b0                 ),
          .aclr1          ( 1'b0                 ),
          .sclr           ( 1'b0                 ), // Not used.
          .addressstall_a ( 1'b0                 ),
          .addressstall_b ( 1'b0                 ),
          .byteena_a      ( 1'b1                 ),
          .byteena_b      ( 1'b1                 ),
          .clock1         ( 1'b1                 ),
          .clocken0       ( 1'b1                 ),
          .clocken1       ( 1'b1                 ),
          .clocken2       ( 1'b1                 ),
          .clocken3       ( 1'b1                 ),
          .data_b         ( {32 {1'b1}}          ), // Not used.
          .eccencbypass   ( 1'b0                 ), // Not used.
          .eccencparity   ( 8'b0                 ), // Not used.
          .q_a            (                      ), // Not used.
          .rden_a         ( 1'b1                 ), // Not used.
          .wren_b         ( 1'b0                 ), // Not used.
          .eccstatus      (                      )  // Not used.
          );
        defparam
         data_par_memory.address_aclr_b              = "NONE",
         data_par_memory.address_reg_b               = "CLOCK0",
         data_par_memory.clock_enable_input_a        = "BYPASS",
         data_par_memory.clock_enable_input_b        = "BYPASS",
         data_par_memory.clock_enable_output_b       = "BYPASS",
         data_par_memory.enable_ecc                  = "FALSE",
         data_par_memory.ecc_pipeline_stage_enabled  = "FALSE",
         data_par_memory.intended_device_family      = FAMILY,
         data_par_memory.lpm_type                    = "altera_syncram",
         data_par_memory.maximum_depth               = 512,
         data_par_memory.numwords_a                  = MEM_SIZE,
         data_par_memory.numwords_b                  = MEM_SIZE,
         data_par_memory.operation_mode              = "DUAL_PORT",
         data_par_memory.outdata_aclr_b              = "NONE",
         data_par_memory.outdata_reg_b               = "CLOCK0",
         data_par_memory.power_up_uninitialized      = "FALSE",
         data_par_memory.ram_block_type              = MEM_TYPE,
         data_par_memory.widthad_a                   = MEM_SIZE_WIDTH,
         data_par_memory.widthad_b                   = MEM_SIZE_WIDTH,
         data_par_memory.width_a                     = UI_WIDTH/8,
         data_par_memory.width_b                     = UI_WIDTH/8,
         //data_par_memory.enable_ecc                  = "TRUE",
         //data_par_memory.ecc_pipeline_stage_enabled  = "TRUE", // this really is required for respectable Q speed with ECC
         data_par_memory.width_byteena_a             = 1;
       end
    
      altera_syncram  hdr_par_memory
      (
        .clock0         ( clk                  ),
        .wren_a         ( header_write         ),
        .address_a      ( wr_ptr               ),
        .data_a         ( wheader_par          ),
        .rden_b         ( 1'b1                 ),
        .address_b      ( rd_ptr               ),
        .q_b            ( rheader_par_wire     ),
        .aclr0          ( 1'b0                 ),
        .aclr1          ( 1'b0                 ),
        .sclr           ( 1'b0                 ), // Not used.
        .addressstall_a ( 1'b0                 ),
        .addressstall_b ( 1'b0                 ),
        .byteena_a      ( 1'b1                 ),
        .byteena_b      ( 1'b1                 ),
        .clock1         ( 1'b1                 ),
        .clocken0       ( 1'b1                 ),
        .clocken1       ( 1'b1                 ),
        .clocken2       ( 1'b1                 ),
        .clocken3       ( 1'b1                 ),
        .data_b         ( {32 {1'b1}}          ), // Not used.
        .eccencbypass   ( 1'b0                 ), // Not used.
        .eccencparity   ( 8'b0                 ), // Not used.
        .q_a            (                      ), // Not used.
        .rden_a         ( 1'b1                 ), // Not used.
        .wren_b         ( 1'b0                 ), // Not used.
        .eccstatus      (                      )  // Not used.
        );
      defparam
       hdr_par_memory.address_aclr_b              = "NONE",
       hdr_par_memory.address_reg_b               = "CLOCK0",
       hdr_par_memory.clock_enable_input_a        = "BYPASS",
       hdr_par_memory.clock_enable_input_b        = "BYPASS",
       hdr_par_memory.clock_enable_output_b       = "BYPASS",
       hdr_par_memory.enable_ecc                  = "FALSE",
       hdr_par_memory.ecc_pipeline_stage_enabled  = "FALSE",
       hdr_par_memory.intended_device_family      = FAMILY,
       hdr_par_memory.lpm_type                    = "altera_syncram",
       hdr_par_memory.maximum_depth               = 512,
       hdr_par_memory.numwords_a                  = MEM_SIZE,
       hdr_par_memory.numwords_b                  = MEM_SIZE,
       hdr_par_memory.operation_mode              = "DUAL_PORT",
       hdr_par_memory.outdata_aclr_b              = "NONE",
       hdr_par_memory.outdata_reg_b               = "CLOCK0",
       hdr_par_memory.power_up_uninitialized      = "FALSE",
       hdr_par_memory.ram_block_type              = MEM_TYPE,
       hdr_par_memory.widthad_a                   = MEM_SIZE_WIDTH,
       hdr_par_memory.widthad_b                   = MEM_SIZE_WIDTH,
       hdr_par_memory.width_a                     = HEADER_PAR_MEM_WIDTH,
       hdr_par_memory.width_b                     = HEADER_PAR_MEM_WIDTH,
       //hdr_memory.enable_ecc                  = "TRUE",
       //hdr_memory.ecc_pipeline_stage_enabled  = "TRUE", // this really is required for respectable Q speed with ECC
       hdr_par_memory.width_byteena_a             = 1;
    
    
    
    end
    
    
    for (i=0; i < NUM_HEADER_MEM; i=i+1 ) begin : header_fifo_memory
      altera_syncram  hdr_memory
      (
        .clock0         ( clk                              ),
        .wren_a         ( header_write                     ),
        .address_a      ( wr_ptr                           ),
        .data_a         ( wheader[32*i+31:32*i]            ),
        .rden_b         ( 1'b1                             ),
        .address_b      ( rd_ptr                           ),
        .q_b            ( rheader_wire[32*i+31:32*i]         ),
        .aclr0          ( 1'b0                 ),
        .aclr1          ( 1'b0                 ),
        .sclr           ( 1'b0                 ), // Not used.
        .addressstall_a ( 1'b0                 ),
        .addressstall_b ( 1'b0                 ),
        .byteena_a      ( 1'b1                 ),
        .byteena_b      ( 1'b1                 ),
        .clock1         ( 1'b1                 ),
        .clocken0       ( 1'b1                 ),
        .clocken1       ( 1'b1                 ),
        .clocken2       ( 1'b1                 ),
        .clocken3       ( 1'b1                 ),
        .data_b         ( {32 {1'b1}}          ), // Not used.
        .eccencbypass   ( 1'b0                 ), // Not used.
        .eccencparity   ( 8'b0                 ), // Not used.
        .q_a            (                      ), // Not used.
        .rden_a         ( 1'b1                 ), // Not used.
        .wren_b         ( 1'b0                 ), // Not used.
        .eccstatus      (                      )  // Not used.
        );
      defparam
       hdr_memory.address_aclr_b              = "NONE",
       hdr_memory.address_reg_b               = "CLOCK0",
       hdr_memory.clock_enable_input_a        = "BYPASS",
       hdr_memory.clock_enable_input_b        = "BYPASS",
       hdr_memory.clock_enable_output_b       = "BYPASS",
       hdr_memory.enable_ecc                  = "FALSE",
       hdr_memory.ecc_pipeline_stage_enabled  = "FALSE",
       hdr_memory.intended_device_family      = FAMILY,
       hdr_memory.lpm_type                    = "altera_syncram",
       hdr_memory.maximum_depth               = 512,
       hdr_memory.numwords_a                  = MEM_SIZE,
       hdr_memory.numwords_b                  = MEM_SIZE,
       hdr_memory.operation_mode              = "DUAL_PORT",
       hdr_memory.outdata_aclr_b              = "NONE",
       hdr_memory.outdata_reg_b               = "CLOCK0",
       hdr_memory.power_up_uninitialized      = "FALSE",
       hdr_memory.ram_block_type              = MEM_TYPE,
       hdr_memory.widthad_a                   = MEM_SIZE_WIDTH,
       hdr_memory.widthad_b                   = MEM_SIZE_WIDTH,
       hdr_memory.width_a                     = 32,
       hdr_memory.width_b                     = 32,
       //hdr_memory.enable_ecc                  = "TRUE",
       //hdr_memory.ecc_pipeline_stage_enabled  = "TRUE", // this really is required for respectable Q speed with ECC
       hdr_memory.width_byteena_a             = 1;
   end
   alt_mlab control_memory (
      .wclk               (clk                               ),
      .wena               (incr_wr_ptr                       ),
      .waddr_reg          (wr_ptr                            ),
      .wdata_reg          ({tx_st_sop_rr,tx_st_eop_rr,tx_st_err_rr}            ),
      .raddr              (rd_ptr                            ),
      .rdata              ({tx_st_sop_hip_wire,tx_st_eop_hip_wire,tx_st_err_hip_wire}            )
      );
    defparam control_memory .SIM_EMULATE = SIM_EMULATE;
    defparam control_memory .WIDTH = 3;
    defparam control_memory .ADDR_WIDTH = MEM_SIZE_WIDTH;
    /* altera_syncram  control_memory
      (
        .clock0         ( clk                              ),
        .wren_a         ( incr_wr_ptr                      ),
        .address_a      ( wr_ptr                           ),
        .data_a         ( {tx_st_sop_rr,tx_st_eop_rr,tx_st_err_rr} ),
        .rden_b         ( 1'b1                             ),
        .address_b      ( rd_ptr                           ),
        .q_b            ( {tx_st_sop_hip_wire,tx_st_eop_hip_wire,tx_st_err_hip_wire}),
        .aclr0          ( 1'b0                 ),
        .aclr1          ( 1'b0                 ),
        .sclr           ( 1'b0                 ), // Not used.
        .addressstall_a ( 1'b0                 ),
        .addressstall_b ( 1'b0                 ),
        .byteena_a      ( 1'b1                 ),
        .byteena_b      ( 1'b1                 ),
        .clock1         ( 1'b1                 ),
        .clocken0       ( 1'b1                 ),
        .clocken1       ( 1'b1                 ),
        .clocken2       ( 1'b1                 ),
        .clocken3       ( 1'b1                 ),
        .data_b         ( {32 {1'b1}}          ), // Not used.
        .eccencbypass   ( 1'b0                 ), // Not used.
        .eccencparity   ( 8'b0                 ), // Not used.
        .q_a            (                      ), // Not used.
        .rden_a         ( 1'b1                 ), // Not used.
        .wren_b         ( 1'b0                 ), // Not used.
        .eccstatus      (                      )  // Not used.
        );
      defparam
       control_memory.address_aclr_b              = "NONE",
       control_memory.address_reg_b               = "CLOCK0",
       control_memory.clock_enable_input_a        = "BYPASS",
       control_memory.clock_enable_input_b        = "BYPASS",
       control_memory.clock_enable_output_b       = "BYPASS",
       control_memory.enable_ecc                  = "FALSE",
       control_memory.ecc_pipeline_stage_enabled  = "FALSE",
       control_memory.intended_device_family      = FAMILY,
       control_memory.lpm_type                    = "altera_syncram",
       control_memory.maximum_depth               = 512,
       control_memory.numwords_a                  = MEM_SIZE,
       control_memory.numwords_b                  = MEM_SIZE,
       control_memory.operation_mode              = "DUAL_PORT",
       control_memory.outdata_aclr_b              = "NONE",
       control_memory.outdata_reg_b               = "CLOCK0",
       control_memory.power_up_uninitialized      = "FALSE",
       control_memory.ram_block_type              = MEM_TYPE,
       control_memory.widthad_a                   = MEM_SIZE_WIDTH,
       control_memory.widthad_b                   = MEM_SIZE_WIDTH,
       control_memory.width_a                     = 3,
       control_memory.width_b                     = 3,
       control_memory.width_byteena_a             = 1; */


    for (j=0; j < WIDTH_RATIO; j=j+1 ) begin : data_fifo_memory_write
      always_ff @(posedge clk) begin
        if (rst) begin
          data_write[j] <= '0;
        end
        else begin
          if (tx_st_valid_r & select_line[j]) begin
            data_write[j] <= 1'b1;
          end
          else begin
            data_write[j] <= 1'b0;
          end
        end
      end
    end


  endgenerate




endmodule
