// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//This module is to decode CII write to ARI CAP register for PF0 and create shadow register for the following:
//1. ARI_ACS_FUN_GRP_CAP (bit 1) 
//2. ARI_ACS_FUN_GRP_EN (bit 17)
//3. ARI_FUN_GRP (bit 22:20)

//These bits are read-write bits in which user able to overwrite it through CII write. 
//During CII override workaround for ARI_NEXT_FUN_NUM, these bits will be taken from shadow register.

module pcie_cii_ari_decode #(
	parameter PF_COUNT = 1 
) (
	
	input clk,
	input reset_n, 

	input        aib_cii_req_i,
	input        aib_cii_wr_i,
	input [9:0]  aib_cii_addr_i,
	input [31:0] aib_cii_dout_i,
	input        aib_cii_wr_vf_active_i,

	output reg       ari_acs_fun_grp_en_o,
	output reg [2:0] ari_fun_grp_o,
	output reg       ari_acs_fun_grp_cap_o

);

//ARI address
localparam ADDR = 10'h5e;

reg addr_hit_byte;

assign addr_hit_byte  = (aib_cii_addr_i == ADDR) ? 1'b1 : 1'b0;

always @ (posedge clk) begin
	if (~reset_n) begin
		ari_acs_fun_grp_en_o  <= 1'b0;
		ari_fun_grp_o         <= 3'h0;
		ari_acs_fun_grp_cap_o <= 1'b0;
	end
	else if (aib_cii_req_i && aib_cii_wr_i && addr_hit_byte && ~aib_cii_wr_vf_active_i) begin 
		if (PF_COUNT == 1) begin
			ari_acs_fun_grp_en_o  <= aib_cii_dout_i[17];
			ari_fun_grp_o         <= aib_cii_dout_i[22:20];
			ari_acs_fun_grp_cap_o <= 1'b1; 
		end else begin 
			ari_acs_fun_grp_en_o  <= 1'b0;
			ari_fun_grp_o         <= 3'h0;
			ari_acs_fun_grp_cap_o <= 1'b0;
		end
	end 
	else begin
		ari_acs_fun_grp_en_o  <= ari_acs_fun_grp_en_o;
		ari_fun_grp_o         <= ari_fun_grp_o;
		ari_acs_fun_grp_cap_o <= ari_acs_fun_grp_cap_o;
	end
end

endmodule
