// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//This module is to provide internal override on targeted address through CII interface as workaround.

//PF Overwrite
//1. AER: On WRITE of 0x108 or 0x10C, set bits [24:25] to 0. (HSD#: 1409768853) 
//        0x108: Bit 24 --> ATOMIC_EGRESS_BLOCKED_ERR_STATUS
//               Bit 25 --> TLP_PRFX_BLOCKED_ERR_STATUS
//        0x10C: Bit 24 --> ATOMIC_EGRESS_BLOCKED_ERR_MASK
//               Bit 25 --> TLP_PRFX_BLOCKED_ERR_MASK

//2. ARI: On READ of 0x17C, set bitsc [8:15] to 0 for last enabled PF. 
//        0x17C: Bits [8:15] --> ARI_NEXT_FUN_NUM

//VF Overwrtie (VF_ACTIVE == 1)
//3. Link Status: On READ of 0x080, set bits [16:31] to 0.

//4. Device Control2: On READ of 0x098, set bit 12 to 0.
//                    0x098: Bit 12 --> PCIE_CAP_10BITS_TAG_REQ_EN

//5. ATS: On WRITE of 0x1A0, sets bits [16:20] to 0.
//        0x1A0: Bits [16:20] --> STU

module pcie_cii_workaround #(
 parameter integer OVR_NUM 		    = 3,
 parameter [9:0]   OVR_ADDR [7:0] 	= {10'h0, 10'h0, 10'h0, 10'h0, 10'h0, 10'h5f, 10'h5f , 10'h5f}, // maximum 8 function
 parameter [3:0]   OVR_BE   [7:0] 	= {4'b0, 4'b0, 4'b0, 4'b0, 4'b0, 4'b0110, 4'b0010, 4'b0001} ,
 parameter bit	   OVR_WR   [7:0] 	= {1'b0, 1'b0, 1'b0, 1'b0, 1'b0, 1'b0, 1'b1, 1'b1}, 
 parameter bit     OVR_ALL_PFVF  [7:0]  = {1'b0, 1'b0, 1'b0, 1'b0, 1'b0, 1'b0, 1'b0, 1'b0},
 parameter integer OVR_FUNC_NUM  [7:0] 	= {8'd0, 8'd0, 8'd0, 8'd0, 8'd0, 8'd0, 8'd1, 8'd2} ,
 parameter bit 	   OVR_VF_ACTIVE [7:0]	= {1'b0, 1'b0, 1'b0, 1'b0, 1'b0, 1'b1, 1'b0, 1'b0} ,
 //parameter [31:0] OVR_DATA[7:0] 	= {32'h0, 32'h0, 32'h0, 32'h0, 32'h0, 32'hffff_00ff, 32'hffff_0000 , 32'hffff_00ff},
 parameter [31:0]  OVR_BIT_SEL   [7:0]  = {32'h0, 32'h0, 32'h0, 32'h0, 32'h0, 32'h0000_00ff, 32'h0000_000f , 32'h0000_ff00}

// add selector for data out 
//check OVR_NUM =0 

  )(  
        input clk,
        input reset_n,

        //override data from tl_cfg_decode block
        input [255:0] ovr_data_i,

        //interface from/to aib 
        input         aib_cii_req_i,
        input         aib_cii_hdr_poisoned_i,
        input  [3:0]  aib_cii_hdr_first_be_i,
        input  [2:0]  aib_cii_func_num_i,
        input         aib_cii_wr_vf_active_i,
        input  [10:0] aib_cii_vf_num_i,
        input         aib_cii_wr_i,
        input  [9:0]  aib_cii_addr_i,
        input  [31:0] aib_cii_dout_i,

        output reg        aib_cii_override_en_o,
        output reg [31:0] aib_cii_override_din_o,
        output reg        aib_cii_halt_o,

        //interface from/to virtio/user 
        output reg        cii_req_o,
        output reg        cii_hdr_poisoned_o,
        output reg [3:0]  cii_hdr_first_be_o,
        output reg [2:0]  cii_func_num_o,
        output reg        cii_wr_vf_active_o,
        output reg [10:0] cii_vf_num_o,
        output reg	      cii_wr_o,
        output reg [9:0]  cii_addr_o,
        output reg [31:0] cii_dout_o,
        input         cii_override_en_i,
        input  [31:0] cii_override_din_i,
        input 	      cii_halt_i   
);
 


//concatenated data output of all the generate req_ovr modules
reg [OVR_NUM-1:0] cii_ovr_en_int_c;
reg [8*32-1:0]    cii_ovr_din_c;
reg [OVR_NUM-1:0] match_params_c;

//reg for input flop
reg         cii_req_r;
reg         cii_hdr_poisoned_r;
reg  [3:0]  cii_hdr_first_be_r;
reg  [2:0]  cii_func_num_r;
reg         cii_wr_vf_active_r;
reg  [10:0] cii_vf_num_r;
reg         cii_wr_r;
reg  [9:0]  cii_addr_r;
reg  [31:0] cii_dout_r;  


//internal signals for internal override trigger 
reg 		cii_req_int;
reg 		cii_req_int_r;
reg 		cii_req_int_r2;
reg 		cii_req_int_r3;
reg 		cii_req_int_r4;
reg [31:0]  cii_dout_int;
reg [31:0]  cii_dout_int_r;
reg [31:0]  cii_dout_int_r2;
reg [2:0]   cii_func_num_int;   
reg         cii_hdr_poisoned_int;
reg [3:0]   cii_hdr_first_be_int;
reg         cii_wr_vf_active_int;
reg [10:0]  cii_vf_num_int;
reg         cii_wr_int;
reg [9:0]   cii_addr_int;
reg         cii_override_en_int;
reg [31:0]  cii_override_din_int;
reg         cii_req_rise;
reg         cii_req_rise_r;
reg         cii_req_rise_r2;
reg         cii_req_rise_r3;
reg         no_match;
reg         no_match_r;

//register to hold param values. bit width determine by OVR_NUM.
reg [(OVR_NUM)*3-1:0]   pr_ovr_func_num;
reg [(OVR_NUM)*1-1:0]   pr_ovr_all_pfvf;
reg [(OVR_NUM)*10-1:0]  pr_ovr_addr;
reg [(OVR_NUM)*4-1:0]   pr_ovr_be;
reg [(OVR_NUM)*1-1:0]   pr_ovr_wr;
reg [(OVR_NUM)*1-1:0]   pr_ovr_vf_active;
reg [(OVR_NUM)*32-1:0]  pr_ovr_data; //Replaced by input instead of param
reg [(OVR_NUM)*32-1:0]  pr_ovr_bit_sel;
  
reg [31:0] ovr_din_num1;
reg [31:0] ovr_din_num2;
reg [31:0] ovr_din_num3;
reg [31:0] ovr_din_num4;
reg [31:0] ovr_din_num5;
reg [31:0] ovr_din_num6;
reg [31:0] ovr_din_num7;
reg [31:0] ovr_din_num8;

reg [31:0] cii_din_int_req;

reg        cii_override_en_r;
reg        cii_override_en_r2;
reg        cii_override_en_r3;
reg        cii_override_en_r4;
reg [31:0] cii_override_din_r;
reg [31:0] cii_override_din_r2;
reg [31:0] cii_override_din_r3;
reg [31:0] cii_override_din_r4;
reg        cii_halt_r;
reg        cii_halt_r2;
reg        cii_halt_r3;
reg        cii_halt_r4;


reg match_params_r;
reg match_params_r2;
reg match_params_r3;

//Output to user/virtio
always@ (posedge clk) begin
   if (~reset_n) begin
      cii_hdr_poisoned_o <=   1'h0;
      cii_hdr_first_be_o <=   4'h0;
      cii_func_num_o     <=   3'h0;
      cii_wr_vf_active_o <=   1'h0;
      cii_vf_num_o       <=   11'h0;
      cii_wr_o           <=   1'h0;
      cii_addr_o         <=   10'h0;
      cii_dout_o         <=   32'h0;
   end
   else begin
      cii_hdr_poisoned_o <=   cii_hdr_poisoned_r;
      cii_hdr_first_be_o <=   cii_hdr_first_be_r;
      cii_func_num_o     <=   cii_func_num_r;
      cii_wr_vf_active_o <=   cii_wr_vf_active_r;
      cii_vf_num_o       <=   cii_vf_num_r;
      cii_wr_o           <=   cii_wr_r;
      cii_addr_o         <=   cii_addr_r;
      cii_dout_o         <=   cii_dout_r;
   end 
end

always @(posedge clk) begin
	cii_hdr_poisoned_r   <= aib_cii_hdr_poisoned_i;
	cii_hdr_first_be_r   <= aib_cii_hdr_first_be_i;
	cii_func_num_r       <= aib_cii_func_num_i;
	cii_wr_vf_active_r   <= aib_cii_wr_vf_active_i;
    cii_vf_num_r         <= aib_cii_vf_num_i;
    cii_wr_r             <= aib_cii_wr_i;
    cii_addr_r           <= aib_cii_addr_i;
    cii_dout_r           <= aib_cii_dout_i;
    cii_req_r            <= aib_cii_req_i;
end

//Internal signal for internal override
assign   cii_hdr_poisoned_int	=   cii_hdr_poisoned_r;
assign   cii_hdr_first_be_int   =   cii_hdr_first_be_r;
assign   cii_func_num_int       =   cii_func_num_r;
assign   cii_wr_vf_active_int   =   cii_wr_vf_active_r;
assign   cii_vf_num_int         =   cii_vf_num_r;
assign   cii_wr_int             =   cii_wr_r;
assign   cii_addr_int           =   cii_addr_r;
assign   cii_dout_int           =   cii_dout_r;
assign   cii_req_int            =   cii_req_r;


//finding edge for cii_req, flopping inputs
always@( posedge clk )
begin
 if (~reset_n) begin 
   cii_req_int_r    <= 'b0;
   cii_req_int_r2   <= 'b0;
   cii_req_int_r3   <= 'b0;
   cii_req_int_r4   <= 'b0;
   cii_dout_int_r   <= 'b0;
   cii_dout_int_r2  <= 'b0;
   cii_req_rise_r   <= 'b0;
   cii_req_rise_r2  <= 'b0;
   cii_req_rise_r3  <= 'b0;

   cii_halt_r          <= 1'b0;
   cii_halt_r2         <= 1'b0;
   cii_halt_r3         <= 1'b0;
   cii_halt_r4         <= 1'b0;
   cii_override_en_r   <= 1'b0;
   cii_override_en_r2  <= 1'b0;
   cii_override_en_r3  <= 1'b0;
   cii_override_en_r4  <= 1'b0;
   cii_override_din_r  <= 32'b0;
   cii_override_din_r2 <= 32'b0;
   cii_override_din_r3 <= 32'b0;
   cii_override_din_r4 <= 32'b0;

   match_params_r2  <= 1'b0;
   match_params_r3  <= 1'b0;

 end 
 else begin
   cii_req_int_r    <=   cii_req_int;
   cii_req_int_r2   <=   cii_req_int_r;
   cii_req_int_r3   <=   cii_req_int_r2;
   cii_req_int_r4   <=   cii_req_int_r3; //used to output to virtio/user
   cii_dout_int_r   <=   cii_dout_int;
   cii_dout_int_r2  <=   cii_dout_int;
   cii_req_rise_r   <=   cii_req_rise;
   cii_req_rise_r2  <=   cii_req_rise_r;
   cii_req_rise_r3  <=   cii_req_rise_r2;
   
   cii_halt_r          <= cii_halt_i;
   cii_halt_r2         <= cii_halt_r;
   cii_halt_r3         <= cii_halt_r2;
   cii_halt_r4         <= cii_halt_r3;
   cii_override_en_r   <= cii_override_en_i;
   cii_override_en_r2  <= cii_override_en_r;
   cii_override_en_r3  <= cii_override_en_r2;
   cii_override_en_r4  <= cii_override_en_r3;
   cii_override_din_r  <= cii_override_din_i;
   cii_override_din_r2 <= cii_override_din_r;
   cii_override_din_r3 <= cii_override_din_r2;
   cii_override_din_r4 <= cii_override_din_r3;

   match_params_r2 <= match_params_r;
   match_params_r3 <= match_params_r2;
 end 
end

assign cii_req_rise = !cii_req_int_r && cii_req_int;

//generate based on OVR_NUM params inputs
  genvar i; 
  generate 
     for ( i=1; i<=OVR_NUM ; i=i+1) begin : map_params
        assign pr_ovr_func_num  [(i-1)*3  +:3]  = OVR_FUNC_NUM[i-1] ; 
        assign pr_ovr_all_pfvf  [(i-1)*1  +:1]  = OVR_ALL_PFVF[i-1] ; 
        assign pr_ovr_addr      [(i-1)*10 +:10] = OVR_ADDR[i-1] ;
        assign pr_ovr_be        [(i-1)*4 +:4] 	= OVR_BE[i-1] ;
        assign pr_ovr_wr        [(i-1)*1 +:1] 	= OVR_WR[i-1] ;
        assign pr_ovr_vf_active	[(i-1)*1 +:1] 	= OVR_VF_ACTIVE[i-1] ;
        assign pr_ovr_bit_sel  	[(i-1)*32 +:32] = OVR_BIT_SEL[i-1] ;
 
        //pr_ovr_data get the data from input instead of param
        assign pr_ovr_data   	[(i-1)*32 +:32] = ovr_data_i[(i-1)*32 +:32] ;
 

        req_ovr nst_req_ovr (
          .clk                  (clk),	
          .reset_n              (reset_n),
          .pr_ovr_func_num      (pr_ovr_func_num [(i-1)*3  +:3]),
          .pr_ovr_all_pfvf      (pr_ovr_all_pfvf [(i-1)*1  +:1]),
          .pr_ovr_addr          (pr_ovr_addr	 [(i-1)*10 +:10]),
          .pr_ovr_be            (pr_ovr_be       [(i-1)*4 +:4] ),
          .pr_ovr_wr            (pr_ovr_wr       [(i-1)*1 +:1]),
          .pr_ovr_vf_active     (pr_ovr_vf_active[(i-1)*1 +:1]),
          .pr_ovr_data          (pr_ovr_data	 [(i-1)*32 +:32]),
          .pr_ovr_bit_sel       (pr_ovr_bit_sel  [(i-1)*32 +:32]),
          .cii_req_int          (cii_req_int),
          .cii_req_rise         (cii_req_rise_r),
          .cii_func_num_int     (cii_func_num_int),
          .cii_addr_int         (cii_addr_int),
          .cii_be_int           (cii_hdr_first_be_int),
          .cii_wr_int           (cii_wr_int),
          .cii_wr_vf_active_int (cii_wr_vf_active_int),
          .cii_vf_num_int       (cii_vf_num_int),
          .cii_dout_int         (cii_dout_int),
          .cii_override_en_int	(cii_ovr_en_int_c[(i-1)]), 
          .cii_override_din_int (cii_ovr_din_c   [(i-1)*32 +:32]), 
      	  .match_params_int     (match_params_c  [(i-1)])
      );
      end //for loop
 endgenerate 



//###############################################################################################################################################################

//mapping internal din override to registers  
//CKL: review flop for performance improvement (not needed if no timing issue)
always@( posedge clk )
begin
 if (~reset_n) begin 
  ovr_din_num1 <= 32'b0;
  ovr_din_num2 <= 32'b0; 
  ovr_din_num3 <= 32'b0; 
  ovr_din_num4 <= 32'b0; 
  ovr_din_num5 <= 32'b0; 
  ovr_din_num6 <= 32'b0; 
  ovr_din_num7 <= 32'b0; 
  ovr_din_num8 <= 32'b0; 
 end 
 else if (OVR_NUM==1) begin
  ovr_din_num1 <=  cii_ovr_din_c[31:0];
  ovr_din_num2 <=  32'b0;
  ovr_din_num3 <=  32'b0;
  ovr_din_num4 <=  32'b0;
  ovr_din_num5 <=  32'b0;
  ovr_din_num6 <=  32'b0;
  ovr_din_num7 <=  32'b0;
  ovr_din_num8 <=  32'b0;
 end 
 else if(OVR_NUM ==2 ) begin 
  ovr_din_num1 <=  cii_ovr_din_c[31:0];
  ovr_din_num2 <=  cii_ovr_din_c[63:32];
  ovr_din_num3 <=  32'b0;
  ovr_din_num4 <=  32'b0;
  ovr_din_num5 <=  32'b0;
  ovr_din_num6 <=  32'b0;
  ovr_din_num7 <=  32'b0;
  ovr_din_num8 <=  32'b0;
 end
 else if(OVR_NUM ==3 ) begin 
  ovr_din_num1 <=  cii_ovr_din_c[31:0];
  ovr_din_num2 <=  cii_ovr_din_c[63:32];
  ovr_din_num3 <=  cii_ovr_din_c[95:64];
  ovr_din_num4 <=  32'b0;
  ovr_din_num5 <=  32'b0;
  ovr_din_num6 <=  32'b0;
  ovr_din_num7 <=  32'b0;
  ovr_din_num8 <=  32'b0;
 end 
 else if(OVR_NUM ==4 ) begin 
  ovr_din_num1 <=  cii_ovr_din_c[31:0];
  ovr_din_num2 <=  cii_ovr_din_c[63:32];
  ovr_din_num3 <=  cii_ovr_din_c[95:64];
  ovr_din_num4 <=  cii_ovr_din_c[127:96];
  ovr_din_num5 <=  32'b0;
  ovr_din_num6 <=  32'b0;
  ovr_din_num7 <=  32'b0;
  ovr_din_num8 <=  32'b0;
 end 
 else if(OVR_NUM ==5 ) begin 
  ovr_din_num1 <=  cii_ovr_din_c[31:0];
  ovr_din_num2 <=  cii_ovr_din_c[63:32];
  ovr_din_num3 <=  cii_ovr_din_c[95:64];
  ovr_din_num4 <=  cii_ovr_din_c[127:96];
  ovr_din_num5 <=  cii_ovr_din_c[159:128];
  ovr_din_num6 <=  32'b0;
  ovr_din_num7 <=  32'b0;
  ovr_din_num8 <=  32'b0;
 end 
 else if(OVR_NUM ==6 ) begin 
  ovr_din_num1 <=  cii_ovr_din_c[31:0];
  ovr_din_num2 <=  cii_ovr_din_c[63:32];
  ovr_din_num3 <=  cii_ovr_din_c[95:64];
  ovr_din_num4 <=  cii_ovr_din_c[127:96];
  ovr_din_num5 <=  cii_ovr_din_c[159:128];
  ovr_din_num6 <=  cii_ovr_din_c[191:160];
  ovr_din_num7 <=  32'b0;
  ovr_din_num8 <=  32'b0;
 end 
 else if(OVR_NUM ==7 ) begin 
  ovr_din_num1 <=  cii_ovr_din_c[31:0];
  ovr_din_num2 <=  cii_ovr_din_c[63:32];
  ovr_din_num3 <=  cii_ovr_din_c[95:64];
  ovr_din_num4 <=  cii_ovr_din_c[127:96];
  ovr_din_num5 <=  cii_ovr_din_c[159:128];
  ovr_din_num6 <=  cii_ovr_din_c[191:160];
  ovr_din_num7 <=  cii_ovr_din_c[223:192];
  ovr_din_num8 <=  32'b0;
 end 
 else if(OVR_NUM ==8 ) begin 
  ovr_din_num1 <=  cii_ovr_din_c[31:0];
  ovr_din_num2 <=  cii_ovr_din_c[63:32];
  ovr_din_num3 <=  cii_ovr_din_c[95:64];
  ovr_din_num4 <=  cii_ovr_din_c[127:96];
  ovr_din_num5 <=  cii_ovr_din_c[159:128];
  ovr_din_num6 <=  cii_ovr_din_c[191:160];
  ovr_din_num7 <=  cii_ovr_din_c[223:192];
  ovr_din_num8 <=  cii_ovr_din_c[255:224];
 end 
 else begin 
  ovr_din_num1 <= 32'b0;
  ovr_din_num2 <= 32'b0; 
  ovr_din_num3 <= 32'b0; 
  ovr_din_num4 <= 32'b0; 
  ovr_din_num5 <= 32'b0; 
  ovr_din_num6 <= 32'b0; 
  ovr_din_num7 <= 32'b0; 
  ovr_din_num8 <= 32'b0; 
 end 
end

  for ( i=0; i<32 ; i=i+1) begin : bitwise_cii_din_req 
assign cii_din_int_req[i] = |{ovr_din_num8[i],ovr_din_num7[i],ovr_din_num6[i],ovr_din_num5[i],ovr_din_num4[i],ovr_din_num3[i],ovr_din_num2[i],ovr_din_num1[i]};
  end 

//flop the match_params
always@(posedge clk ) begin
   if (~reset_n) begin
      match_params_r <= 1'b0;
   end
   else begin 
      match_params_r <= |match_params_c[OVR_NUM-1:0];
   end
end

// cii_halt and override
always@( posedge clk )
begin
    if( ~reset_n ) begin
        aib_cii_override_en_o  <= 1'b0;
        aib_cii_halt_o         <= 1'b1;
        aib_cii_override_din_o <= 32'b0;
    end
    else begin
        if (cii_req_rise_r3 == 1'b1) begin
            if (match_params_r2 == 1'b1) begin
                aib_cii_override_en_o  <= 1'b1;
                aib_cii_halt_o         <= 1'b0;
                aib_cii_override_din_o <= cii_din_int_req;
            end
            else begin 
                aib_cii_override_en_o  <= 1'b0;
                aib_cii_halt_o         <= 1'b1;
                aib_cii_override_din_o <= 32'b0;
            end
        end
        else begin
            if (cii_req_int_r4 == 1'b1) begin
                if (match_params_r3 == 1'b1) begin
                    aib_cii_override_en_o  <= 1'b0;
                    aib_cii_halt_o         <= 1'b1;
                    aib_cii_override_din_o <= 32'b0;
                end
                else begin
                    aib_cii_override_en_o  <= cii_override_en_r4;
                    aib_cii_halt_o         <= cii_halt_r4;
                    aib_cii_override_din_o <= cii_override_din_r4;
                end
            end
            else begin 
                aib_cii_override_en_o  <= 1'b0;
                aib_cii_halt_o         <= 1'b1;
                aib_cii_override_din_o <= 32'b0;
            end	
        end
    end
end //end always

//output req to virtio/user. req & addr hit to give a cii_req=1 to virtio/user 
always@( posedge clk )
begin
    if( ~reset_n ) begin
        cii_req_o   <=  1'b0;
        no_match    <=  1'b0;
        no_match_r  <=  1'b0;
    end
    else begin
        cii_req_o   <= no_match_r ? cii_req_int   : 1'b0 ;
        no_match    <= (cii_req_rise_r2 & (|cii_ovr_en_int_c[OVR_NUM-1:0]==1'b0) )  || (no_match && cii_req_int_r);
    	no_match_r  <= no_match;
	end
end

endmodule 
//end pcie_cii_workaround






//###############################################################################################################################################################
//-------------------------------------------------------------
// Module : Masking at address hit
//-------------------------------------------------------------

module req_ovr
// parameter MAX_F_NUM = 1
(

     input          clk, 
	 input          reset_n,
     input [2:0]    pr_ovr_func_num,	
     input          pr_ovr_all_pfvf,	
     input [9:0]    pr_ovr_addr,		
     input [3:0]    pr_ovr_be,
     input          pr_ovr_wr,	  
	 input          pr_ovr_vf_active,
     input [31:0]   pr_ovr_data, 
     input [31:0]   pr_ovr_bit_sel, 
     input          cii_req_int,
	 input          cii_req_rise, 
     input [2:0]    cii_func_num_int,
     input [9:0]    cii_addr_int, 
     input [3:0]    cii_be_int,
     input          cii_wr_int,  		
     input 	        cii_wr_vf_active_int, 	
     input [10:0]   cii_vf_num_int,	 
     input [31:0]   cii_dout_int,

     output reg         cii_override_en_int,  
     output reg [31:0]  cii_override_din_int,  
     output reg         match_params_int

);

//match cii inputs, check if the match the parameters
reg [3:0] match_be; 
reg match_params;

assign match_be[0] = (pr_ovr_be[0]) ? (pr_ovr_be[0]== cii_be_int[0]) : 1'b1;
assign match_be[1] = (pr_ovr_be[1]) ? (pr_ovr_be[1]== cii_be_int[1]) : 1'b1;
assign match_be[2] = (pr_ovr_be[2]) ? (pr_ovr_be[2]== cii_be_int[2]) : 1'b1;
assign match_be[3] = (pr_ovr_be[3]) ? (pr_ovr_be[3]== cii_be_int[3]) : 1'b1;

assign match_params = (cii_req_int) ? (pr_ovr_all_pfvf == 1'b1) ? 
		 ((pr_ovr_addr == cii_addr_int) && (match_be[3:0]==4'b1111) && (pr_ovr_wr == cii_wr_int) && (pr_ovr_vf_active == cii_wr_vf_active_int)) 
		:((pr_ovr_func_num == cii_func_num_int) && (pr_ovr_addr == cii_addr_int) && (match_be[3:0]==4'b1111) && (pr_ovr_wr == cii_wr_int) && (pr_ovr_vf_active == cii_wr_vf_active_int)) : 1'b0;



always@( posedge clk )
begin
    if( ~reset_n )
    begin
        cii_override_en_int <= 1'b0;
		match_params_int    <= 1'b0;
    end

    else
    begin
		match_params_int    <= match_params;
        cii_override_en_int <= cii_req_rise & match_params_int;	
    end
end

//CKL: Review the flop 
always@( posedge clk )
begin
    if(match_params_int)
      begin
		cii_override_din_int[31]  <= (pr_ovr_bit_sel[31]) ? pr_ovr_data[31] : cii_dout_int[31];
		cii_override_din_int[30]  <= (pr_ovr_bit_sel[30]) ? pr_ovr_data[30] : cii_dout_int[30];
		cii_override_din_int[29]  <= (pr_ovr_bit_sel[29]) ? pr_ovr_data[29] : cii_dout_int[29];
		cii_override_din_int[28]  <= (pr_ovr_bit_sel[28]) ? pr_ovr_data[28] : cii_dout_int[28];
		cii_override_din_int[27]  <= (pr_ovr_bit_sel[27]) ? pr_ovr_data[27] : cii_dout_int[27];
		cii_override_din_int[26]  <= (pr_ovr_bit_sel[26]) ? pr_ovr_data[26] : cii_dout_int[26];
		cii_override_din_int[25]  <= (pr_ovr_bit_sel[25]) ? pr_ovr_data[25] : cii_dout_int[25];
		cii_override_din_int[24]  <= (pr_ovr_bit_sel[24]) ? pr_ovr_data[24] : cii_dout_int[24];
		cii_override_din_int[23]  <= (pr_ovr_bit_sel[23]) ? pr_ovr_data[23] : cii_dout_int[23];
		cii_override_din_int[22]  <= (pr_ovr_bit_sel[22]) ? pr_ovr_data[22] : cii_dout_int[22];
		cii_override_din_int[21]  <= (pr_ovr_bit_sel[21]) ? pr_ovr_data[21] : cii_dout_int[21];
		cii_override_din_int[20]  <= (pr_ovr_bit_sel[20]) ? pr_ovr_data[20] : cii_dout_int[20];
		cii_override_din_int[19]  <= (pr_ovr_bit_sel[19]) ? pr_ovr_data[19] : cii_dout_int[19];
		cii_override_din_int[18]  <= (pr_ovr_bit_sel[18]) ? pr_ovr_data[18] : cii_dout_int[18];
		cii_override_din_int[17]  <= (pr_ovr_bit_sel[17]) ? pr_ovr_data[17] : cii_dout_int[17];
		cii_override_din_int[16]  <= (pr_ovr_bit_sel[16]) ? pr_ovr_data[16] : cii_dout_int[16];
		cii_override_din_int[15]  <= (pr_ovr_bit_sel[15]) ? pr_ovr_data[15] : cii_dout_int[15];
		cii_override_din_int[14]  <= (pr_ovr_bit_sel[14]) ? pr_ovr_data[14] : cii_dout_int[14];
		cii_override_din_int[13]  <= (pr_ovr_bit_sel[13]) ? pr_ovr_data[13] : cii_dout_int[13];
		cii_override_din_int[12]  <= (pr_ovr_bit_sel[12]) ? pr_ovr_data[12] : cii_dout_int[12];
		cii_override_din_int[11]  <= (pr_ovr_bit_sel[11]) ? pr_ovr_data[11] : cii_dout_int[11];
		cii_override_din_int[10]  <= (pr_ovr_bit_sel[10]) ? pr_ovr_data[10] : cii_dout_int[10];
		cii_override_din_int[9]   <= (pr_ovr_bit_sel[9])  ? pr_ovr_data[9] : cii_dout_int[9];
		cii_override_din_int[8]   <= (pr_ovr_bit_sel[8])  ? pr_ovr_data[8] : cii_dout_int[8];
		cii_override_din_int[7]   <= (pr_ovr_bit_sel[7])  ? pr_ovr_data[7] : cii_dout_int[7];
		cii_override_din_int[6]   <= (pr_ovr_bit_sel[6])  ? pr_ovr_data[6] : cii_dout_int[6];
		cii_override_din_int[5]   <= (pr_ovr_bit_sel[5])  ? pr_ovr_data[5] : cii_dout_int[5];
		cii_override_din_int[4]   <= (pr_ovr_bit_sel[4])  ? pr_ovr_data[4] : cii_dout_int[4];
		cii_override_din_int[3]   <= (pr_ovr_bit_sel[3])  ? pr_ovr_data[3] : cii_dout_int[3];
		cii_override_din_int[2]   <= (pr_ovr_bit_sel[2])  ? pr_ovr_data[2] : cii_dout_int[2];
		cii_override_din_int[1]   <= (pr_ovr_bit_sel[1])  ? pr_ovr_data[1] : cii_dout_int[1];
		cii_override_din_int[0]   <= (pr_ovr_bit_sel[0])  ? pr_ovr_data[0] : cii_dout_int[0];

	end
    else begin
       	//	cii_override_din_int <=  32'h0000_ffff;
	        cii_override_din_int <=  32'h0000_0000;
	end
end    

endmodule 
