// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

//-----------------------------------------------------------------------------
// Description   : WHR PCI Express VirtIO Capability PF and VF Register Set.
//-----------------------------------------------------------------------------
// File          : intel_pcie_virtio_cap_pfvfreg.v
// Author        : Intel Corporation
//-----------------------------------------------------------------------------
//

module intel_pcie_virtio_cap_pfvfreg  # 
    (
        parameter       TOTAL_PF_COUNT                      = 8, // Total number of PFs 
        parameter       TOTAL_PF_COUNT_WIDTH                = 3, 
        parameter       TOTAL_VF_COUNT                      = 4, // Total number of VFs
        parameter       TOTAL_VF_COUNT_WIDTH                = 11,
        parameter       REG_ADDR_WIDTH                      = 10,   // The double word register address.
        parameter       PF0_VF_COUNT                        = 11'd1024, // Number of VFs attached to PF 0
        parameter       PF1_VF_COUNT                        = 11'd1024, // Number of VFs attached to PF 1
        parameter       PF2_VF_COUNT                        = 11'd0,    // Number of VFs attached to PF 2
        parameter       PF3_VF_COUNT                        = 11'd0,    // Number of VFs attached to PF 3
        parameter       PF4_VF_COUNT                        = 11'd0,    // Number of VFs attached to PF 4
        parameter       PF5_VF_COUNT                        = 11'd0,    // Number of VFs attached to PF 5
        parameter       PF6_VF_COUNT                        = 11'd0,    // Number of VFs attached to PF 6
        parameter       PF7_VF_COUNT                        = 11'd0,    // Number of VFs attached to PF 7
        parameter       TOTAL_PF_COUNT_MAX                  = 8,
        //****************************************************************************************************
        // VirtIO Specific Parameters PF0
        //****************************************************************************************************   
        // parameter       PF0_VIRTIO_CAPABILITY_PRESENT                    = 0,            //Indicates VIRTIO Capability is Present             
        parameter       PF0_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT           = 0,            //This is optional capability for VIRTIO
   
        parameter       PF0_VIRTIO_CMN_CONFIG_BAR_INDICATOR              = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF0_VIRTIO_NOTIFICATION_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF0_VIRTIO_ISRSTATUS_BAR_INDICATOR               = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF0_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR        = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF0_VIRTIO_DEVSPECIFIC_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF0_VIRTIO_CMN_CONFIG_BAR_OFFSET                 = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF0_VIRTIO_NOTIFICATION_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF0_VIRTIO_ISRSTATUS_BAR_OFFSET                  = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF0_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET           = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF0_VIRTIO_DEVSPECIFIC_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF0_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH           = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF0_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF0_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH            = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF0_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH     = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF0_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF0_VIRTIO_NOTIFY_OFF_MULTIPLIER                 = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF1
        //****************************************************************************************************
        // parameter       PF1_VIRTIO_CAPABILITY_PRESENT                    = 0,            //Indicates VIRTIO Capability is Present             
        parameter       PF1_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT           = 0,            //This is optional capability for VIRTIO
   
        parameter       PF1_VIRTIO_CMN_CONFIG_BAR_INDICATOR              = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF1_VIRTIO_NOTIFICATION_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF1_VIRTIO_ISRSTATUS_BAR_INDICATOR               = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF1_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR        = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF1_VIRTIO_DEVSPECIFIC_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF1_VIRTIO_CMN_CONFIG_BAR_OFFSET                 = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF1_VIRTIO_NOTIFICATION_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF1_VIRTIO_ISRSTATUS_BAR_OFFSET                  = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF1_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET           = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF1_VIRTIO_DEVSPECIFIC_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF1_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH           = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF1_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF1_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH            = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF1_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH     = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF1_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF1_VIRTIO_NOTIFY_OFF_MULTIPLIER                 = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF2
        //****************************************************************************************************
        // parameter       PF2_VIRTIO_CAPABILITY_PRESENT                    = 0,            //Indicates VIRTIO Capability is Present             
        parameter       PF2_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT           = 0,            //This is optional capability for VIRTIO
   
        parameter       PF2_VIRTIO_CMN_CONFIG_BAR_INDICATOR              = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF2_VIRTIO_NOTIFICATION_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF2_VIRTIO_ISRSTATUS_BAR_INDICATOR               = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF2_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR        = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF2_VIRTIO_DEVSPECIFIC_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF2_VIRTIO_CMN_CONFIG_BAR_OFFSET                 = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF2_VIRTIO_NOTIFICATION_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF2_VIRTIO_ISRSTATUS_BAR_OFFSET                  = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF2_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET           = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF2_VIRTIO_DEVSPECIFIC_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF2_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH           = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF2_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF2_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH            = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF2_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH     = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF2_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF2_VIRTIO_NOTIFY_OFF_MULTIPLIER                 = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF3
        //****************************************************************************************************
        // parameter       PF3_VIRTIO_CAPABILITY_PRESENT                    = 0,            //Indicates VIRTIO Capability is Present             
        parameter       PF3_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT           = 0,            //This is optional capability for VIRTIO
   
        parameter       PF3_VIRTIO_CMN_CONFIG_BAR_INDICATOR              = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF3_VIRTIO_NOTIFICATION_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF3_VIRTIO_ISRSTATUS_BAR_INDICATOR               = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF3_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR        = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF3_VIRTIO_DEVSPECIFIC_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF3_VIRTIO_CMN_CONFIG_BAR_OFFSET                 = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF3_VIRTIO_NOTIFICATION_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF3_VIRTIO_ISRSTATUS_BAR_OFFSET                  = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF3_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET           = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF3_VIRTIO_DEVSPECIFIC_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF3_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH           = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF3_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF3_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH            = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF3_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH     = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF3_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF3_VIRTIO_NOTIFY_OFF_MULTIPLIER                 = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF4
        //****************************************************************************************************   
        // parameter       PF4_VIRTIO_CAPABILITY_PRESENT                    = 0,            //Indicates VIRTIO Capability is Present             
        parameter       PF4_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT           = 0,            //This is optional capability for VIRTIO
   
        parameter       PF4_VIRTIO_CMN_CONFIG_BAR_INDICATOR              = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF4_VIRTIO_NOTIFICATION_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF4_VIRTIO_ISRSTATUS_BAR_INDICATOR               = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF4_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR        = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF4_VIRTIO_DEVSPECIFIC_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF4_VIRTIO_CMN_CONFIG_BAR_OFFSET                 = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF4_VIRTIO_NOTIFICATION_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF4_VIRTIO_ISRSTATUS_BAR_OFFSET                  = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF4_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET           = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF4_VIRTIO_DEVSPECIFIC_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF4_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH           = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF4_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF4_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH            = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF4_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH     = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF4_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF4_VIRTIO_NOTIFY_OFF_MULTIPLIER                 = 32'h00000008, //Indicates Multiplier for queue_notify_off  

        //****************************************************************************************************
        // VirtIO Specific Parameters PF5
        //****************************************************************************************************   
        // parameter       PF5_VIRTIO_CAPABILITY_PRESENT                    = 0,            //Indicates VIRTIO Capability is Present             
        parameter       PF5_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT           = 0,            //This is optional capability for VIRTIO
   
        parameter       PF5_VIRTIO_CMN_CONFIG_BAR_INDICATOR              = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF5_VIRTIO_NOTIFICATION_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF5_VIRTIO_ISRSTATUS_BAR_INDICATOR               = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF5_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR        = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF5_VIRTIO_DEVSPECIFIC_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF5_VIRTIO_CMN_CONFIG_BAR_OFFSET                 = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF5_VIRTIO_NOTIFICATION_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF5_VIRTIO_ISRSTATUS_BAR_OFFSET                  = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF5_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET           = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF5_VIRTIO_DEVSPECIFIC_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF5_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH           = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF5_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF5_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH            = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF5_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH     = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF5_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF5_VIRTIO_NOTIFY_OFF_MULTIPLIER                 = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF6
        //****************************************************************************************************   
        // parameter       PF6_VIRTIO_CAPABILITY_PRESENT                    = 0,            //Indicates VIRTIO Capability is Present             
        parameter       PF6_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT           = 0,            //This is optional capability for VIRTIO
   
        parameter       PF6_VIRTIO_CMN_CONFIG_BAR_INDICATOR              = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF6_VIRTIO_NOTIFICATION_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF6_VIRTIO_ISRSTATUS_BAR_INDICATOR               = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF6_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR        = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF6_VIRTIO_DEVSPECIFIC_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF6_VIRTIO_CMN_CONFIG_BAR_OFFSET                 = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF6_VIRTIO_NOTIFICATION_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF6_VIRTIO_ISRSTATUS_BAR_OFFSET                  = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF6_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET           = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF6_VIRTIO_DEVSPECIFIC_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF6_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH           = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF6_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF6_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH            = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF6_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH     = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF6_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF6_VIRTIO_NOTIFY_OFF_MULTIPLIER                 = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF7
        //****************************************************************************************************   
        // parameter       PF7_VIRTIO_CAPABILITY_PRESENT                    = 0,            //Indicates VIRTIO Capability is Present             
        parameter       PF7_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT           = 0,            //This is optional capability for VIRTIO
   
        parameter       PF7_VIRTIO_CMN_CONFIG_BAR_INDICATOR              = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF7_VIRTIO_NOTIFICATION_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF7_VIRTIO_ISRSTATUS_BAR_INDICATOR               = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF7_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR        = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF7_VIRTIO_DEVSPECIFIC_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF7_VIRTIO_CMN_CONFIG_BAR_OFFSET                 = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF7_VIRTIO_NOTIFICATION_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF7_VIRTIO_ISRSTATUS_BAR_OFFSET                  = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF7_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET           = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF7_VIRTIO_DEVSPECIFIC_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF7_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH           = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF7_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF7_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH            = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF7_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH     = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF7_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF7_VIRTIO_NOTIFY_OFF_MULTIPLIER                 = 32'h00000008, //Indicates Multiplier for queue_notify_off  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF0 VFs
        //****************************************************************************************************   
        // parameter       PF0VF_VIRTIO_CAPABILITY_PRESENT                  = 0,            //Indicates VIRTIO Capability is Present             
        parameter       PF0VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT         = 0,            //This is optional capability for VIRTIO
   
        parameter       PF0VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF0VF_VIRTIO_NOTIFICATION_BAR_INDICATOR          = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF0VF_VIRTIO_ISRSTATUS_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF0VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR      = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF0VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR           = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF0VF_VIRTIO_CMN_CONFIG_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF0VF_VIRTIO_NOTIFICATION_BAR_OFFSET             = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF0VF_VIRTIO_ISRSTATUS_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF0VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET         = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF0VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET              = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF0VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF0VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH       = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF0VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF0VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH   = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF0VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH        = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF0VF_VIRTIO_NOTIFY_OFF_MULTIPLIER               = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF1 VFs
        //****************************************************************************************************
        // parameter       PF1VF_VIRTIO_CAPABILITY_PRESENT                  = 0,            //Indicates VIRTIO Capability is Present             
        parameter       PF1VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT         = 0,            //This is optional capability for VIRTIO
   
        parameter       PF1VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF1VF_VIRTIO_NOTIFICATION_BAR_INDICATOR          = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF1VF_VIRTIO_ISRSTATUS_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF1VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR      = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF1VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR           = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF1VF_VIRTIO_CMN_CONFIG_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF1VF_VIRTIO_NOTIFICATION_BAR_OFFSET             = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF1VF_VIRTIO_ISRSTATUS_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF1VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET         = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF1VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET              = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF1VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF1VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH       = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF1VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF1VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH   = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF1VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH        = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF1VF_VIRTIO_NOTIFY_OFF_MULTIPLIER               = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF2 VFs
        //****************************************************************************************************
        // parameter       PF2VF_VIRTIO_CAPABILITY_PRESENT                  = 0,            //Indicates VIRTIO Capability is Present             
        parameter       PF2VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT         = 0,            //This is optional capability for VIRTIO
   
        parameter       PF2VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF2VF_VIRTIO_NOTIFICATION_BAR_INDICATOR          = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF2VF_VIRTIO_ISRSTATUS_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF2VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR      = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF2VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR           = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF2VF_VIRTIO_CMN_CONFIG_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF2VF_VIRTIO_NOTIFICATION_BAR_OFFSET             = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF2VF_VIRTIO_ISRSTATUS_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF2VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET         = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF2VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET              = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF2VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF2VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH       = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF2VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF2VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH   = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF2VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH        = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF2VF_VIRTIO_NOTIFY_OFF_MULTIPLIER               = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF3 VFs
        //****************************************************************************************************
        // parameter       PF3VF_VIRTIO_CAPABILITY_PRESENT                  = 0,            //Indicates VIRTIO Capability is Present             
        parameter       PF3VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT         = 0,            //This is optional capability for VIRTIO
   
        parameter       PF3VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF3VF_VIRTIO_NOTIFICATION_BAR_INDICATOR          = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF3VF_VIRTIO_ISRSTATUS_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF3VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR      = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF3VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR           = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF3VF_VIRTIO_CMN_CONFIG_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF3VF_VIRTIO_NOTIFICATION_BAR_OFFSET             = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF3VF_VIRTIO_ISRSTATUS_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF3VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET         = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF3VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET              = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF3VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF3VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH       = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF3VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF3VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH   = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF3VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH        = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF3VF_VIRTIO_NOTIFY_OFF_MULTIPLIER               = 32'h00000008, //Indicates Multiplier for queue_notify_off  

        //****************************************************************************************************
        // VirtIO Specific Parameters PF4 VFs
        //****************************************************************************************************   
        // parameter       PF4VF_VIRTIO_CAPABILITY_PRESENT                  = 0,            //Indicates VIRTIO Capability is Present             
        parameter       PF4VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT         = 0,            //This is optional capability for VIRTIO
   
        parameter       PF4VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF4VF_VIRTIO_NOTIFICATION_BAR_INDICATOR          = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF4VF_VIRTIO_ISRSTATUS_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF4VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR      = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF4VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR           = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF4VF_VIRTIO_CMN_CONFIG_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF4VF_VIRTIO_NOTIFICATION_BAR_OFFSET             = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF4VF_VIRTIO_ISRSTATUS_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF4VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET         = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF4VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET              = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF4VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF4VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH       = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF4VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF4VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH   = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF4VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH        = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF4VF_VIRTIO_NOTIFY_OFF_MULTIPLIER               = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF5 VFs
        //****************************************************************************************************
        // parameter       PF5VF_VIRTIO_CAPABILITY_PRESENT                  = 0,            //Indicates VIRTIO Capability is Present             
        parameter       PF5VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT         = 0,            //This is optional capability for VIRTIO
   
        parameter       PF5VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF5VF_VIRTIO_NOTIFICATION_BAR_INDICATOR          = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF5VF_VIRTIO_ISRSTATUS_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF5VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR      = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF5VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR           = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF5VF_VIRTIO_CMN_CONFIG_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF5VF_VIRTIO_NOTIFICATION_BAR_OFFSET             = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF5VF_VIRTIO_ISRSTATUS_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF5VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET         = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF5VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET              = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF5VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF5VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH       = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF5VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF5VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH   = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF5VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH        = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF5VF_VIRTIO_NOTIFY_OFF_MULTIPLIER               = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF6 VFs
        //****************************************************************************************************
        // parameter       PF6VF_VIRTIO_CAPABILITY_PRESENT                  = 0,            //Indicates VIRTIO Capability is Present             
        parameter       PF6VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT         = 0,            //This is optional capability for VIRTIO
   
        parameter       PF6VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF6VF_VIRTIO_NOTIFICATION_BAR_INDICATOR          = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF6VF_VIRTIO_ISRSTATUS_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF6VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR      = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF6VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR           = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF6VF_VIRTIO_CMN_CONFIG_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF6VF_VIRTIO_NOTIFICATION_BAR_OFFSET             = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF6VF_VIRTIO_ISRSTATUS_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF6VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET         = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF6VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET              = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF6VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF6VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH       = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF6VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF6VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH   = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF6VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH        = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF6VF_VIRTIO_NOTIFY_OFF_MULTIPLIER               = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF7 VFs
        //****************************************************************************************************
        // parameter       PF7VF_VIRTIO_CAPABILITY_PRESENT                  = 0,            //Indicates VIRTIO Capability is Present             
        parameter       PF7VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT         = 0,            //This is optional capability for VIRTIO
   
        parameter       PF7VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF7VF_VIRTIO_NOTIFICATION_BAR_INDICATOR          = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF7VF_VIRTIO_ISRSTATUS_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF7VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR      = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF7VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR           = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF7VF_VIRTIO_CMN_CONFIG_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF7VF_VIRTIO_NOTIFICATION_BAR_OFFSET             = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF7VF_VIRTIO_ISRSTATUS_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF7VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET         = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF7VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET              = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF7VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF7VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH       = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF7VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF7VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH   = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF7VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH        = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF7VF_VIRTIO_NOTIFY_OFF_MULTIPLIER               = 32'h00000008  //Indicates Multiplier for queue_notify_off  
    )
    (
        input                       clk,
        input                       rstn,
        // FLR Interafce
        input  [TOTAL_PF_COUNT-1:0]           flr_rcvd_pf_i,  // 1 bit per PF.  Level signal indicate FLR for corresponding PF is in progress.
        input                                 flr_rcvd_vf_i,  // One-cycle pulse indicates that an FLR was received from host targeting a VF.
        input  [TOTAL_PF_COUNT_WIDTH-1:0]     flr_rcvd_pf_num_i,
        input  [TOTAL_VF_COUNT_WIDTH-1:0]     flr_rcvd_vf_num_i,
        input  [TOTAL_PF_COUNT-1:0]           flr_rcvd_pf_num_match_i,
        // CII Interface
        input                                 cii_req_i,       // Indicates the CFG request is intercepted and all the other CII signals are valid.                                           
        // input                                 cii_hdr_poisoned_i,
        input  [3:0]                          cii_hdr_first_be_i,    // The first dword byte enable field in the received TLP header on CII.
        input  [TOTAL_PF_COUNT_WIDTH-1:0]     cii_func_num_i,        // The function number in the received TLP header on CII.
        input                                 cii_wr_vf_active_i,    // Indicates that the received TLP was targeted to a VF inside the controller, and is identified by cii_vf_num.    
        input  [TOTAL_VF_COUNT_WIDTH-1:0]     cii_vf_num_i,
        input                                 cii_wr_i,        // Indicates that the cii_dout_p0/1 is valid. This signal is asserted only for a configuration write request.
        input  [31:0]                         cii_dout_i,      // Received TLP payload data from the link partner to your application client. 
                                                               // The data is in little endian format. The first received payload byte is in [7:0]
        // RO individual reg read control
        input                              read_cmncfg_header_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_cmncfg_bar_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_cmncfg_bar_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_cmncfg_baroffset_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_cmncfg_baroffset_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_cmncfg_length_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_cmncfg_length_sel_i,
        input                              read_notifications_header_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_notifications_bar_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_notifications_bar_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_notifications_baroffset_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_notifications_baroffset_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_notifications_length_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_notifications_length_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_notifications_notifyoff_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_notifications_notifyoff_sel_i,
        // input                              read_isrstatus_header_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_isrstatus_header_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_isrstatus_header_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_isrstatus_bar_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_isrstatus_bar_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_isrstatus_baroffset_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_isrstatus_baroffset_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_isrstatus_length_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_isrstatus_length_sel_i,
        input                              read_devspecific_header_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_devspecific_bar_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_devspecific_bar_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_devspecific_baroffset_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_devspecific_baroffset_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_devspecific_length_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_devspecific_length_sel_i,
        input                              write_roreg_sel_i,
        input                              read_pcicfgaccess_header_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_pcicfgaccess_bar_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      write_pf_pcicfgaccess_bar_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_pcicfgaccess_baroffset_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      write_pf_pcicfgaccess_baroffset_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_pcicfgaccess_length_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      write_pf_pcicfgaccess_length_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_pf_pcicfgaccess_cfgdata_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      write_pf_pcicfgaccess_cfgdata_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_pcicfgaccess_bar_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      write_vf_pcicfgaccess_bar_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_pcicfgaccess_baroffset_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      write_vf_pcicfgaccess_baroffset_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_pcicfgaccess_length_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      write_vf_pcicfgaccess_length_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      read_vf_pcicfgaccess_cfgdata_sel_i,
        input  [TOTAL_PF_COUNT-1 : 0]      write_vf_pcicfgaccess_cfgdata_sel_i,

        // Read Data Interfaces
        output [31:0]            virtio_cmncfg_header_data_o,
        output [7:0 ]            virtio_cmncfg_bar_data_o,
        output [31:0]            virtio_cmncfg_length_data_o,
        output [31:0]            virtio_cmncfg_baroffset_data_o,
        output [31:0]            virtio_notifications_header_data_o,
        output [7:0 ]            virtio_notifications_bar_data_o,
        output [31:0]            virtio_notifications_length_data_o,
        output [31:0]            virtio_notifications_baroffset_data_o,
        output [31:0]            virtio_notifications_notifyoff_data_o,
        output [31:0]            virtio_isrstatus_header_data_o,
        output [7:0 ]            virtio_isrstatus_bar_data_o,
        output [31:0]            virtio_isrstatus_length_data_o,
        output [31:0]            virtio_isrstatus_baroffset_data_o,
        output [31:0]            virtio_devspecific_header_data_o,
        output [7:0 ]            virtio_devspecific_bar_data_o,
        output [31:0]            virtio_devspecific_length_data_o,
        output [31:0]            virtio_devspecific_baroffset_data_o,
        
        // PCI Config Access
        output [31:0]            virtio_pcicfgaccess_header_data_o,
        output [31:0]            virtio_pcicfgaccess_cfgdata_data_o,
        output [7:0 ]            virtio_pcicfgaccess_bar_data_o,
        output [31:0]            virtio_pcicfgaccess_length_data_o,
        output [31:0]            virtio_pcicfgaccess_baroffset_data_o,
        
        // User Hooks. VirtIO PCI Config Access Interface
        output reg                                virtio_pcicfg_vfaccess_o,        
        output reg [TOTAL_VF_COUNT_WIDTH-1:0]     virtio_pcicfg_vfnum_o,
        output reg [TOTAL_PF_COUNT_WIDTH-1:0]     virtio_pcicfg_pfnum_o,
        output reg [7:0]             virtio_pcicfg_bar_o,
        output reg [31:0]            virtio_pcicfg_length_o,
        output reg [31:0]            virtio_pcicfg_baroffset_o,
        output reg [31:0]            virtio_pcicfg_cfgdata_o,
        output reg                   virtio_pcicfg_cfgwr_o,        
        output reg                   virtio_pcicfg_cfgrd_o, 
              
        output                                virtio_cfgwr_req_ack_o,
        output reg                            virtio_cfgrd_req_ack_o,
        
        // input                                 virtio_pcicfg_appvf_i,     // Application indicates the read ack data targets a VF.
        input  [TOTAL_VF_COUNT_WIDTH-1:0]     virtio_pcicfg_appvfnum_i,  // Application indicates for which VF, if virtio_pcicfg_appvf_i is 1.
        input  [TOTAL_PF_COUNT_WIDTH-1:0]     virtio_pcicfg_apppfnum_i,  // Application indicates for which PF, if virtio_pcicfg_appvf_i is 0.
        input                                 virtio_pcicfg_rdack_i,     // Application Read Data Ack to store config data.
        input  [3:0]                          virtio_pcicfg_rdbe_i,      // Application indicates which byte to store.
        input  [31:0]                         virtio_pcicfg_data_i       // Data to be stored in config data registers.
                        
    );
    
    // Local Parameter Declaration
    // VirtIO Capability Local Parameters
    localparam VIRTIO_CAPID                          = 8'h09;
    
    localparam VIRTIO_CMN_CONFIG_NEXT_CAP_PTR        = 8'h58;    // Points to VirtIO Notification Capability
    localparam VIRTIO_CMN_CONFIG_TYPE                = 8'h01;    // 01 indicates capability structure is common config 
    localparam VIRTIO_CMN_CONFIG_CAP_LENGTH          = 8'h10;    // Length of common config capability    
    
    localparam VIRTIO_NOTIFICATION_NEXT_CAP_PTR      = 8'hBC;    // Points to VirtIO ISR STATUS Capability
    localparam VIRTIO_NOTIFICATION_TYPE              = 8'h02;    // 02 indicates capability structure is notification 
    localparam VIRTIO_NOTIFICATION_CAP_LENGTH        = 8'h14;    // Length of notification capability  // note the 32-bit additional notify_off_multiplier field
    
    localparam VIRTIO_ISR_STATUS_TYPE                = 8'h03;    // 03 indicates capability structure is ISR STATUS 
    localparam VIRTIO_ISR_STATUS_CAP_LENGTH          = 8'h10;    // Length of ISR STATUS capability  
    
    localparam VIRTIO_DEV_SPECIFIC_NEXT_CAP_PTR      = 8'hDC;    // Points to VirtIO PCI Config Access Capability
    localparam VIRTIO_DEV_SPECIFIC_TYPE              = 8'h04;    // 04 indicates capability structure is Device Specific 
    localparam VIRTIO_DEV_SPECIFIC_CAP_LENGTH        = 8'h10;    // Length of Device Specific capability   
    
    localparam VIRTIO_PCICFG_ACCESS_NEXT_CAP_PTR     = 8'h00;   // NULL Pointer
    localparam VIRTIO_PCICFG_ACCESS_TYPE             = 8'h05;   // 05 indicates capability structure is PCI Config Access 
    localparam VIRTIO_PCICFG_ACCESS_CAP_LENGTH       = 8'h14;   // Length of PCI Config Access capability    // note the additional pci_cfg_data field
    
    localparam TOTAL_VF_COUNT_WIDTH_MAX              = 12;
    localparam PFVF_COUNT    = {PF7_VF_COUNT[TOTAL_VF_COUNT_WIDTH_MAX-1:0],PF6_VF_COUNT[TOTAL_VF_COUNT_WIDTH_MAX-1:0],
                                PF5_VF_COUNT[TOTAL_VF_COUNT_WIDTH_MAX-1:0],PF4_VF_COUNT[TOTAL_VF_COUNT_WIDTH_MAX-1:0],
                                PF3_VF_COUNT[TOTAL_VF_COUNT_WIDTH_MAX-1:0],PF2_VF_COUNT[TOTAL_VF_COUNT_WIDTH_MAX-1:0],
                                PF1_VF_COUNT[TOTAL_VF_COUNT_WIDTH_MAX-1:0],PF0_VF_COUNT[TOTAL_VF_COUNT_WIDTH_MAX-1:0]};
    
    // Signal Declaration
    genvar i;
    // Virtio Cap Parameters collaboration
    wire   [TOTAL_PF_COUNT_MAX-1:0]      pf_virtio_device_specific_cap_present;
    wire   [TOTAL_PF_COUNT_MAX-1:0]      vf_virtio_device_specific_cap_present;
    wire   [(8*TOTAL_PF_COUNT_MAX)-1:0]     pf_virtio_isr_status_next_cap_ptr;
    wire   [(8*TOTAL_PF_COUNT_MAX)-1:0]     vf_virtio_isr_status_next_cap_ptr;
    
    wire   [(8*TOTAL_PF_COUNT_MAX)-1:0]     pf_virtio_cmn_config_bar_indicator;
    wire   [(8*TOTAL_PF_COUNT_MAX)-1:0]     vf_virtio_cmn_config_bar_indicator;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    pf_virtio_cmn_config_bar_offset;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    vf_virtio_cmn_config_bar_offset;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    pf_virtio_cmn_config_structure_length;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    vf_virtio_cmn_config_structure_length;
    wire   [(8*TOTAL_PF_COUNT_MAX)-1:0]     pf_virtio_notifications_bar_indicator;
    wire   [(8*TOTAL_PF_COUNT_MAX)-1:0]     vf_virtio_notifications_bar_indicator;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    pf_virtio_notifications_bar_offset;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    vf_virtio_notifications_bar_offset;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    pf_virtio_notifications_structure_length;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    vf_virtio_notifications_structure_length;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    pf_virtio_notify_off_multiplier;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    vf_virtio_notify_off_multiplier;
    wire   [(8*TOTAL_PF_COUNT_MAX)-1:0]     pf_virtio_isrstatus_bar_indicator;
    wire   [(8*TOTAL_PF_COUNT_MAX)-1:0]     vf_virtio_isrstatus_bar_indicator;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    pf_virtio_isrstatus_bar_offset;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    vf_virtio_isrstatus_bar_offset;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    pf_virtio_isrstatus_structure_length;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    vf_virtio_isrstatus_structure_length;
    wire   [(8*TOTAL_PF_COUNT_MAX)-1:0]     pf_virtio_devspecific_bar_indicator;
    wire   [(8*TOTAL_PF_COUNT_MAX)-1:0]     vf_virtio_devspecific_bar_indicator;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    pf_virtio_devspecific_bar_offset;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    vf_virtio_devspecific_bar_offset;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    pf_virtio_devspecific_structure_length;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    vf_virtio_devspecific_structure_length;
    wire   [(8*TOTAL_PF_COUNT_MAX)-1:0]     pf_virtio_pcicfgaccess_bar_indicator;
    wire   [(8*TOTAL_PF_COUNT_MAX)-1:0]     vf_virtio_pcicfgaccess_bar_indicator;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    pf_virtio_pcicfgaccess_bar_offset;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    vf_virtio_pcicfgaccess_bar_offset;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    pf_virtio_pcicfgaccess_structure_length;
    wire   [(32*TOTAL_PF_COUNT_MAX)-1:0]    vf_virtio_pcicfgaccess_structure_length;
    // First and second(dataor) Stage Read Data handling internal signals
    wire   [31:0]        virtio_cmncfg_header_data;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_cmncfg_bar_data;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_cmncfg_bar_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_cmncfg_bar_dataor;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_cmncfg_bar_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_cmncfg_baroffset_data;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_cmncfg_baroffset_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_cmncfg_baroffset_dataor;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_cmncfg_baroffset_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_cmncfg_length_data;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_cmncfg_length_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_cmncfg_length_dataor;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_cmncfg_length_dataor;
    wire   [31:0]        virtio_notifications_header_data;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_notifications_bar_data;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_notifications_bar_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_notifications_bar_dataor;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_notifications_bar_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_notifications_baroffset_data;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_notifications_baroffset_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_notifications_baroffset_dataor;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_notifications_baroffset_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_notifications_length_data;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_notifications_length_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_notifications_length_dataor;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_notifications_length_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_notifications_notifyoff_data;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_notifications_notifyoff_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_notifications_notifyoff_dataor;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_notifications_notifyoff_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_isrstatus_header_data;  // depends on devspecific
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_isrstatus_header_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_isrstatus_header_dataor;  // depends on devspecific
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_isrstatus_header_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_isrstatus_bar_data;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_isrstatus_bar_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_isrstatus_bar_dataor;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_isrstatus_bar_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_isrstatus_baroffset_data;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_isrstatus_baroffset_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_isrstatus_baroffset_dataor;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_isrstatus_baroffset_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_isrstatus_length_data;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_isrstatus_length_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_isrstatus_length_dataor;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_isrstatus_length_dataor;
    wire   [31:0]        virtio_devspecific_header_data;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_devspecific_bar_data;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_devspecific_bar_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_devspecific_bar_dataor;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_devspecific_bar_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_devspecific_baroffset_data;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_devspecific_baroffset_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_devspecific_baroffset_dataor;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_devspecific_baroffset_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_devspecific_length_data;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_devspecific_length_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_devspecific_length_dataor;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_devspecific_length_dataor;
    wire   [31:0]        virtio_pcicfgaccess_header_data;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_pcicfgaccess_bar_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_pcicfgaccess_bar_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_pcicfgaccess_bar_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_pcicfgaccess_bar_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_pcicfgaccess_baroffset_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_pcicfgaccess_baroffset_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_pcicfgaccess_baroffset_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_pcicfgaccess_baroffset_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_pcicfgaccess_length_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_pcicfgaccess_length_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_pcicfgaccess_length_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_pcicfgaccess_length_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_pf_pcicfgaccess_cfgdata_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_pf_pcicfgaccess_cfgdata_dataor;
    wire   [(32*TOTAL_PF_COUNT)-1:0]        virtio_vf_pcicfgaccess_cfgdata_data;
    wire   [(32*(TOTAL_PF_COUNT+1))-1:0]    virtio_vf_pcicfgaccess_cfgdata_dataor;
    wire   [TOTAL_PF_COUNT-1:0]             virtio_pf_pcicfgaccess_datawr_reg;  // for virtio_pcicfg_cfgwr_o
    wire   [TOTAL_PF_COUNT-1:0]             virtio_pf_pcicfgaccess_datard_reg;
    wire   [TOTAL_PF_COUNT-1:0]             virtio_vf_pcicfgaccess_datawr_reg;  // for virtio_pcicfg_cfgwr_o
    wire   [TOTAL_PF_COUNT-1:0]             virtio_vf_pcicfgaccess_datard_reg;
    wire   [TOTAL_PF_COUNT:0]               virtio_pf_pcicfgaccess_datawror;  // for virtio_pcicfg_cfgwr_o
    wire   [TOTAL_PF_COUNT:0]               virtio_vf_pcicfgaccess_datawror;  // for virtio_pcicfg_cfgwr_o
    wire   [TOTAL_PF_COUNT:0]               virtio_pf_pcicfgaccess_datardor;
    wire   [TOTAL_PF_COUNT:0]               virtio_vf_pcicfgaccess_datardor;
    // reg    [TOTAL_PF_COUNT_WIDTH-1:0]       virtio_pcicfg_pfnum_reg;
    // reg    [TOTAL_VF_COUNT_WIDTH-1:0]       virtio_pcicfg_vfnum_reg;
    wire                                    virtio_cfg_req_ack;
    reg                                     virtio_cfgwr_req_ack_reg;
    reg                                     virtio_cfgrd_req_ack_reg;
    reg                                     virtio_roreg_rdack_reg0;
    reg                                     virtio_roreg_rdack_reg1;
    wire                                    virtio_cmncfg_rdack;
    wire                                    virtio_notifications_rdack;
    wire                                    virtio_isrstatus_rdack;
    wire                                    virtio_devspecific_rdack;
    wire   [TOTAL_PF_COUNT-1:0]             virtio_pf_cfg_req_ack;
    wire   [TOTAL_PF_COUNT-1:0]             virtio_pfvf_cfg_req_ack;
    // Third Stage to arbiter btw PF and VF data
    wire   [7:0 ]            virtio_cmncfg_bar_data;
    wire   [31:0]            virtio_cmncfg_length_data;
    wire   [31:0]            virtio_cmncfg_baroffset_data;
    wire   [7:0 ]            virtio_notifications_bar_data;
    wire   [31:0]            virtio_notifications_length_data;
    wire   [31:0]            virtio_notifications_baroffset_data;
    wire   [31:0]            virtio_notifications_notifyoff_data;
    wire   [31:0]            virtio_isrstatus_header_data;
    wire   [7:0 ]            virtio_isrstatus_bar_data;
    wire   [31:0]            virtio_isrstatus_length_data;
    wire   [31:0]            virtio_isrstatus_baroffset_data;
    wire   [7:0 ]            virtio_devspecific_bar_data;
    wire   [31:0]            virtio_devspecific_length_data;
    wire   [31:0]            virtio_devspecific_baroffset_data;
    wire   [31:0]            virtio_pcicfgaccess_cfgdata_data;
    wire   [7:0 ]            virtio_pcicfgaccess_bar_data;
    wire   [31:0]            virtio_pcicfgaccess_length_data;
    wire   [31:0]            virtio_pcicfgaccess_baroffset_data;
    // Final Stage to register out data
    reg    [31:0]            virtio_cmncfg_header_data_reg;
    reg    [7:0 ]            virtio_cmncfg_bar_data_reg;
    reg    [31:0]            virtio_cmncfg_length_data_reg;
    reg    [31:0]            virtio_cmncfg_baroffset_data_reg;
    reg    [31:0]            virtio_notifications_header_data_reg;
    reg    [7:0 ]            virtio_notifications_bar_data_reg;
    reg    [31:0]            virtio_notifications_length_data_reg;
    reg    [31:0]            virtio_notifications_baroffset_data_reg;
    reg    [31:0]            virtio_notifications_notifyoff_data_reg;
    reg    [31:0]            virtio_isrstatus_header_data_reg;
    reg    [7:0 ]            virtio_isrstatus_bar_data_reg;
    reg    [31:0]            virtio_isrstatus_length_data_reg;
    reg    [31:0]            virtio_isrstatus_baroffset_data_reg;
    reg    [31:0]            virtio_devspecific_header_data_reg;
    reg    [7:0 ]            virtio_devspecific_bar_data_reg;
    reg    [31:0]            virtio_devspecific_length_data_reg;
    reg    [31:0]            virtio_devspecific_baroffset_data_reg;
    reg    [31:0]            virtio_pcicfgaccess_header_data_reg;
    reg    [31:0]            virtio_pcicfgaccess_cfgdata_data_reg;
    reg    [7:0 ]            virtio_pcicfgaccess_bar_data_reg;
    reg    [31:0]            virtio_pcicfgaccess_length_data_reg;
    reg    [31:0]            virtio_pcicfgaccess_baroffset_data_reg;
    
    reg                      cii_wr_reg, cii_wr_q, cii_wr_qq;
    
    // Parameters
    assign pf_virtio_device_specific_cap_present = {PF7_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT[0],PF6_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT[0],
                                                    PF5_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT[0],PF4_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT[0],
                                                    PF3_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT[0],PF2_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT[0],
                                                    PF1_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT[0],PF0_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT[0]};
    assign vf_virtio_device_specific_cap_present = {PF7VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT[0],PF6VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT[0],
                                                    PF5VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT[0],PF4VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT[0],
                                                    PF3VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT[0],PF2VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT[0],
                                                    PF1VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT[0],PF0VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT[0]};
    // Common CFG Cap parameters                                             
    assign pf_virtio_cmn_config_bar_indicator = {PF7_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0],PF6_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0],
                                                 PF5_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0],PF4_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0],
                                                 PF3_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0],PF2_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0],
                                                 PF1_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0],PF0_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0]};
    assign vf_virtio_cmn_config_bar_indicator = {PF7VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0],PF6VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0],
                                                 PF5VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0],PF4VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0],
                                                 PF3VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0],PF2VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0],
                                                 PF1VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0],PF0VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0]};
                                                 
    assign pf_virtio_cmn_config_bar_offset    = {PF7_VIRTIO_CMN_CONFIG_BAR_OFFSET,PF6_VIRTIO_CMN_CONFIG_BAR_OFFSET,
                                                 PF5_VIRTIO_CMN_CONFIG_BAR_OFFSET,PF4_VIRTIO_CMN_CONFIG_BAR_OFFSET,
                                                 PF3_VIRTIO_CMN_CONFIG_BAR_OFFSET,PF2_VIRTIO_CMN_CONFIG_BAR_OFFSET,
                                                 PF1_VIRTIO_CMN_CONFIG_BAR_OFFSET,PF0_VIRTIO_CMN_CONFIG_BAR_OFFSET};
    assign vf_virtio_cmn_config_bar_offset    = {PF7VF_VIRTIO_CMN_CONFIG_BAR_OFFSET,PF6VF_VIRTIO_CMN_CONFIG_BAR_OFFSET,
                                                 PF5VF_VIRTIO_CMN_CONFIG_BAR_OFFSET,PF4VF_VIRTIO_CMN_CONFIG_BAR_OFFSET,
                                                 PF3VF_VIRTIO_CMN_CONFIG_BAR_OFFSET,PF2VF_VIRTIO_CMN_CONFIG_BAR_OFFSET,
                                                 PF1VF_VIRTIO_CMN_CONFIG_BAR_OFFSET,PF0VF_VIRTIO_CMN_CONFIG_BAR_OFFSET};
                                                 
    assign pf_virtio_cmn_config_structure_length   = {PF7_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH,PF6_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH,
                                                      PF5_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH,PF4_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH,
                                                      PF3_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH,PF2_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH,
                                                      PF1_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH,PF0_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH};
    assign vf_virtio_cmn_config_structure_length   = {PF7VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH,PF6VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH,
                                                      PF5VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH,PF4VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH,
                                                      PF3VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH,PF2VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH,
                                                      PF1VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH,PF0VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH};
    // Notifications Cap parameters                                             
    assign pf_virtio_notifications_bar_indicator = {PF7_VIRTIO_NOTIFICATION_BAR_INDICATOR[7:0],PF6_VIRTIO_NOTIFICATION_BAR_INDICATOR[7:0],
                                                    PF5_VIRTIO_NOTIFICATION_BAR_INDICATOR[7:0],PF4_VIRTIO_NOTIFICATION_BAR_INDICATOR[7:0],
                                                    PF3_VIRTIO_NOTIFICATION_BAR_INDICATOR[7:0],PF2_VIRTIO_NOTIFICATION_BAR_INDICATOR[7:0],
                                                    PF1_VIRTIO_NOTIFICATION_BAR_INDICATOR[7:0],PF0_VIRTIO_NOTIFICATION_BAR_INDICATOR[7:0]};
    assign vf_virtio_notifications_bar_indicator = {PF7VF_VIRTIO_NOTIFICATION_BAR_INDICATOR[7:0],PF6VF_VIRTIO_NOTIFICATION_BAR_INDICATOR[7:0],
                                                    PF5VF_VIRTIO_NOTIFICATION_BAR_INDICATOR[7:0],PF4VF_VIRTIO_NOTIFICATION_BAR_INDICATOR[7:0],
                                                    PF3VF_VIRTIO_NOTIFICATION_BAR_INDICATOR[7:0],PF2VF_VIRTIO_NOTIFICATION_BAR_INDICATOR[7:0],
                                                    PF1VF_VIRTIO_NOTIFICATION_BAR_INDICATOR[7:0],PF0VF_VIRTIO_NOTIFICATION_BAR_INDICATOR[7:0]};
                                                 
    assign pf_virtio_notifications_bar_offset    = {PF7_VIRTIO_NOTIFICATION_BAR_OFFSET,PF6_VIRTIO_NOTIFICATION_BAR_OFFSET,
                                                    PF5_VIRTIO_NOTIFICATION_BAR_OFFSET,PF4_VIRTIO_NOTIFICATION_BAR_OFFSET,
                                                    PF3_VIRTIO_NOTIFICATION_BAR_OFFSET,PF2_VIRTIO_NOTIFICATION_BAR_OFFSET,
                                                    PF1_VIRTIO_NOTIFICATION_BAR_OFFSET,PF0_VIRTIO_NOTIFICATION_BAR_OFFSET};
    assign vf_virtio_notifications_bar_offset    = {PF7VF_VIRTIO_NOTIFICATION_BAR_OFFSET,PF6VF_VIRTIO_NOTIFICATION_BAR_OFFSET,
                                                    PF5VF_VIRTIO_NOTIFICATION_BAR_OFFSET,PF4VF_VIRTIO_NOTIFICATION_BAR_OFFSET,
                                                    PF3VF_VIRTIO_NOTIFICATION_BAR_OFFSET,PF2VF_VIRTIO_NOTIFICATION_BAR_OFFSET,
                                                    PF1VF_VIRTIO_NOTIFICATION_BAR_OFFSET,PF0VF_VIRTIO_NOTIFICATION_BAR_OFFSET};
                                                 
    assign pf_virtio_notifications_structure_length   = {PF7_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH,PF6_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH,
                                                         PF5_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH,PF4_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH,
                                                         PF3_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH,PF2_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH,
                                                         PF1_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH,PF0_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH};
    assign vf_virtio_notifications_structure_length   = {PF7VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH,PF6VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH,
                                                         PF5VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH,PF4VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH,
                                                         PF3VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH,PF2VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH,
                                                         PF1VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH,PF0VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH};
                                                         
    assign pf_virtio_notify_off_multiplier            = {PF7_VIRTIO_NOTIFY_OFF_MULTIPLIER,PF6_VIRTIO_NOTIFY_OFF_MULTIPLIER,
                                                         PF5_VIRTIO_NOTIFY_OFF_MULTIPLIER,PF4_VIRTIO_NOTIFY_OFF_MULTIPLIER,
                                                         PF3_VIRTIO_NOTIFY_OFF_MULTIPLIER,PF2_VIRTIO_NOTIFY_OFF_MULTIPLIER,
                                                         PF1_VIRTIO_NOTIFY_OFF_MULTIPLIER,PF0_VIRTIO_NOTIFY_OFF_MULTIPLIER};
    assign vf_virtio_notify_off_multiplier            = {PF7VF_VIRTIO_NOTIFY_OFF_MULTIPLIER,PF6VF_VIRTIO_NOTIFY_OFF_MULTIPLIER,
                                                         PF5VF_VIRTIO_NOTIFY_OFF_MULTIPLIER,PF4VF_VIRTIO_NOTIFY_OFF_MULTIPLIER,
                                                         PF3VF_VIRTIO_NOTIFY_OFF_MULTIPLIER,PF2VF_VIRTIO_NOTIFY_OFF_MULTIPLIER,
                                                         PF1VF_VIRTIO_NOTIFY_OFF_MULTIPLIER,PF0VF_VIRTIO_NOTIFY_OFF_MULTIPLIER};
    // ISR Cap parameters                                             
    assign pf_virtio_isrstatus_bar_indicator = {PF7_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0],PF6_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0],
                                                PF5_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0],PF4_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0],
                                                PF3_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0],PF2_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0],
                                                PF1_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0],PF0_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0]};
    assign vf_virtio_isrstatus_bar_indicator = {PF7VF_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0],PF6VF_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0],
                                                PF5VF_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0],PF4VF_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0],
                                                PF3VF_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0],PF2VF_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0],
                                                PF1VF_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0],PF0VF_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0]};
                                                 
    assign pf_virtio_isrstatus_bar_offset    = {PF7_VIRTIO_ISRSTATUS_BAR_OFFSET,PF6_VIRTIO_ISRSTATUS_BAR_OFFSET,
                                                PF5_VIRTIO_ISRSTATUS_BAR_OFFSET,PF4_VIRTIO_ISRSTATUS_BAR_OFFSET,
                                                PF3_VIRTIO_ISRSTATUS_BAR_OFFSET,PF2_VIRTIO_ISRSTATUS_BAR_OFFSET,
                                                PF1_VIRTIO_ISRSTATUS_BAR_OFFSET,PF0_VIRTIO_ISRSTATUS_BAR_OFFSET};
    assign vf_virtio_isrstatus_bar_offset    = {PF7VF_VIRTIO_ISRSTATUS_BAR_OFFSET,PF6VF_VIRTIO_ISRSTATUS_BAR_OFFSET,
                                                PF5VF_VIRTIO_ISRSTATUS_BAR_OFFSET,PF4VF_VIRTIO_ISRSTATUS_BAR_OFFSET,
                                                PF3VF_VIRTIO_ISRSTATUS_BAR_OFFSET,PF2VF_VIRTIO_ISRSTATUS_BAR_OFFSET,
                                                PF1VF_VIRTIO_ISRSTATUS_BAR_OFFSET,PF0VF_VIRTIO_ISRSTATUS_BAR_OFFSET};
                                                 
    assign pf_virtio_isrstatus_structure_length   = {PF7_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH,PF6_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH,
                                                     PF5_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH,PF4_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH,
                                                     PF3_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH,PF2_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH,
                                                     PF1_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH,PF0_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH};
    assign vf_virtio_isrstatus_structure_length   = {PF7VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH,PF6VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH,
                                                     PF5VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH,PF4VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH,
                                                     PF3VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH,PF2VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH,
                                                     PF1VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH,PF0VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH};
    // Device Specific Cap parameters                                             
    assign pf_virtio_devspecific_bar_indicator = {PF7_VIRTIO_DEVSPECIFIC_BAR_INDICATOR[7:0],PF6_VIRTIO_DEVSPECIFIC_BAR_INDICATOR[7:0],
                                                  PF5_VIRTIO_DEVSPECIFIC_BAR_INDICATOR[7:0],PF4_VIRTIO_DEVSPECIFIC_BAR_INDICATOR[7:0],
                                                  PF3_VIRTIO_DEVSPECIFIC_BAR_INDICATOR[7:0],PF2_VIRTIO_DEVSPECIFIC_BAR_INDICATOR[7:0],
                                                  PF1_VIRTIO_DEVSPECIFIC_BAR_INDICATOR[7:0],PF0_VIRTIO_DEVSPECIFIC_BAR_INDICATOR[7:0]};
    assign vf_virtio_devspecific_bar_indicator = {PF7VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR[7:0],PF6VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR[7:0],
                                                  PF5VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR[7:0],PF4VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR[7:0],
                                                  PF3VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR[7:0],PF2VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR[7:0],
                                                  PF1VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR[7:0],PF0VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR[7:0]};
                                                 
    assign pf_virtio_devspecific_bar_offset    = {PF7_VIRTIO_DEVSPECIFIC_BAR_OFFSET,PF6_VIRTIO_DEVSPECIFIC_BAR_OFFSET,
                                                  PF5_VIRTIO_DEVSPECIFIC_BAR_OFFSET,PF4_VIRTIO_DEVSPECIFIC_BAR_OFFSET,
                                                  PF3_VIRTIO_DEVSPECIFIC_BAR_OFFSET,PF2_VIRTIO_DEVSPECIFIC_BAR_OFFSET,
                                                  PF1_VIRTIO_DEVSPECIFIC_BAR_OFFSET,PF0_VIRTIO_DEVSPECIFIC_BAR_OFFSET};
    assign vf_virtio_devspecific_bar_offset    = {PF7VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET,PF6VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET,
                                                  PF5VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET,PF4VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET,
                                                  PF3VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET,PF2VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET,
                                                  PF1VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET,PF0VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET};
                                                 
    assign pf_virtio_devspecific_structure_length   = {PF7_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH,PF6_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH,
                                                       PF5_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH,PF4_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH,
                                                       PF3_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH,PF2_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH,
                                                       PF1_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH,PF0_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH};
    assign vf_virtio_devspecific_structure_length   = {PF7VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH,PF6VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH,
                                                       PF5VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH,PF4VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH,
                                                       PF3VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH,PF2VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH,
                                                       PF1VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH,PF0VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH};
                        
    // PCI CFG Access Cap parameters                                             
    assign pf_virtio_pcicfgaccess_bar_indicator = {PF7_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR[7:0],PF6_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR[7:0],
                                                   PF5_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR[7:0],PF4_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR[7:0],
                                                   PF3_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR[7:0],PF2_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR[7:0],
                                                   PF1_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR[7:0],PF0_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR[7:0]};
    assign vf_virtio_pcicfgaccess_bar_indicator = {PF7VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR[7:0],PF6VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR[7:0],
                                                   PF5VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR[7:0],PF4VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR[7:0],
                                                   PF3VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR[7:0],PF2VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR[7:0],
                                                   PF1VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR[7:0],PF0VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR[7:0]};
                                                 
    assign pf_virtio_pcicfgaccess_bar_offset    = {PF7_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET,PF6_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET,
                                                   PF5_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET,PF4_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET,
                                                   PF3_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET,PF2_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET,
                                                   PF1_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET,PF0_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET};
    assign vf_virtio_pcicfgaccess_bar_offset    = {PF7VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET,PF6VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET,
                                                   PF5VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET,PF4VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET,
                                                   PF3VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET,PF2VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET,
                                                   PF1VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET,PF0VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET};
                                                 
    assign pf_virtio_pcicfgaccess_structure_length   = {PF7_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH,PF6_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH,
                                                        PF5_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH,PF4_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH,
                                                        PF3_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH,PF2_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH,
                                                        PF1_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH,PF0_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH};
    assign vf_virtio_pcicfgaccess_structure_length   = {PF7VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH,PF6VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH,
                                                        PF5VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH,PF4VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH,
                                                        PF3VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH,PF2VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH,
                                                        PF1VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH,PF0VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH};
    
    // Initiate xxxxxor[31:0] to 0
    assign virtio_pf_cmncfg_bar_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_cmncfg_bar_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_cmncfg_baroffset_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_cmncfg_baroffset_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_cmncfg_length_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_cmncfg_length_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_notifications_bar_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_notifications_bar_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_notifications_baroffset_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_notifications_baroffset_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_notifications_length_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_notifications_length_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_notifications_notifyoff_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_notifications_notifyoff_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_isrstatus_header_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_isrstatus_header_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_isrstatus_bar_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_isrstatus_bar_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_isrstatus_baroffset_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_isrstatus_baroffset_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_isrstatus_length_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_isrstatus_length_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_devspecific_bar_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_devspecific_bar_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_devspecific_baroffset_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_devspecific_baroffset_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_devspecific_length_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_devspecific_length_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_pcicfgaccess_bar_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_pcicfgaccess_bar_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_pcicfgaccess_baroffset_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_pcicfgaccess_baroffset_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_pcicfgaccess_length_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_pcicfgaccess_length_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_pcicfgaccess_cfgdata_dataor[31:0] = {32{1'b0}};
    assign virtio_vf_pcicfgaccess_cfgdata_dataor[31:0] = {32{1'b0}};
    assign virtio_pf_pcicfgaccess_datawror[0] = 1'b0;
    assign virtio_vf_pcicfgaccess_datawror[0] = 1'b0;
    assign virtio_pf_pcicfgaccess_datardor[0] = 1'b0;
    assign virtio_vf_pcicfgaccess_datardor[0] = 1'b0;
    

    // ISR Nxtptr, depends on devspecific enablement
    generate
        for (i=0; i<TOTAL_PF_COUNT; i=i+1) begin:isrnxtptr    
            // Points to VirtIO Device Specific capability if present else points to PCI Config Access Capability        
            assign pf_virtio_isr_status_next_cap_ptr[(8*(i+1))-1:(i*8)]     = pf_virtio_device_specific_cap_present[i] ? 8'hCC : 8'hDC; 
            assign vf_virtio_isr_status_next_cap_ptr[(8*(i+1))-1:(i*8)]     = vf_virtio_device_specific_cap_present[i] ? 8'hCC : 8'hDC; 
        end
    endgenerate
        
    
    //======================================
    // Virtio Capability Write Path
    //======================================
    // Read Write Reg
    // PFs
    // no matter which PF the req targets to, single set of PF read data is returned. Depends on cii_func_num_i
    
    // The check of cii_func_num_i based on xxx_VIRTIO_CAPABILITY_PRESENT is done in rdwr_ctrl, by passing ctrl signals.
    generate
        for (i=0; i<TOTAL_PF_COUNT; i=i+1) begin:pfcfgaccesscap
            intel_pcie_virtio_cfg_access_cap #
            (
                .TOTAL_PF_COUNT           (TOTAL_PF_COUNT),
                .TOTAL_PF_COUNT_WIDTH     (TOTAL_PF_COUNT_WIDTH),
                .TOTAL_VF_COUNT_WIDTH     (TOTAL_VF_COUNT_WIDTH),
                .FUNC_COUNT               (1),
                .FUNC_COUNT_WIDTH         (TOTAL_PF_COUNT_WIDTH),
                .PF_NUM                   (i)
            )
            pf_pci_cfg_access_cap
            (
                .Clk_i                    (clk),
                .Rstn_i                   (rstn),
                .virtio_pciconfig_access_bar_indicator     (pf_virtio_pcicfgaccess_bar_indicator[(8*(i+1))-1:(i*8)]),
                .virtio_pciconfig_access_structure_length  (pf_virtio_pcicfgaccess_structure_length[(32*(i+1))-1:(i*32)]),
                .virtio_pciconfig_access_bar_offset        (pf_virtio_pcicfgaccess_bar_offset[(32*(i+1))-1:(i*32)]),

                .cfg_req_func_num_i       (cii_func_num_i),
                .cfg_app_func_num_i       (virtio_pcicfg_apppfnum_i),
                .cfg_write_req_i          (write_pf_pcicfgaccess_cfgdata_sel_i[i]),
                .cfg_write_data_i         (cii_dout_i),
                .cfg_write_be_i           (cii_hdr_first_be_i),
                .cfg_read_req_i           (read_pf_pcicfgaccess_cfgdata_sel_i[i]),
                .cfg_req_ack_o            (virtio_pf_cfg_req_ack[i]),
                .cfg_read_ack_i           (virtio_pcicfg_rdack_i),
                .cfg_read_ack_be_i        (virtio_pcicfg_rdbe_i),
                .cfg_rdack_data_i         (virtio_pcicfg_data_i),
                .cfg_barwr_req_i          (write_pf_pcicfgaccess_bar_sel_i[i]),
                .cfg_barrd_req_i          (read_pf_pcicfgaccess_bar_sel_i[i]),
                .cfg_lengthwr_req_i       (write_pf_pcicfgaccess_length_sel_i[i]),
                .cfg_lengthrd_req_i       (read_pf_pcicfgaccess_length_sel_i[i]),
                .cfg_baroffsetwr_req_i    (write_pf_pcicfgaccess_baroffset_sel_i[i]),
                .cfg_baroffsetrd_req_i    (read_pf_pcicfgaccess_baroffset_sel_i[i]),
              
                .cfg_read_data_o          (virtio_pf_pcicfgaccess_cfgdata_data[(32*(i+1))-1:(i*32)]),
                .cfg_bar_o                (virtio_pf_pcicfgaccess_bar_data[((i+1)*32)-1-(32-8):(i*32)]),  // 8-bit
                .cfg_length_o             (virtio_pf_pcicfgaccess_length_data[(32*(i+1))-1:(i*32)]),
                .cfg_baroffset_o          (virtio_pf_pcicfgaccess_baroffset_data[(32*(i+1))-1:(i*32)]),
                .cfg_access_datawr_o      (virtio_pf_pcicfgaccess_datawr_reg[i]),
                .cfg_access_datard_o      (virtio_pf_pcicfgaccess_datard_reg[i]),

                // FLR inputs
                .flr_rcvd_vf_i            (1'b0),
                .flr_rcvd_vf_num_i        (flr_rcvd_vf_num_i),
                .flr_rcvd_pf_i            (flr_rcvd_pf_i[i]),
                .flr_rcvd_pf_num_i        (flr_rcvd_pf_num_i),
                .flr_rcvd_pf_num_match_i  (flr_rcvd_pf_num_match_i)
            );
            assign virtio_pf_pcicfgaccess_cfgdata_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_pcicfgaccess_cfgdata_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_pf_pcicfgaccess_cfgdata_data[(32*(i+1))-1:(i*32)];
            assign virtio_pf_pcicfgaccess_bar_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_pcicfgaccess_bar_dataor [(32*(i+1))-1:(i*32)] 
                                                     | {24'd0, virtio_pf_pcicfgaccess_bar_data[((i+1)*32)-1-(32-8):(i*32)]};
            assign virtio_pf_pcicfgaccess_length_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_pcicfgaccess_length_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_pf_pcicfgaccess_length_data[(32*(i+1))-1:(i*32)];
            assign virtio_pf_pcicfgaccess_baroffset_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_pcicfgaccess_baroffset_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_pf_pcicfgaccess_baroffset_data[(32*(i+1))-1:(i*32)];
                                                     
            assign virtio_pf_pcicfgaccess_datawror [(1*(i+2))-1:((i+1)*1)]  =  
                                                     virtio_pf_pcicfgaccess_datawror [(1*(i+1))-1:(i*1)] 
                                                     | virtio_pf_pcicfgaccess_datawr_reg[(1*(i+1))-1:(i*1)];
            assign virtio_pf_pcicfgaccess_datardor [(1*(i+2))-1:((i+1)*1)]  =  
                                                     virtio_pf_pcicfgaccess_datardor [(1*(i+1))-1:(i*1)] 
                                                     | virtio_pf_pcicfgaccess_datard_reg[(1*(i+1))-1:(i*1)];
        end
    endgenerate    
        
    // each PF's VFs
    generate
        for (i=0; i<TOTAL_PF_COUNT; i=i+1) begin:vfcfgaccesscap
            // PF has VFs attached. PF num matches iteration & Req targets a VF are covered in rdwr_ctrl
            // XXTODO. Check virtio_pcicfg_apppfnum_i ? Depends on user side response latency, in order or might cause mismatch.
            if (PFVF_COUNT[(TOTAL_VF_COUNT_WIDTH_MAX*(i+1))-1:(i*TOTAL_VF_COUNT_WIDTH_MAX)] != 0) begin
                intel_pcie_virtio_cfg_access_cap #
                (
                    .TOTAL_PF_COUNT           (TOTAL_PF_COUNT),
                    .TOTAL_PF_COUNT_WIDTH     (TOTAL_PF_COUNT_WIDTH),
                    .TOTAL_VF_COUNT_WIDTH     (TOTAL_VF_COUNT_WIDTH),
                    .FUNC_COUNT               (PFVF_COUNT[(TOTAL_VF_COUNT_WIDTH_MAX*(i+1))-1:(i*TOTAL_VF_COUNT_WIDTH_MAX)]),
                    .FUNC_COUNT_WIDTH         (TOTAL_VF_COUNT_WIDTH),
                    .PF_NUM                   (i)
                )
                vf_pci_cfg_access_cap
                (
                    .Clk_i                    (clk),
                    .Rstn_i                   (rstn),
                    .virtio_pciconfig_access_bar_indicator     (vf_virtio_pcicfgaccess_bar_indicator[(8*(i+1))-1:(i*8)]),
                    .virtio_pciconfig_access_structure_length  (vf_virtio_pcicfgaccess_structure_length[(32*(i+1))-1:(i*32)]),
                    .virtio_pciconfig_access_bar_offset        (vf_virtio_pcicfgaccess_bar_offset[(32*(i+1))-1:(i*32)]),

                    .cfg_req_func_num_i       (cii_vf_num_i),
                    .cfg_app_func_num_i       (virtio_pcicfg_appvfnum_i),
                    .cfg_write_req_i          (write_vf_pcicfgaccess_cfgdata_sel_i[i]),
                    .cfg_write_data_i         (cii_dout_i),
                    .cfg_write_be_i           (cii_hdr_first_be_i),
                    .cfg_read_req_i           (read_vf_pcicfgaccess_cfgdata_sel_i[i]),
                    .cfg_req_ack_o            (virtio_pfvf_cfg_req_ack[i]),
                    .cfg_read_ack_i           (virtio_pcicfg_rdack_i),
                    .cfg_read_ack_be_i        (virtio_pcicfg_rdbe_i),
                    .cfg_rdack_data_i         (virtio_pcicfg_data_i),
                    .cfg_barwr_req_i          (write_vf_pcicfgaccess_bar_sel_i[i]),
                    .cfg_barrd_req_i          (read_vf_pcicfgaccess_bar_sel_i[i]),
                    .cfg_lengthwr_req_i       (write_vf_pcicfgaccess_length_sel_i[i]),
                    .cfg_lengthrd_req_i       (read_vf_pcicfgaccess_length_sel_i[i]),
                    .cfg_baroffsetwr_req_i    (write_vf_pcicfgaccess_baroffset_sel_i[i]),
                    .cfg_baroffsetrd_req_i    (read_vf_pcicfgaccess_baroffset_sel_i[i]),
                  
                    .cfg_read_data_o          (virtio_vf_pcicfgaccess_cfgdata_data[(32*(i+1))-1:(i*32)]),
                    .cfg_bar_o                (virtio_vf_pcicfgaccess_bar_data[((i+1)*32)-1-(32-8):(i*32)]),  // 8-bit
                    .cfg_length_o             (virtio_vf_pcicfgaccess_length_data[(32*(i+1))-1:(i*32)]),
                    .cfg_baroffset_o          (virtio_vf_pcicfgaccess_baroffset_data[(32*(i+1))-1:(i*32)]),
                    .cfg_access_datawr_o      (virtio_vf_pcicfgaccess_datawr_reg[i]),
                    .cfg_access_datard_o      (virtio_vf_pcicfgaccess_datard_reg[i]),

                    // FLR inputs
                    .flr_rcvd_vf_i            (flr_rcvd_vf_i),
                    .flr_rcvd_vf_num_i        (flr_rcvd_vf_num_i),
                    .flr_rcvd_pf_i            (flr_rcvd_pf_i[i]),
                    .flr_rcvd_pf_num_i        (flr_rcvd_pf_num_i),
                    .flr_rcvd_pf_num_match_i  (flr_rcvd_pf_num_match_i)
                );
            end
            else begin
                assign virtio_pfvf_cfg_req_ack[i]  =  0;
                assign virtio_vf_pcicfgaccess_cfgdata_data[(32*(i+1))-1:(i*32)]    =  0;
                assign virtio_vf_pcicfgaccess_bar_data[(32*(i+1))-1:(i*32)]        =  0;
                assign virtio_vf_pcicfgaccess_length_data[(32*(i+1))-1:(i*32)]     =  0;
                assign virtio_vf_pcicfgaccess_baroffset_data[(32*(i+1))-1:(i*32)]  =  0;
                assign virtio_vf_pcicfgaccess_datawr_reg[i]  =  0;
                assign virtio_vf_pcicfgaccess_datard_reg[i]  =  0;
            end

            assign virtio_vf_pcicfgaccess_cfgdata_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_pcicfgaccess_cfgdata_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_vf_pcicfgaccess_cfgdata_data[(32*(i+1))-1:(i*32)];
            assign virtio_vf_pcicfgaccess_bar_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_pcicfgaccess_bar_dataor [(32*(i+1))-1:(i*32)] 
                                                     | {24'd0, virtio_vf_pcicfgaccess_bar_data[((i+1)*32)-1-(32-8):(i*32)]};
            assign virtio_vf_pcicfgaccess_length_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_pcicfgaccess_length_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_vf_pcicfgaccess_length_data[(32*(i+1))-1:(i*32)];
            assign virtio_vf_pcicfgaccess_baroffset_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_pcicfgaccess_baroffset_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_vf_pcicfgaccess_baroffset_data[(32*(i+1))-1:(i*32)];
                                                     
            assign virtio_vf_pcicfgaccess_datawror [(1*(i+2))-1:((i+1)*1)]  =  
                                                     virtio_vf_pcicfgaccess_datawror [(1*(i+1))-1:(i*1)] 
                                                     | virtio_vf_pcicfgaccess_datawr_reg[(1*(i+1))-1:(i*1)];
            assign virtio_vf_pcicfgaccess_datardor [(1*(i+2))-1:((i+1)*1)]  =  
                                                     virtio_vf_pcicfgaccess_datardor [(1*(i+1))-1:(i*1)] 
                                                     | virtio_vf_pcicfgaccess_datard_reg[(1*(i+1))-1:(i*1)];
        end
    endgenerate
        
    // Hooks. Generate VIRTIO outputs to user side



    always @ (posedge clk) begin
    virtio_pcicfg_bar_o       <= virtio_pf_pcicfgaccess_bar_dataor[((TOTAL_PF_COUNT+1)*32)-1-(32-8): TOTAL_PF_COUNT*32] | 
                                 virtio_vf_pcicfgaccess_bar_dataor[((TOTAL_PF_COUNT+1)*32)-1-(32-8): TOTAL_PF_COUNT*32];
    virtio_pcicfg_length_o    <= virtio_pf_pcicfgaccess_length_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32] | 
                                 virtio_vf_pcicfgaccess_length_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32];
    virtio_pcicfg_baroffset_o <= virtio_pf_pcicfgaccess_baroffset_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32] | 
                                 virtio_vf_pcicfgaccess_baroffset_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32];
    virtio_pcicfg_cfgdata_o   <= virtio_pf_pcicfgaccess_cfgdata_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32] | 
                                 virtio_vf_pcicfgaccess_cfgdata_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32];
    virtio_pcicfg_cfgwr_o     <= virtio_pf_pcicfgaccess_datawror[TOTAL_PF_COUNT] | 
                                 virtio_vf_pcicfgaccess_datawror[TOTAL_PF_COUNT];
    virtio_pcicfg_cfgrd_o     <= virtio_pf_pcicfgaccess_datardor[TOTAL_PF_COUNT] | 
                                 virtio_vf_pcicfgaccess_datardor[TOTAL_PF_COUNT];
    virtio_pcicfg_vfnum_o        <= cii_vf_num_i;
    virtio_pcicfg_pfnum_o        <= cii_func_num_i;
    virtio_pcicfg_vfaccess_o     <= cii_wr_vf_active_i;
	end
        
    //======================================
    // Virtio Capability Read Path
    //======================================
    // Common CFG CAP Reg                                             
    assign virtio_cmncfg_header_data = read_cmncfg_header_sel_i ? 
                                         {
                                          VIRTIO_CMN_CONFIG_TYPE,         // Config Type
                                          VIRTIO_CMN_CONFIG_CAP_LENGTH,   // Length
                                          VIRTIO_CMN_CONFIG_NEXT_CAP_PTR, // Next cap ptr
                                          VIRTIO_CAPID // Cap ID
                                        } : {32{1'b0}};

    generate                                   
        for (i=0; i<TOTAL_PF_COUNT; i=i+1) begin:cmncfgrdpath                                   
            assign virtio_pf_cmncfg_bar_data[(32*(i+1))-1:(i*32)]    = read_pf_cmncfg_bar_sel_i[i] ? 
                                                                       {
                                                                        24'h000000, // Pad Zeros
                                                                        pf_virtio_cmn_config_bar_indicator[(8*(i+1))-1:(i*8)]     //PF0_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0] // Bar Indicator
                                                                       } : {32{1'b0}};
                                       
            assign virtio_vf_cmncfg_bar_data[(32*(i+1))-1:(i*32)]    = read_vf_cmncfg_bar_sel_i[i] ? 
                                                                       {
                                                                        24'h000000, // Pad Zeros
                                                                        vf_virtio_cmn_config_bar_indicator[(8*(i+1))-1:(i*8)]     //PF0VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR[7:0] // Bar Indicator
                                                                       } : {32{1'b0}};
                                       
            assign virtio_pf_cmncfg_baroffset_data[(32*(i+1))-1:(i*32)]    = read_pf_cmncfg_baroffset_sel_i[i] ? 
                                                                             pf_virtio_cmn_config_bar_offset[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                                       
            assign virtio_vf_cmncfg_baroffset_data[(32*(i+1))-1:(i*32)]    = read_vf_cmncfg_baroffset_sel_i[i] ? 
                                                                             vf_virtio_cmn_config_bar_offset[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                                       
            assign virtio_pf_cmncfg_length_data[(32*(i+1))-1:(i*32)]       = read_pf_cmncfg_length_sel_i[i] ? 
                                                                             pf_virtio_cmn_config_structure_length[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                                       
            assign virtio_vf_cmncfg_length_data[(32*(i+1))-1:(i*32)]       = read_vf_cmncfg_length_sel_i[i] ? 
                                                                             vf_virtio_cmn_config_structure_length[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                                       

            assign virtio_pf_cmncfg_bar_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_cmncfg_bar_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_pf_cmncfg_bar_data[(32*(i+1))-1:(i*32)];
            assign virtio_pf_cmncfg_length_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_cmncfg_length_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_pf_cmncfg_length_data[(32*(i+1))-1:(i*32)];
            assign virtio_pf_cmncfg_baroffset_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_cmncfg_baroffset_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_pf_cmncfg_baroffset_data[(32*(i+1))-1:(i*32)];
            assign virtio_vf_cmncfg_bar_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_cmncfg_bar_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_vf_cmncfg_bar_data[(32*(i+1))-1:(i*32)];
            assign virtio_vf_cmncfg_length_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_cmncfg_length_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_vf_cmncfg_length_data[(32*(i+1))-1:(i*32)];
            assign virtio_vf_cmncfg_baroffset_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_cmncfg_baroffset_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_vf_cmncfg_baroffset_data[(32*(i+1))-1:(i*32)];
        end
    endgenerate    

    assign virtio_cmncfg_rdack = (read_cmncfg_header_sel_i | (|read_pf_cmncfg_bar_sel_i) | (|read_vf_cmncfg_bar_sel_i)
                                 | (|read_pf_cmncfg_baroffset_sel_i) | (|read_vf_cmncfg_baroffset_sel_i) 
                                 | (|read_pf_cmncfg_length_sel_i) | (|read_vf_cmncfg_length_sel_i)) ? 'b1 : 'b0;
    
    // Notifications CAP Reg
    assign virtio_notifications_header_data = read_notifications_header_sel_i ? 
                                         {
                                          VIRTIO_NOTIFICATION_TYPE,         // Config Type
                                          VIRTIO_NOTIFICATION_CAP_LENGTH,   // Length
                                          VIRTIO_NOTIFICATION_NEXT_CAP_PTR, // Next cap ptr
                                          VIRTIO_CAPID // Cap ID
                                        } : {32{1'b0}};

    generate                                   
        for (i=0; i<TOTAL_PF_COUNT; i=i+1) begin:notificationsrdpath                             
            assign virtio_pf_notifications_bar_data[(32*(i+1))-1:(i*32)]    = read_pf_notifications_bar_sel_i[i] ? 
                                                                              {
                                                                               24'h000000, // Pad Zeros
                                                                               pf_virtio_notifications_bar_indicator[(8*(i+1))-1:(i*8)]     //PF0_VIRTIO_NOTIFICATIONS_BAR_INDICATOR[7:0] // Bar Indicator
                                                                              } : {32{1'b0}};
                                       
            assign virtio_vf_notifications_bar_data[(32*(i+1))-1:(i*32)]    = read_vf_notifications_bar_sel_i[i] ? 
                                                                              {
                                                                               24'h000000, // Pad Zeros
                                                                               vf_virtio_notifications_bar_indicator[(8*(i+1))-1:(i*8)]     //PF0VF_VIRTIO_NOTIFICATIONS_BAR_INDICATOR[7:0] // Bar Indicator
                                                                              } : {32{1'b0}};
                                       
            assign virtio_pf_notifications_baroffset_data[(32*(i+1))-1:(i*32)]    = read_pf_notifications_baroffset_sel_i[i] ? 
                                                                                    pf_virtio_notifications_bar_offset[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                                       
            assign virtio_vf_notifications_baroffset_data[(32*(i+1))-1:(i*32)]    = read_vf_notifications_baroffset_sel_i[i] ? 
                                                                                    vf_virtio_notifications_bar_offset[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                                       
            assign virtio_pf_notifications_length_data[(32*(i+1))-1:(i*32)]       = read_pf_notifications_length_sel_i[i] ? 
                                                                                    pf_virtio_notifications_structure_length[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                                       
            assign virtio_vf_notifications_length_data[(32*(i+1))-1:(i*32)]       = read_vf_notifications_length_sel_i[i] ? 
                                                                                    vf_virtio_notifications_structure_length[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                                                                                    
            assign virtio_pf_notifications_notifyoff_data[(32*(i+1))-1:(i*32)]    = read_pf_notifications_notifyoff_sel_i[i] ? 
                                                                                    pf_virtio_notify_off_multiplier[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                                       
            assign virtio_vf_notifications_notifyoff_data[(32*(i+1))-1:(i*32)]    = read_vf_notifications_notifyoff_sel_i[i] ? 
                                                                                    vf_virtio_notify_off_multiplier[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                                                                                    
                                                                                    
            assign virtio_pf_notifications_bar_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_notifications_bar_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_pf_notifications_bar_data[(32*(i+1))-1:(i*32)];
            assign virtio_pf_notifications_length_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_notifications_length_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_pf_notifications_length_data[(32*(i+1))-1:(i*32)];
            assign virtio_pf_notifications_baroffset_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_notifications_baroffset_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_pf_notifications_baroffset_data[(32*(i+1))-1:(i*32)];
            assign virtio_pf_notifications_notifyoff_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_notifications_notifyoff_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_pf_notifications_notifyoff_data[(32*(i+1))-1:(i*32)];
            assign virtio_vf_notifications_bar_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_notifications_bar_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_vf_notifications_bar_data[(32*(i+1))-1:(i*32)];
            assign virtio_vf_notifications_length_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_notifications_length_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_vf_notifications_length_data[(32*(i+1))-1:(i*32)];
            assign virtio_vf_notifications_baroffset_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_notifications_baroffset_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_vf_notifications_baroffset_data[(32*(i+1))-1:(i*32)];
            assign virtio_vf_notifications_notifyoff_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_notifications_notifyoff_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_vf_notifications_notifyoff_data[(32*(i+1))-1:(i*32)];
        end
    endgenerate    

    assign virtio_notifications_rdack = (read_notifications_header_sel_i | (|read_pf_notifications_bar_sel_i) | (|read_vf_notifications_bar_sel_i)
                                        | (|read_pf_notifications_baroffset_sel_i) | (|read_vf_notifications_baroffset_sel_i) 
                                        | (|read_pf_notifications_length_sel_i) | (|read_vf_notifications_length_sel_i)
                                        | (|read_pf_notifications_notifyoff_sel_i) | (|read_vf_notifications_notifyoff_sel_i)) ? 'b1 : 'b0;
    
    // ISR CAP Reg
    generate                                   
        for (i=0; i<TOTAL_PF_COUNT; i=i+1) begin:isrstatusrdpath        
            assign virtio_pf_isrstatus_header_data[(32*(i+1))-1:(i*32)] = read_pf_isrstatus_header_sel_i[i] ? 
                                                                            {
                                                                             VIRTIO_ISR_STATUS_TYPE,         // Config Type
                                                                             VIRTIO_ISR_STATUS_CAP_LENGTH,   // Length
                                                                             pf_virtio_isr_status_next_cap_ptr[(8*(i+1))-1:(i*8)], // Next cap ptr
                                                                             VIRTIO_CAPID // Cap ID
                                                                           } : {32{1'b0}};

            assign virtio_vf_isrstatus_header_data[(32*(i+1))-1:(i*32)] = read_vf_isrstatus_header_sel_i[i] ? 
                                                                            {
                                                                             VIRTIO_ISR_STATUS_TYPE,         // Config Type
                                                                             VIRTIO_ISR_STATUS_CAP_LENGTH,   // Length
                                                                             vf_virtio_isr_status_next_cap_ptr[(8*(i+1))-1:(i*8)], // Next cap ptr
                                                                             VIRTIO_CAPID // Cap ID
                                                                           } : {32{1'b0}};

            assign virtio_pf_isrstatus_bar_data[(32*(i+1))-1:(i*32)]    = read_pf_isrstatus_bar_sel_i[i] ? 
                                                                          {
                                                                           24'h000000, // Pad Zeros
                                                                           pf_virtio_isrstatus_bar_indicator[(8*(i+1))-1:(i*8)]     //PF0_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0] // Bar Indicator
                                                                          } : {32{1'b0}};
                                       
            assign virtio_vf_isrstatus_bar_data[(32*(i+1))-1:(i*32)]    = read_vf_isrstatus_bar_sel_i[i] ? 
                                                                          {
                                                                           24'h000000, // Pad Zeros
                                                                           vf_virtio_isrstatus_bar_indicator[(8*(i+1))-1:(i*8)]     //PF0VF_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0] // Bar Indicator
                                                                          } : {32{1'b0}};
                                       
            assign virtio_pf_isrstatus_baroffset_data[(32*(i+1))-1:(i*32)]    = read_pf_isrstatus_baroffset_sel_i[i] ? 
                                                                                pf_virtio_isrstatus_bar_offset[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                                       
            assign virtio_vf_isrstatus_baroffset_data[(32*(i+1))-1:(i*32)]    = read_vf_isrstatus_baroffset_sel_i[i] ? 
                                                                                vf_virtio_isrstatus_bar_offset[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                                       
            assign virtio_pf_isrstatus_length_data[(32*(i+1))-1:(i*32)]       = read_pf_isrstatus_length_sel_i[i] ? 
                                                                                pf_virtio_isrstatus_structure_length[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                                       
            assign virtio_vf_isrstatus_length_data[(32*(i+1))-1:(i*32)]       = read_vf_isrstatus_length_sel_i[i] ? 
                                                                                vf_virtio_isrstatus_structure_length[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                                       
                                       
            assign virtio_pf_isrstatus_header_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_isrstatus_header_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_pf_isrstatus_header_data[(32*(i+1))-1:(i*32)];
            assign virtio_pf_isrstatus_bar_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_isrstatus_bar_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_pf_isrstatus_bar_data[(32*(i+1))-1:(i*32)];
            assign virtio_pf_isrstatus_length_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_isrstatus_length_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_pf_isrstatus_length_data[(32*(i+1))-1:(i*32)];
            assign virtio_pf_isrstatus_baroffset_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_isrstatus_baroffset_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_pf_isrstatus_baroffset_data[(32*(i+1))-1:(i*32)];
            assign virtio_vf_isrstatus_header_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_isrstatus_header_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_vf_isrstatus_header_data[(32*(i+1))-1:(i*32)];
            assign virtio_vf_isrstatus_bar_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_isrstatus_bar_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_vf_isrstatus_bar_data[(32*(i+1))-1:(i*32)];
            assign virtio_vf_isrstatus_length_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_isrstatus_length_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_vf_isrstatus_length_data[(32*(i+1))-1:(i*32)];
            assign virtio_vf_isrstatus_baroffset_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_isrstatus_baroffset_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_vf_isrstatus_baroffset_data[(32*(i+1))-1:(i*32)];
        end
    endgenerate   
    
    assign virtio_isrstatus_rdack = ((|read_pf_isrstatus_header_sel_i) | (|read_vf_isrstatus_header_sel_i)
                                    | (|read_pf_isrstatus_bar_sel_i) | (|read_vf_isrstatus_bar_sel_i)
                                    | (|read_pf_isrstatus_baroffset_sel_i) | (|read_vf_isrstatus_baroffset_sel_i) 
                                    | (|read_pf_isrstatus_length_sel_i) | (|read_vf_isrstatus_length_sel_i)) ? 'b1 : 'b0;
    
    // Device Specific CAP Reg  // Optional cap
    assign virtio_devspecific_header_data = read_devspecific_header_sel_i ? 
                                         {
                                          VIRTIO_DEV_SPECIFIC_TYPE,         // Config Type
                                          VIRTIO_DEV_SPECIFIC_CAP_LENGTH,   // Length
                                          VIRTIO_DEV_SPECIFIC_NEXT_CAP_PTR, // Next cap ptr
                                          VIRTIO_CAPID // Cap ID
                                        } : {32{1'b0}};

    generate                                   
        for (i=0; i<TOTAL_PF_COUNT; i=i+1) begin:devspecificrdpath                                   
            assign virtio_pf_devspecific_bar_data[(32*(i+1))-1:(i*32)]    = read_pf_devspecific_bar_sel_i[i] ? 
                                                                            {
                                                                             24'h000000, // Pad Zeros
                                                                             pf_virtio_devspecific_bar_indicator[(8*(i+1))-1:(i*8)]     //PF0_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0] // Bar Indicator
                                                                            } : {32{1'b0}};
                                       
            assign virtio_vf_devspecific_bar_data[(32*(i+1))-1:(i*32)]    = read_vf_devspecific_bar_sel_i[i] ? 
                                                                            {
                                                                             24'h000000, // Pad Zeros
                                                                             vf_virtio_devspecific_bar_indicator[(8*(i+1))-1:(i*8)]     //PF0VF_VIRTIO_ISRSTATUS_BAR_INDICATOR[7:0] // Bar Indicator
                                                                            } : {32{1'b0}};
                                       
            assign virtio_pf_devspecific_baroffset_data[(32*(i+1))-1:(i*32)]    = read_pf_devspecific_baroffset_sel_i[i] ? 
                                                                                  pf_virtio_devspecific_bar_offset[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                                       
            assign virtio_vf_devspecific_baroffset_data[(32*(i+1))-1:(i*32)]    = read_vf_devspecific_baroffset_sel_i[i] ? 
                                                                                  vf_virtio_devspecific_bar_offset[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                                       
            assign virtio_pf_devspecific_length_data[(32*(i+1))-1:(i*32)]       = read_pf_devspecific_length_sel_i[i] ? 
                                                                                  pf_virtio_devspecific_structure_length[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                                       
            assign virtio_vf_devspecific_length_data[(32*(i+1))-1:(i*32)]       = read_vf_devspecific_length_sel_i[i] ? 
                                                                                  vf_virtio_devspecific_structure_length[(32*(i+1))-1:(i*32)] : {32{1'b0}};
                

            assign virtio_pf_devspecific_bar_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_devspecific_bar_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_pf_devspecific_bar_data[(32*(i+1))-1:(i*32)];
            assign virtio_pf_devspecific_length_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_devspecific_length_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_pf_devspecific_length_data[(32*(i+1))-1:(i*32)];
            assign virtio_pf_devspecific_baroffset_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_pf_devspecific_baroffset_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_pf_devspecific_baroffset_data[(32*(i+1))-1:(i*32)];
            assign virtio_vf_devspecific_bar_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_devspecific_bar_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_vf_devspecific_bar_data[(32*(i+1))-1:(i*32)];
            assign virtio_vf_devspecific_length_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_devspecific_length_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_vf_devspecific_length_data[(32*(i+1))-1:(i*32)];
            assign virtio_vf_devspecific_baroffset_dataor [(32*(i+2))-1:((i+1)*32)]  =  
                                                     virtio_vf_devspecific_baroffset_dataor [(32*(i+1))-1:(i*32)] 
                                                     | virtio_vf_devspecific_baroffset_data[(32*(i+1))-1:(i*32)];
        end
    endgenerate  
    
    assign virtio_devspecific_rdack = (read_devspecific_header_sel_i | (|read_pf_devspecific_bar_sel_i) | (|read_vf_devspecific_bar_sel_i)
                                    | (|read_pf_devspecific_baroffset_sel_i) | (|read_vf_devspecific_baroffset_sel_i) 
                                    | (|read_pf_devspecific_length_sel_i) | (|read_vf_devspecific_length_sel_i)) ? 'b1 : 'b0;
                                    
    // PCI CFG Access header
    assign virtio_pcicfgaccess_header_data = read_pcicfgaccess_header_sel_i ? 
                                               {
                                                VIRTIO_PCICFG_ACCESS_TYPE,         // Config Type
                                                VIRTIO_PCICFG_ACCESS_CAP_LENGTH,   // Length
                                                VIRTIO_PCICFG_ACCESS_NEXT_CAP_PTR, // Next cap ptr
                                                VIRTIO_CAPID // Cap ID
                                              } : {32{1'b0}};

    assign virtio_pcicfgaccess_cfgdata_data   = 
                virtio_vf_pcicfgaccess_cfgdata_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32] | 
                virtio_pf_pcicfgaccess_cfgdata_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32];
    assign virtio_pcicfgaccess_bar_data       = 
                virtio_vf_pcicfgaccess_bar_dataor[((TOTAL_PF_COUNT+1)*32)-1-(32-8): TOTAL_PF_COUNT*32] | 
                virtio_pf_pcicfgaccess_bar_dataor[((TOTAL_PF_COUNT+1)*32)-1-(32-8): TOTAL_PF_COUNT*32];
    assign virtio_pcicfgaccess_length_data    = 
                virtio_vf_pcicfgaccess_length_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32] | 
                virtio_pf_pcicfgaccess_length_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32];
    assign virtio_pcicfgaccess_baroffset_data = 
                virtio_vf_pcicfgaccess_baroffset_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32] | 
                virtio_pf_pcicfgaccess_baroffset_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32];
                                           
                                       
    // Read data back outputs. ARBT for a PF data and a VF data
    // bar data is 8-bit, others are 32-bit data after OR.
    assign virtio_cmncfg_bar_data       =
                virtio_vf_cmncfg_bar_dataor[((TOTAL_PF_COUNT+1)*32)-1-(32-8): TOTAL_PF_COUNT*32] | 
                virtio_pf_cmncfg_bar_dataor[((TOTAL_PF_COUNT+1)*32)-1-(32-8): TOTAL_PF_COUNT*32];
    assign virtio_cmncfg_length_data    = 
                virtio_vf_cmncfg_length_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32] | 
                virtio_pf_cmncfg_length_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32];
    assign virtio_cmncfg_baroffset_data = 
                virtio_vf_cmncfg_baroffset_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32] | 
                virtio_pf_cmncfg_baroffset_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32];
    
    assign virtio_notifications_bar_data       = 
                virtio_vf_notifications_bar_dataor[((TOTAL_PF_COUNT+1)*32)-1-(32-8): TOTAL_PF_COUNT*32] | 
                virtio_pf_notifications_bar_dataor[((TOTAL_PF_COUNT+1)*32)-1-(32-8): TOTAL_PF_COUNT*32];
    assign virtio_notifications_length_data    = 
                virtio_vf_notifications_length_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32] | 
                virtio_pf_notifications_length_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32];
    assign virtio_notifications_baroffset_data = 
                virtio_vf_notifications_baroffset_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32] | 
                virtio_pf_notifications_baroffset_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32];
    assign virtio_notifications_notifyoff_data = 
                virtio_vf_notifications_notifyoff_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32] | 
                virtio_pf_notifications_notifyoff_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32];
    
    assign virtio_isrstatus_header_data          = 
                virtio_vf_isrstatus_header_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32] | 
                virtio_pf_isrstatus_header_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32];
    assign virtio_isrstatus_bar_data          = 
                virtio_vf_isrstatus_bar_dataor[((TOTAL_PF_COUNT+1)*32)-1-(32-8): TOTAL_PF_COUNT*32] | 
                virtio_pf_isrstatus_bar_dataor[((TOTAL_PF_COUNT+1)*32)-1-(32-8): TOTAL_PF_COUNT*32];
    assign virtio_isrstatus_length_data       = 
                virtio_vf_isrstatus_length_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32] | 
                virtio_pf_isrstatus_length_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32];
    assign virtio_isrstatus_baroffset_data    = 
                virtio_vf_isrstatus_baroffset_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32] | 
                virtio_pf_isrstatus_baroffset_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32];
    
    assign virtio_devspecific_bar_data        = 
                virtio_vf_devspecific_bar_dataor[((TOTAL_PF_COUNT+1)*32)-1-(32-8): TOTAL_PF_COUNT*32] | 
                virtio_pf_devspecific_bar_dataor[((TOTAL_PF_COUNT+1)*32)-1-(32-8): TOTAL_PF_COUNT*32];
    assign virtio_devspecific_length_data     = 
                virtio_vf_devspecific_length_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32] | 
                virtio_pf_devspecific_length_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32];
    assign virtio_devspecific_baroffset_data  = 
                virtio_vf_devspecific_baroffset_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32] | 
                virtio_pf_devspecific_baroffset_dataor[((TOTAL_PF_COUNT+1)*32)-1: TOTAL_PF_COUNT*32];
    
    // Generate pci cfg access ack    
    assign virtio_cfg_req_ack = (|virtio_pf_cfg_req_ack) | (|virtio_pfvf_cfg_req_ack);
    
    // Avoid rdreq ack pulse b2b to a wrreq ack
    always @ (posedge clk) begin
        if (~rstn) begin 
            cii_wr_reg      <= 1'b0;
            cii_wr_q        <= '0;
            cii_wr_qq       <= '0;
        end    
        else begin
            cii_wr_q        <= cii_wr_i;
            cii_wr_qq       <= cii_wr_q;
            cii_wr_reg      <= cii_wr_qq;
        end
    end  
    
    always @ (posedge clk) begin
        if (~rstn) begin
            virtio_cmncfg_header_data_reg           <= {32{1'b0}};
            virtio_cmncfg_bar_data_reg              <= {8{1'b0}};
            virtio_cmncfg_length_data_reg           <= {32{1'b0}};
            virtio_cmncfg_baroffset_data_reg        <= {32{1'b0}};
            virtio_notifications_header_data_reg        <= {32{1'b0}};
            virtio_notifications_bar_data_reg           <= {8{1'b0}};
            virtio_notifications_length_data_reg        <= {32{1'b0}};
            virtio_notifications_baroffset_data_reg     <= {32{1'b0}};
            virtio_notifications_notifyoff_data_reg     <= {32{1'b0}};
            virtio_isrstatus_header_data_reg        <= {32{1'b0}};
            virtio_isrstatus_bar_data_reg           <= {8{1'b0}};
            virtio_isrstatus_length_data_reg        <= {32{1'b0}};
            virtio_isrstatus_baroffset_data_reg     <= {32{1'b0}};
            virtio_devspecific_header_data_reg          <= {32{1'b0}};
            virtio_devspecific_bar_data_reg             <= {8{1'b0}};
            virtio_devspecific_length_data_reg          <= {32{1'b0}};
            virtio_devspecific_baroffset_data_reg       <= {32{1'b0}};
            virtio_pcicfgaccess_header_data_reg      <= {32{1'b0}};
            virtio_pcicfgaccess_bar_data_reg         <= {8{1'b0}};
            virtio_pcicfgaccess_length_data_reg      <= {32{1'b0}};
            virtio_pcicfgaccess_baroffset_data_reg   <= {32{1'b0}};
            virtio_pcicfgaccess_cfgdata_data_reg     <= {32{1'b0}};
            virtio_cfgwr_req_ack_reg      <= 1'b0;
            virtio_cfgrd_req_ack_reg      <= 1'b0;
            virtio_cfgrd_req_ack_o        <= 1'b0;
            virtio_roreg_rdack_reg0       <= 1'b0;
            virtio_roreg_rdack_reg1       <= 1'b0;
        end 
        else begin
            virtio_cmncfg_header_data_reg           <= virtio_cmncfg_header_data;
            virtio_cmncfg_bar_data_reg              <= virtio_cmncfg_bar_data;
            virtio_cmncfg_length_data_reg           <= virtio_cmncfg_length_data;
            virtio_cmncfg_baroffset_data_reg        <= virtio_cmncfg_baroffset_data;
            virtio_notifications_header_data_reg        <= virtio_notifications_header_data;
            virtio_notifications_bar_data_reg           <= virtio_notifications_bar_data;
            virtio_notifications_length_data_reg        <= virtio_notifications_length_data;
            virtio_notifications_baroffset_data_reg     <= virtio_notifications_baroffset_data;
            virtio_notifications_notifyoff_data_reg     <= virtio_notifications_notifyoff_data;
            virtio_isrstatus_header_data_reg        <= virtio_isrstatus_header_data;
            virtio_isrstatus_bar_data_reg           <= virtio_isrstatus_bar_data;
            virtio_isrstatus_length_data_reg        <= virtio_isrstatus_length_data;
            virtio_isrstatus_baroffset_data_reg     <= virtio_isrstatus_baroffset_data;
            virtio_devspecific_header_data_reg          <= virtio_devspecific_header_data;
            virtio_devspecific_bar_data_reg             <= virtio_devspecific_bar_data;
            virtio_devspecific_length_data_reg          <= virtio_devspecific_length_data;
            virtio_devspecific_baroffset_data_reg       <= virtio_devspecific_baroffset_data;
            virtio_pcicfgaccess_header_data_reg      <= virtio_pcicfgaccess_header_data;
            virtio_pcicfgaccess_bar_data_reg         <= virtio_pcicfgaccess_bar_data;
            virtio_pcicfgaccess_length_data_reg      <= virtio_pcicfgaccess_length_data;
            virtio_pcicfgaccess_baroffset_data_reg   <= virtio_pcicfgaccess_baroffset_data;
            virtio_pcicfgaccess_cfgdata_data_reg     <= virtio_pcicfgaccess_cfgdata_data;
            
            if ((|read_pcicfgaccess_header_sel_i) | virtio_cmncfg_rdack | virtio_notifications_rdack
                 | virtio_isrstatus_rdack | virtio_devspecific_rdack) begin
                virtio_roreg_rdack_reg0          <= 1'b1;
                virtio_roreg_rdack_reg1          <= virtio_roreg_rdack_reg0;
            end
            else begin
                virtio_roreg_rdack_reg1          <= 1'b0;
            end
            // Read ack. PCI cfg access rd to RW ack, or rd to RO dummy ack
            if ((~cii_wr_reg & virtio_cfg_req_ack) | virtio_roreg_rdack_reg1) begin 
                virtio_cfgrd_req_ack_reg         <= 1'b1;
                virtio_cfgrd_req_ack_o           <= virtio_cfgrd_req_ack_reg;
            end
            else begin
                virtio_cfgrd_req_ack_o           <= 1'b0;
            end
            // Write ack. PCI cfg access wr to RW ack, or wr to RO dummy ack
            virtio_cfgwr_req_ack_reg             <= (cii_wr_reg & virtio_cfg_req_ack) | write_roreg_sel_i;
        end
    end

    assign  virtio_cmncfg_header_data_o            = virtio_cmncfg_header_data_reg;
    assign  virtio_cmncfg_bar_data_o               = virtio_cmncfg_bar_data_reg;
    assign  virtio_cmncfg_length_data_o            = virtio_cmncfg_length_data_reg;
    assign  virtio_cmncfg_baroffset_data_o         = virtio_cmncfg_baroffset_data_reg;
    assign  virtio_notifications_header_data_o         = virtio_notifications_header_data_reg;
    assign  virtio_notifications_bar_data_o            = virtio_notifications_bar_data_reg;
    assign  virtio_notifications_length_data_o         = virtio_notifications_length_data_reg;
    assign  virtio_notifications_baroffset_data_o      = virtio_notifications_baroffset_data_reg;
    assign  virtio_notifications_notifyoff_data_o      = virtio_notifications_notifyoff_data_reg;
    assign  virtio_isrstatus_header_data_o         = virtio_isrstatus_header_data_reg;
    assign  virtio_isrstatus_bar_data_o            = virtio_isrstatus_bar_data_reg;
    assign  virtio_isrstatus_length_data_o         = virtio_isrstatus_length_data_reg;
    assign  virtio_isrstatus_baroffset_data_o      = virtio_isrstatus_baroffset_data_reg;
    assign  virtio_devspecific_header_data_o           = virtio_devspecific_header_data_reg;
    assign  virtio_devspecific_bar_data_o              = virtio_devspecific_bar_data_reg;
    assign  virtio_devspecific_length_data_o           = virtio_devspecific_length_data_reg;
    assign  virtio_devspecific_baroffset_data_o        = virtio_devspecific_baroffset_data_reg;
    assign  virtio_pcicfgaccess_header_data_o       = virtio_pcicfgaccess_header_data_reg;
    assign  virtio_pcicfgaccess_bar_data_o          = virtio_pcicfgaccess_bar_data_reg;
    assign  virtio_pcicfgaccess_length_data_o       = virtio_pcicfgaccess_length_data_reg;
    assign  virtio_pcicfgaccess_baroffset_data_o    = virtio_pcicfgaccess_baroffset_data_reg;
    assign  virtio_pcicfgaccess_cfgdata_data_o      = virtio_pcicfgaccess_cfgdata_data_reg;
    
    assign virtio_cfgwr_req_ack_o         = virtio_cfgwr_req_ack_reg;
    // assign virtio_cfgrd_req_ack_o         = virtio_cfgrd_req_ack_reg;
    
endmodule

