// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

//-----------------------------------------------------------------------------
// Description   : WHR PCI Express PF and VF VirtIO Top level module
//-----------------------------------------------------------------------------
// File          : intel_pcie_virtio_top.v
// Author        : Intel Corporation
//-----------------------------------------------------------------------------
//

module intel_pcie_virtio_top #
    (
        //****************************************************************************************************
        // High-level parameters
        //****************************************************************************************************
        parameter TOTAL_PF_COUNT                      = 8, // Total number of PFs 
        parameter TOTAL_PF_COUNT_WIDTH                = 3, 
        parameter TOTAL_VF_COUNT                      = 5, // Total number of VFs
                                                              // 1 - 8 (in steps of 1),
                                                              // 8 - 256 (in multiples of 4)
                                                              // 256 - 1024 (in multiples of 64)
                                                              // 1024 - 4096 (in multiples of 512)
                                                              // Sum of PF0_VF_COUNT, PF1_VF_COUNT, PF2_VF_COUNT and
                                                              // PF3_VF_COUNT must equal TOTAL_VF_COUNT
        parameter TOTAL_VF_COUNT_WIDTH                = 3,   // Number of bits needed to represent TOTAL_VF_COUNT
        parameter PF0_VF_COUNT                        = 11'd1, // Number of VFs attached to PF 0
        parameter PF1_VF_COUNT                        = 11'd4,   // Number of VFs attached to PF 1
        parameter PF2_VF_COUNT                        = 11'd0,    // Number of VFs attached to PF 2
        parameter PF3_VF_COUNT                        = 11'd0,    // Number of VFs attached to PF 3
        parameter PF4_VF_COUNT                        = 11'd0,    // Number of VFs attached to PF 4
        parameter PF5_VF_COUNT                        = 11'd0,    // Number of VFs attached to PF 5
        parameter PF6_VF_COUNT                        = 11'd0,    // Number of VFs attached to PF 6
        parameter PF7_VF_COUNT                        = 11'd0,    // Number of VFs attached to PF 7
        // parameter VIRTIO_CFGACCESS_DATA_ACK           = 1,    // Data Ack Option for VirtIO CFG Read Access to CFG Data
                                                              // // If true, only store the returned data to pci_cfg_data register without sending back to the wire side.
        parameter REG_ADDR_WIDTH                      = 10,   // The double word register address.
        parameter VIRTIO_CMN_CONFIG_BA                = 8'h48,    // Byte Address

        //****************************************************************************************************
        // VirtIO Specific Parameters PF0
        // //****************************************************************************************************   
        parameter       PF0_VIRTIO_CAPABILITY_PRESENT                    = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter       PF0_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT           = 1'b0,            //This is optional capability for VIRTIO
   
        parameter       PF0_VIRTIO_CMN_CONFIG_BAR_INDICATOR              = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF0_VIRTIO_NOTIFICATION_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF0_VIRTIO_ISRSTATUS_BAR_INDICATOR               = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF0_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR        = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF0_VIRTIO_DEVSPECIFIC_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF0_VIRTIO_CMN_CONFIG_BAR_OFFSET                 = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF0_VIRTIO_NOTIFICATION_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF0_VIRTIO_ISRSTATUS_BAR_OFFSET                  = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF0_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET           = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF0_VIRTIO_DEVSPECIFIC_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF0_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH           = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF0_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF0_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH            = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF0_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH     = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF0_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF0_VIRTIO_NOTIFY_OFF_MULTIPLIER                 = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF1
        //****************************************************************************************************
        parameter       PF1_VIRTIO_CAPABILITY_PRESENT                    = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter       PF1_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT           = 1'b0,            //This is optional capability for VIRTIO
   
        parameter       PF1_VIRTIO_CMN_CONFIG_BAR_INDICATOR              = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF1_VIRTIO_NOTIFICATION_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF1_VIRTIO_ISRSTATUS_BAR_INDICATOR               = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF1_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR        = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF1_VIRTIO_DEVSPECIFIC_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF1_VIRTIO_CMN_CONFIG_BAR_OFFSET                 = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF1_VIRTIO_NOTIFICATION_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF1_VIRTIO_ISRSTATUS_BAR_OFFSET                  = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF1_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET           = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF1_VIRTIO_DEVSPECIFIC_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF1_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH           = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF1_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF1_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH            = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF1_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH     = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF1_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF1_VIRTIO_NOTIFY_OFF_MULTIPLIER                 = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF2
        //****************************************************************************************************
        parameter       PF2_VIRTIO_CAPABILITY_PRESENT                    = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter       PF2_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT           = 1'b0,            //This is optional capability for VIRTIO
   
        parameter       PF2_VIRTIO_CMN_CONFIG_BAR_INDICATOR              = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF2_VIRTIO_NOTIFICATION_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF2_VIRTIO_ISRSTATUS_BAR_INDICATOR               = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF2_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR        = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF2_VIRTIO_DEVSPECIFIC_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF2_VIRTIO_CMN_CONFIG_BAR_OFFSET                 = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF2_VIRTIO_NOTIFICATION_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF2_VIRTIO_ISRSTATUS_BAR_OFFSET                  = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF2_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET           = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF2_VIRTIO_DEVSPECIFIC_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF2_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH           = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF2_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF2_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH            = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF2_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH     = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF2_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF2_VIRTIO_NOTIFY_OFF_MULTIPLIER                 = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF3
        //****************************************************************************************************
        parameter       PF3_VIRTIO_CAPABILITY_PRESENT                    = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter       PF3_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT           = 1'b0,            //This is optional capability for VIRTIO
   
        parameter       PF3_VIRTIO_CMN_CONFIG_BAR_INDICATOR              = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF3_VIRTIO_NOTIFICATION_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF3_VIRTIO_ISRSTATUS_BAR_INDICATOR               = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF3_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR        = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF3_VIRTIO_DEVSPECIFIC_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF3_VIRTIO_CMN_CONFIG_BAR_OFFSET                 = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF3_VIRTIO_NOTIFICATION_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF3_VIRTIO_ISRSTATUS_BAR_OFFSET                  = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF3_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET           = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF3_VIRTIO_DEVSPECIFIC_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF3_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH           = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF3_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF3_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH            = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF3_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH     = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF3_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF3_VIRTIO_NOTIFY_OFF_MULTIPLIER                 = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF4
        //****************************************************************************************************   
        parameter       PF4_VIRTIO_CAPABILITY_PRESENT                    = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter       PF4_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT           = 1'b0,            //This is optional capability for VIRTIO
   
        parameter       PF4_VIRTIO_CMN_CONFIG_BAR_INDICATOR              = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF4_VIRTIO_NOTIFICATION_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF4_VIRTIO_ISRSTATUS_BAR_INDICATOR               = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF4_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR        = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF4_VIRTIO_DEVSPECIFIC_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF4_VIRTIO_CMN_CONFIG_BAR_OFFSET                 = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF4_VIRTIO_NOTIFICATION_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF4_VIRTIO_ISRSTATUS_BAR_OFFSET                  = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF4_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET           = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF4_VIRTIO_DEVSPECIFIC_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF4_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH           = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF4_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF4_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH            = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF4_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH     = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF4_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF4_VIRTIO_NOTIFY_OFF_MULTIPLIER                 = 32'h00000008, //Indicates Multiplier for queue_notify_off  

        //****************************************************************************************************
        // VirtIO Specific Parameters PF5
        //****************************************************************************************************   
        parameter       PF5_VIRTIO_CAPABILITY_PRESENT                    = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter       PF5_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT           = 1'b0,            //This is optional capability for VIRTIO
   
        parameter       PF5_VIRTIO_CMN_CONFIG_BAR_INDICATOR              = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF5_VIRTIO_NOTIFICATION_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF5_VIRTIO_ISRSTATUS_BAR_INDICATOR               = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF5_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR        = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF5_VIRTIO_DEVSPECIFIC_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF5_VIRTIO_CMN_CONFIG_BAR_OFFSET                 = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF5_VIRTIO_NOTIFICATION_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF5_VIRTIO_ISRSTATUS_BAR_OFFSET                  = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF5_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET           = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF5_VIRTIO_DEVSPECIFIC_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF5_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH           = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF5_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF5_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH            = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF5_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH     = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF5_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF5_VIRTIO_NOTIFY_OFF_MULTIPLIER                 = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF6
        //****************************************************************************************************   
        parameter       PF6_VIRTIO_CAPABILITY_PRESENT                    = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter       PF6_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT           = 1'b0,            //This is optional capability for VIRTIO
   
        parameter       PF6_VIRTIO_CMN_CONFIG_BAR_INDICATOR              = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF6_VIRTIO_NOTIFICATION_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF6_VIRTIO_ISRSTATUS_BAR_INDICATOR               = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF6_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR        = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF6_VIRTIO_DEVSPECIFIC_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF6_VIRTIO_CMN_CONFIG_BAR_OFFSET                 = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF6_VIRTIO_NOTIFICATION_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF6_VIRTIO_ISRSTATUS_BAR_OFFSET                  = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF6_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET           = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF6_VIRTIO_DEVSPECIFIC_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF6_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH           = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF6_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF6_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH            = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF6_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH     = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF6_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF6_VIRTIO_NOTIFY_OFF_MULTIPLIER                 = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF7
        //****************************************************************************************************   
        parameter       PF7_VIRTIO_CAPABILITY_PRESENT                    = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter       PF7_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT           = 1'b0,            //This is optional capability for VIRTIO
   
        parameter       PF7_VIRTIO_CMN_CONFIG_BAR_INDICATOR              = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF7_VIRTIO_NOTIFICATION_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF7_VIRTIO_ISRSTATUS_BAR_INDICATOR               = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF7_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR        = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF7_VIRTIO_DEVSPECIFIC_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF7_VIRTIO_CMN_CONFIG_BAR_OFFSET                 = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF7_VIRTIO_NOTIFICATION_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF7_VIRTIO_ISRSTATUS_BAR_OFFSET                  = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF7_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET           = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF7_VIRTIO_DEVSPECIFIC_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF7_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH           = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF7_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF7_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH            = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF7_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH     = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF7_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF7_VIRTIO_NOTIFY_OFF_MULTIPLIER                 = 32'h00000008, //Indicates Multiplier for queue_notify_off  
   
        //****************************************************************************************************
        // VirtIO Specific Parameters PF0 VFs
        //****************************************************************************************************   
        parameter       PF0VF_VIRTIO_CAPABILITY_PRESENT                  = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter       PF0VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT         = 1'b0,            //This is optional capability for VIRTIO
   
        parameter       PF0VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF0VF_VIRTIO_NOTIFICATION_BAR_INDICATOR          = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF0VF_VIRTIO_ISRSTATUS_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF0VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR      = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF0VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR           = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF0VF_VIRTIO_CMN_CONFIG_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF0VF_VIRTIO_NOTIFICATION_BAR_OFFSET             = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF0VF_VIRTIO_ISRSTATUS_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF0VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET         = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF0VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET              = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF0VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF0VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH       = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF0VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF0VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH   = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF0VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH        = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF0VF_VIRTIO_NOTIFY_OFF_MULTIPLIER               = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF1 VFs
        //****************************************************************************************************
        parameter       PF1VF_VIRTIO_CAPABILITY_PRESENT                  = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter       PF1VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT         = 1'b0,            //This is optional capability for VIRTIO
   
        parameter       PF1VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF1VF_VIRTIO_NOTIFICATION_BAR_INDICATOR          = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF1VF_VIRTIO_ISRSTATUS_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF1VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR      = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF1VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR           = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF1VF_VIRTIO_CMN_CONFIG_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF1VF_VIRTIO_NOTIFICATION_BAR_OFFSET             = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF1VF_VIRTIO_ISRSTATUS_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF1VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET         = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF1VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET              = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF1VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF1VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH       = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF1VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF1VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH   = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF1VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH        = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF1VF_VIRTIO_NOTIFY_OFF_MULTIPLIER               = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF2 VFs
        //****************************************************************************************************
        parameter       PF2VF_VIRTIO_CAPABILITY_PRESENT                  = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter       PF2VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT         = 1'b0,            //This is optional capability for VIRTIO
   
        parameter       PF2VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF2VF_VIRTIO_NOTIFICATION_BAR_INDICATOR          = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF2VF_VIRTIO_ISRSTATUS_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF2VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR      = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF2VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR           = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF2VF_VIRTIO_CMN_CONFIG_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF2VF_VIRTIO_NOTIFICATION_BAR_OFFSET             = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF2VF_VIRTIO_ISRSTATUS_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF2VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET         = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF2VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET              = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF2VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF2VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH       = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF2VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF2VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH   = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF2VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH        = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF2VF_VIRTIO_NOTIFY_OFF_MULTIPLIER               = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF3 VFs
        //****************************************************************************************************
        parameter       PF3VF_VIRTIO_CAPABILITY_PRESENT                  = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter       PF3VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT         = 1'b0,            //This is optional capability for VIRTIO
   
        parameter       PF3VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF3VF_VIRTIO_NOTIFICATION_BAR_INDICATOR          = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF3VF_VIRTIO_ISRSTATUS_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF3VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR      = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF3VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR           = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF3VF_VIRTIO_CMN_CONFIG_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF3VF_VIRTIO_NOTIFICATION_BAR_OFFSET             = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF3VF_VIRTIO_ISRSTATUS_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF3VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET         = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF3VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET              = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF3VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF3VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH       = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF3VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF3VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH   = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF3VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH        = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF3VF_VIRTIO_NOTIFY_OFF_MULTIPLIER               = 32'h00000008, //Indicates Multiplier for queue_notify_off  

        //****************************************************************************************************
        // VirtIO Specific Parameters PF4 VFs
        //****************************************************************************************************   
        parameter       PF4VF_VIRTIO_CAPABILITY_PRESENT                  = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter       PF4VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT         = 1'b0,            //This is optional capability for VIRTIO
   
        parameter       PF4VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF4VF_VIRTIO_NOTIFICATION_BAR_INDICATOR          = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF4VF_VIRTIO_ISRSTATUS_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF4VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR      = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF4VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR           = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF4VF_VIRTIO_CMN_CONFIG_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF4VF_VIRTIO_NOTIFICATION_BAR_OFFSET             = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF4VF_VIRTIO_ISRSTATUS_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF4VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET         = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF4VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET              = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF4VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF4VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH       = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF4VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF4VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH   = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF4VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH        = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF4VF_VIRTIO_NOTIFY_OFF_MULTIPLIER               = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF5 VFs
        //****************************************************************************************************
        parameter       PF5VF_VIRTIO_CAPABILITY_PRESENT                  = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter       PF5VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT         = 1'b0,            //This is optional capability for VIRTIO
   
        parameter       PF5VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF5VF_VIRTIO_NOTIFICATION_BAR_INDICATOR          = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF5VF_VIRTIO_ISRSTATUS_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF5VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR      = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF5VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR           = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF5VF_VIRTIO_CMN_CONFIG_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF5VF_VIRTIO_NOTIFICATION_BAR_OFFSET             = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF5VF_VIRTIO_ISRSTATUS_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF5VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET         = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF5VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET              = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF5VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF5VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH       = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF5VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF5VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH   = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF5VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH        = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF5VF_VIRTIO_NOTIFY_OFF_MULTIPLIER               = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF6 VFs
        //****************************************************************************************************
        parameter       PF6VF_VIRTIO_CAPABILITY_PRESENT                  = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter       PF6VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT         = 1'b0,            //This is optional capability for VIRTIO
   
        parameter       PF6VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF6VF_VIRTIO_NOTIFICATION_BAR_INDICATOR          = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF6VF_VIRTIO_ISRSTATUS_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF6VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR      = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF6VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR           = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF6VF_VIRTIO_CMN_CONFIG_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF6VF_VIRTIO_NOTIFICATION_BAR_OFFSET             = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF6VF_VIRTIO_ISRSTATUS_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF6VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET         = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF6VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET              = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF6VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF6VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH       = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF6VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF6VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH   = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF6VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH        = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF6VF_VIRTIO_NOTIFY_OFF_MULTIPLIER               = 32'h00000008, //Indicates Multiplier for queue_notify_off  
  
        //****************************************************************************************************
        // VirtIO Specific Parameters PF7 VFs
        //****************************************************************************************************
        parameter       PF7VF_VIRTIO_CAPABILITY_PRESENT                  = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter       PF7VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT         = 1'b0,            //This is optional capability for VIRTIO
   
        parameter       PF7VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR            = 8'h00,        //Indicates BAR holding the Common Config Structure
        parameter       PF7VF_VIRTIO_NOTIFICATION_BAR_INDICATOR          = 8'h00,        //Indicates BAR holding the Notification Structure
        parameter       PF7VF_VIRTIO_ISRSTATUS_BAR_INDICATOR             = 8'h00,        //Indicates BAR holding the ISR STATUS Structure 
        parameter       PF7VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR      = 8'h00,        //Indicates BAR holding the PCI Config Access Structure 
        parameter       PF7VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR           = 8'h00,        //Indicates BAR holding the Device Specific Configuration Structure    
                                                                    
        parameter       PF7VF_VIRTIO_CMN_CONFIG_BAR_OFFSET               = 32'h00000000, //Indicates Starting position of Common Config Structure in given BAR
        parameter       PF7VF_VIRTIO_NOTIFICATION_BAR_OFFSET             = 32'h00000000, //Indicates Starting position of Notification Structure in given BAR
        parameter       PF7VF_VIRTIO_ISRSTATUS_BAR_OFFSET                = 32'h00000000, //Indicates Starting position of ISR STATUS Structure in given BAR
        parameter       PF7VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET         = 32'h00000000, //Indicates Starting position of PCI Config Access Structure in given BAR
        parameter       PF7VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET              = 32'h00000000, //Indicates Starting position of Device Specific Configuration Structure in given BAR
                                                                    
        parameter       PF7VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH         = 32'h00000008, //Indicates length of Common Config Structure
        parameter       PF7VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH       = 32'h00000008, //Indicates length of Notification Structure
        parameter       PF7VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH          = 32'h00000008, //Indicates length of ISR STATUS Structure 
        parameter       PF7VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH   = 32'h00000008, //Indicates length of PCI Config Access Structure 
        parameter       PF7VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH        = 32'h00000008, //Indicates length of Device Specific Configuration Structure    
        parameter       PF7VF_VIRTIO_NOTIFY_OFF_MULTIPLIER               = 32'h00000008  //Indicates Multiplier for queue_notify_off  
    
    )
    (
        input                                 pld_clk,
        input                                 pld_rst_n, 
        // FLR Interafce
        input  [TOTAL_PF_COUNT-1:0]           flr_rcvd_pf_i,  // 1 bit per PF.  Level signal indicate FLR for corresponding PF is in progress.
        input                                 flr_rcvd_vf_i,  // One-cycle pulse indicates that an FLR was received from host targeting a VF.
        input  [TOTAL_PF_COUNT_WIDTH-1:0]     flr_rcvd_pf_num_i,
        input  [TOTAL_VF_COUNT_WIDTH-1:0]     flr_rcvd_vf_num_i,
        // CII HIP Interface
        input                                 cii_req_i,       // Indicates the CFG request is intercepted and all the other CII signals are valid.                                           
        input                                 cii_hdr_poisoned_i,
        input  [3:0]                          cii_hdr_first_be_i,    // The first dword byte enable field in the received TLP header on CII.
        input  [TOTAL_PF_COUNT_WIDTH-1:0]     cii_func_num_i,        // The function number in the received TLP header on CII.
        input                                 cii_wr_vf_active_i,    // Indicates that the received TLP was targeted to a VF inside the controller, and is identified by cii_vf_num.    
        input  [TOTAL_VF_COUNT_WIDTH-1:0]     cii_vf_num_i,
        input                                 cii_wr_i,        // Indicates that the cii_dout_p0/1 is valid. This signal is asserted only for a configuration write request.
        input  [REG_ADDR_WIDTH-1:0]           cii_addr_i,      // The dword register address in the received TLP header on CII.
        input  [31:0]                         cii_dout_i,      // Received TLP payload data from the link partner to your application client. 
                                                               // The data is in little endian format. The first received payload byte is in [7:0]
        output                                cii_override_en_o,     // When your application logic asserts this output, the HIP overrides the CfgWr payload 
                                                                     // or CfgRd completion using the data supplied by your application logic on cii_override_din.
        output [31:0]                         cii_override_din_o,
        output                                cii_halt_o,      // When cii_halt_p0/1 is asserted, the HIP halts the processing of CFG reqs for the PCIe config space regs.

        // CII User Interface
        output                                pin_cii_req_o,       // Indicates the CFG request is intercepted and all the other CII signals are valid.                                           
        output                                pin_cii_hdr_poisoned_o,
        output [3:0]                          pin_cii_hdr_first_be_o,    // The first dword byte enable field in the received TLP header on CII.
        output [TOTAL_PF_COUNT_WIDTH-1:0]     pin_cii_func_num_o,        // The function number in the received TLP header on CII.
        output                                pin_cii_wr_vf_active_o,    // Indicates that the received TLP was targeted to a VF inside the controller, and is identified by cii_vf_num.    
        output [TOTAL_VF_COUNT_WIDTH-1:0]     pin_cii_vf_num_o,
        output                                pin_cii_wr_o,        // Indicates that the cii_dout_p0/1 is valid. This signal is asserted only for a configuration write request.
        output [REG_ADDR_WIDTH-1:0]           pin_cii_addr_o,      // The dword register address in the received TLP header on CII.
        output [31:0]                         pin_cii_dout_o,      // Received TLP payload data from the link partner to your application client. 
                                                               // The data is in little endian format. The first received payload byte is in [7:0]
        input                                 pin_cii_override_en_i,     // When your application logic asserts this output, the HIP overrides the CfgWr payload 
                                                                     // or CfgRd completion using the data supplied by your application logic on cii_override_din.
        input  [31:0]                         pin_cii_override_din_i,
        input                                 pin_cii_halt_i,      // When cii_halt_p0/1 is asserted, the HIP halts the processing of CFG reqs for the PCIe config space regs.

        // VirtIO PCI Config Access Interface
        output                                virtio_pcicfg_vfaccess_o,        
        output [TOTAL_VF_COUNT_WIDTH-1:0]     virtio_pcicfg_vfnum_o,
        output [TOTAL_PF_COUNT_WIDTH-1:0]     virtio_pcicfg_pfnum_o,
        output [7:0]                          virtio_pcicfg_bar_o,
        output [31:0]                         virtio_pcicfg_length_o,
        output [31:0]                         virtio_pcicfg_baroffset_o,
        output [31:0]                         virtio_pcicfg_cfgdata_o,
        output                                virtio_pcicfg_cfgwr_o,        
        output                                virtio_pcicfg_cfgrd_o, 
              
        // input                                 virtio_pcicfg_appvf_i,     // Application indicates the read ack data targets a VF.
        input  [TOTAL_VF_COUNT_WIDTH-1:0]     virtio_pcicfg_appvfnum_i,  // Application indicates for which VF offset.
        input  [TOTAL_PF_COUNT_WIDTH-1:0]     virtio_pcicfg_apppfnum_i,  // Application indicates for which PF.
        input                                 virtio_pcicfg_rdack_i,     // Application Read Data Ack to store config data.
        input  [3:0]                          virtio_pcicfg_rdbe_i,      // Application indicates which byte to store.
        input  [31:0]                         virtio_pcicfg_data_i       // Data to be stored in config data registers.
    );
    
    localparam TOTAL_PF_COUNT_MAX               = 8;
    localparam VIRTIO_CMN_CONFIG_REG_ADDR       = VIRTIO_CMN_CONFIG_BA >> 2;  // Divide by 4. Convert to Reg Address
    localparam VIRTIO_NOTIFICATION_REG_ADDR     = 8'h16;
    localparam VIRTIO_ISR_STATUS_REG_ADDR       = 8'h2F;
    localparam VIRTIO_DEV_SPECIFIC_REG_ADDR     = 8'h33;
    localparam VIRTIO_PCICFG_ACCESS_REG_ADDR    = 8'h37;
    localparam TOTAL_VF_COUNT_LCL               = (TOTAL_VF_COUNT==0)? 1 : TOTAL_VF_COUNT;  // avoid negative bit declarition when there's no VF enabled for virtio
   
    // Signal Declarations
    wire                             read_cmncfg_header_sel, read_notifications_header_sel, read_devspecific_header_sel, write_roreg_sel;
    wire                             read_pcicfgaccess_header_sel_o, targeted_cap_not_present, noregbank_selected;
    wire [TOTAL_PF_COUNT-1 : 0]      read_pf_cmncfg_bar_sel, read_vf_cmncfg_bar_sel, read_pf_cmncfg_baroffset_sel;
    wire [TOTAL_PF_COUNT-1 : 0]      read_vf_cmncfg_baroffset_sel, read_pf_cmncfg_length_sel, read_vf_cmncfg_length_sel;
    wire [TOTAL_PF_COUNT-1 : 0]      read_pf_notifications_bar_sel, read_vf_notifications_bar_sel, read_pf_notifications_baroffset_sel;
    wire [TOTAL_PF_COUNT-1 : 0]      read_vf_notifications_baroffset_sel, read_pf_notifications_length_sel, read_vf_notifications_length_sel;
    wire [TOTAL_PF_COUNT-1 : 0]      read_pf_notifications_notifyoff_sel, read_vf_notifications_notifyoff_sel;
    wire [TOTAL_PF_COUNT-1 : 0]      read_pf_isrstatus_header_sel, read_vf_isrstatus_header_sel;
    wire [TOTAL_PF_COUNT-1 : 0]      read_pf_isrstatus_bar_sel, read_vf_isrstatus_bar_sel, read_pf_isrstatus_baroffset_sel;
    wire [TOTAL_PF_COUNT-1 : 0]      read_vf_isrstatus_baroffset_sel, read_pf_isrstatus_length_sel, read_vf_isrstatus_length_sel;
    wire [TOTAL_PF_COUNT-1 : 0]      read_pf_devspecific_bar_sel, read_vf_devspecific_bar_sel, read_pf_devspecific_baroffset_sel;
    wire [TOTAL_PF_COUNT-1 : 0]      read_vf_devspecific_baroffset_sel, read_pf_devspecific_length_sel, read_vf_devspecific_length_sel;
    wire [TOTAL_PF_COUNT-1 : 0]      read_pf_pcicfgaccess_bar_sel, write_pf_pcicfgaccess_bar_sel, read_pf_pcicfgaccess_baroffset_sel;
    wire [TOTAL_PF_COUNT-1 : 0]      write_pf_pcicfgaccess_baroffset_sel, read_pf_pcicfgaccess_length_sel, write_pf_pcicfgaccess_length_sel;
    wire [TOTAL_PF_COUNT-1 : 0]      read_pf_pcicfgaccess_cfgdata_sel, write_pf_pcicfgaccess_cfgdata_sel;
    wire [TOTAL_PF_COUNT-1 : 0]      read_vf_pcicfgaccess_bar_sel, write_vf_pcicfgaccess_bar_sel;
    wire [TOTAL_PF_COUNT-1 : 0]      read_vf_pcicfgaccess_baroffset_sel, write_vf_pcicfgaccess_baroffset_sel;
    wire [TOTAL_PF_COUNT-1 : 0]      read_vf_pcicfgaccess_length_sel, write_vf_pcicfgaccess_length_sel;
    wire [TOTAL_PF_COUNT-1 : 0]      read_vf_pcicfgaccess_cfgdata_sel, write_vf_pcicfgaccess_cfgdata_sel;

    wire                     virtio_cfgwr_req_ack, virtio_cfgrd_req_ack;
    wire   [31:0]            virtio_cmncfg_header_data, virtio_notifications_header_data, virtio_isrstatus_header_data;
    wire   [31:0]            virtio_devspecific_header_data, virtio_pcicfgaccess_header_data;
    wire   [7:0 ]            virtio_cmncfg_bar_data, virtio_notifications_bar_data, virtio_isrstatus_bar_data;
    wire   [7:0 ]            virtio_devspecific_bar_data, virtio_pcicfgaccess_bar_data;
    wire   [31:0]            virtio_cmncfg_length_data, virtio_cmncfg_baroffset_data, virtio_notifications_length_data;
    wire   [31:0]            virtio_notifications_baroffset_data, virtio_notifications_notifyoff_data;
    wire   [31:0]            virtio_isrstatus_length_data, virtio_isrstatus_baroffset_data, virtio_devspecific_length_data;
    wire   [31:0]            virtio_devspecific_baroffset_data, virtio_pcicfgaccess_cfgdata_data;
    wire   [31:0]            virtio_pcicfgaccess_length_data, virtio_pcicfgaccess_baroffset_data;

    // FLR Interafce
    reg [TOTAL_PF_COUNT-1:0]           flr_rcvd_pf_q;
    reg                                flr_rcvd_vf_q;
    reg [TOTAL_PF_COUNT_WIDTH-1:0]     flr_rcvd_pf_num_q;
    reg [TOTAL_VF_COUNT_WIDTH-1:0]     flr_rcvd_vf_num_q;
    reg [TOTAL_PF_COUNT-1:0]           flr_rcvd_pf_num_match;
    wire [TOTAL_PF_COUNT-1:0]           flr_rcvd_pf_num_match_w;


//Timing fix -convert to 1hot
genvar i;
    generate
        for (i=0; i<TOTAL_PF_COUNT; i=i+1) begin:pf_num_match    
            assign flr_rcvd_pf_num_match_w[i]     = (flr_rcvd_pf_num_i == i); 
        end
    endgenerate

    always @ (posedge pld_clk) begin
        if (~pld_rst_n) begin
            flr_rcvd_pf_q      <= '0;
            flr_rcvd_vf_q      <= '0;
        end
        else begin
            flr_rcvd_pf_q      <= flr_rcvd_pf_i;
            flr_rcvd_vf_q      <= flr_rcvd_vf_i;
            flr_rcvd_pf_num_q  <= flr_rcvd_pf_num_i;
            flr_rcvd_vf_num_q  <= flr_rcvd_vf_num_i;
            flr_rcvd_pf_num_match <= flr_rcvd_pf_num_match_w;
        end
    end

    // Instantiate PF/VF WR/RD cap select generation module
    intel_pcie_virtio_cfgmnt # 
    (
        .TOTAL_PF_COUNT                  (TOTAL_PF_COUNT),
        .TOTAL_PF_COUNT_WIDTH            (TOTAL_PF_COUNT_WIDTH),
        .TOTAL_VF_COUNT                  (TOTAL_VF_COUNT_LCL),
        .TOTAL_VF_COUNT_WIDTH            (TOTAL_VF_COUNT_WIDTH), 
        .PF0_VF_COUNT                    (PF0_VF_COUNT),
        .PF1_VF_COUNT                    (PF1_VF_COUNT),
        .PF2_VF_COUNT                    (PF2_VF_COUNT),
        .PF3_VF_COUNT                    (PF3_VF_COUNT),
        .PF4_VF_COUNT                    (PF4_VF_COUNT),
        .PF5_VF_COUNT                    (PF5_VF_COUNT),
        .PF6_VF_COUNT                    (PF6_VF_COUNT),
        .PF7_VF_COUNT                    (PF7_VF_COUNT),
        .TOTAL_PF_COUNT_MAX              (TOTAL_PF_COUNT_MAX),
        .REG_ADDR_WIDTH                  (REG_ADDR_WIDTH),
        .VIRTIO_CMN_CONFIG_REG_ADDR      (VIRTIO_CMN_CONFIG_REG_ADDR),
        .VIRTIO_NOTIFICATION_REG_ADDR    (VIRTIO_NOTIFICATION_REG_ADDR),
        .VIRTIO_ISR_STATUS_REG_ADDR      (VIRTIO_ISR_STATUS_REG_ADDR),
        .VIRTIO_DEV_SPECIFIC_REG_ADDR    (VIRTIO_DEV_SPECIFIC_REG_ADDR),
        .VIRTIO_PCICFG_ACCESS_REG_ADDR   (VIRTIO_PCICFG_ACCESS_REG_ADDR),
        .PF0_VIRTIO_CAPABILITY_PRESENT     (PF0_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF1_VIRTIO_CAPABILITY_PRESENT     (PF1_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF2_VIRTIO_CAPABILITY_PRESENT     (PF2_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF3_VIRTIO_CAPABILITY_PRESENT     (PF3_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF4_VIRTIO_CAPABILITY_PRESENT     (PF4_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF5_VIRTIO_CAPABILITY_PRESENT     (PF5_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF6_VIRTIO_CAPABILITY_PRESENT     (PF6_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF7_VIRTIO_CAPABILITY_PRESENT     (PF7_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF0VF_VIRTIO_CAPABILITY_PRESENT   (PF0VF_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF1VF_VIRTIO_CAPABILITY_PRESENT   (PF1VF_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF2VF_VIRTIO_CAPABILITY_PRESENT   (PF2VF_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF3VF_VIRTIO_CAPABILITY_PRESENT   (PF3VF_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF4VF_VIRTIO_CAPABILITY_PRESENT   (PF4VF_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF5VF_VIRTIO_CAPABILITY_PRESENT   (PF5VF_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF6VF_VIRTIO_CAPABILITY_PRESENT   (PF6VF_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF7VF_VIRTIO_CAPABILITY_PRESENT   (PF7VF_VIRTIO_CAPABILITY_PRESENT  )    
    )
    cfgmnt (
        .clk                                     (pld_clk),
        .rstn                                    (pld_rst_n),
        .cii_req_i                               (cii_req_i),
        .cii_wr_i                                (cii_wr_i),
        .cii_wr_vf_active_i                      (cii_wr_vf_active_i),  
        .cii_vf_num_i                            (cii_vf_num_i),
        .cii_addr_i                              (cii_addr_i),
        .cii_func_num_i                          (cii_func_num_i),
        .flr_rcvd_pf_i                           (flr_rcvd_pf_q),
        .flr_rcvd_vf_i                           (flr_rcvd_vf_q),
        .flr_rcvd_pf_num_i                       (flr_rcvd_pf_num_q),
        .flr_rcvd_vf_num_i                       (flr_rcvd_vf_num_q),
        .read_cmncfg_header_sel_o                (read_cmncfg_header_sel),
        .read_pf_cmncfg_bar_sel_o                (read_pf_cmncfg_bar_sel),
        .read_vf_cmncfg_bar_sel_o                (read_vf_cmncfg_bar_sel),
        .read_pf_cmncfg_baroffset_sel_o          (read_pf_cmncfg_baroffset_sel),
        .read_vf_cmncfg_baroffset_sel_o          (read_vf_cmncfg_baroffset_sel),
        .read_pf_cmncfg_length_sel_o             (read_pf_cmncfg_length_sel),
        .read_vf_cmncfg_length_sel_o             (read_vf_cmncfg_length_sel),
        .read_notifications_header_sel_o         (read_notifications_header_sel),
        .read_pf_notifications_bar_sel_o         (read_pf_notifications_bar_sel),
        .read_vf_notifications_bar_sel_o         (read_vf_notifications_bar_sel),
        .read_pf_notifications_baroffset_sel_o   (read_pf_notifications_baroffset_sel),
        .read_vf_notifications_baroffset_sel_o   (read_vf_notifications_baroffset_sel),
        .read_pf_notifications_length_sel_o      (read_pf_notifications_length_sel),
        .read_vf_notifications_length_sel_o      (read_vf_notifications_length_sel),
        .read_pf_notifications_notifyoff_sel_o   (read_pf_notifications_notifyoff_sel),
        .read_vf_notifications_notifyoff_sel_o   (read_vf_notifications_notifyoff_sel),
        .read_pf_isrstatus_header_sel_o          (read_pf_isrstatus_header_sel),
        .read_vf_isrstatus_header_sel_o          (read_vf_isrstatus_header_sel),
        .read_pf_isrstatus_bar_sel_o             (read_pf_isrstatus_bar_sel),
        .read_vf_isrstatus_bar_sel_o             (read_vf_isrstatus_bar_sel),
        .read_pf_isrstatus_baroffset_sel_o       (read_pf_isrstatus_baroffset_sel),
        .read_vf_isrstatus_baroffset_sel_o       (read_vf_isrstatus_baroffset_sel),
        .read_pf_isrstatus_length_sel_o          (read_pf_isrstatus_length_sel),
        .read_vf_isrstatus_length_sel_o          (read_vf_isrstatus_length_sel),
        .read_devspecific_header_sel_o           (read_devspecific_header_sel),
        .read_pf_devspecific_bar_sel_o           (read_pf_devspecific_bar_sel),
        .read_vf_devspecific_bar_sel_o           (read_vf_devspecific_bar_sel),
        .read_pf_devspecific_baroffset_sel_o     (read_pf_devspecific_baroffset_sel),
        .read_vf_devspecific_baroffset_sel_o     (read_vf_devspecific_baroffset_sel),
        .read_pf_devspecific_length_sel_o        (read_pf_devspecific_length_sel),
        .read_vf_devspecific_length_sel_o        (read_vf_devspecific_length_sel),
        .write_roreg_sel_o                       (write_roreg_sel),
        .read_pcicfgaccess_header_sel_o          (read_pcicfgaccess_header_sel),
        .read_pf_pcicfgaccess_bar_sel_o          (read_pf_pcicfgaccess_bar_sel),
        .write_pf_pcicfgaccess_bar_sel_o         (write_pf_pcicfgaccess_bar_sel),
        .read_vf_pcicfgaccess_bar_sel_o          (read_vf_pcicfgaccess_bar_sel),
        .write_vf_pcicfgaccess_bar_sel_o         (write_vf_pcicfgaccess_bar_sel),
        .read_pf_pcicfgaccess_baroffset_sel_o    (read_pf_pcicfgaccess_baroffset_sel),
        .write_pf_pcicfgaccess_baroffset_sel_o   (write_pf_pcicfgaccess_baroffset_sel),
        .read_vf_pcicfgaccess_baroffset_sel_o    (read_vf_pcicfgaccess_baroffset_sel),
        .write_vf_pcicfgaccess_baroffset_sel_o   (write_vf_pcicfgaccess_baroffset_sel),
        .read_pf_pcicfgaccess_length_sel_o       (read_pf_pcicfgaccess_length_sel),
        .write_pf_pcicfgaccess_length_sel_o      (write_pf_pcicfgaccess_length_sel),
        .read_vf_pcicfgaccess_length_sel_o       (read_vf_pcicfgaccess_length_sel),
        .write_vf_pcicfgaccess_length_sel_o      (write_vf_pcicfgaccess_length_sel),
        .read_pf_pcicfgaccess_cfgdata_sel_o      (read_pf_pcicfgaccess_cfgdata_sel),
        .write_pf_pcicfgaccess_cfgdata_sel_o     (write_pf_pcicfgaccess_cfgdata_sel),
        .read_vf_pcicfgaccess_cfgdata_sel_o      (read_vf_pcicfgaccess_cfgdata_sel),
        .write_vf_pcicfgaccess_cfgdata_sel_o     (write_vf_pcicfgaccess_cfgdata_sel),
        .targeted_cap_not_present_o              (targeted_cap_not_present),
        .noregbank_selected_o                    (noregbank_selected)
    );

    // Instantiate VirtIO Capability
    intel_pcie_virtio_cap_pfvfreg # 
    (
        .TOTAL_PF_COUNT                  (TOTAL_PF_COUNT),
        .TOTAL_PF_COUNT_WIDTH            (TOTAL_PF_COUNT_WIDTH),
        .TOTAL_VF_COUNT                  (TOTAL_VF_COUNT_LCL),
        .TOTAL_VF_COUNT_WIDTH            (TOTAL_VF_COUNT_WIDTH), 
        .REG_ADDR_WIDTH                  (REG_ADDR_WIDTH),
        .PF0_VF_COUNT                    (PF0_VF_COUNT),
        .PF1_VF_COUNT                    (PF1_VF_COUNT),
        .PF2_VF_COUNT                    (PF2_VF_COUNT),
        .PF3_VF_COUNT                    (PF3_VF_COUNT),
        .PF4_VF_COUNT                    (PF4_VF_COUNT),
        .PF5_VF_COUNT                    (PF5_VF_COUNT),
        .PF6_VF_COUNT                    (PF6_VF_COUNT),
        .PF7_VF_COUNT                    (PF7_VF_COUNT),
        .TOTAL_PF_COUNT_MAX              (TOTAL_PF_COUNT_MAX),
        .PF0_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT    (PF0_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT),
        .PF0_VIRTIO_CMN_CONFIG_BAR_INDICATOR       (PF0_VIRTIO_CMN_CONFIG_BAR_INDICATOR),
        .PF0_VIRTIO_NOTIFICATION_BAR_INDICATOR     (PF0_VIRTIO_NOTIFICATION_BAR_INDICATOR),
        .PF0_VIRTIO_ISRSTATUS_BAR_INDICATOR        (PF0_VIRTIO_ISRSTATUS_BAR_INDICATOR),
        .PF0_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR (PF0_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR),
        .PF0_VIRTIO_DEVSPECIFIC_BAR_INDICATOR      (PF0_VIRTIO_DEVSPECIFIC_BAR_INDICATOR),
        .PF0_VIRTIO_CMN_CONFIG_BAR_OFFSET          (PF0_VIRTIO_CMN_CONFIG_BAR_OFFSET),
        .PF0_VIRTIO_NOTIFICATION_BAR_OFFSET        (PF0_VIRTIO_NOTIFICATION_BAR_OFFSET),
        .PF0_VIRTIO_ISRSTATUS_BAR_OFFSET           (PF0_VIRTIO_ISRSTATUS_BAR_OFFSET),
        .PF0_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET    (PF0_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET),
        .PF0_VIRTIO_DEVSPECIFIC_BAR_OFFSET         (PF0_VIRTIO_DEVSPECIFIC_BAR_OFFSET),
        .PF0_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH       (PF0_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH),
        .PF0_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH     (PF0_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH),
        .PF0_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH        (PF0_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH),
        .PF0_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH (PF0_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH),
        .PF0_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH      (PF0_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH),
        .PF0_VIRTIO_NOTIFY_OFF_MULTIPLIER             (PF0_VIRTIO_NOTIFY_OFF_MULTIPLIER),
        .PF1_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT    (PF1_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT),
        .PF1_VIRTIO_CMN_CONFIG_BAR_INDICATOR       (PF1_VIRTIO_CMN_CONFIG_BAR_INDICATOR),
        .PF1_VIRTIO_NOTIFICATION_BAR_INDICATOR     (PF1_VIRTIO_NOTIFICATION_BAR_INDICATOR),
        .PF1_VIRTIO_ISRSTATUS_BAR_INDICATOR        (PF1_VIRTIO_ISRSTATUS_BAR_INDICATOR),
        .PF1_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR (PF1_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR),
        .PF1_VIRTIO_DEVSPECIFIC_BAR_INDICATOR      (PF1_VIRTIO_DEVSPECIFIC_BAR_INDICATOR),
        .PF1_VIRTIO_CMN_CONFIG_BAR_OFFSET          (PF1_VIRTIO_CMN_CONFIG_BAR_OFFSET),
        .PF1_VIRTIO_NOTIFICATION_BAR_OFFSET        (PF1_VIRTIO_NOTIFICATION_BAR_OFFSET),
        .PF1_VIRTIO_ISRSTATUS_BAR_OFFSET           (PF1_VIRTIO_ISRSTATUS_BAR_OFFSET),
        .PF1_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET    (PF1_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET),
        .PF1_VIRTIO_DEVSPECIFIC_BAR_OFFSET         (PF1_VIRTIO_DEVSPECIFIC_BAR_OFFSET),
        .PF1_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH       (PF1_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH),
        .PF1_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH     (PF1_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH),
        .PF1_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH        (PF1_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH),
        .PF1_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH (PF1_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH),
        .PF1_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH      (PF1_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH),
        .PF1_VIRTIO_NOTIFY_OFF_MULTIPLIER             (PF1_VIRTIO_NOTIFY_OFF_MULTIPLIER),
        .PF2_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT    (PF2_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT),
        .PF2_VIRTIO_CMN_CONFIG_BAR_INDICATOR       (PF2_VIRTIO_CMN_CONFIG_BAR_INDICATOR),
        .PF2_VIRTIO_NOTIFICATION_BAR_INDICATOR     (PF2_VIRTIO_NOTIFICATION_BAR_INDICATOR),
        .PF2_VIRTIO_ISRSTATUS_BAR_INDICATOR        (PF2_VIRTIO_ISRSTATUS_BAR_INDICATOR),
        .PF2_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR (PF2_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR),
        .PF2_VIRTIO_DEVSPECIFIC_BAR_INDICATOR      (PF2_VIRTIO_DEVSPECIFIC_BAR_INDICATOR),
        .PF2_VIRTIO_CMN_CONFIG_BAR_OFFSET          (PF2_VIRTIO_CMN_CONFIG_BAR_OFFSET),
        .PF2_VIRTIO_NOTIFICATION_BAR_OFFSET        (PF2_VIRTIO_NOTIFICATION_BAR_OFFSET),
        .PF2_VIRTIO_ISRSTATUS_BAR_OFFSET           (PF2_VIRTIO_ISRSTATUS_BAR_OFFSET),
        .PF2_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET    (PF2_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET),
        .PF2_VIRTIO_DEVSPECIFIC_BAR_OFFSET         (PF2_VIRTIO_DEVSPECIFIC_BAR_OFFSET),
        .PF2_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH       (PF2_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH),
        .PF2_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH     (PF2_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH),
        .PF2_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH        (PF2_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH),
        .PF2_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH (PF2_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH),
        .PF2_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH      (PF2_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH),
        .PF2_VIRTIO_NOTIFY_OFF_MULTIPLIER             (PF2_VIRTIO_NOTIFY_OFF_MULTIPLIER),
        .PF3_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT    (PF3_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT),
        .PF3_VIRTIO_CMN_CONFIG_BAR_INDICATOR       (PF3_VIRTIO_CMN_CONFIG_BAR_INDICATOR),
        .PF3_VIRTIO_NOTIFICATION_BAR_INDICATOR     (PF3_VIRTIO_NOTIFICATION_BAR_INDICATOR),
        .PF3_VIRTIO_ISRSTATUS_BAR_INDICATOR        (PF3_VIRTIO_ISRSTATUS_BAR_INDICATOR),
        .PF3_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR (PF3_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR),
        .PF3_VIRTIO_DEVSPECIFIC_BAR_INDICATOR      (PF3_VIRTIO_DEVSPECIFIC_BAR_INDICATOR),
        .PF3_VIRTIO_CMN_CONFIG_BAR_OFFSET          (PF3_VIRTIO_CMN_CONFIG_BAR_OFFSET),
        .PF3_VIRTIO_NOTIFICATION_BAR_OFFSET        (PF3_VIRTIO_NOTIFICATION_BAR_OFFSET),
        .PF3_VIRTIO_ISRSTATUS_BAR_OFFSET           (PF3_VIRTIO_ISRSTATUS_BAR_OFFSET),
        .PF3_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET    (PF3_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET),
        .PF3_VIRTIO_DEVSPECIFIC_BAR_OFFSET         (PF3_VIRTIO_DEVSPECIFIC_BAR_OFFSET),
        .PF3_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH       (PF3_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH),
        .PF3_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH     (PF3_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH),
        .PF3_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH        (PF3_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH),
        .PF3_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH (PF3_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH),
        .PF3_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH      (PF3_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH),
        .PF3_VIRTIO_NOTIFY_OFF_MULTIPLIER             (PF3_VIRTIO_NOTIFY_OFF_MULTIPLIER),
        .PF4_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT    (PF4_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT),
        .PF4_VIRTIO_CMN_CONFIG_BAR_INDICATOR       (PF4_VIRTIO_CMN_CONFIG_BAR_INDICATOR),
        .PF4_VIRTIO_NOTIFICATION_BAR_INDICATOR     (PF4_VIRTIO_NOTIFICATION_BAR_INDICATOR),
        .PF4_VIRTIO_ISRSTATUS_BAR_INDICATOR        (PF4_VIRTIO_ISRSTATUS_BAR_INDICATOR),
        .PF4_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR (PF4_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR),
        .PF4_VIRTIO_DEVSPECIFIC_BAR_INDICATOR      (PF4_VIRTIO_DEVSPECIFIC_BAR_INDICATOR),
        .PF4_VIRTIO_CMN_CONFIG_BAR_OFFSET          (PF4_VIRTIO_CMN_CONFIG_BAR_OFFSET),
        .PF4_VIRTIO_NOTIFICATION_BAR_OFFSET        (PF4_VIRTIO_NOTIFICATION_BAR_OFFSET),
        .PF4_VIRTIO_ISRSTATUS_BAR_OFFSET           (PF4_VIRTIO_ISRSTATUS_BAR_OFFSET),
        .PF4_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET    (PF4_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET),
        .PF4_VIRTIO_DEVSPECIFIC_BAR_OFFSET         (PF4_VIRTIO_DEVSPECIFIC_BAR_OFFSET),
        .PF4_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH       (PF4_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH),
        .PF4_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH     (PF4_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH),
        .PF4_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH        (PF4_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH),
        .PF4_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH (PF4_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH),
        .PF4_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH      (PF4_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH),
        .PF4_VIRTIO_NOTIFY_OFF_MULTIPLIER             (PF4_VIRTIO_NOTIFY_OFF_MULTIPLIER),
        .PF5_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT    (PF5_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT),
        .PF5_VIRTIO_CMN_CONFIG_BAR_INDICATOR       (PF5_VIRTIO_CMN_CONFIG_BAR_INDICATOR),
        .PF5_VIRTIO_NOTIFICATION_BAR_INDICATOR     (PF5_VIRTIO_NOTIFICATION_BAR_INDICATOR),
        .PF5_VIRTIO_ISRSTATUS_BAR_INDICATOR        (PF5_VIRTIO_ISRSTATUS_BAR_INDICATOR),
        .PF5_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR (PF5_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR),
        .PF5_VIRTIO_DEVSPECIFIC_BAR_INDICATOR      (PF5_VIRTIO_DEVSPECIFIC_BAR_INDICATOR),
        .PF5_VIRTIO_CMN_CONFIG_BAR_OFFSET          (PF5_VIRTIO_CMN_CONFIG_BAR_OFFSET),
        .PF5_VIRTIO_NOTIFICATION_BAR_OFFSET        (PF5_VIRTIO_NOTIFICATION_BAR_OFFSET),
        .PF5_VIRTIO_ISRSTATUS_BAR_OFFSET           (PF5_VIRTIO_ISRSTATUS_BAR_OFFSET),
        .PF5_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET    (PF5_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET),
        .PF5_VIRTIO_DEVSPECIFIC_BAR_OFFSET         (PF5_VIRTIO_DEVSPECIFIC_BAR_OFFSET),
        .PF5_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH       (PF5_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH),
        .PF5_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH     (PF5_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH),
        .PF5_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH        (PF5_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH),
        .PF5_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH (PF5_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH),
        .PF5_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH      (PF5_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH),
        .PF5_VIRTIO_NOTIFY_OFF_MULTIPLIER             (PF5_VIRTIO_NOTIFY_OFF_MULTIPLIER),
        .PF6_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT    (PF6_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT),
        .PF6_VIRTIO_CMN_CONFIG_BAR_INDICATOR       (PF6_VIRTIO_CMN_CONFIG_BAR_INDICATOR),
        .PF6_VIRTIO_NOTIFICATION_BAR_INDICATOR     (PF6_VIRTIO_NOTIFICATION_BAR_INDICATOR),
        .PF6_VIRTIO_ISRSTATUS_BAR_INDICATOR        (PF6_VIRTIO_ISRSTATUS_BAR_INDICATOR),
        .PF6_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR (PF6_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR),
        .PF6_VIRTIO_DEVSPECIFIC_BAR_INDICATOR      (PF6_VIRTIO_DEVSPECIFIC_BAR_INDICATOR),
        .PF6_VIRTIO_CMN_CONFIG_BAR_OFFSET          (PF6_VIRTIO_CMN_CONFIG_BAR_OFFSET),
        .PF6_VIRTIO_NOTIFICATION_BAR_OFFSET        (PF6_VIRTIO_NOTIFICATION_BAR_OFFSET),
        .PF6_VIRTIO_ISRSTATUS_BAR_OFFSET           (PF6_VIRTIO_ISRSTATUS_BAR_OFFSET),
        .PF6_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET    (PF6_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET),
        .PF6_VIRTIO_DEVSPECIFIC_BAR_OFFSET         (PF6_VIRTIO_DEVSPECIFIC_BAR_OFFSET),
        .PF6_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH       (PF6_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH),
        .PF6_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH     (PF6_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH),
        .PF6_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH        (PF6_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH),
        .PF6_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH (PF6_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH),
        .PF6_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH      (PF6_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH),
        .PF6_VIRTIO_NOTIFY_OFF_MULTIPLIER             (PF6_VIRTIO_NOTIFY_OFF_MULTIPLIER),
        .PF7_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT    (PF7_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT),
        .PF7_VIRTIO_CMN_CONFIG_BAR_INDICATOR       (PF7_VIRTIO_CMN_CONFIG_BAR_INDICATOR),
        .PF7_VIRTIO_NOTIFICATION_BAR_INDICATOR     (PF7_VIRTIO_NOTIFICATION_BAR_INDICATOR),
        .PF7_VIRTIO_ISRSTATUS_BAR_INDICATOR        (PF7_VIRTIO_ISRSTATUS_BAR_INDICATOR),
        .PF7_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR (PF7_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR),
        .PF7_VIRTIO_DEVSPECIFIC_BAR_INDICATOR      (PF7_VIRTIO_DEVSPECIFIC_BAR_INDICATOR),
        .PF7_VIRTIO_CMN_CONFIG_BAR_OFFSET          (PF7_VIRTIO_CMN_CONFIG_BAR_OFFSET),
        .PF7_VIRTIO_NOTIFICATION_BAR_OFFSET        (PF7_VIRTIO_NOTIFICATION_BAR_OFFSET),
        .PF7_VIRTIO_ISRSTATUS_BAR_OFFSET           (PF7_VIRTIO_ISRSTATUS_BAR_OFFSET),
        .PF7_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET    (PF7_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET),
        .PF7_VIRTIO_DEVSPECIFIC_BAR_OFFSET         (PF7_VIRTIO_DEVSPECIFIC_BAR_OFFSET),
        .PF7_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH       (PF7_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH),
        .PF7_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH     (PF7_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH),
        .PF7_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH        (PF7_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH),
        .PF7_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH (PF7_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH),
        .PF7_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH      (PF7_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH),
        .PF7_VIRTIO_NOTIFY_OFF_MULTIPLIER             (PF7_VIRTIO_NOTIFY_OFF_MULTIPLIER),
        .PF0VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT    (PF0VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT),
        .PF0VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR       (PF0VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR),
        .PF0VF_VIRTIO_NOTIFICATION_BAR_INDICATOR     (PF0VF_VIRTIO_NOTIFICATION_BAR_INDICATOR),
        .PF0VF_VIRTIO_ISRSTATUS_BAR_INDICATOR        (PF0VF_VIRTIO_ISRSTATUS_BAR_INDICATOR),
        .PF0VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR (PF0VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR),
        .PF0VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR      (PF0VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR),
        .PF0VF_VIRTIO_CMN_CONFIG_BAR_OFFSET          (PF0VF_VIRTIO_CMN_CONFIG_BAR_OFFSET),
        .PF0VF_VIRTIO_NOTIFICATION_BAR_OFFSET        (PF0VF_VIRTIO_NOTIFICATION_BAR_OFFSET),
        .PF0VF_VIRTIO_ISRSTATUS_BAR_OFFSET           (PF0VF_VIRTIO_ISRSTATUS_BAR_OFFSET),
        .PF0VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET    (PF0VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET),
        .PF0VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET         (PF0VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET),
        .PF0VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH       (PF0VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH),
        .PF0VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH     (PF0VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH),
        .PF0VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH        (PF0VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH),
        .PF0VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH (PF0VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH),
        .PF0VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH      (PF0VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH),
        .PF0VF_VIRTIO_NOTIFY_OFF_MULTIPLIER             (PF0VF_VIRTIO_NOTIFY_OFF_MULTIPLIER),
        .PF1VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT    (PF1VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT),
        .PF1VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR       (PF1VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR),
        .PF1VF_VIRTIO_NOTIFICATION_BAR_INDICATOR     (PF1VF_VIRTIO_NOTIFICATION_BAR_INDICATOR),
        .PF1VF_VIRTIO_ISRSTATUS_BAR_INDICATOR        (PF1VF_VIRTIO_ISRSTATUS_BAR_INDICATOR),
        .PF1VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR (PF1VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR),
        .PF1VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR      (PF1VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR),
        .PF1VF_VIRTIO_CMN_CONFIG_BAR_OFFSET          (PF1VF_VIRTIO_CMN_CONFIG_BAR_OFFSET),
        .PF1VF_VIRTIO_NOTIFICATION_BAR_OFFSET        (PF1VF_VIRTIO_NOTIFICATION_BAR_OFFSET),
        .PF1VF_VIRTIO_ISRSTATUS_BAR_OFFSET           (PF1VF_VIRTIO_ISRSTATUS_BAR_OFFSET),
        .PF1VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET    (PF1VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET),
        .PF1VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET         (PF1VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET),
        .PF1VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH       (PF1VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH),
        .PF1VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH     (PF1VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH),
        .PF1VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH        (PF1VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH),
        .PF1VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH (PF1VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH),
        .PF1VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH      (PF1VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH),
        .PF1VF_VIRTIO_NOTIFY_OFF_MULTIPLIER             (PF1VF_VIRTIO_NOTIFY_OFF_MULTIPLIER),
        .PF2VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT    (PF2VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT),
        .PF2VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR       (PF2VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR),
        .PF2VF_VIRTIO_NOTIFICATION_BAR_INDICATOR     (PF2VF_VIRTIO_NOTIFICATION_BAR_INDICATOR),
        .PF2VF_VIRTIO_ISRSTATUS_BAR_INDICATOR        (PF2VF_VIRTIO_ISRSTATUS_BAR_INDICATOR),
        .PF2VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR (PF2VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR),
        .PF2VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR      (PF2VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR),
        .PF2VF_VIRTIO_CMN_CONFIG_BAR_OFFSET          (PF2VF_VIRTIO_CMN_CONFIG_BAR_OFFSET),
        .PF2VF_VIRTIO_NOTIFICATION_BAR_OFFSET        (PF2VF_VIRTIO_NOTIFICATION_BAR_OFFSET),
        .PF2VF_VIRTIO_ISRSTATUS_BAR_OFFSET           (PF2VF_VIRTIO_ISRSTATUS_BAR_OFFSET),
        .PF2VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET    (PF2VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET),
        .PF2VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET         (PF2VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET),
        .PF2VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH       (PF2VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH),
        .PF2VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH     (PF2VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH),
        .PF2VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH        (PF2VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH),
        .PF2VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH (PF2VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH),
        .PF2VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH      (PF2VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH),
        .PF2VF_VIRTIO_NOTIFY_OFF_MULTIPLIER             (PF2VF_VIRTIO_NOTIFY_OFF_MULTIPLIER),
        .PF3VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT    (PF3VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT),
        .PF3VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR       (PF3VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR),
        .PF3VF_VIRTIO_NOTIFICATION_BAR_INDICATOR     (PF3VF_VIRTIO_NOTIFICATION_BAR_INDICATOR),
        .PF3VF_VIRTIO_ISRSTATUS_BAR_INDICATOR        (PF3VF_VIRTIO_ISRSTATUS_BAR_INDICATOR),
        .PF3VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR (PF3VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR),
        .PF3VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR      (PF3VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR),
        .PF3VF_VIRTIO_CMN_CONFIG_BAR_OFFSET          (PF3VF_VIRTIO_CMN_CONFIG_BAR_OFFSET),
        .PF3VF_VIRTIO_NOTIFICATION_BAR_OFFSET        (PF3VF_VIRTIO_NOTIFICATION_BAR_OFFSET),
        .PF3VF_VIRTIO_ISRSTATUS_BAR_OFFSET           (PF3VF_VIRTIO_ISRSTATUS_BAR_OFFSET),
        .PF3VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET    (PF3VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET),
        .PF3VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET         (PF3VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET),
        .PF3VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH       (PF3VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH),
        .PF3VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH     (PF3VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH),
        .PF3VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH        (PF3VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH),
        .PF3VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH (PF3VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH),
        .PF3VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH      (PF3VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH),
        .PF3VF_VIRTIO_NOTIFY_OFF_MULTIPLIER             (PF3VF_VIRTIO_NOTIFY_OFF_MULTIPLIER),
        .PF4VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT    (PF4VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT),
        .PF4VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR       (PF4VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR),
        .PF4VF_VIRTIO_NOTIFICATION_BAR_INDICATOR     (PF4VF_VIRTIO_NOTIFICATION_BAR_INDICATOR),
        .PF4VF_VIRTIO_ISRSTATUS_BAR_INDICATOR        (PF4VF_VIRTIO_ISRSTATUS_BAR_INDICATOR),
        .PF4VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR (PF4VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR),
        .PF4VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR      (PF4VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR),
        .PF4VF_VIRTIO_CMN_CONFIG_BAR_OFFSET          (PF4VF_VIRTIO_CMN_CONFIG_BAR_OFFSET),
        .PF4VF_VIRTIO_NOTIFICATION_BAR_OFFSET        (PF4VF_VIRTIO_NOTIFICATION_BAR_OFFSET),
        .PF4VF_VIRTIO_ISRSTATUS_BAR_OFFSET           (PF4VF_VIRTIO_ISRSTATUS_BAR_OFFSET),
        .PF4VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET    (PF4VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET),
        .PF4VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET         (PF4VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET),
        .PF4VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH       (PF4VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH),
        .PF4VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH     (PF4VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH),
        .PF4VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH        (PF4VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH),
        .PF4VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH (PF4VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH),
        .PF4VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH      (PF4VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH),
        .PF4VF_VIRTIO_NOTIFY_OFF_MULTIPLIER             (PF4VF_VIRTIO_NOTIFY_OFF_MULTIPLIER),
        .PF5VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT    (PF5VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT),
        .PF5VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR       (PF5VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR),
        .PF5VF_VIRTIO_NOTIFICATION_BAR_INDICATOR     (PF5VF_VIRTIO_NOTIFICATION_BAR_INDICATOR),
        .PF5VF_VIRTIO_ISRSTATUS_BAR_INDICATOR        (PF5VF_VIRTIO_ISRSTATUS_BAR_INDICATOR),
        .PF5VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR (PF5VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR),
        .PF5VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR      (PF5VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR),
        .PF5VF_VIRTIO_CMN_CONFIG_BAR_OFFSET          (PF5VF_VIRTIO_CMN_CONFIG_BAR_OFFSET),
        .PF5VF_VIRTIO_NOTIFICATION_BAR_OFFSET        (PF5VF_VIRTIO_NOTIFICATION_BAR_OFFSET),
        .PF5VF_VIRTIO_ISRSTATUS_BAR_OFFSET           (PF5VF_VIRTIO_ISRSTATUS_BAR_OFFSET),
        .PF5VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET    (PF5VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET),
        .PF5VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET         (PF5VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET),
        .PF5VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH       (PF5VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH),
        .PF5VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH     (PF5VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH),
        .PF5VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH        (PF5VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH),
        .PF5VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH (PF5VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH),
        .PF5VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH      (PF5VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH),
        .PF5VF_VIRTIO_NOTIFY_OFF_MULTIPLIER             (PF5VF_VIRTIO_NOTIFY_OFF_MULTIPLIER),
        .PF6VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT    (PF6VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT),
        .PF6VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR       (PF6VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR),
        .PF6VF_VIRTIO_NOTIFICATION_BAR_INDICATOR     (PF6VF_VIRTIO_NOTIFICATION_BAR_INDICATOR),
        .PF6VF_VIRTIO_ISRSTATUS_BAR_INDICATOR        (PF6VF_VIRTIO_ISRSTATUS_BAR_INDICATOR),
        .PF6VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR (PF6VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR),
        .PF6VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR      (PF6VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR),
        .PF6VF_VIRTIO_CMN_CONFIG_BAR_OFFSET          (PF6VF_VIRTIO_CMN_CONFIG_BAR_OFFSET),
        .PF6VF_VIRTIO_NOTIFICATION_BAR_OFFSET        (PF6VF_VIRTIO_NOTIFICATION_BAR_OFFSET),
        .PF6VF_VIRTIO_ISRSTATUS_BAR_OFFSET           (PF6VF_VIRTIO_ISRSTATUS_BAR_OFFSET),
        .PF6VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET    (PF6VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET),
        .PF6VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET         (PF6VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET),
        .PF6VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH       (PF6VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH),
        .PF6VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH     (PF6VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH),
        .PF6VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH        (PF6VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH),
        .PF6VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH (PF6VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH),
        .PF6VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH      (PF6VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH),
        .PF6VF_VIRTIO_NOTIFY_OFF_MULTIPLIER             (PF6VF_VIRTIO_NOTIFY_OFF_MULTIPLIER),
        .PF7VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT    (PF7VF_VIRTIO_DEVICE_SPECIFIC_CAP_PRESENT),
        .PF7VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR       (PF7VF_VIRTIO_CMN_CONFIG_BAR_INDICATOR),
        .PF7VF_VIRTIO_NOTIFICATION_BAR_INDICATOR     (PF7VF_VIRTIO_NOTIFICATION_BAR_INDICATOR),
        .PF7VF_VIRTIO_ISRSTATUS_BAR_INDICATOR        (PF7VF_VIRTIO_ISRSTATUS_BAR_INDICATOR),
        .PF7VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR (PF7VF_VIRTIO_PCICONFIG_ACCESS_BAR_INDICATOR),
        .PF7VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR      (PF7VF_VIRTIO_DEVSPECIFIC_BAR_INDICATOR),
        .PF7VF_VIRTIO_CMN_CONFIG_BAR_OFFSET          (PF7VF_VIRTIO_CMN_CONFIG_BAR_OFFSET),
        .PF7VF_VIRTIO_NOTIFICATION_BAR_OFFSET        (PF7VF_VIRTIO_NOTIFICATION_BAR_OFFSET),
        .PF7VF_VIRTIO_ISRSTATUS_BAR_OFFSET           (PF7VF_VIRTIO_ISRSTATUS_BAR_OFFSET),
        .PF7VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET    (PF7VF_VIRTIO_PCICONFIG_ACCESS_BAR_OFFSET),
        .PF7VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET         (PF7VF_VIRTIO_DEVSPECIFIC_BAR_OFFSET),
        .PF7VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH       (PF7VF_VIRTIO_CMN_CONFIG_STRUCTURE_LENGTH),
        .PF7VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH     (PF7VF_VIRTIO_NOTIFICATION_STRUCTURE_LENGTH),
        .PF7VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH        (PF7VF_VIRTIO_ISRSTATUS_STRUCTURE_LENGTH),
        .PF7VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH (PF7VF_VIRTIO_PCICONFIG_ACCESS_STRUCTURE_LENGTH),
        .PF7VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH      (PF7VF_VIRTIO_DEVSPECIFIC_STRUCTURE_LENGTH),
        .PF7VF_VIRTIO_NOTIFY_OFF_MULTIPLIER             (PF7VF_VIRTIO_NOTIFY_OFF_MULTIPLIER)
    )
    pfvfreg (
        .clk                                     (pld_clk),
        .rstn                                    (pld_rst_n),
        .flr_rcvd_pf_i                           (flr_rcvd_pf_q),
        .flr_rcvd_vf_i                           (flr_rcvd_vf_q),
        .flr_rcvd_pf_num_i                       (flr_rcvd_pf_num_q),
        .flr_rcvd_pf_num_match_i                 (flr_rcvd_pf_num_match),
        .flr_rcvd_vf_num_i                       (flr_rcvd_vf_num_q),
        .cii_req_i                               (cii_req_i),
        .cii_hdr_first_be_i                      (cii_hdr_first_be_i),
        .cii_func_num_i                          (cii_func_num_i),
        .cii_wr_vf_active_i                      (cii_wr_vf_active_i),
        .cii_vf_num_i                            (cii_vf_num_i),
        .cii_wr_i                                (cii_wr_i),
        .cii_dout_i                              (cii_dout_i),
        .read_cmncfg_header_sel_i                (read_cmncfg_header_sel),
        .read_pf_cmncfg_bar_sel_i                (read_pf_cmncfg_bar_sel),
        .read_vf_cmncfg_bar_sel_i                (read_vf_cmncfg_bar_sel),
        .read_pf_cmncfg_baroffset_sel_i          (read_pf_cmncfg_baroffset_sel),
        .read_vf_cmncfg_baroffset_sel_i          (read_vf_cmncfg_baroffset_sel),
        .read_pf_cmncfg_length_sel_i             (read_pf_cmncfg_length_sel),
        .read_vf_cmncfg_length_sel_i             (read_vf_cmncfg_length_sel),
        .read_notifications_header_sel_i         (read_notifications_header_sel),
        .read_pf_notifications_bar_sel_i         (read_pf_notifications_bar_sel),
        .read_vf_notifications_bar_sel_i         (read_vf_notifications_bar_sel),
        .read_pf_notifications_baroffset_sel_i   (read_pf_notifications_baroffset_sel),
        .read_vf_notifications_baroffset_sel_i   (read_vf_notifications_baroffset_sel),
        .read_pf_notifications_length_sel_i      (read_pf_notifications_length_sel),
        .read_vf_notifications_length_sel_i      (read_vf_notifications_length_sel),
        .read_pf_notifications_notifyoff_sel_i   (read_pf_notifications_notifyoff_sel),
        .read_vf_notifications_notifyoff_sel_i   (read_vf_notifications_notifyoff_sel),
        .read_pf_isrstatus_header_sel_i          (read_pf_isrstatus_header_sel),
        .read_vf_isrstatus_header_sel_i          (read_vf_isrstatus_header_sel),
        .read_pf_isrstatus_bar_sel_i             (read_pf_isrstatus_bar_sel),
        .read_vf_isrstatus_bar_sel_i             (read_vf_isrstatus_bar_sel),
        .read_pf_isrstatus_baroffset_sel_i       (read_pf_isrstatus_baroffset_sel),
        .read_vf_isrstatus_baroffset_sel_i       (read_vf_isrstatus_baroffset_sel),
        .read_pf_isrstatus_length_sel_i          (read_pf_isrstatus_length_sel),
        .read_vf_isrstatus_length_sel_i          (read_vf_isrstatus_length_sel),
        .read_devspecific_header_sel_i           (read_devspecific_header_sel),
        .read_pf_devspecific_bar_sel_i           (read_pf_devspecific_bar_sel),
        .read_vf_devspecific_bar_sel_i           (read_vf_devspecific_bar_sel),
        .read_pf_devspecific_baroffset_sel_i     (read_pf_devspecific_baroffset_sel),
        .read_vf_devspecific_baroffset_sel_i     (read_vf_devspecific_baroffset_sel),
        .read_pf_devspecific_length_sel_i        (read_pf_devspecific_length_sel),
        .read_vf_devspecific_length_sel_i        (read_vf_devspecific_length_sel),
        .write_roreg_sel_i                       (write_roreg_sel),
        .read_pcicfgaccess_header_sel_i          (read_pcicfgaccess_header_sel),
        .read_pf_pcicfgaccess_bar_sel_i          (read_pf_pcicfgaccess_bar_sel),
        .write_pf_pcicfgaccess_bar_sel_i         (write_pf_pcicfgaccess_bar_sel),
        .read_vf_pcicfgaccess_bar_sel_i          (read_vf_pcicfgaccess_bar_sel),
        .write_vf_pcicfgaccess_bar_sel_i         (write_vf_pcicfgaccess_bar_sel),
        .read_pf_pcicfgaccess_baroffset_sel_i    (read_pf_pcicfgaccess_baroffset_sel),
        .write_pf_pcicfgaccess_baroffset_sel_i   (write_pf_pcicfgaccess_baroffset_sel),
        .read_vf_pcicfgaccess_baroffset_sel_i    (read_vf_pcicfgaccess_baroffset_sel),
        .write_vf_pcicfgaccess_baroffset_sel_i   (write_vf_pcicfgaccess_baroffset_sel),
        .read_pf_pcicfgaccess_length_sel_i       (read_pf_pcicfgaccess_length_sel),
        .write_pf_pcicfgaccess_length_sel_i      (write_pf_pcicfgaccess_length_sel),
        .read_vf_pcicfgaccess_length_sel_i       (read_vf_pcicfgaccess_length_sel),
        .write_vf_pcicfgaccess_length_sel_i      (write_vf_pcicfgaccess_length_sel),
        .read_pf_pcicfgaccess_cfgdata_sel_i      (read_pf_pcicfgaccess_cfgdata_sel),
        .write_pf_pcicfgaccess_cfgdata_sel_i     (write_pf_pcicfgaccess_cfgdata_sel),
        .read_vf_pcicfgaccess_cfgdata_sel_i      (read_vf_pcicfgaccess_cfgdata_sel),
        .write_vf_pcicfgaccess_cfgdata_sel_i     (write_vf_pcicfgaccess_cfgdata_sel),
        .virtio_cmncfg_header_data_o             (virtio_cmncfg_header_data),
        .virtio_cmncfg_bar_data_o                (virtio_cmncfg_bar_data),
        .virtio_cmncfg_length_data_o             (virtio_cmncfg_length_data),
        .virtio_cmncfg_baroffset_data_o          (virtio_cmncfg_baroffset_data),
        .virtio_notifications_header_data_o      (virtio_notifications_header_data),
        .virtio_notifications_bar_data_o         (virtio_notifications_bar_data),
        .virtio_notifications_length_data_o      (virtio_notifications_length_data),
        .virtio_notifications_baroffset_data_o   (virtio_notifications_baroffset_data),
        .virtio_notifications_notifyoff_data_o   (virtio_notifications_notifyoff_data),
        .virtio_isrstatus_header_data_o          (virtio_isrstatus_header_data),
        .virtio_isrstatus_bar_data_o             (virtio_isrstatus_bar_data),
        .virtio_isrstatus_length_data_o          (virtio_isrstatus_length_data),
        .virtio_isrstatus_baroffset_data_o       (virtio_isrstatus_baroffset_data),
        .virtio_devspecific_header_data_o        (virtio_devspecific_header_data),
        .virtio_devspecific_bar_data_o           (virtio_devspecific_bar_data),
        .virtio_devspecific_length_data_o        (virtio_devspecific_length_data),
        .virtio_devspecific_baroffset_data_o     (virtio_devspecific_baroffset_data),
        .virtio_pcicfgaccess_header_data_o       (virtio_pcicfgaccess_header_data),
        .virtio_pcicfgaccess_cfgdata_data_o      (virtio_pcicfgaccess_cfgdata_data),
        .virtio_pcicfgaccess_bar_data_o          (virtio_pcicfgaccess_bar_data),
        .virtio_pcicfgaccess_length_data_o       (virtio_pcicfgaccess_length_data),
        .virtio_pcicfgaccess_baroffset_data_o    (virtio_pcicfgaccess_baroffset_data),
        .virtio_pcicfg_vfaccess_o                (virtio_pcicfg_vfaccess_o),
        .virtio_pcicfg_vfnum_o                   (virtio_pcicfg_vfnum_o),
        .virtio_pcicfg_pfnum_o                   (virtio_pcicfg_pfnum_o),
        .virtio_pcicfg_bar_o                     (virtio_pcicfg_bar_o),
        .virtio_pcicfg_length_o                  (virtio_pcicfg_length_o),
        .virtio_pcicfg_baroffset_o               (virtio_pcicfg_baroffset_o),
        .virtio_pcicfg_cfgdata_o                 (virtio_pcicfg_cfgdata_o),
        .virtio_pcicfg_cfgwr_o                   (virtio_pcicfg_cfgwr_o),
        .virtio_pcicfg_cfgrd_o                   (virtio_pcicfg_cfgrd_o),
        .virtio_cfgwr_req_ack_o                  (virtio_cfgwr_req_ack),
        .virtio_cfgrd_req_ack_o                  (virtio_cfgrd_req_ack),
        .virtio_pcicfg_appvfnum_i                (virtio_pcicfg_appvfnum_i),
        .virtio_pcicfg_apppfnum_i                (virtio_pcicfg_apppfnum_i),
        .virtio_pcicfg_rdack_i                   (virtio_pcicfg_rdack_i),
        .virtio_pcicfg_rdbe_i                    (virtio_pcicfg_rdbe_i),
        .virtio_pcicfg_data_i                    (virtio_pcicfg_data_i)
    );
    
    // Instantiate VirtIO CFG Dataflow Module
    intel_pcie_virtio_dataflow #(
        .TOTAL_PF_COUNT_WIDTH            (TOTAL_PF_COUNT_WIDTH),
        .TOTAL_VF_COUNT_WIDTH            (TOTAL_VF_COUNT_WIDTH),
        .REG_ADDR_WIDTH                  (REG_ADDR_WIDTH)
    ) dataflowctrl (
        .clk                                     (pld_clk),           
        .rstn                                    (pld_rst_n),
        .virtio_cmncfg_header_data_i             (virtio_cmncfg_header_data),
        .virtio_cmncfg_bar_data_i                ({24'd0, virtio_cmncfg_bar_data}),
        .virtio_cmncfg_length_data_i             (virtio_cmncfg_length_data),
        .virtio_cmncfg_baroffset_data_i          (virtio_cmncfg_baroffset_data),
        .virtio_notifications_header_data_i      (virtio_notifications_header_data),
        .virtio_notifications_bar_data_i         ({24'd0, virtio_notifications_bar_data}),
        .virtio_notifications_length_data_i      (virtio_notifications_length_data),
        .virtio_notifications_baroffset_data_i   (virtio_notifications_baroffset_data),
        .virtio_notifications_notifyoff_data_i   (virtio_notifications_notifyoff_data),
        .virtio_isrstatus_header_data_i          (virtio_isrstatus_header_data),
        .virtio_isrstatus_bar_data_i             ({24'd0, virtio_isrstatus_bar_data}),
        .virtio_isrstatus_length_data_i          (virtio_isrstatus_length_data),
        .virtio_isrstatus_baroffset_data_i       (virtio_isrstatus_baroffset_data),
        .virtio_devspecific_header_data_i        (virtio_devspecific_header_data),
        .virtio_devspecific_bar_data_i           ({24'd0, virtio_devspecific_bar_data}),
        .virtio_devspecific_length_data_i        (virtio_devspecific_length_data),
        .virtio_devspecific_baroffset_data_i     (virtio_devspecific_baroffset_data),
        .virtio_pcicfgaccess_header_data_i       (virtio_pcicfgaccess_header_data),
        .virtio_pcicfgaccess_bar_data_i          ({24'd0, virtio_pcicfgaccess_bar_data}),
        .virtio_pcicfgaccess_length_data_i       (virtio_pcicfgaccess_length_data),
        .virtio_pcicfgaccess_baroffset_data_i    (virtio_pcicfgaccess_baroffset_data),
        .virtio_pcicfgaccess_cfgdata_data_i      (virtio_pcicfgaccess_cfgdata_data),
        .virtio_cfgwr_req_ack_i                  (virtio_cfgwr_req_ack),
        .virtio_cfgrd_req_ack_i                  (virtio_cfgrd_req_ack),
        .targeted_cap_not_present_i              (targeted_cap_not_present),
        .noregbank_selected_i                    (noregbank_selected),
        // from/to HIP interface
        .cii_override_en_o                       (cii_override_en_o),
        .cii_override_din_o                      (cii_override_din_o),
        .cii_halt_o                              (cii_halt_o),
        .cii_req_i                               (cii_req_i),
        .cii_hdr_poisoned_i                      (cii_hdr_poisoned_i),
        .cii_hdr_first_be_i                      (cii_hdr_first_be_i),
        .cii_func_num_i                          (cii_func_num_i),
        .cii_wr_vf_active_i                      (cii_wr_vf_active_i),
        .cii_vf_num_i                            (cii_vf_num_i),
        .cii_wr_i                                (cii_wr_i),
        .cii_addr_i                              (cii_addr_i),
        .cii_dout_i                              (cii_dout_i),
        // from/to user interface
        .pin_cii_override_en_i                   (pin_cii_override_en_i),
        .pin_cii_override_din_i                  (pin_cii_override_din_i),
        .pin_cii_halt_i                          (pin_cii_halt_i),
        .pin_cii_req_o                           (pin_cii_req_o),
        .pin_cii_hdr_poisoned_o                  (pin_cii_hdr_poisoned_o),
        .pin_cii_hdr_first_be_o                  (pin_cii_hdr_first_be_o),
        .pin_cii_func_num_o                      (pin_cii_func_num_o),
        .pin_cii_wr_vf_active_o                  (pin_cii_wr_vf_active_o),
        .pin_cii_vf_num_o                        (pin_cii_vf_num_o),
        .pin_cii_wr_o                            (pin_cii_wr_o),
        .pin_cii_addr_o                          (pin_cii_addr_o),
        .pin_cii_dout_o                          (pin_cii_dout_o)
    );

endmodule
