// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//-----------------------------------------------------------------------------------
// This confidential and proprietary software may be used only as authorized by
// a licensing agreement from Intel
// copyright notice must be reproduced on all authorized copies.
//-----------------------------------------------------------------------------------
// Copyright (C) 2017 Intel Corporation. All rights reserved.  Intel products are
// protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
// other intellectual property laws.
//-----------------------------------------------------------------------------------
//  Project Name:  wharf_rock gen4x8 256 bits @ 500MHZ to 512 bits @ 250 MHz adapter
//  Module Name :  intel_pcie_whr_gen4x8_flr_if
//  Author      :  sdsawant
//  Date        :  Wed Sep 28 16:52:43 2019
//  Description :
//-----------------------------------------------------------------------------------

module  intel_pcie_whr_gen4x8_flr_if
 #(
     parameter intended_device_family = "Stratix 10"
  )
  (
   input               clk500,
   input               rst_clk500_n,
   input               clk250,
   input               rst_clk250_n,
   input  logic [7:0]  flr_rcvd_pf_i,
   input               flr_rcvd_vf_i,
   input  logic [2:0]  flr_rcvd_pf_num_i,
   input  logic [10:0] flr_rcvd_vf_num_i,
   input  logic [7:0]  flr_completed_pf_i,
   input               flr_completed_vf_i,
   input  logic [2:0]  flr_completed_pf_num_i,
   input  logic [10:0] flr_completed_vf_num_i,
   output logic [7:0]  flr_rcvd_pf_o,
   output logic        flr_rcvd_vf_o,
   output logic [2:0]  flr_rcvd_pf_num_o,
   output logic [10:0] flr_rcvd_vf_num_o,
   output logic [7:0]  flr_completed_pf_o,
   output logic        flr_completed_vf_o,
   output logic [2:0]  flr_completed_pf_num_o,
   output logic [10:0] flr_completed_vf_num_o
  );

//********************************************************************
// SYNCHRONIZATION OF HIP -> APP SIGNALS
//********************************************************************

  //******************************************************************************
  // DEFINE VARIABLES
  //******************************************************************************
  typedef struct packed {
  logic        flr_rcvd_vf;
  logic [2:0]  flr_rcvd_pf_num;
  logic [10:0] flr_rcvd_vf_num;
  } flr_rcvd_vf_data_t;

  logic flr_rcvd_vf_fifo_rdreq, flr_rcvd_vf_fifo_rdreq_q, flr_rcvd_vf_empty;
  logic [3:0] flr_vf_rd_numdata;
  flr_rcvd_vf_data_t flr_rcvd_vf_data_in, flr_rcvd_vf_data_out;
  logic [7:0]  flr_rcvd_pf_q;
  logic        flr_rcvd_vf_q;
  logic [2:0]  flr_rcvd_pf_num_q;
  logic [10:0] flr_rcvd_vf_num_q;

  //******************************************************************************
  // BIT SYNCHRONIZERS TO SYNCHRONIZE PF FLR REQUESTS
  //******************************************************************************
  always @(posedge clk500) begin //negedge rst_clk500_n or 
    if (rst_clk500_n == 0)
      flr_rcvd_pf_q <= '0;
    else
      flr_rcvd_pf_q <= flr_rcvd_pf_i; 
  end

  genvar i;
  generate
     for (i=0; i<8; i=i+1) begin : flr_rcvd_pf_loop
       intel_pcie_whr_gen4x8_sync_bit #(.DWIDTH(1), .RESET_VAL(0))
              flr_rcvd_pf_sync (.clk(clk250), .rst_n(rst_clk250_n), .din(flr_rcvd_pf_q[i]), .dout(flr_rcvd_pf_o[i]));
     end
  endgenerate

  //******************************************************************************
  // FIFO TO SYNCHRONIZE VF FLR REQUESTS
  //******************************************************************************
    always @ (posedge clk500) begin //negedge rst_clk500_n or 
      if (rst_clk500_n == 0)
        flr_rcvd_vf_q     <= '0;
      else begin
        flr_rcvd_vf_q     <= flr_rcvd_vf_i;
      end
    end

    always @ (posedge clk500) begin
      flr_rcvd_pf_num_q <= flr_rcvd_pf_num_i;
      flr_rcvd_vf_num_q <= flr_rcvd_vf_num_i;
    end

  assign flr_rcvd_vf_data_in = {flr_rcvd_vf_q,flr_rcvd_pf_num_q,flr_rcvd_vf_num_q};

  intel_pcie_whr_gen4x8_dcfifo
    #(
      .DWIDTH ($bits(flr_rcvd_vf_data_in)),
      .AWIDTH (3),
      .intended_device_family(intended_device_family)
     ) flr_rcvd_vf_fifo
    (
     .wr_rst_n(rst_clk500_n),                         // Write Domain Active low Reset
     .wr_clk(clk500),                                 // Write Domain Clock
     .wr_en(flr_rcvd_vf_q),                           // Write Data Enable
     .wr_data(flr_rcvd_vf_data_in),                   // Write Data In
     .rd_rst_n(rst_clk250_n),                         // Read Domain Active low Reset
     .rd_clk(clk250),                                 // Read Domain Clock
     .rd_en(flr_rcvd_vf_fifo_rdreq),                  // Read Data Enable
     .r_pempty(3'd1),                                 // FIFO partially empty threshold
     .r_pfull(3'd6),                                  // FIFO partially full threshold
     .rd_data(flr_rcvd_vf_data_out),                  // Read Data Out
     .rd_numdata(flr_vf_rd_numdata),                  // Number of Data available in Read clock
     .wr_numdata(),                                   // Number of Data available in Write clock
     .wr_empty(),                                     // FIFO Empty
     .wr_pempty(),                                    // FIFO Partial Empty
     .wr_full(),                                      // FIFO Full
     .wr_pfull(),                                     // FIFO Parial Full
     .rd_empty(flr_rcvd_vf_empty),                    // FIFO Empty
     .rd_pempty(),                                    // FIFO Partial Empty
     .rd_full(),                                      // FIFO Full
     .rd_pfull()                                      // FIFO Partial Full
     );

  assign flr_rcvd_vf_fifo_rdreq = ~(flr_rcvd_vf_empty | (flr_vf_rd_numdata == 4'd1 & ~flr_rcvd_vf_empty & flr_rcvd_vf_fifo_rdreq_q));

  always @ (posedge clk250) begin //negedge rst_clk250_n or 
    if (rst_clk250_n == 0) begin
      flr_rcvd_vf_o            <= '0;
      flr_rcvd_vf_fifo_rdreq_q <= '0;
    end
    else begin
      flr_rcvd_vf_o            <= '0;
      flr_rcvd_vf_fifo_rdreq_q <= flr_rcvd_vf_fifo_rdreq;
      if (flr_rcvd_vf_fifo_rdreq_q) begin
        flr_rcvd_vf_o     <= flr_rcvd_vf_data_out.flr_rcvd_vf;
      end
    end
  end

  always @ (posedge clk250) begin
    flr_rcvd_pf_num_o <= flr_rcvd_vf_data_out.flr_rcvd_pf_num;
    flr_rcvd_vf_num_o <= flr_rcvd_vf_data_out.flr_rcvd_vf_num;
  end

//********************************************************************
// SYNCHRONIZATION OF APP -> HIP SIDE SIGNALS
//********************************************************************
  //******************************************************************************
  // DEFINE VARIABLES
  //******************************************************************************

  typedef struct packed {
  logic        flr_completed_vf;
  logic [2:0]  flr_completed_pf_num;
  logic [10:0] flr_completed_vf_num;
  } flr_completed_vf_data_t;

  logic flr_completed_vf_fifo_rdreq, flr_completed_vf_fifo_rdreq_q, flr_completed_vf_empty;
  flr_completed_vf_data_t flr_completed_vf_data_in, flr_completed_vf_data_out;
  logic [3:0] flr_completed_vf_rd_numdata;
  logic [7:0]  flr_completed_pf_q;
  logic        flr_completed_vf_q;     /* synthesis preserve_syn_only */
  logic [2:0]  flr_completed_pf_num_q; /* synthesis preserve_syn_only */
  logic [10:0] flr_completed_vf_num_q; /* synthesis preserve_syn_only */
  //******************************************************************************
  // PULSE SYNCHRONIZERS TO SYNCHRONIZE PF COMPLETION SIGNALS
  //******************************************************************************
  
  always @(posedge clk250) begin //negedge rst_clk500_n or 
    if (rst_clk250_n == 0)
      flr_completed_pf_q <= '0;
    else begin
      flr_completed_pf_q <= flr_completed_pf_i;
    end 
  end

  genvar j;
  generate
    for (j=0; j<8; j=j+1) begin : flr_completed_pf_loop
      intel_pcie_whr_gen4x8_sync_pulse flr_completed_pf_sync
        (
         .wr_clk(clk250),
         .rd_clk(clk500),
         .wr_rst_n(rst_clk250_n),
         .rd_rst_n(rst_clk500_n),
         .din(flr_completed_pf_q[j]),
         .dout(flr_completed_pf_o[j])
         );
    end
  endgenerate

  //******************************************************************************
  // FIFO TO SYNCHRONIZE VF COMPLETION SIGNALS
  //******************************************************************************
  always @ (posedge clk250) begin //negedge rst_clk250_n or 
    if (rst_clk250_n == 0)
      flr_completed_vf_q     <= '0;
    else begin
      flr_completed_vf_q     <= flr_completed_vf_i;
      end
    end

  always @ (posedge clk250) begin
      flr_completed_pf_num_q <= flr_completed_pf_num_i;
      flr_completed_vf_num_q <= flr_completed_vf_num_i;
  end


  assign flr_completed_vf_data_in = {flr_completed_vf_q,flr_completed_pf_num_q,flr_completed_vf_num_q};

  intel_pcie_whr_gen4x8_dcfifo
    #(
      .DWIDTH ($bits(flr_completed_vf_data_in)),
      .AWIDTH (3),
      .intended_device_family(intended_device_family)
     ) flr_completed_vf_fifo
    (
     .wr_rst_n(rst_clk250_n),                         // Write Domain Active low Reset
     .wr_clk(clk250),                                 // Write Domain Clock
     .wr_en(flr_completed_vf_q),                      // Write Data Enable
     .wr_data(flr_completed_vf_data_in),              // Write Data In
     .rd_rst_n(rst_clk500_n),                         // Read Domain Active low Reset
     .rd_clk(clk500),                                 // Read Domain Clock
     .rd_en(flr_completed_vf_fifo_rdreq),             // Read Data Enable
     .r_pempty(3'd2),                                 // FIFO partially empty threshold
     .r_pfull(3'd6),                                  // FIFO partially full threshold
     .rd_data(flr_completed_vf_data_out),             // Read Data Out
     .rd_numdata(flr_completed_vf_rd_numdata),        // Number of Data available in Read clock
     .wr_numdata(),                                   // Number of Data available in Write clock
     .wr_empty(),                                     // FIFO Empty
     .wr_pempty(),                                    // FIFO Partial Empty
     .wr_full(),                                      // FIFO Full
     .wr_pfull(),                                     // FIFO Parial Full
     .rd_empty(flr_completed_vf_empty),               // FIFO Empty
     .rd_pempty(),                                    // FIFO Partial Empty
     .rd_full(),                                      // FIFO Full
     .rd_pfull()                                      // FIFO Partial Full
     );

   assign flr_completed_vf_fifo_rdreq = ~(flr_completed_vf_empty | (flr_completed_vf_rd_numdata == 4'd1 & ~flr_completed_vf_empty & flr_completed_vf_fifo_rdreq_q));

   always @ (posedge clk500) begin //negedge rst_clk500_n or 
     if (rst_clk500_n == 0) begin
       flr_completed_vf_o            <= '0;
       flr_completed_vf_fifo_rdreq_q <= '0;
     end
     else begin
       flr_completed_vf_o            <= '0;
       flr_completed_vf_fifo_rdreq_q <= flr_completed_vf_fifo_rdreq;
       if (flr_completed_vf_fifo_rdreq_q) begin
         flr_completed_vf_o     <= flr_completed_vf_data_out.flr_completed_vf;
       end
     end
   end

   always @ (posedge clk500) begin
     flr_completed_pf_num_o <= flr_completed_vf_data_out.flr_completed_pf_num;
     flr_completed_vf_num_o <= flr_completed_vf_data_out.flr_completed_vf_num;
   end
endmodule
