// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//-----------------------------------------------------------------------------------
// This confidential and proprietary software may be used only as authorized by
// a licensing agreement from Intel
// copyright notice must be reproduced on all authorized copies.
//-----------------------------------------------------------------------------------
// Copyright (C) 2017 Intel Corporation. All rights reserved.  Intel products are
// protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
// other intellectual property laws.
//-----------------------------------------------------------------------------------
//  Project Name:  wharf_rock gen4x8 256 bits @ 500MHZ to 512 bits @ 250 MHz adapter
//  Module Name :  intel_pcie_whr_gen4x8_avmm_hard_ip_status_if
//  Author      :  sdsawant
//  Date        :  Wed Sep 28 16:52:43 2019
//  Description :
//-----------------------------------------------------------------------------------

module  intel_pcie_whr_gen4x8_avmm_hard_ip_status_if
 #(
     parameter intended_device_family = "Stratix 10"
  )
  (
   input  logic       clk500,
   input  logic       rst_clk500_n,
   input  logic       clk250,
   input  logic       rst_clk250_n,
   input  logic       link_up_i,
   input  logic       dl_up_i,
   input  logic [5:0] ltssm_state_i,
   output logic       link_up_o,
   output logic       dl_up_o,
   output logic [5:0] ltssm_state_o
  );

//********************************************************************
// Two stage synchronizer for link_up and dl_up signals
//********************************************************************

  intel_pcie_whr_gen4x8_avmm_sync_bit #(.DWIDTH(1)) link_up_sync (.clk(clk250), .rst_n(rst_clk250_n), .din(link_up_i), .dout(link_up_o));

  intel_pcie_whr_gen4x8_avmm_sync_bit #(.DWIDTH(1)) dl_up_sync (.clk(clk250), .rst_n(rst_clk250_n), .din(dl_up_i), .dout(dl_up_o));

//********************************************************************
// FIFO TO SYNCHRONIZE LTSSM STATE
//********************************************************************
  //******************************************************************************
  // DEFINE VARIABLES
  //******************************************************************************
  logic [5:0] ltssm_state_q, ltssm_state_qq;
  logic       ltssm_fifo_empty;
  logic [5:0] ltssm_fifo_out;
  logic       ltssm_fifo_rdreq, ltssm_fifo_rdreq_q;
  logic [3:0] ltssm_rd_numdata;
  logic       ltssm_fifo_wrreq, ltssm_fifo_wrreq_q;

//********************************************************************
// WRITE CLOCK DOMAIN
//********************************************************************  
  always @ (negedge rst_clk500_n or posedge clk500) begin
    if (rst_clk500_n == 0) begin
      ltssm_state_q       <= '0;
      ltssm_state_qq      <= '0;
      ltssm_fifo_wrreq_q  <= '0;
    end 
    else begin
      ltssm_fifo_wrreq_q  <= ltssm_fifo_wrreq;
      ltssm_state_q       <= ltssm_state_i;
      ltssm_state_qq      <= ltssm_state_q;
    end
  end

  assign ltssm_fifo_wrreq = (ltssm_state_qq != ltssm_state_q);

//********************************************************************
// STORAGE
//********************************************************************
  intel_pcie_whr_gen4x8_avmm_dcfifo
  #(
    .DWIDTH ($bits(ltssm_state_i)),
    .AWIDTH (3),
    .intended_device_family(intended_device_family)
   ) ltssm_fifo
  (
   .wr_rst_n(rst_clk500_n),                         // Write Domain Active low Reset
   .wr_clk(clk500),                                 // Write Domain Clock
   .wr_en(ltssm_fifo_wrreq_q),                      // Write Data Enable
   .wr_data(ltssm_state_qq),                        // Write Data In
   .rd_rst_n(rst_clk250_n),                         // Read Domain Active low Reset
   .rd_clk(clk250),                                 // Read Domain Clock
   .rd_en(ltssm_fifo_rdreq),                        // Read Data Enable
   .r_pempty(3'd2),                                 // FIFO partially empty threshold
   .r_pfull(3'd6),                                  // FIFO partially full threshold
   .rd_data(ltssm_fifo_out),                        // Read Data Out
   .rd_numdata(ltssm_rd_numdata),                   // Number of Data available in Read clock
   .wr_numdata(),                                   // Number of Data available in Write clock
   .wr_empty(),                                     // FIFO Empty
   .wr_pempty(),                                    // FIFO Partial Empty
   .wr_full(),                                      // FIFO Full
   .wr_pfull(),                                     // FIFO Parial Full
   .rd_empty(ltssm_fifo_empty),                     // FIFO Empty
   .rd_pempty(),                                    // FIFO Partial Empty
   .rd_full(),                                      // FIFO Full
   .rd_pfull()                                      // FIFO Partial Full
   );

//********************************************************************
// READ CLOCK DOMAIN
//********************************************************************
  assign ltssm_fifo_rdreq = ~(ltssm_fifo_empty | (ltssm_rd_numdata == 4'd1 & ~ltssm_fifo_empty & ltssm_fifo_rdreq_q));

  always @ (negedge rst_clk250_n or posedge clk250) begin
    if (rst_clk250_n == 0) begin
      ltssm_state_o      <= '0;
      ltssm_fifo_rdreq_q <= '0;
    end
    else begin
      ltssm_fifo_rdreq_q <= ltssm_fifo_rdreq;
      if (ltssm_fifo_rdreq_q) begin
        ltssm_state_o <= ltssm_fifo_out;
      end
    end
  end
endmodule
