// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//-----------------------------------------------------------------------------------
// This confidential and proprietary software may be used only as authorized by
// a licensing agreement from Intel
// copyright notice must be reproduced on all authorized copies.
//-----------------------------------------------------------------------------------
// Copyright (C) 2017 Intel Corporation. All rights reserved.  Intel products are
// protected under numerous U.S. and foreign patents, maskwork rights, copyrights and
// other intellectual property laws.
//-----------------------------------------------------------------------------------
//  Project Name:  wharf_rock gen4x8 256 bits @ 500MHZ to 512 bits @ 250 MHz adapter
//  Module Name :  intel_pcie_whr_gen4x8_avmm_sync_vec
//  Author      :  sdsawant
//  Date        :  Wed Sep 28 16:52:43 2019
//  Description :
//-----------------------------------------------------------------------------------

module intel_pcie_whr_gen4x8_avmm_sync_vec
  #(
     parameter DWIDTH           = 2
  )(
  // Inputs
   input  logic               wr_clk,        // write clock
   input  logic               rd_clk,        // read clock
   input  logic               wr_rst_n,      // async write reset
   input  logic               rd_rst_n,      // async read reset
   input  logic [DWIDTH-1:0]  din,           // data in
   output logic [DWIDTH-1:0]  dout           // data out
  );

//******************************************************************************
// DEFINE VARIABLES
//******************************************************************************
  logic  [DWIDTH-1:0] data_in_d0;
  logic               req_wr_clk;
  logic               req_rd_clk;
  logic               ack_wr_clk;
  logic               ack_rd_clk;
  logic               req_rd_clk_d0;

//******************************************************************************
// WRITE CLOCK DOMAIN: Generate req & Store data when synchroniztion is not
// already in progress 
//******************************************************************************
  always @(negedge wr_rst_n or posedge wr_clk) begin
     if (wr_rst_n == 1'b0) begin
       data_in_d0 <= {DWIDTH{1'b0}};
       req_wr_clk <= 1'b0; 
     end
     else begin
       // Store data when Write Req equals Write Ack
       if (req_wr_clk == ack_wr_clk) begin
         data_in_d0 <= din;
       end
       
       // Generate a Req when there is change in data 
       if ((req_wr_clk == ack_wr_clk) & (data_in_d0 != din)) begin
         req_wr_clk <= ~req_wr_clk;
       end
     end
  end

//******************************************************************************
// WRITE CLOCK DOMAIN:  
//******************************************************************************
  intel_pcie_whr_gen4x8_avmm_sync_bit
  #(
  .DWIDTH      (1),         // Sync Data input
  .RESET_VAL   (0)          // Reset value
  )
  ack_wr_clk_sync
     (
     .clk      (wr_clk),
     .rst_n    (wr_rst_n),
     .din      (ack_rd_clk),
     .dout     (ack_wr_clk)
     );
  assign ack_rd_clk = req_rd_clk_d0;

//******************************************************************************
// READ CLOCK DOMAIN:  
//******************************************************************************
  intel_pcie_whr_gen4x8_avmm_sync_bit
  #(
  .DWIDTH      (1),         // Sync Data input
  .RESET_VAL   (0)          // Reset value
  )
  u_req_rd_clk_sync
  (
     .clk      (rd_clk),
     .rst_n    (rd_rst_n),
     .din      (req_wr_clk),
     .dout     (req_rd_clk)
  );

//******************************************************************************
// READ CLOCK DOMAIN:  
//******************************************************************************
  always @(negedge rd_rst_n or posedge rd_clk) begin
    if (rd_rst_n == 1'b0) begin
      dout          <= {DWIDTH{1'b0}};
      req_rd_clk_d0 <= 1'b0;
    end
    else begin 
      req_rd_clk_d0 <= req_rd_clk;
      if (req_rd_clk_d0 != req_rd_clk) begin
        dout <= data_in_d0;
      end
    end
  end
endmodule //sync_vec

