// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

//-----------------------------------------------------------------------------
// Title         : Virtio Capability Structure for VFs
//-----------------------------------------------------------------------------
// File          : intel_pcie_virtio_cfg_access_cap.v
// Author        : Intel Corporation
//-----------------------------------------------------------------------------
//

module intel_pcie_virtio_cfg_access_cap #
    (
        parameter TOTAL_PF_COUNT                      = 8, // Total number of PFs 
        parameter TOTAL_PF_COUNT_WIDTH                = 3, 
        parameter TOTAL_VF_COUNT_WIDTH                = 11,
        parameter FUNC_COUNT                          = 1,
        parameter FUNC_COUNT_WIDTH                    = 3,
        parameter PF_NUM                              = 1
    )
    (
        input                        Clk_i,
        input                        Rstn_i,
        
        input  [7:0 ]                virtio_pciconfig_access_bar_indicator,
        input  [31:0]                virtio_pciconfig_access_structure_length,
        input  [31:0]                virtio_pciconfig_access_bar_offset,
        
        input  [FUNC_COUNT_WIDTH-1:0]  cfg_req_func_num_i,
        input  [FUNC_COUNT_WIDTH-1:0]  cfg_app_func_num_i,      // App side input
        input                          cfg_write_req_i,
        input  [31:0]                  cfg_write_data_i, 
        input  [3:0 ]                  cfg_write_be_i,
        input                          cfg_read_req_i,
        input                          cfg_read_ack_i,        // App side input
        input  [3:0 ]                  cfg_read_ack_be_i,     // App side input
        input  [31:0]                  cfg_rdack_data_i,      // App side input
        input                          cfg_barwr_req_i,
        input                          cfg_barrd_req_i,
        input                          cfg_lengthwr_req_i,
        input                          cfg_lengthrd_req_i,
        input                          cfg_baroffsetwr_req_i,
        input                          cfg_baroffsetrd_req_i,
   
        output reg [31:0]              cfg_read_data_o,
        output reg                     cfg_req_ack_o,
        output reg [7:0 ]              cfg_bar_o,
        output reg [31:0]              cfg_length_o,
        output reg [31:0]              cfg_baroffset_o,
        output reg                     cfg_access_datawr_o,
        output reg                     cfg_access_datard_o,
        
        // FLR reset inputs
        input                                 flr_rcvd_pf_i,  // 1 bit per PF.  Level signal indicate FLR for corresponding PF is in progress.
        input                                 flr_rcvd_vf_i,  // One-cycle pulse indicates that an FLR was received from host targeting a VF.
        input  [TOTAL_PF_COUNT_WIDTH-1:0]     flr_rcvd_pf_num_i,
        input  [TOTAL_PF_COUNT-1:0]           flr_rcvd_pf_num_match_i,
        input  [TOTAL_VF_COUNT_WIDTH-1:0]     flr_rcvd_vf_num_i
    );
    
    // Local Parameter Declaration
    localparam RAM_ADDR_WIDTH = (FUNC_COUNT > 1024)? 11:
                   (FUNC_COUNT > 512)? 10:
                   (FUNC_COUNT > 256)? 9:
                   (FUNC_COUNT > 128)? 8:
                   (FUNC_COUNT > 64)? 7:
                   (FUNC_COUNT > 32)? 6:
                   (FUNC_COUNT > 16)? 5:
                   (FUNC_COUNT > 8)? 4:
                   (FUNC_COUNT > 4)? 3:
                   (FUNC_COUNT > 2)? 2: 1;
                   
    localparam RESET_ST      = 2'd0;
    localparam RAM_INIT_ST   = 2'd1;
    localparam READY_ST      = 2'd2;
    localparam WAIT_FOR_IDLE = 2'd3;

//    reg  [FUNC_COUNT_WIDTH-1:0]   cfg_req_func_num_q; //not used
//    reg  [31:0]                   cfg_write_data_q;  //not used
//    reg  [3:0 ]                   cfg_write_be_q; //not used
    reg  [RAM_ADDR_WIDTH-1:0]     write_addr;
    reg  [31:0]                   write_data;
    reg  [3:0 ]                   write_enable;
    reg  [RAM_ADDR_WIDTH-1:0]     rdack_addr;
    reg  [31:0]                   rdack_data;
    reg  [3:0 ]                   rdack_enable;
    wire [31:0]                   read_data;
    wire [7:0 ]                   read_bar_data;
    wire [31:0]                   read_length_data;
    wire [31:0]                   read_baroffset_data;
    wire [31:0]                   read_ramdata;
    wire [7:0]                    read_bar_ramdata;
    wire [31:0]                   read_length_ramdata;
    wire [31:0]                   read_baroffset_ramdata;
//    reg  [31:0]                   read_ramdata_q; //not used
//    reg  [31:0]                   read_bar_ramdata_q; //not used
//    reg  [31:0]                   read_length_ramdata_q; //not used
//    reg  [31:0]                   read_baroffset_ramdata_q; //not used
//    reg  [31:0]                   read_ramdata_reg; //not used
//    reg  [31:0]                   read_bar_ramdata_reg; //not used
//    reg  [31:0]                   read_length_ramdata_reg; //not used
//    reg  [31:0]                   read_baroffset_ramdata_reg; //not used
    reg  [RAM_ADDR_WIDTH-1:0]     write_addr_reg;
    reg  [31:0]                   write_data_reg;
    reg  [7:0]                    write_bar_reg;
    reg  [31:0]                   write_length_reg;
    reg  [31:0]                   write_baroffset_reg;
    reg  [3:0 ]                   write_enable_reg;
//    reg  [RAM_ADDR_WIDTH-1:0]     read_addr_reg; //not used
//    reg  [31:0]                   read_data_reg; //not used
//    reg  [7:0 ]                   read_bar_data_reg; //not used
//    reg  [31:0]                   read_length_data_reg; //not used
//    reg  [31:0]                   read_baroffset_data_reg; //not used
    // PCI CFG Access Data Req registers
    reg virtio_pcicfg_cfgwr_reg;
    reg virtio_pcicfg_cfgaccesswr_dly0;
    reg virtio_pcicfg_cfgaccesswr_dly1;
    reg virtio_pcicfg_cfgaccesswr_dly2;
    reg virtio_pcicfg_cfgrd_reg;
    reg virtio_pcicfg_cfgaccessrd_dly0;
    reg virtio_pcicfg_cfgaccessrd_dly1;
    reg virtio_pcicfg_cfgaccessrd_dly2;
    
    reg  [1:0 ]            state;
    reg                    read_ack;
    reg                    read_ack_1;
    reg                    read_ack_2;
    reg                    read_ack_3;
    reg                    read_ack_4;
    reg                    write_ack;
//    reg  [12:0]            iter_cnt; //not used
//    reg                    rst_end_tag;
//    reg                    rst_end_tag_reg;
//    reg  [FUNC_COUNT-1:0]  fst_wrdata_tag;
//    reg  [FUNC_COUNT-1:0]  fst_wrbar_tag;
//    reg  [FUNC_COUNT-1:0]  fst_wrlength_tag;
//    reg  [FUNC_COUNT-1:0]  fst_wrbaroffset_tag;
    reg                    vf_num_changed_tag;
    reg                    cfg_read_ack_reg0;
    reg                    cfg_read_ack_reg1;
    reg                    cfg_read_ack_reg2;
    wire                   cfg_read_ack_2cycle;
    wire [FUNC_COUNT_WIDTH-1:0] index;

    reg  cfg_write_req_dly1_flag,cfg_write_req_dly2_flag,cfg_write_req_dly3_flag;
    
    //======================================
    // RAM Init State Machine
    //======================================
    
     assign index = (FUNC_COUNT == 1) ? 0 : cfg_req_func_num_i;
       
//     always @(posedge Clk_i) begin
//       cfg_req_func_num_q <= cfg_req_func_num_i;
//       cfg_write_data_q   <= cfg_write_data_i; 
//       cfg_write_be_q     <= cfg_write_be_i;
//     end 

     always @(posedge Clk_i) begin
        if (~Rstn_i) begin
            state        <= RESET_ST;
            write_enable <= 4'd0;
            write_addr   <= {RAM_ADDR_WIDTH{1'b0}};
            write_data   <= 32'd0;
            rdack_enable <= 4'd0;
            rdack_addr   <= {RAM_ADDR_WIDTH{1'b0}};
            rdack_data   <= 32'd0;
            write_ack    <= 1'd0;
            cfg_read_ack_reg0  <= 1'b0;
        end
        else begin
            case(state)
                RESET_ST:
                begin
                    write_enable <= 4'd0;
                    write_addr   <= {RAM_ADDR_WIDTH{1'b0}};
                    write_data   <= 32'd0;
                    rdack_enable <= 4'd0;
                    rdack_addr   <= {RAM_ADDR_WIDTH{1'b0}};
                    rdack_data   <= 32'd0;
                    write_ack    <= 1'd0;
                    state        <= READY_ST;
//                    fst_wrdata_tag       <= 'b0;
//                    fst_wrbar_tag        <= 'b0;
//                    fst_wrlength_tag     <= 'b0;
//                    fst_wrbaroffset_tag  <= 'b0;
                    cfg_read_ack_reg0  <= 1'b0;
                end
                READY_ST:
                begin

//                    if(flr_rcvd_vf_i & PF_NUM == flr_rcvd_pf_num_i) begin
//                    if(flr_rcvd_vf_i & flr_rcvd_pf_num_match_i) begin
//                        fst_wrdata_tag[flr_rcvd_vf_num_i]              <= 1'b0;
//                        fst_wrbar_tag[flr_rcvd_vf_num_i]               <= 1'b0;
//                        fst_wrlength_tag[flr_rcvd_vf_num_i]            <= 1'b0;
//                        fst_wrbaroffset_tag[flr_rcvd_vf_num_i]         <= 1'b0;
//                    end

//                    if (flr_rcvd_pf_i) begin
//                        state <= RESET_ST;
//                    end
                    // Process Config Writes and FLR
                    // higher priority for read ack
//                    else if (cfg_read_ack_i & (cfg_app_func_num_i == cfg_req_func_num_i)) begin
                    if (cfg_read_ack_i & cfg_read_req_i) begin
                        rdack_addr   <= cfg_req_func_num_i[RAM_ADDR_WIDTH-1:0];
                        rdack_data   <= cfg_rdack_data_i;
                        rdack_enable <= cfg_read_ack_be_i;
                        state        <= WAIT_FOR_IDLE;
//                        if(cfg_read_req_i) begin
//                          fst_wrdata_tag[index]       <= 1'b1;
//                        end
                        cfg_read_ack_reg0  <= 1'b1;
                    end
                    else if ((cfg_write_req_i) | (cfg_barwr_req_i) | (cfg_lengthwr_req_i) | (cfg_baroffsetwr_req_i)) begin
//CY                        write_addr   <= cfg_req_func_num_q[RAM_ADDR_WIDTH-1:0];
//CY                        write_data   <= cfg_write_data_q;
//CY                        write_enable <= cfg_write_be_q;
                        write_addr   <= cfg_req_func_num_i[RAM_ADDR_WIDTH-1:0];
                        write_data   <= cfg_write_data_i;
                        write_enable <= cfg_write_be_i;
                        write_ack    <= 1'd1;
                        state        <= WAIT_FOR_IDLE;
//                        if (cfg_write_req_i) begin
//                            fst_wrdata_tag[index]       <= 1'b1;
//                        end
//                        else if (cfg_barwr_req_i) begin
//                            fst_wrbar_tag[index]        <= 1'b1;
//                        end
//                        else if (cfg_lengthwr_req_i) begin
//                            fst_wrlength_tag[index]     <= 1'b1;
//                        end
//                        else if (cfg_baroffsetwr_req_i) begin
//                            fst_wrbaroffset_tag[index]  <= 1'b1;
//                        end
                    end
                    else begin
                        state        <= READY_ST;
                    end
                end // case: READY_ST
                WAIT_FOR_IDLE:
                begin
                    // Wait for request de-assertion
                    write_enable   <= 'b0;
                    rdack_enable   <= 'b0;
                    if (~(cfg_write_req_i) & ~(cfg_barwr_req_i) & ~(cfg_lengthwr_req_i) & ~(cfg_baroffsetwr_req_i) & ~(cfg_read_req_i)) begin
                        state      <= READY_ST;
                        write_ack  <= 1'b0;  // more than 1 cycle
                        cfg_read_ack_reg0 <= 1'b0;
                    end
                    else begin
                        state      <= WAIT_FOR_IDLE;
                    end
                end
            endcase // case (state)
        end // else
    end // always
            
            
    //======================================
    // RAM Control Signals
    //======================================
    // Read side
//    always @(posedge Clk_i) begin
        //if ((cfg_read_req_i) | (cfg_barrd_req_i) | (cfg_lengthrd_req_i) | (cfg_baroffsetrd_req_i)) begin
//      read_addr_reg <= cfg_req_func_num_q[RAM_ADDR_WIDTH-1:0];
        //end
        //else begin
        //    read_addr_reg <= {RAM_ADDR_WIDTH{1'b0}};
        //end
//    end
    

    // if cfg_read_ack_i is only one cycle from user side, need to extend it in order to register rdack_data correctly
    always @(posedge Clk_i) begin
        if (~Rstn_i) begin
            cfg_read_ack_reg1  <= 1'b0;
            cfg_read_ack_reg2  <= 1'b0;
        end
        else begin
            cfg_read_ack_reg1  <= cfg_read_ack_reg0;
            cfg_read_ack_reg2  <= cfg_read_ack_reg1;
        end
    end
    // 2 cycle pulse
    assign cfg_read_ack_2cycle = cfg_read_ack_reg0 & ~cfg_read_ack_reg2;

    always @(posedge Clk_i) begin
        if (~Rstn_i) begin
            write_addr_reg     <= 'd0;
            write_enable_reg   <= 4'd0;
            // rdack_enable_reg <= 4'd0;
        end
        else begin
            if (cfg_read_ack_reg0) begin
                write_addr_reg   <= rdack_addr;
                write_enable_reg <= rdack_enable;
            end
            else begin                
                write_addr_reg    <= write_addr;
                write_enable_reg  <= write_enable;
            end
        end
    end

    always @(posedge Clk_i) begin
        // higher priority for read ack
        if (cfg_read_ack_2cycle) begin  // rdack  // write to update cfgdata
        // store the first cap.length bytes in pci_cfg_data.  
        // assume each VF driver is slow rd, two contiguous cfg_rd (cfg_rd to pci_cfg_data to grab user structure data and store, 
        // cfg_rd to pci_cfg_data to read back user structure data) are sent to read user structure back.        
            write_data_reg       <= rdack_data;
        end
        else if (cfg_write_req_i) begin  // VIRTIO_PCICFG_ACCESS_CFGDATA_ADDR
            write_data_reg       <= write_data;
        end
        else if (cfg_barwr_req_i) begin  // VIRTIO_PCICFG_ACCESS_BIR_ADDR
            write_bar_reg        <= write_data[7:0];
        end
        else if (cfg_lengthwr_req_i) begin  // VIRTIO_PCICFG_ACCESS_LENGTH_ADDR
            write_length_reg     <= write_data;
        end 
        else if (cfg_baroffsetwr_req_i) begin  // VIRTIO_PCICFG_ACCESS_BAR_OFFSET_ADDR
            write_baroffset_reg  <= write_data;
        end
        //else begin
        //    write_data_reg       <= {32{1'b0}};
        //    write_bar_reg        <= {{24{1'b0}} , virtio_pciconfig_access_bar_indicator};
        //    write_length_reg     <= virtio_pciconfig_access_structure_length;
        //    write_baroffset_reg  <= virtio_pciconfig_access_bar_offset;
        //end
    end
    
    //======================================
    // RAM Write Read
    //======================================
    // VIRTIO_PCICFG_ACCESS_CFGDATA_ADDR
    // cfg access, both write and rdack
    altera_syncram  cfgdata_reg_ram
    (
        .clock0         (Clk_i),
        .address_a      (write_addr_reg),
        .data_a         (write_data_reg),
        .wren_a         ((cfg_write_req_i) | cfg_read_ack_reg1),
        .rden_b         (1'b1),
        .address_b      (cfg_req_func_num_i[RAM_ADDR_WIDTH-1:0]),
        .q_b            (read_ramdata),
        .aclr0          (1'b0                 ),
        .aclr1          (1'b0                 ),
        .sclr           (1'b0                 ), // Not used.
        .addressstall_a (1'b0                 ),
        .addressstall_b (1'b0                 ),
        .byteena_a      (write_enable_reg     ),
        .byteena_b      (1'b1                 ),
        .clock1         (1'b1                 ),
        .clocken0       (1'b1                 ),
        .clocken1       (1'b1                 ),
        .clocken2       (1'b1                 ),
        .clocken3       (1'b1                 ),
        .data_b         ({32{1'b1}}           ), // Not used.
        .eccencbypass   (1'b0                 ), // Not used.
        .eccencparity   (8'b0                 ), // Not used.
        .q_a            (                     ), // Not used.
        .rden_a         (1'b1                 ), // Not used.
        .address2_a     (1'b1                 ), // Not used.
        .address2_b     (1'b1                 ), // Not used.
        .wren_b         (1'b0                 )  // Not used.
    );
    defparam
        cfgdata_reg_ram.address_aclr_b              = "NONE",
        cfgdata_reg_ram.address_reg_b               = "CLOCK0",
        cfgdata_reg_ram.clock_enable_input_a        = "BYPASS",
        cfgdata_reg_ram.clock_enable_input_b        = "BYPASS",
        cfgdata_reg_ram.clock_enable_output_b       = "BYPASS",
        cfgdata_reg_ram.enable_ecc                  = "FALSE",
        cfgdata_reg_ram.ecc_pipeline_stage_enabled  = "FALSE",
        cfgdata_reg_ram.width_eccstatus             = 2,
        cfgdata_reg_ram.intended_device_family      = "Stratix 10",
        cfgdata_reg_ram.lpm_type                    = "altera_syncram",
        cfgdata_reg_ram.maximum_depth               = 2048,
        cfgdata_reg_ram.numwords_a                  = (1 << RAM_ADDR_WIDTH),
        cfgdata_reg_ram.numwords_b                  = (1 << RAM_ADDR_WIDTH),
        cfgdata_reg_ram.operation_mode              = "DUAL_PORT",
        cfgdata_reg_ram.outdata_aclr_b              = "NONE",
        cfgdata_reg_ram.outdata_reg_b               = "CLOCK0",
        cfgdata_reg_ram.power_up_uninitialized      = "FALSE",
        cfgdata_reg_ram.ram_block_type              = "M20K",
        cfgdata_reg_ram.read_during_write_mode_mixed_ports  = "DONT_CARE",
        cfgdata_reg_ram.widthad_a                   = RAM_ADDR_WIDTH,
        cfgdata_reg_ram.widthad_b                   = RAM_ADDR_WIDTH,
        cfgdata_reg_ram.width_a                     = 32,
        cfgdata_reg_ram.width_b                     = 32,
        cfgdata_reg_ram.width_byteena_a             = 4;
        
    // VIRTIO_PCICFG_ACCESS_BIR_ADDR    
    altera_syncram  bar_reg_ram
    (
        .clock0         (Clk_i),
        .address_a      (write_addr_reg),
        .data_a         (write_bar_reg),
        .wren_a         (cfg_barwr_req_i),
        .rden_b         (1'b1),
        .address_b      (cfg_req_func_num_i[RAM_ADDR_WIDTH-1:0]),
        .q_b            (read_bar_ramdata),
        .aclr0          (1'b0                 ),
        .aclr1          (1'b0                 ),
        .sclr           (1'b0                 ), // Not used.
        .addressstall_a (1'b0                 ),
        .addressstall_b (1'b0                 ),
        .byteena_a      (write_enable_reg[0]  ),
        .byteena_b      (1'b1                 ),
        .clock1         (1'b1                 ),
        .clocken0       (1'b1                 ),
        .clocken1       (1'b1                 ),
        .clocken2       (1'b1                 ),
        .clocken3       (1'b1                 ),
        .data_b         ({8{1'b1}}            ), // Not used.
        .eccencbypass   (1'b0                 ), // Not used.
        .eccencparity   (8'b0                 ), // Not used.
        .q_a            (                     ), // Not used.
        .rden_a         (1'b1                 ), // Not used.
        .address2_a     (1'b1                 ), // Not used.
        .address2_b     (1'b1                 ), // Not used.
        .wren_b         (1'b0                 )  // Not used.
    );
    defparam
        bar_reg_ram.address_aclr_b              = "NONE",
        bar_reg_ram.address_reg_b               = "CLOCK0",
        bar_reg_ram.clock_enable_input_a        = "BYPASS",
        bar_reg_ram.clock_enable_input_b        = "BYPASS",
        bar_reg_ram.clock_enable_output_b       = "BYPASS",
        bar_reg_ram.enable_ecc                  = "FALSE",
        bar_reg_ram.ecc_pipeline_stage_enabled  = "FALSE",
        bar_reg_ram.width_eccstatus             = 2,
        bar_reg_ram.intended_device_family      = "Stratix 10",
        bar_reg_ram.lpm_type                    = "altera_syncram",
        bar_reg_ram.maximum_depth               = 2048,
        bar_reg_ram.numwords_a                  = (1 << RAM_ADDR_WIDTH),
        bar_reg_ram.numwords_b                  = (1 << RAM_ADDR_WIDTH),
        bar_reg_ram.operation_mode              = "DUAL_PORT",
        bar_reg_ram.outdata_aclr_b              = "NONE",
        bar_reg_ram.outdata_reg_b               = "CLOCK0",
        bar_reg_ram.power_up_uninitialized      = "FALSE",
        bar_reg_ram.ram_block_type              = "M20K",
        bar_reg_ram.read_during_write_mode_mixed_ports  = "DONT_CARE",
        bar_reg_ram.widthad_a                   = RAM_ADDR_WIDTH,
        bar_reg_ram.widthad_b                   = RAM_ADDR_WIDTH,
        bar_reg_ram.width_a                     = 8,
        bar_reg_ram.width_b                     = 8,
        bar_reg_ram.width_byteena_a             = 1;
        
    // VIRTIO_PCICFG_ACCESS_LENGTH_ADDR
    altera_syncram  length_reg_ram
    (
        .clock0         (Clk_i),
        .address_a      (write_addr_reg),
        .data_a         (write_length_reg),
        .wren_a         (cfg_lengthwr_req_i),
        .rden_b         (1'b1),
        .address_b      (cfg_req_func_num_i[RAM_ADDR_WIDTH-1:0]),
        .q_b            (read_length_ramdata),
        .aclr0          (1'b0                 ),
        .aclr1          (1'b0                 ),
        .sclr           (1'b0                 ), // Not used.
        .addressstall_a (1'b0                 ),
        .addressstall_b (1'b0                 ),
        .byteena_a      (write_enable_reg     ),
        .byteena_b      (1'b1                 ),
        .clock1         (1'b1                 ),
        .clocken0       (1'b1                 ),
        .clocken1       (1'b1                 ),
        .clocken2       (1'b1                 ),
        .clocken3       (1'b1                 ),
        .data_b         ({32{1'b1}}           ), // Not used.
        .eccencbypass   (1'b0                 ), // Not used.
        .eccencparity   (8'b0                 ), // Not used.
        .q_a            (                     ), // Not used.
        .rden_a         (1'b1                 ), // Not used.
        .address2_a     (1'b1                 ), // Not used.
        .address2_b     (1'b1                 ), // Not used.
        .wren_b         (1'b0                 )  // Not used.
    );
    defparam
        length_reg_ram.address_aclr_b              = "NONE",
        length_reg_ram.address_reg_b               = "CLOCK0",
        length_reg_ram.clock_enable_input_a        = "BYPASS",
        length_reg_ram.clock_enable_input_b        = "BYPASS",
        length_reg_ram.clock_enable_output_b       = "BYPASS",
        length_reg_ram.enable_ecc                  = "FALSE",
        length_reg_ram.ecc_pipeline_stage_enabled  = "FALSE",
        length_reg_ram.width_eccstatus             = 2,
        length_reg_ram.intended_device_family      = "Stratix 10",
        length_reg_ram.lpm_type                    = "altera_syncram",
        length_reg_ram.maximum_depth               = 2048,
        length_reg_ram.numwords_a                  = (1 << RAM_ADDR_WIDTH),
        length_reg_ram.numwords_b                  = (1 << RAM_ADDR_WIDTH),
        length_reg_ram.operation_mode              = "DUAL_PORT",
        length_reg_ram.outdata_aclr_b              = "NONE",
        length_reg_ram.outdata_reg_b               = "CLOCK0",
        length_reg_ram.power_up_uninitialized      = "FALSE",
        length_reg_ram.ram_block_type              = "M20K",
        length_reg_ram.read_during_write_mode_mixed_ports  = "DONT_CARE",
        length_reg_ram.widthad_a                   = RAM_ADDR_WIDTH,
        length_reg_ram.widthad_b                   = RAM_ADDR_WIDTH,
        length_reg_ram.width_a                     = 32,
        length_reg_ram.width_b                     = 32,
        length_reg_ram.width_byteena_a             = 4;

    // VIRTIO_PCICFG_ACCESS_BAR_OFFSET_ADDR
    altera_syncram  baroffset_reg_ram
    (
        .clock0         (Clk_i),
        .address_a      (write_addr_reg),
        .data_a         (write_baroffset_reg),
        .wren_a         (cfg_baroffsetwr_req_i),
        .rden_b         (1'b1),
        .address_b      (cfg_req_func_num_i[RAM_ADDR_WIDTH-1:0]),
        .q_b            (read_baroffset_ramdata),
        .aclr0          (1'b0                 ),
        .aclr1          (1'b0                 ),
        .sclr           (1'b0                 ), // Not used.
        .addressstall_a (1'b0                 ),
        .addressstall_b (1'b0                 ),
        .byteena_a      (write_enable_reg     ),
        .byteena_b      (1'b1                 ),
        .clock1         (1'b1                 ),
        .clocken0       (1'b1                 ),
        .clocken1       (1'b1                 ),
        .clocken2       (1'b1                 ),
        .clocken3       (1'b1                 ),
        .data_b         ({32{1'b1}}           ), // Not used.
        .eccencbypass   (1'b0                 ), // Not used.
        .eccencparity   (8'b0                 ), // Not used.
        .q_a            (                     ), // Not used.
        .rden_a         (1'b1                 ), // Not used.
        .address2_a     (1'b1                 ), // Not used.
        .address2_b     (1'b1                 ), // Not used.
        .wren_b         (1'b0                 )  // Not used.
    );
    defparam
        baroffset_reg_ram.address_aclr_b              = "NONE",
        baroffset_reg_ram.address_reg_b               = "CLOCK0",
        baroffset_reg_ram.clock_enable_input_a        = "BYPASS",
        baroffset_reg_ram.clock_enable_input_b        = "BYPASS",
        baroffset_reg_ram.clock_enable_output_b       = "BYPASS",
        baroffset_reg_ram.enable_ecc                  = "FALSE",
        baroffset_reg_ram.ecc_pipeline_stage_enabled  = "FALSE",
        baroffset_reg_ram.width_eccstatus             = 2,
        baroffset_reg_ram.intended_device_family      = "Stratix 10",
        baroffset_reg_ram.lpm_type                    = "altera_syncram",
        baroffset_reg_ram.maximum_depth               = 2048,
        baroffset_reg_ram.numwords_a                  = (1 << RAM_ADDR_WIDTH),
        baroffset_reg_ram.numwords_b                  = (1 << RAM_ADDR_WIDTH),
        baroffset_reg_ram.operation_mode              = "DUAL_PORT",
        baroffset_reg_ram.outdata_aclr_b              = "NONE",
        baroffset_reg_ram.outdata_reg_b               = "CLOCK0",
        baroffset_reg_ram.power_up_uninitialized      = "FALSE",
        baroffset_reg_ram.ram_block_type              = "M20K",
        baroffset_reg_ram.read_during_write_mode_mixed_ports  = "DONT_CARE",
        baroffset_reg_ram.widthad_a                   = RAM_ADDR_WIDTH,
        baroffset_reg_ram.widthad_b                   = RAM_ADDR_WIDTH,
        baroffset_reg_ram.width_a                     = 32,
        baroffset_reg_ram.width_b                     = 32,
        baroffset_reg_ram.width_byteena_a             = 4;

//    always @(posedge Clk_i) begin
//      read_ramdata_q            <= read_ramdata;
//      read_bar_ramdata_q        <= read_bar_ramdata;
//      read_length_ramdata_q     <= read_length_ramdata;
//      read_baroffset_ramdata_q  <= read_baroffset_ramdata;
//    end

//    always @(posedge Clk_i) begin
//        read_ramdata_reg            <= {32{1'b0}};
//        read_bar_ramdata_reg        <= {{24{1'b0}} , virtio_pciconfig_access_bar_indicator};
//        read_length_ramdata_reg     <= virtio_pciconfig_access_structure_length;
//        read_baroffset_ramdata_reg  <= virtio_pciconfig_access_bar_offset;

//        if (fst_wrdata_tag[index]) begin
//            read_ramdata_reg            <= read_ramdata_q;
//        end
//        if (fst_wrbar_tag[index]) begin
//            read_bar_ramdata_reg        <= read_bar_ramdata_q;
//        end
//        if (fst_wrlength_tag[index]) begin
//            read_length_ramdata_reg     <= read_length_ramdata_q;
//        end
//        if (fst_wrbaroffset_tag[index]) begin
//            read_baroffset_ramdata_reg  <= read_baroffset_ramdata_q;
//        end
//    end


    always @(posedge Clk_i) begin
      cfg_write_req_dly1_flag <= ((write_enable_reg!='0) || cfg_write_req_dly1_flag) && cfg_write_req_i;
      cfg_write_req_dly2_flag <= cfg_write_req_dly1_flag;
      cfg_write_req_dly3_flag <= cfg_write_req_dly1_flag && cfg_write_req_dly2_flag;
    end
 
//    reg [31:0]      read_ramdata_mux, read_length_ramdata_mux, read_baroffset_ramdata_mux;
//    reg [7:0]       read_bar_ramdata_mux;
//    assign read_ramdata_mux    = fst_wrdata_tag[index] ? read_ramdata : 'b0;
    assign read_data           = (cfg_write_req_dly3_flag) ? read_ramdata :
                                ((cfg_read_req_i) ? read_ramdata : 'b0);
 //CY timing fix       assign read_data           = (cfg_write_req_i) ? write_data_reg : read_ramdata_reg;
//    assign read_bar_ramdata_mux = fst_wrbar_tag[index] ? read_bar_ramdata : virtio_pciconfig_access_bar_indicator;
    assign read_bar_data       = cfg_barrd_req_i ? read_bar_ramdata[7:0] : 'b0;

//    assign read_length_ramdata_mux = fst_wrlength_tag[index] ? read_length_ramdata : virtio_pciconfig_access_structure_length;
    assign read_length_data    = cfg_lengthrd_req_i ? read_length_ramdata : 'b0;

//    assign read_baroffset_ramdata_mux = fst_wrbaroffset_tag[index] ? read_baroffset_ramdata : virtio_pciconfig_access_bar_offset; 
    assign read_baroffset_data = cfg_baroffsetrd_req_i ? read_baroffset_ramdata : 'b0;

//    always @(posedge Clk_i or negedge Rstn_i) begin
//        if (~Rstn_i) begin
//            read_data_reg             <= 'd0;
//            read_bar_data_reg         <= 'd0;
//            read_length_data_reg      <= 'd0;
//            read_baroffset_data_reg   <= 'd0;
//        end
//        else begin
//            read_data_reg           <= read_data;
//            read_bar_data_reg       <= read_bar_data;
//            read_length_data_reg    <= read_length_data;
//            read_baroffset_data_reg <= read_baroffset_data;
//        end
//    end

    // PCI CFG Access Data Req registers feed to user side, one cycle pulse
    always @(posedge Clk_i) begin
        if (~Rstn_i) begin
            virtio_pcicfg_cfgwr_reg          <= 1'b0;
            virtio_pcicfg_cfgaccesswr_dly0   <= 1'b0;
            virtio_pcicfg_cfgaccesswr_dly1   <= 1'b0;
            virtio_pcicfg_cfgaccesswr_dly2   <= 1'b0;
            
            virtio_pcicfg_cfgrd_reg          <= 1'b0;
            virtio_pcicfg_cfgaccessrd_dly0   <= 1'b0;
            virtio_pcicfg_cfgaccessrd_dly1   <= 1'b0;
            virtio_pcicfg_cfgaccessrd_dly2   <= 1'b0;
        end
        else begin
            virtio_pcicfg_cfgaccesswr_dly0   <= ((write_enable_reg!='0) || virtio_pcicfg_cfgaccesswr_dly0) && cfg_write_req_i;
            virtio_pcicfg_cfgaccesswr_dly1   <= ~virtio_pcicfg_cfgaccesswr_dly0;
            virtio_pcicfg_cfgaccesswr_dly2   <= virtio_pcicfg_cfgaccesswr_dly1;
            virtio_pcicfg_cfgwr_reg          <= virtio_pcicfg_cfgaccesswr_dly2 & ~virtio_pcicfg_cfgaccesswr_dly1;
            
            virtio_pcicfg_cfgaccessrd_dly0   <= ~(cfg_read_req_i);
            virtio_pcicfg_cfgaccessrd_dly1   <= virtio_pcicfg_cfgaccessrd_dly0;
            virtio_pcicfg_cfgaccessrd_dly2   <= virtio_pcicfg_cfgaccessrd_dly1;
            virtio_pcicfg_cfgrd_reg          <= virtio_pcicfg_cfgaccessrd_dly2 & ~virtio_pcicfg_cfgaccessrd_dly1;
        end
    end    

    // use for both hooks and read back data. 
    // hooks are validated by cfgrd/wr
    // As for read req back data, reset to 0 should be considered 


    always @(posedge Clk_i) begin
        if (~Rstn_i) begin
          cfg_access_datawr_o  <= '0;
          cfg_access_datard_o  <= '0;
        end
        else begin
//CY timing fix          cfg_read_data_o  <= (cfg_write_req_i || cfg_read_req_i) ? read_data_reg : 'b0;
          cfg_read_data_o  <= read_data;
          cfg_bar_o        <= (virtio_pcicfg_cfgrd_reg | virtio_pcicfg_cfgwr_reg) ? read_bar_ramdata[7:0] : read_bar_data; 
          cfg_length_o     <= (virtio_pcicfg_cfgrd_reg | virtio_pcicfg_cfgwr_reg) ? read_length_ramdata : read_length_data;
          cfg_baroffset_o  <= (virtio_pcicfg_cfgrd_reg | virtio_pcicfg_cfgwr_reg) ? read_baroffset_ramdata : read_baroffset_data;
          
          cfg_access_datawr_o  <= virtio_pcicfg_cfgwr_reg;
          cfg_access_datard_o  <= virtio_pcicfg_cfgrd_reg;
        end
    end

    always @(posedge Clk_i) begin
        if (~Rstn_i) begin
            read_ack      <= 1'b0;
            read_ack_1    <= 1'b0;
            read_ack_2    <= 1'b0;
            read_ack_3    <= 1'b0;
            read_ack_4    <= 1'b0;
            cfg_req_ack_o <= 1'b0;
        end
        else begin
            read_ack      <= (cfg_read_req_i && cfg_read_ack_reg1) | (cfg_barrd_req_i) | (cfg_lengthrd_req_i) | (cfg_baroffsetrd_req_i);
            // delay read_ack output 2 cycles after req asserted. In this case, the top cii_override_en_o reg stage num is 7.
            // this is to cooperate with reg stage num of cii_override_din_o (based on req assertion). 
            // For din_o, the longest case is assignment of read_addr_reg (VFs case in general, change of LSB in PF case(eg PF1/3/5/7)).
            // note, there're 2 cycles delay for syncram to read out data after getting intended read_addr_reg.
            read_ack_1    <= read_ack;
            read_ack_2    <= read_ack_1;
            read_ack_3    <= read_ack_2;
            read_ack_4    <= read_ack_3;
            cfg_req_ack_o <= read_ack & read_ack_4 
                             | (((cfg_write_req_i) | (cfg_barwr_req_i) | (cfg_lengthwr_req_i) | (cfg_baroffsetwr_req_i)) & write_ack);
        end
    end

endmodule

       

