// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

//-----------------------------------------------------------------------------
// Description   : WHR PCI Express VirtIO CFG Requests Monitor.
//                 Generate PF/VF Individual Register WR/RD Select Signals.
//-----------------------------------------------------------------------------
// File          : intel_pcie_virtio_cfgmnt.v
// Author        : Intel Corporation
//-----------------------------------------------------------------------------
//

module intel_pcie_virtio_cfgmnt # 
    (
        parameter TOTAL_PF_COUNT                      = 8, // Total number of PFs 
        parameter TOTAL_PF_COUNT_WIDTH                = 3, 
        parameter TOTAL_VF_COUNT                      = 2048, // Total number of VFs
                                                              // 1 - 8 (in steps of 1),
                                                              // 8 - 256 (in multiples of 4)
                                                              // 256 - 1024 (in multiples of 64)
                                                              // 1024 - 4096 (in multiples of 512)
                                                              // Sum of PF0_VF_COUNT, PF1_VF_COUNT, PF2_VF_COUNT and
                                                              // PF3_VF_COUNT must equal TOTAL_VF_COUNT
        parameter TOTAL_VF_COUNT_WIDTH                = 11,   // Number of bits needed to represent TOTAL_VF_COUNT
        parameter PF0_VF_COUNT                        = 1024, // Number of VFs attached to PF 0
        parameter PF1_VF_COUNT                        = 1024, // Number of VFs attached to PF 1
        parameter PF2_VF_COUNT                        = 0,    // Number of VFs attached to PF 2
        parameter PF3_VF_COUNT                        = 0,    // Number of VFs attached to PF 3
        parameter PF4_VF_COUNT                        = 0,    // Number of VFs attached to PF 4
        parameter PF5_VF_COUNT                        = 0,    // Number of VFs attached to PF 5
        parameter PF6_VF_COUNT                        = 0,    // Number of VFs attached to PF 6
        parameter PF7_VF_COUNT                        = 0,    // Number of VFs attached to PF 7
        parameter TOTAL_PF_COUNT_MAX                  = 8,
        parameter REG_ADDR_WIDTH                      = 10,   // The double word register address.
        parameter VIRTIO_CMN_CONFIG_REG_ADDR       = 8'h12,   // Reg Address
        parameter VIRTIO_NOTIFICATION_REG_ADDR     = 8'h16,
        parameter VIRTIO_ISR_STATUS_REG_ADDR       = 8'h2F,
        parameter VIRTIO_DEV_SPECIFIC_REG_ADDR     = 8'h33,
        parameter VIRTIO_PCICFG_ACCESS_REG_ADDR    = 8'h37,
        // VirtIO presence parameters
        parameter PF0_VIRTIO_CAPABILITY_PRESENT       = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF1_VIRTIO_CAPABILITY_PRESENT       = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF2_VIRTIO_CAPABILITY_PRESENT       = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF3_VIRTIO_CAPABILITY_PRESENT       = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF4_VIRTIO_CAPABILITY_PRESENT       = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF5_VIRTIO_CAPABILITY_PRESENT       = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF6_VIRTIO_CAPABILITY_PRESENT       = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF7_VIRTIO_CAPABILITY_PRESENT       = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF0VF_VIRTIO_CAPABILITY_PRESENT     = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF1VF_VIRTIO_CAPABILITY_PRESENT     = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF2VF_VIRTIO_CAPABILITY_PRESENT     = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF3VF_VIRTIO_CAPABILITY_PRESENT     = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF4VF_VIRTIO_CAPABILITY_PRESENT     = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF5VF_VIRTIO_CAPABILITY_PRESENT     = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF6VF_VIRTIO_CAPABILITY_PRESENT     = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF7VF_VIRTIO_CAPABILITY_PRESENT     = 1'b0             //Indicates VIRTIO Capability is Present             
    )
    (
        input                                 clk,
        input                                 rstn,   
        // CII Interface
        input                                 cii_req_i,      // Indicates the CFG request is intercepted and all the other CII signals are valid.                                           
        input                                 cii_wr_i,       // Indicates that the cii_dout_p0/1 is valid. This signal is asserted only for a configuration write request.
        input                                 cii_wr_vf_active_i,    // Indicates that the received TLP was targeted to a VF inside the controller, and is identified by cii_vf_num.    
        input  [TOTAL_VF_COUNT_WIDTH-1:0]     cii_vf_num_i,
        input  [REG_ADDR_WIDTH-1:0]           cii_addr_i,      // The dword register address in the received TLP header on CII.
        input  [TOTAL_PF_COUNT_WIDTH-1:0]     cii_func_num_i,        // The function number in the received TLP header on CII.
        // FLR Interafce
        input  [TOTAL_PF_COUNT-1:0]           flr_rcvd_pf_i,  // 1 bit per PF.  Level signal indicate FLR for corresponding PF is in progress.
        input                                 flr_rcvd_vf_i,  // One-cycle pulse indicates that an FLR was received from host targeting a VF.
        input  [TOTAL_PF_COUNT_WIDTH-1:0]     flr_rcvd_pf_num_i,
        input  [TOTAL_VF_COUNT_WIDTH-1:0]     flr_rcvd_vf_num_i,     // VF offset

        output reg                            read_cmncfg_header_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_cmncfg_bar_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_cmncfg_bar_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_cmncfg_baroffset_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_cmncfg_baroffset_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_cmncfg_length_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_cmncfg_length_sel_o,
        output reg                            read_notifications_header_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_notifications_bar_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_notifications_bar_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_notifications_baroffset_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_notifications_baroffset_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_notifications_length_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_notifications_length_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_notifications_notifyoff_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_notifications_notifyoff_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_isrstatus_header_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_isrstatus_header_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_isrstatus_bar_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_isrstatus_bar_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_isrstatus_baroffset_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_isrstatus_baroffset_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_isrstatus_length_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_isrstatus_length_sel_o,
        output reg                            read_devspecific_header_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_devspecific_bar_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_devspecific_bar_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_devspecific_baroffset_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_devspecific_baroffset_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_devspecific_length_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_devspecific_length_sel_o,

        output reg                            write_roreg_sel_o,

        output reg                            read_pcicfgaccess_header_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_pcicfgaccess_bar_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    write_pf_pcicfgaccess_bar_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_pcicfgaccess_bar_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    write_vf_pcicfgaccess_bar_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_pcicfgaccess_baroffset_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    write_pf_pcicfgaccess_baroffset_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_pcicfgaccess_baroffset_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    write_vf_pcicfgaccess_baroffset_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_pcicfgaccess_length_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    write_pf_pcicfgaccess_length_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_pcicfgaccess_length_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    write_vf_pcicfgaccess_length_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    read_pf_pcicfgaccess_cfgdata_sel_o,
        output reg  [TOTAL_PF_COUNT-1 : 0]    write_pf_pcicfgaccess_cfgdata_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    read_vf_pcicfgaccess_cfgdata_sel_o,
        output wire [TOTAL_PF_COUNT-1 : 0]    write_vf_pcicfgaccess_cfgdata_sel_o,

        output                                targeted_cap_not_present_o,
        output                                noregbank_selected_o  // for no address hit, create halt and override
    );
    
    // Chip Select for VirtIO Common Config Capability
    wire                             virtio_cmncfg_header_cs;
    wire                             virtio_cmncfg_bar_cs;
    wire                             virtio_cmncfg_baroffset_cs;
    wire                             virtio_cmncfg_length_cs;
    // VirtIO Notifications Capability
    wire                             virtio_notifications_header_cs;
    wire                             virtio_notifications_bar_cs;
    wire                             virtio_notifications_baroffset_cs;
    wire                             virtio_notifications_length_cs;
    wire                             virtio_notifications_notifyoff_cs;
    // VirtIO ISR Status Capability
    wire                             virtio_isrstatus_header_cs;
    wire                             virtio_isrstatus_bar_cs;
    wire                             virtio_isrstatus_baroffset_cs;
    wire                             virtio_isrstatus_length_cs;
    // VirtIO Device Specific Capability
    wire                             virtio_devspecific_header_cs;
    wire                             virtio_devspecific_bar_cs;
    wire                             virtio_devspecific_baroffset_cs;
    wire                             virtio_devspecific_length_cs;
    // VirtIO PCI Config Access Capability
    wire                             virtio_pcicfgaccess_header_cs;
    wire                             virtio_pcicfgaccess_bar_cs;
    wire                             virtio_pcicfgaccess_baroffset_cs;
    wire                             virtio_pcicfgaccess_length_cs;
    wire                             virtio_pcicfgaccess_cfgdata_cs;
							         
    wire                             write_roreg_cs;
    // PFs' VF Read Write Control
    wire                             write_pf0vf_ctrl;  // VF offset
    wire                             read_pf0vf_ctrl;  // VF offset
    wire                             write_pf1vf_ctrl;  // VF offset
    wire                             read_pf1vf_ctrl;  // VF offset
    wire                             write_pf2vf_ctrl;  // VF offset
    wire                             read_pf2vf_ctrl;  // VF offset
    wire                             write_pf3vf_ctrl;  // VF offset
    wire                             read_pf3vf_ctrl;  // VF offset
    wire                             write_pf4vf_ctrl;  // VF offset
    wire                             read_pf4vf_ctrl;  // VF offset
    wire                             write_pf5vf_ctrl;  // VF offset
    wire                             read_pf5vf_ctrl;  // VF offset
    wire                             write_pf6vf_ctrl;  // VF offset
    wire                             read_pf6vf_ctrl;  // VF offset
    wire                             write_pf7vf_ctrl;  // VF offset
    wire                             read_pf7vf_ctrl;  // VF offset
    // PF Read Write Control
    wire [TOTAL_PF_COUNT-1 : 0] write_pf_ctrl;
    wire [TOTAL_PF_COUNT-1 : 0] read_pf_ctrl;
    wire                        write_pf_pfvf_ctrl;
    // PCI CFG Access PF VFs
    wire                        read_pf0vf_pcicfgaccess_bar_sel;
    wire                        write_pf0vf_pcicfgaccess_bar_sel;
    wire                        read_pf1vf_pcicfgaccess_bar_sel;
    wire                        write_pf1vf_pcicfgaccess_bar_sel;
    wire                        read_pf2vf_pcicfgaccess_bar_sel;
    wire                        write_pf2vf_pcicfgaccess_bar_sel;
    wire                        read_pf3vf_pcicfgaccess_bar_sel;
    wire                        write_pf3vf_pcicfgaccess_bar_sel;
    wire                        read_pf4vf_pcicfgaccess_bar_sel;
    wire                        write_pf4vf_pcicfgaccess_bar_sel;
    wire                        read_pf5vf_pcicfgaccess_bar_sel;
    wire                        write_pf5vf_pcicfgaccess_bar_sel;
    wire                        read_pf6vf_pcicfgaccess_bar_sel;
    wire                        write_pf6vf_pcicfgaccess_bar_sel;
    wire                        read_pf7vf_pcicfgaccess_bar_sel;
    wire                        write_pf7vf_pcicfgaccess_bar_sel;
    wire                        read_pf0vf_pcicfgaccess_baroffset_sel;
    wire                        write_pf0vf_pcicfgaccess_baroffset_sel;
    wire                        read_pf1vf_pcicfgaccess_baroffset_sel;
    wire                        write_pf1vf_pcicfgaccess_baroffset_sel;
    wire                        read_pf2vf_pcicfgaccess_baroffset_sel;
    wire                        write_pf2vf_pcicfgaccess_baroffset_sel;
    wire                        read_pf3vf_pcicfgaccess_baroffset_sel;
    wire                        write_pf3vf_pcicfgaccess_baroffset_sel;
    wire                        read_pf4vf_pcicfgaccess_baroffset_sel;
    wire                        write_pf4vf_pcicfgaccess_baroffset_sel;
    wire                        read_pf5vf_pcicfgaccess_baroffset_sel;
    wire                        write_pf5vf_pcicfgaccess_baroffset_sel;
    wire                        read_pf6vf_pcicfgaccess_baroffset_sel;
    wire                        write_pf6vf_pcicfgaccess_baroffset_sel;
    wire                        read_pf7vf_pcicfgaccess_baroffset_sel;
    wire                        write_pf7vf_pcicfgaccess_baroffset_sel;
    wire                        read_pf0vf_pcicfgaccess_length_sel;
    wire                        write_pf0vf_pcicfgaccess_length_sel;
    wire                        read_pf1vf_pcicfgaccess_length_sel;
    wire                        write_pf1vf_pcicfgaccess_length_sel;
    wire                        read_pf2vf_pcicfgaccess_length_sel;
    wire                        write_pf2vf_pcicfgaccess_length_sel;
    wire                        read_pf3vf_pcicfgaccess_length_sel;
    wire                        write_pf3vf_pcicfgaccess_length_sel;
    wire                        read_pf4vf_pcicfgaccess_length_sel;
    wire                        write_pf4vf_pcicfgaccess_length_sel;
    wire                        read_pf5vf_pcicfgaccess_length_sel;
    wire                        write_pf5vf_pcicfgaccess_length_sel;
    wire                        read_pf6vf_pcicfgaccess_length_sel;
    wire                        write_pf6vf_pcicfgaccess_length_sel;
    wire                        read_pf7vf_pcicfgaccess_length_sel;
    wire                        write_pf7vf_pcicfgaccess_length_sel;
    wire                        read_pf0vf_pcicfgaccess_cfgdata_sel;
    wire                        write_pf0vf_pcicfgaccess_cfgdata_sel;
    wire                        read_pf1vf_pcicfgaccess_cfgdata_sel;
    wire                        write_pf1vf_pcicfgaccess_cfgdata_sel;
    wire                        read_pf2vf_pcicfgaccess_cfgdata_sel;
    wire                        write_pf2vf_pcicfgaccess_cfgdata_sel;
    wire                        read_pf3vf_pcicfgaccess_cfgdata_sel;
    wire                        write_pf3vf_pcicfgaccess_cfgdata_sel;
    wire                        read_pf4vf_pcicfgaccess_cfgdata_sel;
    wire                        write_pf4vf_pcicfgaccess_cfgdata_sel;
    wire                        read_pf5vf_pcicfgaccess_cfgdata_sel;
    wire                        write_pf5vf_pcicfgaccess_cfgdata_sel;
    wire                        read_pf6vf_pcicfgaccess_cfgdata_sel;
    wire                        write_pf6vf_pcicfgaccess_cfgdata_sel;
    wire                        read_pf7vf_pcicfgaccess_cfgdata_sel;
    wire                        write_pf7vf_pcicfgaccess_cfgdata_sel;    
    
    reg  [7:0]                  read_vf_cmncfg_bar_sel_reg;
    reg  [7:0]                  read_vf_cmncfg_baroffset_sel_reg;
    reg  [7:0]                  read_vf_cmncfg_length_sel_reg;
    reg  [7:0]                  read_vf_notifications_bar_sel_reg;
    reg  [7:0]                  read_vf_notifications_baroffset_sel_reg;
    reg  [7:0]                  read_vf_notifications_length_sel_reg;
    reg  [7:0]                  read_vf_notifications_notifyoff_sel_reg;
    reg  [7:0]                  read_vf_isrstatus_header_sel_reg;
    reg  [7:0]                  read_vf_isrstatus_bar_sel_reg;
    reg  [7:0]                  read_vf_isrstatus_baroffset_sel_reg;
    reg  [7:0]                  read_vf_isrstatus_length_sel_reg;
    reg  [7:0]                  read_vf_devspecific_bar_sel_reg;
    reg  [7:0]                  read_vf_devspecific_baroffset_sel_reg;
    reg  [7:0]                  read_vf_devspecific_length_sel_reg;
    reg  [7:0]                  read_vf_pcicfgaccess_bar_sel_reg;
    reg  [7:0]                  write_vf_pcicfgaccess_bar_sel_reg;
    reg  [7:0]                  read_vf_pcicfgaccess_baroffset_sel_reg;
    reg  [7:0]                  write_vf_pcicfgaccess_baroffset_sel_reg;
    reg  [7:0]                  read_vf_pcicfgaccess_length_sel_reg;
    reg  [7:0]                  write_vf_pcicfgaccess_length_sel_reg;
    reg  [7:0]                  read_vf_pcicfgaccess_cfgdata_sel_reg;
    reg  [7:0]                  write_vf_pcicfgaccess_cfgdata_sel_reg;

    // Instantiate Address Decode Module responsible for CS creation
    intel_pcie_virtio_addrdc # 
    (
        .TOTAL_PF_COUNT_WIDTH             (TOTAL_PF_COUNT_WIDTH),
        .REG_ADDR_WIDTH                   (REG_ADDR_WIDTH ),
        .VIRTIO_CMN_CONFIG_REG_ADDR       (VIRTIO_CMN_CONFIG_REG_ADDR   ),
        .VIRTIO_NOTIFICATION_REG_ADDR     (VIRTIO_NOTIFICATION_REG_ADDR    ),
        .VIRTIO_ISR_STATUS_REG_ADDR       (VIRTIO_ISR_STATUS_REG_ADDR     ),
        .VIRTIO_DEV_SPECIFIC_REG_ADDR     (VIRTIO_DEV_SPECIFIC_REG_ADDR    ),
        .VIRTIO_PCICFG_ACCESS_REG_ADDR    (VIRTIO_PCICFG_ACCESS_REG_ADDR     ),
        .PF0_VIRTIO_CAPABILITY_PRESENT   (PF0_VIRTIO_CAPABILITY_PRESENT),
        .PF1_VIRTIO_CAPABILITY_PRESENT   (PF1_VIRTIO_CAPABILITY_PRESENT),
        .PF2_VIRTIO_CAPABILITY_PRESENT   (PF2_VIRTIO_CAPABILITY_PRESENT),
        .PF3_VIRTIO_CAPABILITY_PRESENT   (PF3_VIRTIO_CAPABILITY_PRESENT),
        .PF4_VIRTIO_CAPABILITY_PRESENT   (PF4_VIRTIO_CAPABILITY_PRESENT),
        .PF5_VIRTIO_CAPABILITY_PRESENT   (PF5_VIRTIO_CAPABILITY_PRESENT),
        .PF6_VIRTIO_CAPABILITY_PRESENT   (PF6_VIRTIO_CAPABILITY_PRESENT),
        .PF7_VIRTIO_CAPABILITY_PRESENT   (PF7_VIRTIO_CAPABILITY_PRESENT),
        .PF0VF_VIRTIO_CAPABILITY_PRESENT (PF0VF_VIRTIO_CAPABILITY_PRESENT),
        .PF1VF_VIRTIO_CAPABILITY_PRESENT (PF1VF_VIRTIO_CAPABILITY_PRESENT),
        .PF2VF_VIRTIO_CAPABILITY_PRESENT (PF2VF_VIRTIO_CAPABILITY_PRESENT),
        .PF3VF_VIRTIO_CAPABILITY_PRESENT (PF3VF_VIRTIO_CAPABILITY_PRESENT),
        .PF4VF_VIRTIO_CAPABILITY_PRESENT (PF4VF_VIRTIO_CAPABILITY_PRESENT),
        .PF5VF_VIRTIO_CAPABILITY_PRESENT (PF5VF_VIRTIO_CAPABILITY_PRESENT),
        .PF6VF_VIRTIO_CAPABILITY_PRESENT (PF6VF_VIRTIO_CAPABILITY_PRESENT),
        .PF7VF_VIRTIO_CAPABILITY_PRESENT (PF7VF_VIRTIO_CAPABILITY_PRESENT)
    )
    addrdecoder (
        .clk                      (clk                    ),                                         
        .rstn                     (rstn                   ),  
        .cii_req_i                (cii_req_i            ),  
        .cii_addr_i               (cii_addr_i          ), 
        .cii_func_num_i           (cii_func_num_i),
        .cii_wr_vf_active_i       (cii_wr_vf_active_i),
        .virtio_cmncfg_header_cs_o             (virtio_cmncfg_header_cs),  
        .virtio_cmncfg_bar_cs_o                (virtio_cmncfg_bar_cs),  
        .virtio_cmncfg_baroffset_cs_o          (virtio_cmncfg_baroffset_cs),  
        .virtio_cmncfg_length_cs_o             (virtio_cmncfg_length_cs),  
        .virtio_notifications_header_cs_o      (virtio_notifications_header_cs),  
        .virtio_notifications_bar_cs_o         (virtio_notifications_bar_cs),  
        .virtio_notifications_baroffset_cs_o   (virtio_notifications_baroffset_cs),  
        .virtio_notifications_length_cs_o      (virtio_notifications_length_cs),  
        .virtio_notifications_notifyoff_cs_o   (virtio_notifications_notifyoff_cs),  
        .virtio_isrstatus_header_cs_o          (virtio_isrstatus_header_cs),  
        .virtio_isrstatus_bar_cs_o             (virtio_isrstatus_bar_cs),  
        .virtio_isrstatus_baroffset_cs_o       (virtio_isrstatus_baroffset_cs),  
        .virtio_isrstatus_length_cs_o          (virtio_isrstatus_length_cs),  
        .virtio_devspecific_header_cs_o        (virtio_devspecific_header_cs), 
        .virtio_devspecific_bar_cs_o           (virtio_devspecific_bar_cs),
        .virtio_devspecific_baroffset_cs_o     (virtio_devspecific_baroffset_cs),
        .virtio_devspecific_length_cs_o        (virtio_devspecific_length_cs),
        .virtio_pcicfgaccess_header_cs_o       (virtio_pcicfgaccess_header_cs),
        .virtio_pcicfgaccess_bar_cs_o          (virtio_pcicfgaccess_bar_cs),
        .virtio_pcicfgaccess_baroffset_cs_o    (virtio_pcicfgaccess_baroffset_cs),
        .virtio_pcicfgaccess_length_cs_o       (virtio_pcicfgaccess_length_cs),
        .virtio_pcicfgaccess_cfgdata_cs_o      (virtio_pcicfgaccess_cfgdata_cs),
        .targeted_cap_not_present_o            (targeted_cap_not_present_o),
        .noreg_selected_o                      (noregbank_selected_o   )
    );

    // Instantiate Read Write Decode module responsible for Read/Write control creation   
    intel_pcie_virtio_rdwr_ctrl # 
    (
        .TOTAL_PF_COUNT         (TOTAL_PF_COUNT ),
        .TOTAL_PF_COUNT_WIDTH   (TOTAL_PF_COUNT_WIDTH ),
        .TOTAL_VF_COUNT         (TOTAL_VF_COUNT  ),      
        .TOTAL_VF_COUNT_WIDTH   (TOTAL_VF_COUNT_WIDTH  ),      
        .PF0_VF_COUNT           (PF0_VF_COUNT  ),      
        .PF1_VF_COUNT           (PF1_VF_COUNT  ),      
        .PF2_VF_COUNT           (PF2_VF_COUNT  ),      
        .PF3_VF_COUNT           (PF3_VF_COUNT  ),      
        .PF4_VF_COUNT           (PF4_VF_COUNT  ),      
        .PF5_VF_COUNT           (PF5_VF_COUNT  ),      
        .PF6_VF_COUNT           (PF6_VF_COUNT  ),      
        .PF7_VF_COUNT           (PF7_VF_COUNT  ),        
        .PF0_VIRTIO_CAPABILITY_PRESENT     (PF0_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF1_VIRTIO_CAPABILITY_PRESENT     (PF1_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF2_VIRTIO_CAPABILITY_PRESENT     (PF2_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF3_VIRTIO_CAPABILITY_PRESENT     (PF3_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF4_VIRTIO_CAPABILITY_PRESENT     (PF4_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF5_VIRTIO_CAPABILITY_PRESENT     (PF5_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF6_VIRTIO_CAPABILITY_PRESENT     (PF6_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF7_VIRTIO_CAPABILITY_PRESENT     (PF7_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF0VF_VIRTIO_CAPABILITY_PRESENT   (PF0VF_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF1VF_VIRTIO_CAPABILITY_PRESENT   (PF1VF_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF2VF_VIRTIO_CAPABILITY_PRESENT   (PF2VF_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF3VF_VIRTIO_CAPABILITY_PRESENT   (PF3VF_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF4VF_VIRTIO_CAPABILITY_PRESENT   (PF4VF_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF5VF_VIRTIO_CAPABILITY_PRESENT   (PF5VF_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF6VF_VIRTIO_CAPABILITY_PRESENT   (PF6VF_VIRTIO_CAPABILITY_PRESENT  ),        
        .PF7VF_VIRTIO_CAPABILITY_PRESENT   (PF7VF_VIRTIO_CAPABILITY_PRESENT  )    
    )
    rdwrctrlgen (
        .clk                  (clk),
        .rstn                 (rstn),
        .flr_rcvd_pf_i        (flr_rcvd_pf_i),                                                    
        .flr_rcvd_vf_i        (flr_rcvd_vf_i),
        .flr_rcvd_pf_num_i    (flr_rcvd_pf_num_i),
        .flr_rcvd_vf_num_i    (flr_rcvd_vf_num_i),
        .cii_req_i            (cii_req_i),
        .cii_wr_i             (cii_wr_i),
        .cii_wr_vf_active_i   (cii_wr_vf_active_i),
        .cii_vf_num_i         (cii_vf_num_i),
        .cii_func_num_i       (cii_func_num_i),
        .write_pf0vf_ctrl_o   (write_pf0vf_ctrl),
        .read_pf0vf_ctrl_o    (read_pf0vf_ctrl),
        .write_pf1vf_ctrl_o   (write_pf1vf_ctrl),
        .read_pf1vf_ctrl_o    (read_pf1vf_ctrl),
        .write_pf2vf_ctrl_o   (write_pf2vf_ctrl),
        .read_pf2vf_ctrl_o    (read_pf2vf_ctrl), 
        .write_pf3vf_ctrl_o   (write_pf3vf_ctrl),
        .read_pf3vf_ctrl_o    (read_pf3vf_ctrl),
        .write_pf4vf_ctrl_o   (write_pf4vf_ctrl),
        .read_pf4vf_ctrl_o    (read_pf4vf_ctrl),
        .write_pf5vf_ctrl_o   (write_pf5vf_ctrl),
        .read_pf5vf_ctrl_o    (read_pf5vf_ctrl),
        .write_pf6vf_ctrl_o   (write_pf6vf_ctrl),
        .read_pf6vf_ctrl_o    (read_pf6vf_ctrl),
        .write_pf7vf_ctrl_o   (write_pf7vf_ctrl),
        .read_pf7vf_ctrl_o    (read_pf7vf_ctrl),
        .write_pf_ctrl_o      (write_pf_ctrl),
        .read_pf_ctrl_o       (read_pf_ctrl)
    );
    
    // Individual Register Read Control
    // For header, it doesn't matter whether it's a PF or a VF. All are the same for one cap.
    // For RO bar, baroffset, length etc., all VFs attached to one PF will have the same default value from GUI. So they use one single select ctrl.
    // As for PCI CFG Access, each single VF might be written with diff RW bar, baroffset, length and cfgdata. Because of diff structure access purpose for each single VF.

    // RO wr select
    assign write_roreg_cs   = virtio_cmncfg_header_cs | virtio_cmncfg_bar_cs | virtio_cmncfg_baroffset_cs | virtio_cmncfg_length_cs
                              | virtio_notifications_header_cs | virtio_notifications_bar_cs | virtio_notifications_baroffset_cs 
                              | virtio_notifications_length_cs | virtio_notifications_notifyoff_cs
                              | virtio_isrstatus_header_cs | virtio_isrstatus_bar_cs | virtio_isrstatus_baroffset_cs | virtio_isrstatus_length_cs
                              | virtio_devspecific_header_cs | virtio_devspecific_bar_cs | virtio_devspecific_baroffset_cs | virtio_devspecific_length_cs
                              | virtio_pcicfgaccess_header_cs;
    assign write_pf_pfvf_ctrl   = (|write_pf_ctrl) | (write_pf7vf_ctrl) | (write_pf6vf_ctrl) | (write_pf5vf_ctrl) | (write_pf4vf_ctrl)
                                  | (write_pf3vf_ctrl) | (write_pf2vf_ctrl) | (write_pf1vf_ctrl) | (write_pf0vf_ctrl);

    // PCI CFG Access Capabilities
    // RO    

    assign read_pf0vf_pcicfgaccess_bar_sel        = (PF0_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_bar_cs==1'b1)? read_pf0vf_ctrl : 'd0); 
    assign write_pf0vf_pcicfgaccess_bar_sel       = (PF0_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_bar_cs==1'b1)? write_pf0vf_ctrl : 'd0); 
    assign read_pf1vf_pcicfgaccess_bar_sel        = (PF1_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_bar_cs==1'b1)? read_pf1vf_ctrl : 'd0); 
    assign write_pf1vf_pcicfgaccess_bar_sel       = (PF1_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_bar_cs==1'b1)? write_pf1vf_ctrl : 'd0); 
    assign read_pf2vf_pcicfgaccess_bar_sel        = (PF2_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_bar_cs==1'b1)? read_pf2vf_ctrl : 'd0); 
    assign write_pf2vf_pcicfgaccess_bar_sel       = (PF2_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_bar_cs==1'b1)? write_pf2vf_ctrl : 'd0); 
    assign read_pf3vf_pcicfgaccess_bar_sel        = (PF3_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_bar_cs==1'b1)? read_pf3vf_ctrl : 'd0); 
    assign write_pf3vf_pcicfgaccess_bar_sel       = (PF3_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_bar_cs==1'b1)? write_pf3vf_ctrl : 'd0); 
    assign read_pf4vf_pcicfgaccess_bar_sel        = (PF4_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_bar_cs==1'b1)? read_pf4vf_ctrl : 'd0); 
    assign write_pf4vf_pcicfgaccess_bar_sel       = (PF4_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_bar_cs==1'b1)? write_pf4vf_ctrl : 'd0); 
    assign read_pf5vf_pcicfgaccess_bar_sel        = (PF5_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_bar_cs==1'b1)? read_pf5vf_ctrl : 'd0); 
    assign write_pf5vf_pcicfgaccess_bar_sel       = (PF5_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_bar_cs==1'b1)? write_pf5vf_ctrl : 'd0); 
    assign read_pf6vf_pcicfgaccess_bar_sel        = (PF6_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_bar_cs==1'b1)? read_pf6vf_ctrl : 'd0); 
    assign write_pf6vf_pcicfgaccess_bar_sel       = (PF6_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_bar_cs==1'b1)? write_pf6vf_ctrl : 'd0); 
    assign read_pf7vf_pcicfgaccess_bar_sel        = (PF7_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_bar_cs==1'b1)? read_pf7vf_ctrl : 'd0); 
    assign write_pf7vf_pcicfgaccess_bar_sel       = (PF7_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_bar_cs==1'b1)? write_pf7vf_ctrl : 'd0); 
    
    assign read_pf0vf_pcicfgaccess_baroffset_sel        = (PF0_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_baroffset_cs==1'b1)? read_pf0vf_ctrl : 'd0); 
    assign write_pf0vf_pcicfgaccess_baroffset_sel       = (PF0_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_baroffset_cs==1'b1)? write_pf0vf_ctrl : 'd0); 
    assign read_pf1vf_pcicfgaccess_baroffset_sel        = (PF1_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_baroffset_cs==1'b1)? read_pf1vf_ctrl : 'd0); 
    assign write_pf1vf_pcicfgaccess_baroffset_sel       = (PF1_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_baroffset_cs==1'b1)? write_pf1vf_ctrl : 'd0); 
    assign read_pf2vf_pcicfgaccess_baroffset_sel        = (PF2_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_baroffset_cs==1'b1)? read_pf2vf_ctrl : 'd0); 
    assign write_pf2vf_pcicfgaccess_baroffset_sel       = (PF2_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_baroffset_cs==1'b1)? write_pf2vf_ctrl : 'd0); 
    assign read_pf3vf_pcicfgaccess_baroffset_sel        = (PF3_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_baroffset_cs==1'b1)? read_pf3vf_ctrl : 'd0); 
    assign write_pf3vf_pcicfgaccess_baroffset_sel       = (PF3_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_baroffset_cs==1'b1)? write_pf3vf_ctrl : 'd0); 
    assign read_pf4vf_pcicfgaccess_baroffset_sel        = (PF4_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_baroffset_cs==1'b1)? read_pf4vf_ctrl : 'd0); 
    assign write_pf4vf_pcicfgaccess_baroffset_sel       = (PF4_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_baroffset_cs==1'b1)? write_pf4vf_ctrl : 'd0); 
    assign read_pf5vf_pcicfgaccess_baroffset_sel        = (PF5_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_baroffset_cs==1'b1)? read_pf5vf_ctrl : 'd0); 
    assign write_pf5vf_pcicfgaccess_baroffset_sel       = (PF5_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_baroffset_cs==1'b1)? write_pf5vf_ctrl : 'd0); 
    assign read_pf6vf_pcicfgaccess_baroffset_sel        = (PF6_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_baroffset_cs==1'b1)? read_pf6vf_ctrl : 'd0); 
    assign write_pf6vf_pcicfgaccess_baroffset_sel       = (PF6_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_baroffset_cs==1'b1)? write_pf6vf_ctrl : 'd0); 
    assign read_pf7vf_pcicfgaccess_baroffset_sel        = (PF7_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_baroffset_cs==1'b1)? read_pf7vf_ctrl : 'd0); 
    assign write_pf7vf_pcicfgaccess_baroffset_sel       = (PF7_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_baroffset_cs==1'b1)? write_pf7vf_ctrl : 'd0); 

    assign read_pf0vf_pcicfgaccess_length_sel        = (PF0_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_length_cs==1'b1)? read_pf0vf_ctrl : 'd0); 
    assign write_pf0vf_pcicfgaccess_length_sel       = (PF0_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_length_cs==1'b1)? write_pf0vf_ctrl : 'd0); 
    assign read_pf1vf_pcicfgaccess_length_sel        = (PF1_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_length_cs==1'b1)? read_pf1vf_ctrl : 'd0); 
    assign write_pf1vf_pcicfgaccess_length_sel       = (PF1_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_length_cs==1'b1)? write_pf1vf_ctrl : 'd0); 
    assign read_pf2vf_pcicfgaccess_length_sel        = (PF2_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_length_cs==1'b1)? read_pf2vf_ctrl : 'd0); 
    assign write_pf2vf_pcicfgaccess_length_sel       = (PF2_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_length_cs==1'b1)? write_pf2vf_ctrl : 'd0); 
    assign read_pf3vf_pcicfgaccess_length_sel        = (PF3_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_length_cs==1'b1)? read_pf3vf_ctrl : 'd0); 
    assign write_pf3vf_pcicfgaccess_length_sel       = (PF3_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_length_cs==1'b1)? write_pf3vf_ctrl : 'd0); 
    assign read_pf4vf_pcicfgaccess_length_sel        = (PF4_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_length_cs==1'b1)? read_pf4vf_ctrl : 'd0); 
    assign write_pf4vf_pcicfgaccess_length_sel       = (PF4_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_length_cs==1'b1)? write_pf4vf_ctrl : 'd0); 
    assign read_pf5vf_pcicfgaccess_length_sel        = (PF5_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_length_cs==1'b1)? read_pf5vf_ctrl : 'd0); 
    assign write_pf5vf_pcicfgaccess_length_sel       = (PF5_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_length_cs==1'b1)? write_pf5vf_ctrl : 'd0); 
    assign read_pf6vf_pcicfgaccess_length_sel        = (PF6_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_length_cs==1'b1)? read_pf6vf_ctrl : 'd0); 
    assign write_pf6vf_pcicfgaccess_length_sel       = (PF6_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_length_cs==1'b1)? write_pf6vf_ctrl : 'd0); 
    assign read_pf7vf_pcicfgaccess_length_sel        = (PF7_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_length_cs==1'b1)? read_pf7vf_ctrl : 'd0); 
    assign write_pf7vf_pcicfgaccess_length_sel       = (PF7_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_length_cs==1'b1)? write_pf7vf_ctrl : 'd0); 

    
    assign read_pf0vf_pcicfgaccess_cfgdata_sel        = (PF0_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_cfgdata_cs==1'b1)? read_pf0vf_ctrl : 'd0); 
    assign write_pf0vf_pcicfgaccess_cfgdata_sel       = (PF0_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_cfgdata_cs==1'b1)? write_pf0vf_ctrl : 'd0); 
    assign read_pf1vf_pcicfgaccess_cfgdata_sel        = (PF1_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_cfgdata_cs==1'b1)? read_pf1vf_ctrl : 'd0); 
    assign write_pf1vf_pcicfgaccess_cfgdata_sel       = (PF1_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_cfgdata_cs==1'b1)? write_pf1vf_ctrl : 'd0); 
    assign read_pf2vf_pcicfgaccess_cfgdata_sel        = (PF2_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_cfgdata_cs==1'b1)? read_pf2vf_ctrl : 'd0); 
    assign write_pf2vf_pcicfgaccess_cfgdata_sel       = (PF2_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_cfgdata_cs==1'b1)? write_pf2vf_ctrl : 'd0); 
    assign read_pf3vf_pcicfgaccess_cfgdata_sel        = (PF3_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_cfgdata_cs==1'b1)? read_pf3vf_ctrl : 'd0); 
    assign write_pf3vf_pcicfgaccess_cfgdata_sel       = (PF3_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_cfgdata_cs==1'b1)? write_pf3vf_ctrl : 'd0); 
    assign read_pf4vf_pcicfgaccess_cfgdata_sel        = (PF4_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_cfgdata_cs==1'b1)? read_pf4vf_ctrl : 'd0); 
    assign write_pf4vf_pcicfgaccess_cfgdata_sel       = (PF4_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_cfgdata_cs==1'b1)? write_pf4vf_ctrl : 'd0); 
    assign read_pf5vf_pcicfgaccess_cfgdata_sel        = (PF5_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_cfgdata_cs==1'b1)? read_pf5vf_ctrl : 'd0); 
    assign write_pf5vf_pcicfgaccess_cfgdata_sel       = (PF5_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_cfgdata_cs==1'b1)? write_pf5vf_ctrl : 'd0); 
    assign read_pf6vf_pcicfgaccess_cfgdata_sel        = (PF6_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_cfgdata_cs==1'b1)? read_pf6vf_ctrl : 'd0); 
    assign write_pf6vf_pcicfgaccess_cfgdata_sel       = (PF6_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_cfgdata_cs==1'b1)? write_pf6vf_ctrl : 'd0); 
    assign read_pf7vf_pcicfgaccess_cfgdata_sel        = (PF7_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_cfgdata_cs==1'b1)? read_pf7vf_ctrl : 'd0); 
    assign write_pf7vf_pcicfgaccess_cfgdata_sel       = (PF7_VF_COUNT==0)? 'd0 : ((virtio_pcicfgaccess_cfgdata_cs==1'b1)? write_pf7vf_ctrl : 'd0); 
 
    always @ (posedge clk) begin
      read_cmncfg_header_sel_o               <= ((|read_pf_ctrl) |                  // Single PF hit
                                                    ((read_pf0vf_ctrl | read_pf1vf_ctrl | read_pf2vf_ctrl | read_pf3vf_ctrl | 
                                                    read_pf4vf_ctrl | read_pf5vf_ctrl | read_pf6vf_ctrl | read_pf7vf_ctrl))) &   // Single VF hit
                                                    virtio_cmncfg_header_cs;
                                         
      read_pf_cmncfg_bar_sel_o               <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_cmncfg_bar_cs}};   // Single PF hit
      read_vf_cmncfg_bar_sel_reg             <= {(read_pf7vf_ctrl) , (read_pf6vf_ctrl) , (read_pf5vf_ctrl) , (read_pf4vf_ctrl) ,
                                                    (read_pf3vf_ctrl) , (read_pf2vf_ctrl) , (read_pf1vf_ctrl) , (read_pf0vf_ctrl)} & 
                                                    {8{virtio_cmncfg_bar_cs}};          // Single VF hit

      read_pf_cmncfg_baroffset_sel_o         <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_cmncfg_baroffset_cs}};
      read_vf_cmncfg_baroffset_sel_reg       <= {(read_pf7vf_ctrl) , (read_pf6vf_ctrl) , (read_pf5vf_ctrl) , (read_pf4vf_ctrl) ,
                                                    (read_pf3vf_ctrl) , (read_pf2vf_ctrl) , (read_pf1vf_ctrl) , (read_pf0vf_ctrl)} & 
                                                    {8{virtio_cmncfg_baroffset_cs}};

      read_pf_cmncfg_length_sel_o            <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_cmncfg_length_cs}};
      read_vf_cmncfg_length_sel_reg          <= {(read_pf7vf_ctrl) , (read_pf6vf_ctrl) , (read_pf5vf_ctrl) , (read_pf4vf_ctrl) ,
                                                    (read_pf3vf_ctrl) , (read_pf2vf_ctrl) , (read_pf1vf_ctrl) , (read_pf0vf_ctrl)} &
                                                    {8{virtio_cmncfg_length_cs}};

      read_notifications_header_sel_o        <= ((|read_pf_ctrl) |                  // Single PF hit
                                                    (|(read_pf0vf_ctrl | read_pf1vf_ctrl | read_pf2vf_ctrl | read_pf3vf_ctrl | 
                                                    read_pf4vf_ctrl | read_pf5vf_ctrl | read_pf6vf_ctrl | read_pf7vf_ctrl))) &   // Single VF hit
                                                    virtio_notifications_header_cs;
                                                    
      read_pf_notifications_bar_sel_o        <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_notifications_bar_cs}};
      read_vf_notifications_bar_sel_reg      <= {(read_pf7vf_ctrl) , (read_pf6vf_ctrl) , (read_pf5vf_ctrl) , (read_pf4vf_ctrl) ,
                                                    (read_pf3vf_ctrl) , (read_pf2vf_ctrl) , (read_pf1vf_ctrl) , (read_pf0vf_ctrl)} &
                                                    {8{virtio_notifications_bar_cs}};

      read_pf_notifications_baroffset_sel_o  <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_notifications_baroffset_cs}};
      read_vf_notifications_baroffset_sel_reg<= {(read_pf7vf_ctrl) , (read_pf6vf_ctrl) , (read_pf5vf_ctrl) , (read_pf4vf_ctrl) ,
                                                    (read_pf3vf_ctrl) , (read_pf2vf_ctrl) , (read_pf1vf_ctrl) , (read_pf0vf_ctrl)} & 
                                                    {8{virtio_notifications_baroffset_cs}};

      read_pf_notifications_length_sel_o     <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_notifications_length_cs}};
      read_vf_notifications_length_sel_reg   <= {(read_pf7vf_ctrl) , (read_pf6vf_ctrl) , (read_pf5vf_ctrl) , (read_pf4vf_ctrl) ,
                                                    (read_pf3vf_ctrl) , (read_pf2vf_ctrl) , (read_pf1vf_ctrl) , (read_pf0vf_ctrl)} &
                                                    {8{virtio_notifications_length_cs}};

      read_pf_notifications_notifyoff_sel_o  <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_notifications_notifyoff_cs}};
      read_vf_notifications_notifyoff_sel_reg<= {(read_pf7vf_ctrl) , (read_pf6vf_ctrl) , (read_pf5vf_ctrl) , (read_pf4vf_ctrl) ,
                                                    (read_pf3vf_ctrl) , (read_pf2vf_ctrl) , (read_pf1vf_ctrl) , (read_pf0vf_ctrl)} &
                                                    {8{virtio_notifications_notifyoff_cs}};
                                         
      read_pf_isrstatus_header_sel_o         <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_isrstatus_header_cs}};
      read_vf_isrstatus_header_sel_reg       <= {(read_pf7vf_ctrl) , (read_pf6vf_ctrl) , (read_pf5vf_ctrl) , (read_pf4vf_ctrl) ,
                                                    (read_pf3vf_ctrl) , (read_pf2vf_ctrl) , (read_pf1vf_ctrl) , (read_pf0vf_ctrl)} & 
                                                    {8{virtio_isrstatus_header_cs}}; 

      read_pf_isrstatus_bar_sel_o            <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_isrstatus_bar_cs}};
      read_vf_isrstatus_bar_sel_reg          <= {(read_pf7vf_ctrl) , (read_pf6vf_ctrl) , (read_pf5vf_ctrl) , (read_pf4vf_ctrl) ,
                                                    (read_pf3vf_ctrl) , (read_pf2vf_ctrl) , (read_pf1vf_ctrl) , (read_pf0vf_ctrl)} & 
                                                    {8{virtio_isrstatus_bar_cs}}; 

      read_pf_isrstatus_baroffset_sel_o      <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_isrstatus_baroffset_cs}};
      read_vf_isrstatus_baroffset_sel_reg    <= {(read_pf7vf_ctrl) , (read_pf6vf_ctrl) , (read_pf5vf_ctrl) , (read_pf4vf_ctrl) ,
                                                    (read_pf3vf_ctrl) , (read_pf2vf_ctrl) , (read_pf1vf_ctrl) , (read_pf0vf_ctrl)} & 
                                                    {8{virtio_isrstatus_baroffset_cs}};

      read_pf_isrstatus_length_sel_o         <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_isrstatus_length_cs}};
      read_vf_isrstatus_length_sel_reg       <= {(read_pf7vf_ctrl) , (read_pf6vf_ctrl) , (read_pf5vf_ctrl) , (read_pf4vf_ctrl) ,
                                                    (read_pf3vf_ctrl) , (read_pf2vf_ctrl) , (read_pf1vf_ctrl) , (read_pf0vf_ctrl)} &
                                                    {8{virtio_isrstatus_length_cs}};

      read_devspecific_header_sel_o          <= ((|read_pf_ctrl) |                  // Single PF hit
                                                    ((read_pf0vf_ctrl | read_pf1vf_ctrl | read_pf2vf_ctrl | read_pf3vf_ctrl | 
                                                    read_pf4vf_ctrl | read_pf5vf_ctrl | read_pf6vf_ctrl | read_pf7vf_ctrl))) &   // Single VF hit
                                                    virtio_devspecific_header_cs;
                                         
      read_pf_devspecific_bar_sel_o          <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_devspecific_bar_cs}};
      read_vf_devspecific_bar_sel_reg        <= {(read_pf7vf_ctrl) , (read_pf6vf_ctrl) , (read_pf5vf_ctrl) , (read_pf4vf_ctrl) ,
                                                    (read_pf3vf_ctrl) , (read_pf2vf_ctrl) , (read_pf1vf_ctrl) , (read_pf0vf_ctrl)} & 
                                                    {8{virtio_devspecific_bar_cs}}; 

      read_pf_devspecific_baroffset_sel_o    <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_devspecific_baroffset_cs}};
      read_vf_devspecific_baroffset_sel_reg  <= {(read_pf7vf_ctrl) , (read_pf6vf_ctrl) , (read_pf5vf_ctrl) , (read_pf4vf_ctrl) ,
                                                    (read_pf3vf_ctrl) , (read_pf2vf_ctrl) , (read_pf1vf_ctrl) , (read_pf0vf_ctrl)} & 
                                                    {8{virtio_devspecific_baroffset_cs}};

      read_pf_devspecific_length_sel_o       <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_devspecific_length_cs}};
      read_vf_devspecific_length_sel_reg     <= {(read_pf7vf_ctrl) , (read_pf6vf_ctrl) , (read_pf5vf_ctrl) , (read_pf4vf_ctrl) ,
                                                    (read_pf3vf_ctrl) , (read_pf2vf_ctrl) , (read_pf1vf_ctrl) , (read_pf0vf_ctrl)} &
                                                    {8{virtio_devspecific_length_cs}};
      
      write_roreg_sel_o                      <= write_pf_pfvf_ctrl & write_roreg_cs;
      read_pcicfgaccess_header_sel_o         <= ((|read_pf_ctrl) |                  // Single PF hit
                                                    (|(read_pf0vf_ctrl | read_pf1vf_ctrl | read_pf2vf_ctrl | read_pf3vf_ctrl | 
                                                    read_pf4vf_ctrl | read_pf5vf_ctrl | read_pf6vf_ctrl | read_pf7vf_ctrl))) &   // Single VF hit
                                                    virtio_pcicfgaccess_header_cs;
    // RW                                    
      read_pf_pcicfgaccess_bar_sel_o         <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_pcicfgaccess_bar_cs}};
      write_pf_pcicfgaccess_bar_sel_o        <= write_pf_ctrl & {TOTAL_PF_COUNT{virtio_pcicfgaccess_bar_cs}};

      read_vf_pcicfgaccess_bar_sel_reg       <= {read_pf7vf_pcicfgaccess_bar_sel,read_pf6vf_pcicfgaccess_bar_sel,
                                                read_pf5vf_pcicfgaccess_bar_sel,read_pf4vf_pcicfgaccess_bar_sel,
                                                read_pf3vf_pcicfgaccess_bar_sel,read_pf2vf_pcicfgaccess_bar_sel,
                                                read_pf1vf_pcicfgaccess_bar_sel,read_pf0vf_pcicfgaccess_bar_sel};
												
      write_vf_pcicfgaccess_bar_sel_reg      <= {write_pf7vf_pcicfgaccess_bar_sel,write_pf6vf_pcicfgaccess_bar_sel,
                                                write_pf5vf_pcicfgaccess_bar_sel,write_pf4vf_pcicfgaccess_bar_sel,
                                                write_pf3vf_pcicfgaccess_bar_sel,write_pf2vf_pcicfgaccess_bar_sel,
                                                write_pf1vf_pcicfgaccess_bar_sel,write_pf0vf_pcicfgaccess_bar_sel};
  
      read_pf_pcicfgaccess_baroffset_sel_o   <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_pcicfgaccess_baroffset_cs}};
      write_pf_pcicfgaccess_baroffset_sel_o  <= write_pf_ctrl & {TOTAL_PF_COUNT{virtio_pcicfgaccess_baroffset_cs}};
      read_vf_pcicfgaccess_baroffset_sel_reg <= {read_pf7vf_pcicfgaccess_baroffset_sel,read_pf6vf_pcicfgaccess_baroffset_sel,
                                                    read_pf5vf_pcicfgaccess_baroffset_sel,read_pf4vf_pcicfgaccess_baroffset_sel,
                                                    read_pf3vf_pcicfgaccess_baroffset_sel,read_pf2vf_pcicfgaccess_baroffset_sel,
                                                    read_pf1vf_pcicfgaccess_baroffset_sel,read_pf0vf_pcicfgaccess_baroffset_sel};
      write_vf_pcicfgaccess_baroffset_sel_reg<= {write_pf7vf_pcicfgaccess_baroffset_sel,write_pf6vf_pcicfgaccess_baroffset_sel,
                                                    write_pf5vf_pcicfgaccess_baroffset_sel,write_pf4vf_pcicfgaccess_baroffset_sel,
                                                    write_pf3vf_pcicfgaccess_baroffset_sel,write_pf2vf_pcicfgaccess_baroffset_sel,
                                                    write_pf1vf_pcicfgaccess_baroffset_sel,write_pf0vf_pcicfgaccess_baroffset_sel};    
    
      read_pf_pcicfgaccess_length_sel_o      <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_pcicfgaccess_length_cs}};
      write_pf_pcicfgaccess_length_sel_o     <= write_pf_ctrl & {TOTAL_PF_COUNT{virtio_pcicfgaccess_length_cs}};
    
      read_vf_pcicfgaccess_length_sel_reg    <= {read_pf7vf_pcicfgaccess_length_sel,read_pf6vf_pcicfgaccess_length_sel,
                                                 read_pf5vf_pcicfgaccess_length_sel,read_pf4vf_pcicfgaccess_length_sel,
                                                 read_pf3vf_pcicfgaccess_length_sel,read_pf2vf_pcicfgaccess_length_sel,
                                                 read_pf1vf_pcicfgaccess_length_sel,read_pf0vf_pcicfgaccess_length_sel};

      write_vf_pcicfgaccess_length_sel_reg   <= {write_pf7vf_pcicfgaccess_length_sel,write_pf6vf_pcicfgaccess_length_sel,
                                                 write_pf5vf_pcicfgaccess_length_sel,write_pf4vf_pcicfgaccess_length_sel,
                                                 write_pf3vf_pcicfgaccess_length_sel,write_pf2vf_pcicfgaccess_length_sel,
                                                 write_pf1vf_pcicfgaccess_length_sel,write_pf0vf_pcicfgaccess_length_sel};    
    
      read_pf_pcicfgaccess_cfgdata_sel_o      <= read_pf_ctrl & {TOTAL_PF_COUNT{virtio_pcicfgaccess_cfgdata_cs}};
      write_pf_pcicfgaccess_cfgdata_sel_o     <= write_pf_ctrl & {TOTAL_PF_COUNT{virtio_pcicfgaccess_cfgdata_cs}};

    
      read_vf_pcicfgaccess_cfgdata_sel_reg    <= {read_pf7vf_pcicfgaccess_cfgdata_sel,read_pf6vf_pcicfgaccess_cfgdata_sel,
                                                 read_pf5vf_pcicfgaccess_cfgdata_sel,read_pf4vf_pcicfgaccess_cfgdata_sel,
                                                 read_pf3vf_pcicfgaccess_cfgdata_sel,read_pf2vf_pcicfgaccess_cfgdata_sel,
                                                 read_pf1vf_pcicfgaccess_cfgdata_sel,read_pf0vf_pcicfgaccess_cfgdata_sel};

      write_vf_pcicfgaccess_cfgdata_sel_reg   <= {write_pf7vf_pcicfgaccess_cfgdata_sel,write_pf6vf_pcicfgaccess_cfgdata_sel,
                                                 write_pf5vf_pcicfgaccess_cfgdata_sel,write_pf4vf_pcicfgaccess_cfgdata_sel,
                                                 write_pf3vf_pcicfgaccess_cfgdata_sel,write_pf2vf_pcicfgaccess_cfgdata_sel,
                                                 write_pf1vf_pcicfgaccess_cfgdata_sel,write_pf0vf_pcicfgaccess_cfgdata_sel};
    end

    assign read_vf_cmncfg_bar_sel_o              = read_vf_cmncfg_bar_sel_reg[TOTAL_PF_COUNT-1:0];
    assign read_vf_cmncfg_baroffset_sel_o        = read_vf_cmncfg_baroffset_sel_reg[TOTAL_PF_COUNT-1:0];
    assign read_vf_cmncfg_length_sel_o           = read_vf_cmncfg_length_sel_reg[TOTAL_PF_COUNT-1:0];
    assign read_vf_notifications_bar_sel_o       = read_vf_notifications_bar_sel_reg[TOTAL_PF_COUNT-1:0];
    assign read_vf_notifications_baroffset_sel_o = read_vf_notifications_baroffset_sel_reg[TOTAL_PF_COUNT-1:0];
    assign read_vf_notifications_length_sel_o    = read_vf_notifications_length_sel_reg[TOTAL_PF_COUNT-1:0];
    assign read_vf_notifications_notifyoff_sel_o = read_vf_notifications_notifyoff_sel_reg[TOTAL_PF_COUNT-1:0];
    assign read_vf_isrstatus_header_sel_o        = read_vf_isrstatus_header_sel_reg[TOTAL_PF_COUNT-1:0];
    assign read_vf_isrstatus_bar_sel_o           = read_vf_isrstatus_bar_sel_reg[TOTAL_PF_COUNT-1:0];
    assign read_vf_isrstatus_baroffset_sel_o     = read_vf_isrstatus_baroffset_sel_reg[TOTAL_PF_COUNT-1:0];
    assign read_vf_isrstatus_length_sel_o        = read_vf_isrstatus_length_sel_reg[TOTAL_PF_COUNT-1:0];
    assign read_vf_devspecific_bar_sel_o         = read_vf_devspecific_bar_sel_reg[TOTAL_PF_COUNT-1:0];
    assign read_vf_devspecific_baroffset_sel_o   = read_vf_devspecific_baroffset_sel_reg[TOTAL_PF_COUNT-1:0];
    assign read_vf_devspecific_length_sel_o      = read_vf_devspecific_length_sel_reg[TOTAL_PF_COUNT-1:0];
    assign read_vf_pcicfgaccess_bar_sel_o        = read_vf_pcicfgaccess_bar_sel_reg[TOTAL_PF_COUNT-1:0];
    assign write_vf_pcicfgaccess_bar_sel_o       = write_vf_pcicfgaccess_bar_sel_reg[TOTAL_PF_COUNT-1:0];
    assign read_vf_pcicfgaccess_baroffset_sel_o  = read_vf_pcicfgaccess_baroffset_sel_reg[TOTAL_PF_COUNT-1:0];
    assign write_vf_pcicfgaccess_baroffset_sel_o = write_vf_pcicfgaccess_baroffset_sel_reg[TOTAL_PF_COUNT-1:0];
    assign read_vf_pcicfgaccess_length_sel_o     = read_vf_pcicfgaccess_length_sel_reg[TOTAL_PF_COUNT-1:0];
    assign write_vf_pcicfgaccess_length_sel_o    = write_vf_pcicfgaccess_length_sel_reg[TOTAL_PF_COUNT-1:0];
    assign read_vf_pcicfgaccess_cfgdata_sel_o    = read_vf_pcicfgaccess_cfgdata_sel_reg[TOTAL_PF_COUNT-1:0];
    assign write_vf_pcicfgaccess_cfgdata_sel_o   = write_vf_pcicfgaccess_cfgdata_sel_reg[TOTAL_PF_COUNT-1:0];

endmodule
