// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

//-----------------------------------------------------------------------------
// Description   : WHR PCI Express CFG Requests rd_ctrl and wr_ctrl decoding 
//                 for PFs or VFs registers.
//-----------------------------------------------------------------------------
// File          : intel_pcie_virtio_rdwr_ctrl.v
// Author        : Intel Corporation
//-----------------------------------------------------------------------------
//

module intel_pcie_virtio_rdwr_ctrl # 
    (
        parameter TOTAL_PF_COUNT                      = 8, // Total number of PFs 
        parameter TOTAL_PF_COUNT_WIDTH                = 3, 
        parameter TOTAL_VF_COUNT                      = 2048, // Total number of VFs
                                                              // 1 - 8 (in steps of 1),
                                                              // 8 - 256 (in multiples of 4)
                                                              // 256 - 1024 (in multiples of 64)
                                                              // 1024 - 4096 (in multiples of 512)
                                                              // Sum of PF0_VF_COUNT, PF1_VF_COUNT, PF2_VF_COUNT and
                                                              // PF3_VF_COUNT must equal TOTAL_VF_COUNT
        parameter TOTAL_VF_COUNT_WIDTH                = 11,   // Number of bits needed to represent TOTAL_VF_COUNT
        parameter PF0_VF_COUNT                        = 1024, // Number of VFs attached to PF 0
        parameter PF1_VF_COUNT                        = 1024, // Number of VFs attached to PF 1
        parameter PF2_VF_COUNT                        = 0,    // Number of VFs attached to PF 2
        parameter PF3_VF_COUNT                        = 0,    // Number of VFs attached to PF 3
        parameter PF4_VF_COUNT                        = 0,    // Number of VFs attached to PF 4
        parameter PF5_VF_COUNT                        = 0,    // Number of VFs attached to PF 5
        parameter PF6_VF_COUNT                        = 0,    // Number of VFs attached to PF 6
        parameter PF7_VF_COUNT                        = 0,    // Number of VFs attached to PF 7
        parameter TOTAL_PF_COUNT_MAX                  = 8,
        // VirtIO presence parameters
        parameter PF0_VIRTIO_CAPABILITY_PRESENT       = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF1_VIRTIO_CAPABILITY_PRESENT       = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF2_VIRTIO_CAPABILITY_PRESENT       = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF3_VIRTIO_CAPABILITY_PRESENT       = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF4_VIRTIO_CAPABILITY_PRESENT       = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF5_VIRTIO_CAPABILITY_PRESENT       = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF6_VIRTIO_CAPABILITY_PRESENT       = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF7_VIRTIO_CAPABILITY_PRESENT       = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF0VF_VIRTIO_CAPABILITY_PRESENT     = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF1VF_VIRTIO_CAPABILITY_PRESENT     = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF2VF_VIRTIO_CAPABILITY_PRESENT     = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF3VF_VIRTIO_CAPABILITY_PRESENT     = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF4VF_VIRTIO_CAPABILITY_PRESENT     = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF5VF_VIRTIO_CAPABILITY_PRESENT     = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF6VF_VIRTIO_CAPABILITY_PRESENT     = 1'b0,            //Indicates VIRTIO Capability is Present             
        parameter PF7VF_VIRTIO_CAPABILITY_PRESENT     = 1'b0             //Indicates VIRTIO Capability is Present             
    )
    (
        input                                   clk,
        input                                   rstn,
        // FLR Interafce
        input  [TOTAL_PF_COUNT-1:0]             flr_rcvd_pf_i,  // 1 bit per PF.  Level signal indicate FLR for corresponding PF is in progress.
        input                                   flr_rcvd_vf_i,  // One-cycle pulse indicates that an FLR was received from host targeting a VF.
        input  [TOTAL_PF_COUNT_WIDTH-1:0]       flr_rcvd_pf_num_i,
        input  [TOTAL_VF_COUNT_WIDTH-1:0]       flr_rcvd_vf_num_i,     // VF offset
        // CII Interface
        input                                   cii_req_i,      // Indicates the CFG request is intercepted and all the other CII signals are valid.                                           
        input                                   cii_wr_i,       // Indicates that the cii_dout_p0/1 is valid. This signal is asserted only for a configuration write request.
        input                                   cii_wr_vf_active_i,    // Indicates that the received TLP was targeted to a VF inside the controller, and is identified by cii_vf_num.    
        input  [TOTAL_VF_COUNT_WIDTH-1:0]       cii_vf_num_i,
        input  [TOTAL_PF_COUNT_WIDTH-1:0]       cii_func_num_i,        // The function number in the received TLP header on CII.
        
        // PFs' VF Read Write Control
        output reg                              write_pf0vf_ctrl_o,  // VF offset
        output reg                              read_pf0vf_ctrl_o,  // VF offset
        output reg                              write_pf1vf_ctrl_o,  // VF offset
        output reg                              read_pf1vf_ctrl_o,  // VF offset
        output reg                              write_pf2vf_ctrl_o,  // VF offset
        output reg                              read_pf2vf_ctrl_o,  // VF offset
        output reg                              write_pf3vf_ctrl_o,  // VF offset
        output reg                              read_pf3vf_ctrl_o,  // VF offset
        output reg                              write_pf4vf_ctrl_o,  // VF offset
        output reg                              read_pf4vf_ctrl_o,  // VF offset
        output reg                              write_pf5vf_ctrl_o,  // VF offset
        output reg                              read_pf5vf_ctrl_o,  // VF offset
        output reg                              write_pf6vf_ctrl_o,  // VF offset
        output reg                              read_pf6vf_ctrl_o,  // VF offset
        output reg                              write_pf7vf_ctrl_o,  // VF offset
        output reg                              read_pf7vf_ctrl_o,  // VF offset
        // PF Read Write Control
        output reg [TOTAL_PF_COUNT-1 : 0]       write_pf_ctrl_o,
        output reg [TOTAL_PF_COUNT-1 : 0]       read_pf_ctrl_o
    );
   
  // Signal Declaration
  //integer i;   
  integer  j;   
  wire    wr_ctrl,rd_ctrl;
  // Virtio Cap Parameters collaboration
  wire   [TOTAL_PF_COUNT_MAX-1:0]      pf_virtio_capability_present;
  wire   [TOTAL_PF_COUNT_MAX-1:0]      vf_virtio_capability_present;
  
  // Parameters
  assign pf_virtio_capability_present = {PF7_VIRTIO_CAPABILITY_PRESENT[0],PF6_VIRTIO_CAPABILITY_PRESENT[0],
                                         PF5_VIRTIO_CAPABILITY_PRESENT[0],PF4_VIRTIO_CAPABILITY_PRESENT[0],
                                         PF3_VIRTIO_CAPABILITY_PRESENT[0],PF2_VIRTIO_CAPABILITY_PRESENT[0],
                                         PF1_VIRTIO_CAPABILITY_PRESENT[0],PF0_VIRTIO_CAPABILITY_PRESENT[0]};
  assign vf_virtio_capability_present = {PF7VF_VIRTIO_CAPABILITY_PRESENT[0],PF6VF_VIRTIO_CAPABILITY_PRESENT[0],
                                         PF5VF_VIRTIO_CAPABILITY_PRESENT[0],PF4VF_VIRTIO_CAPABILITY_PRESENT[0],
                                         PF3VF_VIRTIO_CAPABILITY_PRESENT[0],PF2VF_VIRTIO_CAPABILITY_PRESENT[0],
                                         PF1VF_VIRTIO_CAPABILITY_PRESENT[0],PF0VF_VIRTIO_CAPABILITY_PRESENT[0]};


  assign wr_ctrl  = cii_req_i & cii_wr_i; 
  assign rd_ctrl  = cii_req_i & ~cii_wr_i; 
  
  always @ (posedge clk) begin
    if (~rstn) begin
      // PF Write/Read control
      write_pf_ctrl_o    <= '0;
      read_pf_ctrl_o     <= '0;
      
      // VF Write/Read control
      write_pf0vf_ctrl_o <= 'd0; 
      read_pf0vf_ctrl_o  <= 'd0;
      write_pf1vf_ctrl_o <= 'd0; 
      read_pf1vf_ctrl_o  <= 'd0;
      write_pf2vf_ctrl_o <= 'd0; 
      read_pf2vf_ctrl_o  <= 'd0;
      write_pf3vf_ctrl_o <= 'd0; 
      read_pf3vf_ctrl_o  <= 'd0;
      write_pf4vf_ctrl_o <= 'd0; 
      read_pf4vf_ctrl_o  <= 'd0;
      write_pf5vf_ctrl_o <= 'd0; 
      read_pf5vf_ctrl_o  <= 'd0;
      write_pf6vf_ctrl_o <= 'd0; 
      read_pf6vf_ctrl_o  <= 'd0;
      write_pf7vf_ctrl_o <= 'd0; 
      read_pf7vf_ctrl_o  <= 'd0;
    end
    else begin
    // PF Write/Read control
      //generate
          for (j=0; j<TOTAL_PF_COUNT; j=j+1) begin:pfwrrdctrl
              write_pf_ctrl_o[j] <= (cii_func_num_i == j) & (pf_virtio_capability_present[j] == 1'b1) & wr_ctrl & 
                                    ~flr_rcvd_pf_i[j] & ~cii_wr_vf_active_i;                  
              read_pf_ctrl_o[j]  <= (cii_func_num_i == j) & (pf_virtio_capability_present[j] == 1'b1) & rd_ctrl & 
                                           ~flr_rcvd_pf_i[j] & ~cii_wr_vf_active_i;
          end
      //endgenerate
    
    // VF Write/Read control
      case (cii_func_num_i) 
          'd0:
          begin
              if (PF0_VF_COUNT == 0) begin
                write_pf0vf_ctrl_o <= 0; 
                read_pf0vf_ctrl_o  <= 0;
              end
              else if (vf_virtio_capability_present[0] == 1'b1) begin   // PF0 has VFs and VirtIO enabled
                write_pf0vf_ctrl_o <= wr_ctrl & cii_wr_vf_active_i & ~(flr_rcvd_pf_i[0]) & ~(flr_rcvd_vf_i & (flr_rcvd_pf_num_i == 'd0) & (flr_rcvd_vf_num_i == cii_vf_num_i)); 
                read_pf0vf_ctrl_o  <= rd_ctrl & cii_wr_vf_active_i & ~(flr_rcvd_pf_i[0]) & ~(flr_rcvd_vf_i & (flr_rcvd_pf_num_i == 'd0) & (flr_rcvd_vf_num_i == cii_vf_num_i)); 
              end
          end
          'd1:
          begin
              if (PF1_VF_COUNT == 0) begin
                write_pf1vf_ctrl_o <= 0; 
                read_pf1vf_ctrl_o  <= 0;
              end
              else if (vf_virtio_capability_present[1] == 1'b1) begin   // PF0 has VFs and VirtIO enabled
                write_pf1vf_ctrl_o <= wr_ctrl & cii_wr_vf_active_i & ~(flr_rcvd_pf_i[1]) & ~(flr_rcvd_vf_i & (flr_rcvd_pf_num_i == 'd1) & (flr_rcvd_vf_num_i == cii_vf_num_i)); 
                read_pf1vf_ctrl_o  <= rd_ctrl & cii_wr_vf_active_i & ~(flr_rcvd_pf_i[1]) & ~(flr_rcvd_vf_i & (flr_rcvd_pf_num_i == 'd1) & (flr_rcvd_vf_num_i == cii_vf_num_i)); 
              end
          end
          'd2:
          begin
              if (PF2_VF_COUNT == 0) begin
                write_pf2vf_ctrl_o <= 0; 
                read_pf2vf_ctrl_o  <= 0;
              end
              else if (vf_virtio_capability_present[2] == 1'b1) begin   // PF0 has VFs and VirtIO enabled
                write_pf2vf_ctrl_o <= wr_ctrl & cii_wr_vf_active_i & ~(flr_rcvd_pf_i[2]) & ~(flr_rcvd_vf_i & (flr_rcvd_pf_num_i == 'd2) & (flr_rcvd_vf_num_i == cii_vf_num_i)); 
                read_pf2vf_ctrl_o  <= rd_ctrl & cii_wr_vf_active_i & ~(flr_rcvd_pf_i[2]) & ~(flr_rcvd_vf_i & (flr_rcvd_pf_num_i == 'd2) & (flr_rcvd_vf_num_i == cii_vf_num_i)); 
              end
          end
          'd3:
          begin
              if (PF3_VF_COUNT == 0) begin
                write_pf3vf_ctrl_o <= 0; 
                read_pf3vf_ctrl_o  <= 0;
              end
              else if (vf_virtio_capability_present[3] == 1'b1) begin   // PF0 has VFs and VirtIO enabled
                write_pf3vf_ctrl_o <= wr_ctrl & cii_wr_vf_active_i & ~(flr_rcvd_pf_i[3]) & ~(flr_rcvd_vf_i & (flr_rcvd_pf_num_i == 'd3) & (flr_rcvd_vf_num_i == cii_vf_num_i)); 
                read_pf3vf_ctrl_o  <= rd_ctrl & cii_wr_vf_active_i & ~(flr_rcvd_pf_i[3]) & ~(flr_rcvd_vf_i & (flr_rcvd_pf_num_i == 'd3) & (flr_rcvd_vf_num_i == cii_vf_num_i)); 
              end
          end
          'd4:
          begin
              if (PF4_VF_COUNT == 0) begin
                write_pf4vf_ctrl_o <= 0; 
                read_pf4vf_ctrl_o  <= 0;
              end
              else if (vf_virtio_capability_present[4] == 1'b1) begin   // PF0 has VFs and VirtIO enabled
                write_pf4vf_ctrl_o <= wr_ctrl & cii_wr_vf_active_i & ~(flr_rcvd_pf_i[4]) & ~(flr_rcvd_vf_i & (flr_rcvd_pf_num_i == 'd4) & (flr_rcvd_vf_num_i == cii_vf_num_i)); 
                read_pf4vf_ctrl_o  <= rd_ctrl & cii_wr_vf_active_i & ~(flr_rcvd_pf_i[4]) & ~(flr_rcvd_vf_i & (flr_rcvd_pf_num_i == 'd4) & (flr_rcvd_vf_num_i == cii_vf_num_i)); 
              end
          end
          'd5:
          begin
              if (PF5_VF_COUNT == 0) begin
                write_pf5vf_ctrl_o <= 0; 
                read_pf5vf_ctrl_o  <= 0;
              end
              else if (vf_virtio_capability_present[5] == 1'b1) begin   // PF0 has VFs and VirtIO enabled
                write_pf5vf_ctrl_o <= wr_ctrl & cii_wr_vf_active_i & ~(flr_rcvd_pf_i[5]) & ~(flr_rcvd_vf_i & (flr_rcvd_pf_num_i == 'd5) & (flr_rcvd_vf_num_i == cii_vf_num_i)); 
                read_pf5vf_ctrl_o  <= rd_ctrl & cii_wr_vf_active_i & ~(flr_rcvd_pf_i[5]) & ~(flr_rcvd_vf_i & (flr_rcvd_pf_num_i == 'd5) & (flr_rcvd_vf_num_i == cii_vf_num_i)); 
              end
          end
          'd6:
          begin
              if (PF6_VF_COUNT == 0) begin
                write_pf6vf_ctrl_o <= 0; 
                read_pf6vf_ctrl_o  <= 0;
              end
              else if (vf_virtio_capability_present[6] == 1'b1) begin   // PF0 has VFs and VirtIO enabled
                write_pf6vf_ctrl_o <= wr_ctrl & cii_wr_vf_active_i & ~(flr_rcvd_pf_i[6]) & ~(flr_rcvd_vf_i & (flr_rcvd_pf_num_i == 'd6) & (flr_rcvd_vf_num_i == cii_vf_num_i)); 
                read_pf6vf_ctrl_o  <= rd_ctrl & cii_wr_vf_active_i & ~(flr_rcvd_pf_i[6]) & ~(flr_rcvd_vf_i & (flr_rcvd_pf_num_i == 'd6) & (flr_rcvd_vf_num_i == cii_vf_num_i)); 
              end
          end
          default:
          begin
              if (PF7_VF_COUNT == 0) begin
                write_pf7vf_ctrl_o <= 0; 
                read_pf7vf_ctrl_o  <= 0;
              end
              else if (vf_virtio_capability_present[7] == 1'b1) begin   // PF0 has VFs and VirtIO enabled
                write_pf7vf_ctrl_o <= wr_ctrl & cii_wr_vf_active_i & ~(flr_rcvd_pf_i[7]) & ~(flr_rcvd_vf_i & (flr_rcvd_pf_num_i == 'd7) & (flr_rcvd_vf_num_i == cii_vf_num_i)); 
                read_pf7vf_ctrl_o  <= rd_ctrl & cii_wr_vf_active_i & ~(flr_rcvd_pf_i[7]) & ~(flr_rcvd_vf_i & (flr_rcvd_pf_num_i == 'd7) & (flr_rcvd_vf_num_i == cii_vf_num_i)); 
              end
          end
      endcase
    end
  end
endmodule
