// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`define XGMII_IDLE_D             64'h0707_0707_0707_0707
`define XGMII_IDLE_C             8'b11111111
`define XGMII_START              8'hFB
`define XGMII_TERMINATE          8'hFD
`define FRAME_TYPE_CONSTANT      4'b0001
`define FRAME_TYPE_CONSTANT_CRC  4'b1001
`define FRAME_TYPE_INCREMENT     4'b0010
`define FRAME_TYPE_RANDOM        4'b0100
`define MAX_FRAME                6
`define RX_RECOVERED_CLK_FREQ    257

`timescale 1 ps / 1 ps

module alt_ehipc3_fm_sl_xgmii_client (
    // i_clock,
    i_xgmii_tx_clk,
    i_xgmii_rx_clk,
    i_reset,
    //i_reset_sm,
    
    i_custom_cadence,
    
    o_xgmii_tx_d,
    o_xgmii_tx_c,
    o_xgmii_tx_am,
    o_xgmii_tx_valid,
    i_xgmii_tx_ready,
    
    i_xgmii_rx_d,
    i_xgmii_rx_c,
    i_xgmii_rx_valid,
    i_xgmii_rx_am_valid,
    
    i_rx_ready,
    
    o_checker_pass,
    
    i_avl_mm_clk,       
    i_avl_mm_read,
    i_avl_mm_write,
    o_avl_mm_waitrequest,
    i_avl_mm_baddress,
    o_avl_mm_readdata,
    i_avl_mm_writedata
     
); // module alt_ehipc3_fm_sl_xgmii_client

    // input  wire        clock;
    
    input  wire        i_xgmii_tx_clk;
    input  wire        i_xgmii_rx_clk;
    input  wire        i_reset;
    input  wire        i_custom_cadence;
    output wire [63:0] o_xgmii_tx_d;
    output wire  [7:0] o_xgmii_tx_c;
    output reg         o_xgmii_tx_am;
    output wire        o_xgmii_tx_valid;
    input  wire        i_xgmii_tx_ready;
    input  wire [63:0] i_xgmii_rx_d;
    input  wire  [7:0] i_xgmii_rx_c;
    input  wire        i_xgmii_rx_valid;
    input  wire        i_xgmii_rx_am_valid;
    input  wire        i_rx_ready;
    
    output reg         o_checker_pass;

    input  wire        i_avl_mm_clk;       
    input  wire        i_avl_mm_read;
    input  wire        i_avl_mm_write;
    output wire        o_avl_mm_waitrequest;
    input  wire [11:0] i_avl_mm_baddress;
    output reg  [31:0] o_avl_mm_readdata;
    input  wire [31:0] i_avl_mm_writedata;
    
    parameter          AM_PERIOD = 1276;
    parameter          AM_WIDTH  = 4;
    parameter          AM_ENABLE = 1;        

    wire [13:0] o_match_count;
    wire [13:0] o_mismatch_count;
    
    wire        i_reset_sm;
    wire        xgmii_tx_clk;
    wire [71:0] xgmii_rx;
    wire [71:0] xgmii_tx;
    reg [71:0] xgmii_tx_dc_gen;
    wire [63:0] xgmii_rx_d_mon;
    wire  [7:0] xgmii_rx_c_mon;
    wire [63:0] xgmii_tx_d_mon;
    wire  [7:0] xgmii_tx_c_mon;

    wire         i_reset_sync_tx_clk;
    wire         i_reset_sync_rx_clk;
    wire         i_reset_sync_avl_mm_clk;
    
    genvar g;
    generate
        for (g=0; g<8; g=g+1) begin : ast_translation
            assign xgmii_rx[(g*9)+:8] = i_xgmii_rx_d[(g*8)+:8];
            assign xgmii_rx[(g*9)+8] = i_xgmii_rx_c[g];
            assign o_xgmii_tx_d[(g*8)+:8] = xgmii_tx_dc_gen[(g*9)+:8];
            assign o_xgmii_tx_c[g] = xgmii_tx_dc_gen[(g*9)+8];
        end
    endgenerate

    reg        rx_sync_lock;
    wire       fifo_full;
    reg        test_done;
    reg        frame_sof_reg;
    reg [13:0] frame_length_reg;
    reg  [3:0] frame_type_reg;
    reg [63:0] frame_constant_reg;
    reg [15:0] frame_ifg_reg;
    // reg        rx_ready;
    reg  [3:0] rx_sync_lock_dly_counter;
    wire       frame_done;
    // wire               reset;
    integer    num_frame = 0;
    wire       idle, idle_one_pulse;
    reg        idle_dly;
    
    // wire tx_eth_ready_gated;
    
    reg [13:0] FRAME_LENGTH_LIST[0:(`MAX_FRAME-1)]= '{50, 100, 500, 1000, 1500, 80};
    reg [3:0]  FRAME_TYPE_LIST  [0:(`MAX_FRAME-1)]= '{`FRAME_TYPE_INCREMENT, `FRAME_TYPE_INCREMENT, `FRAME_TYPE_INCREMENT, `FRAME_TYPE_INCREMENT, `FRAME_TYPE_INCREMENT, `FRAME_TYPE_INCREMENT};    

    // assign tx_eth_ready_gated = o_rx_block_lock;
    // assign xgmii_rx_clk     = i_clk_tx;
    // assign xgmii_tx_clk     = xgmii_rx_clk;
    // assign reset = !tx_eth_ready_gated;   
    assign idle_one_pulse = idle & ~idle_dly;
    
 	alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH  (3),
            .WIDTH              (1),
            .INIT_VALUE         (1)
        ) ehip_reset_sync_tx_clk (
            .clk                (i_xgmii_tx_clk),
            .reset              (i_reset),
            .d                  (1'b0),
            .q                  (i_reset_sync_tx_clk)
          );    

	 alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH  (3),
            .WIDTH              (1),
            .INIT_VALUE         (1)
        ) ehip_reset_sync_rx_clk (
            .clk                (i_xgmii_rx_clk),
            .reset              (i_reset),
            .d                  (1'b0),
            .q                  (i_reset_sync_rx_clk)
          );

 	alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH  (3),
            .WIDTH              (1),
            .INIT_VALUE         (1)
        ) ehip_reset_sync_avl_mm_clk (
            .clk                (i_avl_mm_clk),
            .reset              (i_reset),
            .d                  (1'b0),
            .q                  (i_reset_sync_avl_mm_clk)
          );

    always @ (posedge i_xgmii_rx_clk) begin
        if (i_reset_sm) begin
            idle_dly <= 1'b0; 
            // rx_ready <= 1'b0;
        end  
        else if (rx_sync_lock == 1'd1) begin 
            idle_dly <= idle;
            // rx_ready <= 1'b1;
        end 
        else begin
            idle_dly <= 1'b0;
            // rx_ready <= 1'b1;
        end
    end
    
    always @ (posedge i_xgmii_rx_clk) begin
        if (i_reset_sm) begin
            rx_sync_lock_dly_counter <= 4'b0;
            rx_sync_lock <= 1'b0; 
        end
        else begin
            if (rx_sync_lock_dly_counter == 4'b1010)
                rx_sync_lock <= 1'b1;      
            else if (rx_sync_lock == 1'd0)
                rx_sync_lock_dly_counter <= rx_sync_lock_dly_counter + 1'b1;
        end
    end

    always @ (posedge i_xgmii_rx_clk) begin
        if (i_reset_sm) begin
            test_done <= 0;
            num_frame <= 0;
            frame_sof_reg    <= 1'b0;
            frame_length_reg <= FRAME_LENGTH_LIST[0]; 
            frame_type_reg   <= FRAME_TYPE_LIST[0];
            frame_constant_reg <= 0;
            frame_ifg_reg    <= 0;
        end
        else begin
            if ((rx_sync_lock == 1'd1) && i_rx_ready ) begin
                if (!fifo_full && !test_done) begin
                    frame_sof_reg    <= idle_one_pulse;
                    frame_length_reg <= FRAME_LENGTH_LIST[num_frame]; 
                    frame_type_reg   <= FRAME_TYPE_LIST[num_frame];
                    frame_constant_reg <= 2;
                    frame_ifg_reg    <= 20;
                    if (frame_done)
                        num_frame <= num_frame+1;
                end
            end
                 
            if (num_frame == 4)  begin
                test_done <= 1;
            end
        end
    end
            
    // Workaround //////////////////
    reg buffer_write;
    wire buffer_empty;
    wire tx_mii_valid;
    wire tx_mii_am_valid;
    wire tx_mii_am_valid_int;
    wire rx_mii_am_valid;    
    reg  [19:0] tx_am_period; 
 
    assign rx_mii_am_valid = (AM_ENABLE == 1) ? i_xgmii_rx_am_valid : 1'b0;
    assign tx_mii_am_valid = (AM_ENABLE == 1) ? tx_mii_am_valid_int : 1'b0;
    
    always @(posedge xgmii_tx_clk or posedge i_reset_sync_tx_clk) begin
      if (i_reset_sync_tx_clk)
        o_xgmii_tx_am <= 1'b0;
      else
        o_xgmii_tx_am <= tx_mii_am_valid_int;
    end
    
    // TX AM pulse generation
    generate 
    if ((AM_ENABLE == 1) || (AM_ENABLE == 2)) begin:AM_INSERT
      always @(posedge xgmii_tx_clk or posedge i_reset_sync_tx_clk) begin
      if (i_reset_sync_tx_clk)
        tx_am_period <= 20'h00;
      else if ((tx_mii_valid & (AM_ENABLE == 1)) || (i_custom_cadence & (AM_ENABLE == 2)))
        tx_am_period <= (tx_am_period == (AM_PERIOD+AM_WIDTH-1)) ? 20'h00 : tx_am_period + 1'b1;
      end
      
      if (AM_WIDTH == 4) begin
        assign tx_mii_am_valid_int = ((tx_am_period == AM_PERIOD) | (tx_am_period == (AM_PERIOD + 2'h1)) | (tx_am_period == (AM_PERIOD + 2'h2)) | (tx_am_period == (AM_PERIOD + 2'h3)));
      end
      else begin 
        assign tx_mii_am_valid_int = (tx_am_period == AM_PERIOD);
      end
    end 
    else begin
      assign tx_mii_am_valid_int = 1'b0; 
    end
    endgenerate
    
    // assign tx_mii_valid = rx_parallel_data_1d[76];
    // assign tx_mii_valid = o_sl_tx_mii_ready[0];
    assign tx_mii_valid = i_xgmii_tx_ready;
   
    always @(posedge xgmii_tx_clk or posedge i_reset_sync_tx_clk) begin
      if (i_reset_sync_tx_clk)
        buffer_write <= 1'b0;
      else if (frame_sof_reg)
        buffer_write <= 1'b1;
      else if (frame_done)
        buffer_write <= 1'b0;
    end
    
    wire [71:0] xgmii_tx_dc;
   
    alt_ehipc3_fm_sl_xgmii_checker_fifo xgmii_tx_buffer (
        .aclr(i_reset),
        .data(xgmii_tx),
        .rdclk(i_xgmii_tx_clk),
        .rdreq(tx_mii_valid & ~buffer_empty & ~tx_mii_am_valid),
        .wrclk(i_xgmii_tx_clk),
        .wrreq(buffer_write),
        .q(xgmii_tx_dc),
        .rdempty(buffer_empty),
        .wrfull()
    );
    
    // Improve timing
    reg tx_mii_valid_reg;
    
    always @(posedge xgmii_tx_clk) begin
        tx_mii_valid_reg <= tx_mii_valid;
        if (buffer_empty) begin
            xgmii_tx_dc_gen <= 72'h83_c1e0_f078_3c1e_0f07;
        end else begin
            xgmii_tx_dc_gen <= xgmii_tx_dc;
        end
    end
    
    assign o_xgmii_tx_valid = tx_mii_valid_reg;
    // End workaround ////////////
    
    alt_ehipc3_fm_sl_xgmii_src tx_generator (
        .clock(i_xgmii_tx_clk),
        //.reset(i_reset),
        .i_reset_sync_tx_clk(i_reset_sync_tx_clk),
        .i_reset_sync_rx_clk(i_reset_sync_rx_clk),
        .reset_sm(i_reset_sm),
        .frame_sof(frame_sof_reg),
        .frame_length(frame_length_reg),
        .frame_type(frame_type_reg),
        .frame_ifg(frame_ifg_reg),
        .frame_constant(frame_constant_reg),
        .frame_done(frame_done),
        .xgmii_tx_clk(xgmii_tx_clk),
        .xgmii_tx(xgmii_tx),
        .xgmii_rx_clk(i_xgmii_rx_clk),
        .xgmii_rx(xgmii_rx),
        .xgmii_rx_valid(i_xgmii_rx_valid & ~rx_mii_am_valid),
        .xgmii_rx_d_mon(xgmii_rx_d_mon),
        .xgmii_rx_c_mon(xgmii_rx_c_mon),
        .xgmii_tx_d_mon(xgmii_tx_d_mon),
        .xgmii_tx_c_mon(xgmii_tx_c_mon)
    ); // module xgmii_src

    alt_ehipc3_fm_sl_xgmii_sink rx_checker (
        .clock(i_xgmii_rx_clk),
        //.reset(i_reset),
        .i_reset_sync_tx_clk(i_reset_sync_tx_clk),
        .i_reset_sync_rx_clk(i_reset_sync_rx_clk),
        .xgmii_tx_clk(xgmii_tx_clk),
        .xgmii_tx(xgmii_tx),
        .xgmii_rx_clk(i_xgmii_rx_clk),
        .xgmii_rx(xgmii_rx),
        .test_done(test_done),
        .rx_ready(~i_reset_sm),
        .fifo_full(fifo_full),
        .fifo_empty(fifo_empty),
        .read_stored_data(),
        .checker_pass(o_checker_pass),
        .rx_mismatch(rx_mismatch),
        .idle_state(idle),
        .xgmii_rx_d_mon(xgmii_rx_d_mon),
        .xgmii_rx_c_mon(xgmii_rx_c_mon),
        .xgmii_tx_d_mon(xgmii_tx_d_mon),
        .xgmii_tx_c_mon(xgmii_tx_c_mon),
        .o_match_count(o_match_count),
        .o_mismatch_count(o_mismatch_count),
        .rx_valid (i_xgmii_rx_valid & ~rx_mii_am_valid)       
    ); // module xgmii_sink

  // ________________________________________________
  //  AVMM interface
  reg         avl_mm_read_r;
  reg         avl_mm_write_r;
  reg  [11:0] avl_mm_baddress_r;
  reg  [31:0] avl_mm_writedata_r;

  reg         avl_mm_read_r1;
  reg         avl_mm_write_r1;
  reg  [11:0] avl_mm_baddress_r1;
  reg  [31:0] avl_mm_writedata_r1;

  wire [11:0] avl_mm_address = avl_mm_baddress_r[11:0]; // byte to word address
  //wire        blk_sel_pio = (avl_mm_address[9:8] == 2'd3);

  wire        waitrequest_pio;
  reg  [31:0] avl_mm_readdata_pio;
  
  reg         avl_mm_waitrequest;
  wire        avl_mm_read = avl_mm_read_r;
  wire        avl_mm_write = avl_mm_write_r;
  wire [31:0] avl_mm_writedata = avl_mm_writedata_r;

  reg [31:0]  avl_mm_readdata;

  always @ (posedge i_xgmii_tx_clk or posedge i_reset_sync_tx_clk)
   begin
      if (i_reset_sync_tx_clk) begin
         avl_mm_readdata <= 0;
         avl_mm_waitrequest <= 0;
      end
      else begin
         avl_mm_readdata    <= avl_mm_readdata_pio;
         avl_mm_waitrequest <= waitrequest_pio;
      end
   end

// __________________________________________________________________________________
// AVMM Clock crossing bridge
    reg rddly, wrdly, rddly2, wrdly2;
    reg [8:0] rddly1;
    reg [6:0] wrdly1;   
    wire rdempty;
    wire rd_empty;
    wire [71:0] pkt_wr_fifo_read;  
    wire [71:0] pkt_rd_fifo_read; 
    reg avl_mm_write_int_r;
    reg avl_mm_write_int_r1;
    reg avl_mm_waitrequest_r;
    reg rdreq_wr_fifo;
    reg rdreq_rd_fifo;
    wire wredge = i_avl_mm_write& ~wrdly;
    wire rdedge = i_avl_mm_read & ~rddly;
    wire wredge1 = wrdly & ~wrdly2;
    wire rdedge1 = rddly & ~rddly2;
    
    always@(posedge i_avl_mm_clk or posedge i_reset_sync_avl_mm_clk)
    begin
      if(i_reset_sync_avl_mm_clk) 
      begin 
        avl_mm_write_int_r  <= 1'b0;
        avl_mm_write_int_r1 <= 1'b0;
      end 
      else 
      begin 
        avl_mm_write_int_r  <= (i_avl_mm_write | i_avl_mm_read);
        avl_mm_write_int_r1 <= avl_mm_write_int_r;
      end 
    end
 
    assign o_avl_mm_waitrequest = (wredge|rdedge) || (wredge1|rdedge1) || (|wrdly1) || (|rddly1); // Design done with transaction when this goes down 
    
    always@(posedge i_avl_mm_clk or posedge i_reset_sync_avl_mm_clk)
    begin
      if(i_reset_sync_avl_mm_clk) 
      begin 
        wrdly  <= 1'b0; 
        rddly  <= 1'b0;
        wrdly2 <= 1'b0; 
        rddly2 <= 1'b0; 
      end 
      else 
      begin 
        wrdly  <= i_avl_mm_write; 
        rddly  <= i_avl_mm_read; 
        wrdly2 <= wrdly; 
        rddly2 <= rddly; 
      end 
    end
    
    always@(posedge i_avl_mm_clk or posedge i_reset_sync_avl_mm_clk)
    begin
      if(i_reset_sync_avl_mm_clk) 
      begin 
        wrdly1 <= 7'h0; 
        rddly1 <= 9'h0; 
      end 
      else 
      begin 
        wrdly1 <= {wrdly1[5:0],wredge1}; 
        rddly1 <= {rddly1[7:0],rdedge1}; 
      end 
    end

    always@(posedge i_xgmii_tx_clk or posedge i_reset_sync_tx_clk)
    begin
      if(i_reset_sync_tx_clk) 
      begin 
        avl_mm_read_r <= 1'b0;
        avl_mm_write_r <= 1'b0;
        avl_mm_baddress_r <= 12'h0;
        avl_mm_writedata_r <= 32'h0;
      end 
      else 
      begin 
        avl_mm_read_r <= avl_mm_read_r1;
        avl_mm_write_r <= avl_mm_write_r1;
        avl_mm_baddress_r <= avl_mm_baddress_r1;
        avl_mm_writedata_r <= avl_mm_writedata_r1;
      end 
    end
    
    always@(posedge i_xgmii_tx_clk or posedge i_reset_sync_tx_clk)
    begin
      if(i_reset_sync_tx_clk) 
      begin 
        avl_mm_read_r1 <= 1'b0;
        avl_mm_write_r1 <= 1'b0;
        avl_mm_baddress_r1 <= 12'h0;
        avl_mm_writedata_r1 <= 32'h0;
      end 
      else //if (rdreq_wr_fifo)
      begin 
        avl_mm_read_r1 <= rdreq_wr_fifo ? pkt_wr_fifo_read[45] : avl_mm_waitrequest ? 1'b0 : avl_mm_read_r1;
        avl_mm_write_r1 <= rdreq_wr_fifo ? pkt_wr_fifo_read[44] : 1'b0;
        avl_mm_baddress_r1 <= rdreq_wr_fifo ? pkt_wr_fifo_read[43:32] : avl_mm_baddress_r1;
        avl_mm_writedata_r1 <= rdreq_wr_fifo ? pkt_wr_fifo_read[31:0] : avl_mm_writedata_r1;
      end 
    end
   
    always@(posedge i_xgmii_tx_clk or posedge i_reset_sync_tx_clk)
    begin
      if(i_reset_sync_tx_clk) 
      begin 
        rdreq_wr_fifo <= 1'b0;
      end 
      else 
      begin 
        rdreq_wr_fifo <= (~rdempty & ~rdreq_wr_fifo) ? 1'b1 : 1'b0;
      end 
    end

   
    alt_ehipc3_fm_sl_xgmii_checker_fifo pkt_avmm_fifo_wrdata (
      .aclr    (1'b0),
      .data    ({26'h0,i_avl_mm_read,i_avl_mm_write,i_avl_mm_baddress,i_avl_mm_writedata}),    
      .wrreq   (avl_mm_write_int_r & ~avl_mm_write_int_r1),   
      .rdreq   (rdreq_wr_fifo),  
      .wrclk   (i_avl_mm_clk),  
      .rdclk   (i_xgmii_tx_clk),
      .q       (pkt_wr_fifo_read),  
      .rdempty (rdempty), 
      .wrfull  ()   
    );

    always@(posedge i_avl_mm_clk or posedge i_reset_sync_avl_mm_clk)
    begin
      if(i_reset_sync_avl_mm_clk) 
      begin 
        o_avl_mm_readdata <= 32'h0;
      end 
      else if (rdreq_rd_fifo)
      begin 
        o_avl_mm_readdata <= pkt_rd_fifo_read[31:0];
      end 
    end
    
    always@(posedge i_avl_mm_clk or posedge i_reset_sync_avl_mm_clk)
    begin
      if(i_reset_sync_avl_mm_clk) 
      begin 
        rdreq_rd_fifo <= 1'b0;
      end 
      else 
      begin 
        rdreq_rd_fifo <= (~rd_empty & ~rdreq_rd_fifo) ? 1'b1 : 1'b0;
      end 
    end

    always@(posedge i_xgmii_tx_clk or posedge i_reset_sync_tx_clk)
    begin
      if(i_reset_sync_tx_clk) 
      begin 
        avl_mm_waitrequest_r <= 1'b0;
      end 
      else
      begin 
        avl_mm_waitrequest_r <= avl_mm_waitrequest;
      end 
    end
    
    alt_ehipc3_fm_sl_xgmii_checker_fifo pkt_avmm_fifo_rddata (
      .aclr    (1'b0),
      .data    ({40'h0,avl_mm_readdata}),    
      .wrreq   (avl_mm_read_r & avl_mm_waitrequest_r),   
      .rdreq   (rdreq_rd_fifo),  
      .wrclk   (i_xgmii_tx_clk),  
      .rdclk   (i_avl_mm_clk),
      .q       (pkt_rd_fifo_read),  
      .rdempty (rd_empty), 
      .wrfull  ()   
    );   
// __________________________________________________________________________________

  // ________________________________________________
  //  PIO Control
  reg  [31:0] pio_ctl0;
  reg  [31:0] pio_ctl1;
  wire [31:0] pio_status0 = {30'h0, o_checker_pass, i_reset_sm};
  wire [31:0] pio_status1 = 32'h0;
  wire [31:0] pio_status2 = 32'h0;
  wire [31:0] pio_status3 = 32'h0;
  
  assign i_reset_sm = ~pio_ctl0[0];
// ____________________________________________________________________________
// Control register 0
// ____________________________________________________________________________

always @ (posedge i_xgmii_tx_clk or posedge i_reset_sync_tx_clk)
   begin
      if (i_reset_sync_tx_clk) pio_ctl0 <= 32'h0;
      else if (avl_mm_write & avl_mm_address[3:0] == 4'h0) pio_ctl0 <= avl_mm_writedata;
   end

// ____________________________________________________________________________
// Control register 1
// ____________________________________________________________________________

always @ (posedge i_xgmii_tx_clk or posedge i_reset_sync_tx_clk)
   begin
      if (i_reset_sync_tx_clk) pio_ctl1 <= 32'h0;
      else if (avl_mm_write & avl_mm_address[3:0] == 4'h1) pio_ctl1 <= avl_mm_writedata;
   end
   
// Output MUX of registers into readdata bus
   always@(posedge i_xgmii_tx_clk or posedge i_reset_sync_tx_clk)
   begin
      if(i_reset_sync_tx_clk) avl_mm_readdata_pio <= 32'h0;
      else if (avl_mm_read) begin
         case (avl_mm_address[2:0])
            3'h0  : avl_mm_readdata_pio <= pio_ctl0;
            3'h1  : avl_mm_readdata_pio <= pio_ctl1;
            3'h2  : avl_mm_readdata_pio <= pio_status0;
            3'h3  : avl_mm_readdata_pio <= pio_status1;
            3'h4  : avl_mm_readdata_pio <= pio_status2;
            3'h5  : avl_mm_readdata_pio <= pio_status3;
            3'h6  : avl_mm_readdata_pio <= (AM_ENABLE == 2) ? {18'h0,o_match_count} : 32'h0;
            3'h7  : avl_mm_readdata_pio <= (AM_ENABLE == 2) ? {18'h0,o_mismatch_count} : 32'h0;
            default:avl_mm_readdata_pio <= 32'h0;
         endcase
      end
   end

 // ____________________________________________________________
 //
   reg rddly_pio, wrdly_pio;
   always@(posedge i_xgmii_tx_clk or posedge i_reset_sync_tx_clk)
   begin
      if(i_reset_sync_tx_clk) 
      begin 
        wrdly_pio <= 1'b0; 
        rddly_pio <= 1'b0; 
      end 
      else 
      begin 
        wrdly_pio <= avl_mm_write; 
        rddly_pio <= avl_mm_read; 
      end 
   end
  
   wire wredge_pio = avl_mm_write& ~wrdly_pio;
   wire rdedge_pio = avl_mm_read & ~rddly_pio;

   assign waitrequest_pio = (wredge_pio|rdedge_pio); // your design is done with transaction when this goes down
   

endmodule
