// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`define XGMII_IDLE_D             64'h0707_0707_0707_0707
`define XGMII_IDLE_C             8'b11111111
`define XGMII_START              8'hFB
`define XGMII_TERMINATE          8'hFD
`define FRAME_TYPE_CONSTANT      4'b0001
`define FRAME_TYPE_CONSTANT_CRC  4'b1001
`define FRAME_TYPE_INCREMENT     4'b0010
`define FRAME_TYPE_RANDOM        4'b0100
`define MAX_FRAME                6
`define RX_RECOVERED_CLK_FREQ    257

`timescale 1 ps / 1 ps

module alt_ehipc3_fm_sl_xgmii_top (
    // i_clock,
    i_xgmii_tx_clk,
    i_xgmii_rx_clk,
    i_reset,
    i_reset_sm,
    
    o_xgmii_tx_d,
    o_xgmii_tx_c,
    o_xgmii_tx_valid,
    i_xgmii_tx_ready,
    i_xgmii_rx_d,
    i_xgmii_rx_c,
    i_xgmii_rx_valid,
    
    i_rx_ready,
    
    o_checker_pass
); // module alt_ehipc3_fm_sl_xgmii_top

    // input  wire        clock;
    
    input  wire        i_xgmii_tx_clk;
    input  wire        i_xgmii_rx_clk;
    input  wire        i_reset;
    input  wire        i_reset_sm;
    
    output wire [63:0] o_xgmii_tx_d;
    output wire  [7:0] o_xgmii_tx_c;
    output wire        o_xgmii_tx_valid;
    input  wire        i_xgmii_tx_ready;
    input  wire [63:0] i_xgmii_rx_d;
    input  wire  [7:0] i_xgmii_rx_c;
    input  wire        i_xgmii_rx_valid;
    
    input  wire        i_rx_ready;
    
    output reg         o_checker_pass;

    wire        xgmii_tx_clk;
    wire [71:0] xgmii_rx;
    wire [71:0] xgmii_tx;
    reg [71:0] xgmii_tx_dc_gen;
    wire [63:0] xgmii_rx_d_mon;
    wire  [7:0] xgmii_rx_c_mon;
    wire [63:0] xgmii_tx_d_mon;
    wire  [7:0] xgmii_tx_c_mon;
    
    genvar g;
    generate
        for (g=0; g<8; g=g+1) begin : ast_translation
            assign xgmii_rx[(g*9)+:8] = i_xgmii_rx_d[(g*8)+:8];
            assign xgmii_rx[(g*9)+8] = i_xgmii_rx_c[g];
            assign o_xgmii_tx_d[(g*8)+:8] = xgmii_tx_dc_gen[(g*9)+:8];
            assign o_xgmii_tx_c[g] = xgmii_tx_dc_gen[(g*9)+8];
        end
    endgenerate

    reg        rx_sync_lock;
    wire       fifo_full;
    reg        test_done;
    reg        frame_sof_reg;
    reg [13:0] frame_length_reg;
    reg  [3:0] frame_type_reg;
    reg [63:0] frame_constant_reg;
    reg [15:0] frame_ifg_reg;
    // reg        rx_ready;
    reg  [3:0] rx_sync_lock_dly_counter;
    wire       frame_done;
    // wire               reset;
    integer    num_frame = 0;
    wire       idle, idle_one_pulse;
    reg        idle_dly;
    
    // wire tx_eth_ready_gated;
    
    reg [13:0] FRAME_LENGTH_LIST[0:(`MAX_FRAME-1)]= '{50, 100, 500, 1000, 1500, 80};
    reg [3:0]  FRAME_TYPE_LIST  [0:(`MAX_FRAME-1)]= '{`FRAME_TYPE_INCREMENT, `FRAME_TYPE_INCREMENT, `FRAME_TYPE_INCREMENT, `FRAME_TYPE_INCREMENT, `FRAME_TYPE_INCREMENT, `FRAME_TYPE_INCREMENT};    

    // assign tx_eth_ready_gated = o_rx_block_lock;
    // assign xgmii_rx_clk     = i_clk_tx;
    // assign xgmii_tx_clk     = xgmii_rx_clk;
    // assign reset = !tx_eth_ready_gated;   
    assign idle_one_pulse = idle & ~idle_dly;
    
    always @ (posedge i_xgmii_rx_clk) begin
        if (i_reset_sm) begin
            idle_dly <= 1'b0; 
            // rx_ready <= 1'b0;
        end  
        else if (rx_sync_lock == 1'd1) begin 
            idle_dly <= idle;
            // rx_ready <= 1'b1;
        end 
        else begin
            idle_dly <= 1'b0;
            // rx_ready <= 1'b1;
        end
    end
    
    always @ (posedge i_xgmii_rx_clk) begin
        if (i_reset_sm) begin
            rx_sync_lock_dly_counter <= 4'b0;
            rx_sync_lock <= 1'b0; 
        end
        else begin
            if (rx_sync_lock_dly_counter == 4'b1010)
                rx_sync_lock <= 1'b1;      
            else if (rx_sync_lock == 1'd0)
                rx_sync_lock_dly_counter <= rx_sync_lock_dly_counter + 1'b1;
        end
    end

    always @ (posedge i_xgmii_rx_clk) begin
        if (i_reset_sm) begin
            test_done <= 0;
            num_frame <= 0;
            frame_sof_reg    <= 1'b0;
            frame_length_reg <= FRAME_LENGTH_LIST[0]; 
            frame_type_reg   <= FRAME_TYPE_LIST[0];
            frame_constant_reg <= 0;
            frame_ifg_reg    <= 0;
        end
        else begin
            if ((rx_sync_lock == 1'd1) && i_rx_ready ) begin
                if (!fifo_full && !test_done) begin
                    frame_sof_reg    <= idle_one_pulse;
                    frame_length_reg <= FRAME_LENGTH_LIST[num_frame]; 
                    frame_type_reg   <= FRAME_TYPE_LIST[num_frame];
                    frame_constant_reg <= 2;
                    frame_ifg_reg    <= 20;
                    if (frame_done)
                        num_frame <= num_frame+1;
                end
            end
                 
            if (num_frame == 4)  begin
                test_done <= 1;
            end
        end
    end
            
    // Workaround //////////////////
    reg buffer_write;
    wire buffer_empty;
    wire tx_mii_valid;
    
    // assign tx_mii_valid = rx_parallel_data_1d[76];
    // assign tx_mii_valid = o_sl_tx_mii_ready[0];
    assign tx_mii_valid = i_xgmii_tx_ready;
    
    always @(posedge xgmii_tx_clk or posedge i_reset) begin
      if (i_reset)
        buffer_write <= 1'b0;
      else if (frame_sof_reg)
        buffer_write <= 1'b1;
      else if (frame_done)
        buffer_write <= 1'b0;
    end
    
    wire [71:0] xgmii_tx_dc;
   
    alt_ehipc3_fm_sl_xgmii_checker_fifo xgmii_tx_buffer (
        .aclr(i_reset),
        .data(xgmii_tx),
        .rdclk(i_xgmii_tx_clk),
        .rdreq(tx_mii_valid & ~buffer_empty),
        .wrclk(i_xgmii_tx_clk),
        .wrreq(buffer_write),
        .q(xgmii_tx_dc),
        .rdempty(buffer_empty),
        .wrfull()
    );
    
    // Improve timing
    reg tx_mii_valid_reg;
    
    always @(posedge xgmii_tx_clk) begin
        tx_mii_valid_reg <= tx_mii_valid;
        if (buffer_empty) begin
            xgmii_tx_dc_gen <= 72'h83_c1e0_f078_3c1e_0f07;
        end else begin
            xgmii_tx_dc_gen <= xgmii_tx_dc;
        end
    end
    
    assign o_xgmii_tx_valid = tx_mii_valid_reg;
    // End workaround ////////////
    
    alt_ehipc3_fm_sl_xgmii_src tx_generator (
        .clock(i_xgmii_tx_clk),
        .reset(i_reset),
        .reset_sm(i_reset_sm),
        .frame_sof(frame_sof_reg),
        .frame_length(frame_length_reg),
        .frame_type(frame_type_reg),
        .frame_ifg(frame_ifg_reg),
        .frame_constant(frame_constant_reg),
        .frame_done(frame_done),
        .xgmii_tx_clk(xgmii_tx_clk),
        .xgmii_tx(xgmii_tx),
        .xgmii_rx_clk(i_xgmii_rx_clk),
        .xgmii_rx(xgmii_rx),
        .xgmii_rx_valid(i_xgmii_rx_valid),
        .xgmii_rx_d_mon(xgmii_rx_d_mon),
        .xgmii_rx_c_mon(xgmii_rx_c_mon),
        .xgmii_tx_d_mon(xgmii_tx_d_mon),
        .xgmii_tx_c_mon(xgmii_tx_c_mon)
    ); // module xgmii_src

    alt_ehipc3_fm_sl_xgmii_sink rx_checker (
        .clock(i_xgmii_rx_clk),
        .reset(i_reset),
        .xgmii_tx_clk(xgmii_tx_clk),
        .xgmii_tx(xgmii_tx),
        .xgmii_rx_clk(i_xgmii_rx_clk),
        .xgmii_rx(xgmii_rx),
        .test_done(test_done),
        .rx_ready(~i_reset_sm),
        .fifo_full(fifo_full),
        .fifo_empty(fifo_empty),
        .read_stored_data(),
        .checker_pass(o_checker_pass),
        .rx_mismatch(rx_mismatch),
        .idle_state(idle),
        .xgmii_rx_d_mon(xgmii_rx_d_mon),
        .xgmii_rx_c_mon(xgmii_rx_c_mon),
        .xgmii_tx_d_mon(xgmii_tx_d_mon),
        .xgmii_tx_c_mon(xgmii_tx_c_mon),
        .rx_valid (i_xgmii_rx_valid)       
    ); // module xgmii_sink

endmodule
