// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//------------------------------------------------------------------------------
//
// Filename         : alt_ehipc3_fm_sl_ptp_sn_clb_pulse.sv
// Author           : Shoaib Sial <shoaib.sial@intel.com>
// Created On       : Wed Sep 11/09/2019 
//
//------------------------------------------------------------------------------

module alt_ehipc3_fm_sl_ptp_sn_clb_pulse 
(
	input wire 											i_reset_n,
	input wire 											i_clk,
	input wire 											i_all_ready,
	input wire 											i_calibrate,

        input wire [31:0]                                                                               i_apulse_period,


	output wire											o_sclk


 
);


  reg [19:0] roll_over_count;
  reg        o_roll_over_trig;

  wire [19:0] apulse_period_minus_1;

  assign apulse_period_minus_1 = i_apulse_period[18:0] - 19'd1;

  assign o_sclk = o_roll_over_trig;

  always@(posedge i_clk) 
  begin	
    if(!i_reset_n)
      o_roll_over_trig <= 1'b0;
    else if (roll_over_count == apulse_period_minus_1)
      o_roll_over_trig <= ~o_roll_over_trig;
  end

  always@(posedge i_clk) 
  begin	
    if(!i_reset_n) 
      roll_over_count <= 20'd0;
    else if (~i_all_ready || ~i_calibrate)
      roll_over_count <= 20'd0;
    else if (roll_over_count == apulse_period_minus_1)
      roll_over_count <= 20'd0;   
    else
      roll_over_count <= roll_over_count + 1'b1;
  end


endmodule //


