# (C) 2001-2021 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


set sl4_sdc_debug 0


set current_inst [get_current_instance]
set current_inst_rx [string map {"inst_tx_mac" "inst_rx_mac"} $current_inst]
set current_inst_phy [string map {"inst_tx_mac" "inst_sl4_phy"} $current_inst]



# Check if port exists
proc sl4_port_existence {port_name} {
   set port_collection [get_ports -nowarn $port_name]
   if { [get_collection_size $port_collection] > 0 } {
      return 1
   } else {
      return 0
   }
}

# Return existing clock target list
proc sl4_get_clock_target_list {} {
   upvar 1 sl4_sdc_debug sl4_sdc_debug

   set result [list]
   set clocks_collection [get_clocks -nowarn]
   foreach_in_collection clock $clocks_collection {
      set clock_name       [get_clock_info -name $clock]
      set clock_target_col [get_clock_info -targets $clock]
      lappend result       [query_collection -report -all $clock_target_col]
      if {$sl4_sdc_debug} { post_message -type info "clock_name : $clock_name" }
   }
   if {$sl4_sdc_debug} { post_message -type info "SL4 clock_target list: $result" }

   return $result
}

set sl4_clock_target_list [sl4_get_clock_target_list]

#--Set Clock Constraints--

      set sl4_reconfig_clk_ext        [sl4_port_existence reconfig_clk]
      set sl4_reconfig_clk_lsrc       [lsearch -exact $sl4_clock_target_list reconfig_clk]
      set sl4_xcvr_ref_clk_ext    [sl4_port_existence xcvr_ref_clk]
      set sl4_xcvr_ref_clk_lsrc   [lsearch -exact $sl4_clock_target_list xcvr_ref_clk]

      if {$sl4_reconfig_clk_ext && $sl4_reconfig_clk_lsrc == -1} {
         create_clock -period "100 MHz" -name reconfig_clk [get_ports reconfig_clk]
         set_clock_groups -asynchronous -group {reconfig_clk} -group {ALTERA_INSERTED_INTOSC_FOR_TRS|divided_osc_clk}
      }

      if {$sl4_xcvr_ref_clk_ext && $sl4_xcvr_ref_clk_lsrc == -1} {
         create_clock -period "156.250000 MHz" -name xcvr_ref_clk [get_ports xcvr_ref_clk]
      }


#**************************************************************
# Set Max and Min delay for cross clock paths
#**************************************************************
proc set_cross_clk_delay {from_node to_node} {
    # Set the max_skew
    set_max_delay -from $from_node -to $to_node 100
    set_min_delay -from $from_node -to $to_node -100
    set_net_delay -from $from_node -to $to_node -max -get_value_from_clock_period dst_clock_period -value_multiplier 5.0
}


    set reset_controller alt_ehipc3_fm_reset_controller_inst
set L_min1      [expr (4-1)/4]
for {set lane_internal 0} {$lane_internal <= $L_min1} {incr lane_internal} {

    set_max_delay -from [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|RST_CTRL_100G.$reset_controller|ehip_ready_out] -to [get_registers $current_inst|tx_soft_bond_inst|phy_ehip_ready_inst|extended_reg] 100
    set_min_delay -from [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|RST_CTRL_100G.$reset_controller|ehip_ready_out] -to [get_registers $current_inst|tx_soft_bond_inst|phy_ehip_ready_inst|extended_reg] -100
    ##set_net_delay -from [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|RST_CTRL_100G.$reset_controller|ehip_ready_out] -to [get_registers $current_inst|tx_soft_bond_inst|phy_ehip_ready_inst|extended_reg] -max -get_value_from_clock_period dst_clock_period -value_multiplier 5.0
	
    for {set i 0} {$i < 4} {incr i} {
        set j      [expr (($lane_internal)*4 +$i)]
		
        set_max_delay -from [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|RST_CTRL_100G.$reset_controller|rx_eth_ready] -to [get_registers $current_inst_rx|chnl[$j].phy_decode_inst|rx_pcs_ready_inst|u|din_s1] 100
        set_min_delay -from [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|RST_CTRL_100G.$reset_controller|rx_eth_ready] -to [get_registers $current_inst_rx|chnl[$j].phy_decode_inst|rx_pcs_ready_inst|u|din_s1] -100
        set_net_delay -from [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|RST_CTRL_100G.$reset_controller|rx_eth_ready] -to [get_registers $current_inst_rx|chnl[$j].phy_decode_inst|rx_pcs_ready_inst|u|din_s1] -max -get_value_from_clock_period dst_clock_period -value_multiplier 5.0
    }
	
	set_max_delay -from [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|DESKEW.PCSDIRECT.rx_pcs64_dsk_inst|o_deskew_done] -to [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|RST_CTRL_100G.$reset_controller|ready_synchronizers|resync_chains[3].synchronizer_nocut|din_s1] 100
	set_min_delay -from [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|DESKEW.PCSDIRECT.rx_pcs64_dsk_inst|o_deskew_done] -to [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|RST_CTRL_100G.$reset_controller|ready_synchronizers|resync_chains[3].synchronizer_nocut|din_s1] -100
	
	set_max_delay -from [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|DESKEW.PCSDIRECT.rx_pcs64_dsk_inst|o_deskew_done] -to [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|RST_CTRL_100G.$reset_controller|ehip_rst_seqinst|rxpfa_synchronizers|resync_chains[0].synchronizer_nocut|din_s1] 100
	set_min_delay -from [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|DESKEW.PCSDIRECT.rx_pcs64_dsk_inst|o_deskew_done] -to [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|RST_CTRL_100G.$reset_controller|ehip_rst_seqinst|rxpfa_synchronizers|resync_chains[0].synchronizer_nocut|din_s1] -100
	
    set_max_delay -from [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|RST_CTRL_100G.$reset_controller|ehip_rst_seqinst|ehip_tx_reset_out_stage] -to [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|kpfec_cc_gen.cc_timer_reset_sync_inst|din_s1] 100
    set_min_delay -from [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|RST_CTRL_100G.$reset_controller|ehip_rst_seqinst|ehip_tx_reset_out_stage] -to [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|kpfec_cc_gen.cc_timer_reset_sync_inst|din_s1] -100
    set_net_delay -from [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|RST_CTRL_100G.$reset_controller|ehip_rst_seqinst|ehip_tx_reset_out_stage] -to [get_registers $current_inst_phy|inst_ehip_xcvr$lane_internal|kpfec_cc_gen.cc_timer_reset_sync_inst|din_s1] -max -get_value_from_clock_period dst_clock_period -value_multiplier 5.0
}

  #phy_adapter reset_synchronizer
  set_max_delay -from [get_registers $current_inst_phy|inst_phy_adapter|x*_inst.reconfig_clk*_tx_pcs_fec_phy_rst_sync|u|dreg[0]] -to [get_registers $current_inst_phy|inst_ehip_xcvr*|TX_DESKEW.csr_reset_synchronizers3|resync_chains[0].synchronizer_nocut|din_s1] 100
  set_min_delay -from [get_registers $current_inst_phy|inst_phy_adapter|x*_inst.reconfig_clk*_tx_pcs_fec_phy_rst_sync|u|dreg[0]] -to [get_registers $current_inst_phy|inst_ehip_xcvr*|TX_DESKEW.csr_reset_synchronizers3|resync_chains[0].synchronizer_nocut|din_s1] -100



