// (C) 2001-2020 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synopsys translate_off
`timescale 1ps/1ps
`include "testbench_defs.v"
// synopsys translate_on

`define  CONTINUOUS                       0
`define  BURST                            1


module traffic_gen #
  (
    parameter                     LANES                         = 2,              // Number of lanes on the link
    parameter                     STREAM                        = 1,
    parameter   [63:0]            TOTAL_SAMPLES_TO_TRANSFER     = 100,          // Total words transferred during the test
    parameter                     MIN_INTER_BURST_GAP           = 2
  )
  (
    // Clocks and reset
    input                         user_clock,
    input                         user_clock_reset,

    `ifdef SIMULATION

    // Traffic checker ports
    output      [63:0]            burst_descr,
    output                        burst_descr_ready,
    input                         burst_descr_read,
    input                         burst_descr_read_clk,

    `endif

    // Streaming data interface
    output      [(LANES*64)-1:0]  data,
    // output      [21+(LANES*64)-1:0]  data_comb,
    output reg  [7:0]             sync, 
    output reg                    tx_is_usr_cmd,
    output reg  [3:0]             tx_num_valid_bytes_eob,
    output reg  [4:0]             empty,
    output reg                    valid,
    output reg                    start_of_burst,
    output reg                    end_of_burst,
    input                         link_up,
    input                         ready,


    // Test control ports
    input                         tg_enable,
    input                         tg_test_mode,
    input                         tg_enable_stalls,
    input                         fifo_full_stalls,

    // Status interface
    output reg  [26:0]            tg_burst_count,
    output reg  [63:0]            tg_words_transferred,

    //prbs data control
    input prbs_en_data
  );

    localparam   [63:0]            TOTAL_SAMPLES_TO_TRANSFER_M1 = TOTAL_SAMPLES_TO_TRANSFER-1;

    reg                    valid_p1;
    reg                    start_of_burst_p1;
    reg                    end_of_burst_p1;  
    reg                    valid_p;
    reg                    start_of_burst_p;
    reg                    end_of_burst_p;
    reg  [7:0]             gen_sync; 
    reg                    gen_tx_is_usr_cmd;
    reg  [3:0]             gen_tx_num_valid_bytes_eob;
    reg  [4:0]             gen_empty;
    reg                    gen_valid;
    reg                    gen_start_of_burst;
    reg                    gen_end_of_burst;
    reg  [63:0]            gen_tg_words_transferred;
	reg                    tgen_words_count_enable_d;
  localparam                    words = LANES;    // The number of 64-bit words on the user interface is 
  // equal to the number of lanes on the link.
 //   end else begin
 // reg        [(LANES*64)-1:0]            data_comb;

  `ifndef SIMULATION

  //*************************************************************************
  //
  // Instantiate the PRBS generator  
  //
  //*************************************************************************

  wire        [31:0]            prbs;

  prbs_generator #
  (
    .PRBS_INITIAL_VALUE(97),
    .DATA_WIDTH(32),
    .PRBS(23)
  ) 
  prbs_data_generator_inst
  (
    .clk(user_clock),
    .nreset(~user_clock_reset),
    .xaui_word_align(1'b1),
    .dout(prbs),
    .pause(1'b0),
    .insert_error(1'b0)
  );

  `endif


  //
  // tg_enable synchronizer.
  //
  wire  sync_tg_enable;

  seriallite4_dp_sync #
  (
    .DP_WIDTH(1),
    .DP_RESET(1'b0)
  )
  tg_enable_sync
  (
    .async_reset_n(~user_clock_reset),
    .sync_reset_n(1'b1),
    .clk(user_clock),
    .sync_ctrl(2'd2),
    .d(tg_enable),
    .o(sync_tg_enable)
  );

  //
  // tg_enable synchronizer.
  //
  wire  sync_ready;

  seriallite4_dp_sync #
  (
    .DP_WIDTH(1),
    .DP_RESET(1'b0)
  )
  ready_sync
  (
    .async_reset_n(~user_clock_reset),
    .sync_reset_n(1'b1),
    .clk(user_clock),
    .sync_ctrl(2'd2),
    .d(ready),
    .o(sync_ready)
  );


  //
  // tg_test_mode synchronizer.
  //
  wire  sync_tg_test_mode;

  seriallite4_dp_sync #
  (
    .DP_WIDTH(1),
    .DP_RESET(1'b1)
  )
  tg_test_mode_sync
  (
    .async_reset_n(~user_clock_reset),
    .sync_reset_n(1'b1),
    .clk(user_clock),
    .sync_ctrl(2'd2),
    .d(tg_test_mode),
    .o(sync_tg_test_mode)
  );


  //
  // tg_enable_stalls synchronizer.
  //
  wire  sync_tg_enable_stalls;

  seriallite4_dp_sync #
  (
    .DP_WIDTH(1),
    .DP_RESET(1'b0)
  )
  tg_enable_stalls_sync
  (
    .async_reset_n(~user_clock_reset),
    .sync_reset_n(1'b1),
    .clk(user_clock),
    .sync_ctrl(2'd2),
    .d(tg_enable_stalls),
    .o(sync_tg_enable_stalls)
  );
  
  //
  // fifo_full_stalls synchronizer.
  //
  wire  sync_fifo_full_stalls;

  seriallite4_dp_sync #
  (
    .DP_WIDTH(1),
    .DP_RESET(1'b0)
  )
  tg_enable_fifo_stalls_sync
  (
    .async_reset_n(~user_clock_reset),
    .sync_reset_n(1'b1),
    .clk(user_clock),
    .sync_ctrl(2'd2),
    .d(fifo_full_stalls),
    .o(sync_fifo_full_stalls)
  );

  //
  // prbs_en_data synchronizer.
  //
  // wire  sync_prbs_en_data;

  // seriallite4_dp_sync #
  // (
    // .DP_WIDTH(1),
    // .DP_RESET(1'b1)
  // )
  // prbs_en_data_sync
  // (
    // .async_reset_n(~user_clock_reset),
    // .sync_reset_n(1'b1),
    // .clk(user_clock),
    // .sync_ctrl(2'd2),
    // .d(prbs_en_data),
    // .o(sync_prbs_en_data)
  // );

	reg tg_valid;
      always @(posedge user_clock) begin
			if (user_clock_reset) begin
				tg_valid <= 1'b0;
			end else begin
				tg_valid <= 1'b1;
			end
		end

  //
  // prbs_en_data synchronizer.
  //
   reg  sync_prbs_en_data;
   always @(posedge user_clock) begin
      if (~tg_valid) begin
		sync_prbs_en_data <= 1'b0;	
      end else begin
	  	sync_prbs_en_data <= prbs_en_data;		
      end
   end
  //*********************************************************************
  //
  // Traffic Generator Stall Generation
  //
  //*********************************************************************

  localparam                 stall_probability    = 1;     // Probability of starting a stall in a given clock cycle (%). Set to 0 to disable.
  localparam                 stall_max_duration   = 3;     // Maximum number of clock cycles in a stall

  localparam                 IDLE                 = 2'd0;
  localparam                 ACTIVE               = 2'd1;
  localparam                 STALLING             = 2'd2;

  reg            [1:0]       stall_state, next_stall_state;

  reg                        stall_tg, next_stall_tg;

  reg            [2:0]       stall_beats, next_stall_beats;
  reg            [2:0]       beat_count, next_beat_count;

  reg            [7:0]       random_value;
  reg                        stall_tg_d, stall_tg_d1;

  /* synthesis translate_off */

  reg [0:255] stall_state_encode;     // This string is for simulation debug

  always @ (stall_state) begin

    case (stall_state)

      IDLE:       stall_state_encode   <= "IDLE"; 
      ACTIVE:     stall_state_encode   <= "ACTIVE";
      STALLING:   stall_state_encode   <= "STALLING";

    endcase

  end

  /* synthesis translate_on */


  //
  // Traffic Generator Stall State Machine Storage
  //
  always @(posedge user_clock) begin

    if (user_clock_reset) begin

      stall_state    <= IDLE;
      stall_beats    <= 3'd0;
      beat_count     <= 3'd0;
      stall_tg       <= 1'b0;
      random_value   <= 8'd255;

    end else begin

      stall_state    <= next_stall_state;
      stall_beats    <= next_stall_beats;
      beat_count     <= next_beat_count;
      stall_tg       <= next_stall_tg;

      `ifdef SIMULATION
      random_value   <= $urandom_range(100, 1);

      `else
      random_value   <=prbs[6:0];
      `endif

    end

  end


  //
  // Traffic Generator Stall State Machine Decoder
  //
  always @(*) begin

    case (stall_state)

      IDLE: begin

        `ifdef SIMULATION
        if (((gen_start_of_burst == 1'b1) && ($urandom_range(100, 1) <= stall_probability) && (sync_tg_enable_stalls == 1'b1)) || ((gen_start_of_burst == 1'b1) && (sync_fifo_full_stalls ==1'b1))) begin
          `else
          if (((gen_start_of_burst == 1'b1) && (prbs[6:0] <= stall_probability) && (sync_tg_enable_stalls == 1'b1))) begin
            `endif
            next_stall_state     = STALLING;
            next_stall_tg        = 1'b0;
            next_beat_count      = 3'd1;

            `ifdef SIMULATION
            next_stall_beats     = $urandom_range(stall_max_duration, 1);
            `else
            next_stall_beats     = prbs[9:7];
            `endif

          end else if (gen_start_of_burst == 1'b1) begin

            next_stall_state     = ACTIVE;
            next_stall_tg        = 1'b0;
            next_stall_beats     = 3'd0;
            next_beat_count      = 3'd0;

          end else begin

            next_stall_state     = IDLE;
            next_stall_tg        = 1'b0;
            next_stall_beats     = 3'd0;
            next_beat_count      = 3'd0;

          end

        end


        ACTIVE: begin


          if ((gen_end_of_burst == 1'b1)) begin
            next_stall_state     = IDLE;
            next_stall_tg        = 1'b0;
            next_stall_beats     = 3'd0;
            next_beat_count      = 3'd0;
          end else if (((random_value <= stall_probability) && (sync_tg_enable_stalls == 1'b1)) || (sync_fifo_full_stalls ==1'b1)) begin

            next_stall_state     = STALLING;
            next_stall_tg        = 1'b1;
            next_beat_count      = 3'd1;

            `ifdef SIMULATION
            next_stall_beats     = $urandom_range(stall_max_duration, 1);
            `else
            next_stall_beats     = prbs[9:7];
            `endif

          end else begin

            next_stall_state     = ACTIVE;
            next_stall_tg        = 1'b0;
            next_stall_beats     = 3'd0;
            next_beat_count      = 3'd0;

          end

        end


        STALLING: begin

          if (sync_fifo_full_stalls == 1'b1) begin

            next_stall_state     = STALLING;
            next_stall_tg        = 1'b1;
            next_stall_beats     = 3'd0;
            next_beat_count      = 3'd0;

          end else begin

            next_stall_state     = (gen_end_of_burst == 1'b1) ? IDLE : ACTIVE;
            next_stall_tg        = 1'b1;
            next_stall_beats     = stall_beats;
            next_beat_count      = beat_count + 3'd1;

          end

        end


        default: begin

          next_stall_state     = IDLE;
          next_stall_tg        = 1'b0;
          next_stall_beats     = 3'd0;
          next_beat_count      = 3'd0;

        end

        endcase

      end


      //*********************************************************************
      //
      // Traffic Generator Control State Machine
      //
      //*********************************************************************
      localparam                TG_INIT              = 0;
      localparam                TG_IDLE              = 1;
      localparam                TG_PRE_BURST         = 2;
      localparam                TG_PRE_CONT          = 3;
      localparam                TG_ACTIVE_BURST      = 4;
      localparam                TG_ACTIVE_CONT       = 5;
      localparam                TG_ACTIVE_COMPLETE   = 6;

      localparam              max_burst_size       = 100;
      localparam              max_inter_burst_gap  = 16;    
    
      reg   [2:0]             tg_state, next_tg_state  /* synthesis keep */;
      reg                     next_start_of_burst;
      reg                     next_end_of_burst;
      reg   [7:0]             next_sync;
      reg   [3:0]             next_tx_num_valid_bytes_eob;
      reg                     next_tx_is_usr_cmd;
      reg   [4:0]             next_empty;

      reg   [7:0]             inter_burst_count, next_inter_burst_count;

      reg   [7:0]            burst_length, next_burst_length;
      wire   [63:0]            next_tg_words_transferred;

      reg   [7:0]            tg_burst_state_count, next_tg_burst_state_count;

      reg                    tg_active;
      reg           tgen_words_count_enable;
    
   
    
      /* synthesis translate_off */

      reg [0:255] tg_state_encode;     // This string is for simulation debug

      always @ (tg_state) begin

        case (tg_state)

          TG_INIT:             tg_state_encode   <= "INIT"; 
          TG_IDLE:             tg_state_encode   <= "TG_IDLE"; 
          TG_PRE_BURST:        tg_state_encode   <= "TG_PRE_BURST";
          TG_PRE_CONT:         tg_state_encode   <= "TG_PRE_CONT"; 
          TG_ACTIVE_BURST:     tg_state_encode   <= "ACTIVE_BURST";
          TG_ACTIVE_CONT:      tg_state_encode   <= "ACTIVE_CONT";
          TG_ACTIVE_COMPLETE:  tg_state_encode   <= "CONT_COMPLETE";

        endcase

      end


      wire [0:255] tg_test_mode_encode;     // This string is for simulation debug

      assign tg_test_mode_encode = (sync_tg_test_mode == `CONTINUOUS) ? "CONTINUOUS" : "BURST";

      /* synthesis translate_on */


      //
      // Traffic Generator Control State Machine Storage
      //
      always @(posedge user_clock) begin

        if (user_clock_reset) begin

          tg_state                    <= TG_INIT;
          gen_start_of_burst              <= 1'b0;
          gen_end_of_burst                <= 1'b0;
          gen_sync                        <= 8'd0;
          gen_tx_num_valid_bytes_eob      <= 4'd0;
          gen_empty                       <= 5'd0;
          tg_burst_state_count        <= 8'd0;
          inter_burst_count           <= 8'd0;
          tg_active                   <= 1'b0;
		  gen_tg_words_transferred        <= 64'd0;
        end else begin

          if ((tg_words_transferred < TOTAL_SAMPLES_TO_TRANSFER_M1) && (link_up == 1'b1) && (sync_tg_enable == 1'b1)) begin
          // if ((gen_tg_words_transferred < TOTAL_SAMPLES_TO_TRANSFER_M1) && (link_up == 1'b1) && (sync_tg_enable == 1'b1)) begin
            tg_active <= 1'b1;
          end else begin
            tg_active <= 1'b0;
          end

          if (stall_tg_d == 1'b0) begin
            tg_state                 <= next_tg_state;
            gen_start_of_burst           <= next_start_of_burst;
            gen_end_of_burst             <= next_end_of_burst;
            gen_sync                     <= next_sync;
            gen_tx_num_valid_bytes_eob   <= next_tx_num_valid_bytes_eob;
            gen_tx_is_usr_cmd            <= next_tx_is_usr_cmd;
            gen_empty                    <= next_empty;
            tg_burst_state_count     <= next_tg_burst_state_count;
            burst_length             <= next_burst_length;
            inter_burst_count        <= next_inter_burst_count;
		  gen_tg_words_transferred        <= next_tg_words_transferred;
          end else begin

            tg_state                 <= tg_state;
            gen_start_of_burst           <= gen_start_of_burst;
            gen_end_of_burst             <= gen_end_of_burst;
            gen_sync                     <= gen_sync;
            gen_empty                    <= gen_empty;
            gen_tx_num_valid_bytes_eob   <= gen_tx_num_valid_bytes_eob;
            gen_tx_is_usr_cmd            <= gen_tx_is_usr_cmd;
            tg_burst_state_count     <= tg_burst_state_count;
            burst_length             <= burst_length;
            inter_burst_count        <= inter_burst_count;
		  gen_tg_words_transferred        <= gen_tg_words_transferred;
          end

        end

      end

      //
      // Minimum Burst Gap
      //
      wire synth_min_gap;
      generate 
        if (MIN_INTER_BURST_GAP == 1)  begin
          assign synth_min_gap = 1'b0;
        end else begin
          assign synth_min_gap = 1'b1;
        end   
      endgenerate  

      //
      // next_tg_words_transferred counter
      //

  cascaded_counter #
   (
      .NUM_ADD_BLOCK                    (8),
      .ADD_BLOCK_BITS                   (8)
   )
   tgen_word_counter_inst
  ( .clock(user_clock),.reset(~tg_valid),.count_enable(tgen_words_count_enable),.data_in(gen_tg_words_transferred),.data_out(next_tg_words_transferred) ); 
      always @* begin

        case (tg_state)

          TG_INIT: begin
            gen_valid                     = 1'b0;
            tgen_words_count_enable       = 1'b0;

            next_tg_state                 = TG_IDLE;
            next_start_of_burst           = 1'b0;
            next_end_of_burst             = 1'b0;
            next_sync                     = 8'd0;
            next_empty                    = 5'd0;
            next_tx_num_valid_bytes_eob   = 4'd0;
            next_tx_is_usr_cmd            = 1'b0;
            next_tg_burst_state_count     = 8'd0;
            next_burst_length             = 8'd0;


            `ifdef SIMULATION
            next_inter_burst_count     = $urandom_range(max_inter_burst_gap, MIN_INTER_BURST_GAP) - 8'd1;
            `else
            next_inter_burst_count = {prbs[15:9],synth_min_gap};
            `endif
          end

         TG_IDLE: begin
            next_tg_burst_state_count     = 8'd0;

            gen_valid                     = 1'b0;
            tgen_words_count_enable    	  = 1'b0;   

            next_start_of_burst           = 1'b0;
            next_end_of_burst             = 1'b0;
            next_tx_num_valid_bytes_eob   = 4'd0;
            next_tx_is_usr_cmd            = 1'b0;
            next_sync                     = 8'd0;
            next_empty                    = 5'd0;
            next_inter_burst_count        = (inter_burst_count != 8'd0) ? inter_burst_count - 8'd1 : inter_burst_count;
            next_burst_length             = 8'd0;
   

            `ifdef SIMULATION

            if ((sync_tg_test_mode == `CONTINUOUS) && (tg_active == 1'b1) && (tg_words_transferred != TOTAL_SAMPLES_TO_TRANSFER_M1)) begin
              next_tg_state              = TG_PRE_CONT;
            end else if ((sync_tg_test_mode == `BURST) && (tg_active == 1'b1) && (inter_burst_count == 0) && (tg_words_transferred != TOTAL_SAMPLES_TO_TRANSFER_M1)) begin
              next_tg_state              = TG_PRE_BURST;
            end else begin
              next_tg_state              = TG_IDLE;
            end

            `else

            if ((sync_tg_test_mode == `CONTINUOUS) && (tg_active == 1'b1) ) begin
              next_tg_state              = TG_PRE_CONT;
            end else if ((sync_tg_test_mode == `BURST) && (tg_active == 1'b1) && (inter_burst_count == 0) ) begin
              next_tg_state              = TG_PRE_BURST;
            end else begin
              next_tg_state              = TG_IDLE;
            end

            `endif



          end

          TG_PRE_CONT: begin
            next_tx_num_valid_bytes_eob  = 4'd0;
            next_tg_burst_state_count  = 8'd0;
            next_tg_state              = TG_ACTIVE_CONT;


            gen_valid                  = 1'b0;
            tgen_words_count_enable    = 1'b0; 


            next_start_of_burst        = 1'b1;
            next_end_of_burst          = 1'b0;
            next_tx_is_usr_cmd         = 1'b1;
            next_inter_burst_count     = 8'd0;

            next_burst_length          = burst_length;


            `ifdef SIMULATION
            next_sync                  = $urandom_range(256, 0);
            next_empty                 = $urandom_range(32, 0);
            next_tx_is_usr_cmd         = $urandom_range(1, 0);


            $write("\n");
            $write("   Traffic Generator: %0d sample continuous transfer started at time %0t\n", TOTAL_SAMPLES_TO_TRANSFER, $time);
            `else
            next_sync                  = prbs[7:0];
            next_empty                 = prbs[12:8];
            `endif

          end

            TG_PRE_BURST: begin
            next_tg_burst_state_count  = 8'd0;
            next_tx_num_valid_bytes_eob   = 4'd0;


            `ifdef SIMULATION
            next_burst_length           = (TOTAL_SAMPLES_TO_TRANSFER_M1 - tg_words_transferred < 100) ? TOTAL_SAMPLES_TO_TRANSFER_M1 - tg_words_transferred : $urandom_range(max_burst_size, 1);
            next_tg_state              = ((TOTAL_SAMPLES_TO_TRANSFER_M1 - tg_words_transferred) == 1) ? TG_ACTIVE_COMPLETE:TG_ACTIVE_BURST;	    
            next_end_of_burst          = ((TOTAL_SAMPLES_TO_TRANSFER_M1 - tg_words_transferred) == 1) ? 1:0;
            `else
            next_burst_length          =  {prbs[7:1],1'd1};
            next_tg_state              = TG_ACTIVE_BURST;
            next_end_of_burst          = 1'b0;
            `endif
      
            gen_valid                  = 1'b0;
            tgen_words_count_enable    = 1'b0; 


            next_tx_is_usr_cmd         = 1'b1;
            next_start_of_burst        = 1'b1;



            next_inter_burst_count     = 8'd0;
            `ifdef SIMULATION
            next_sync                  = $urandom_range(256, 0);
            next_empty                 = $urandom_range(32, 0);
            next_tx_is_usr_cmd         = $urandom_range(1, 0);
            $write("\n");
            $write("   Traffic Generator: %0d sample burst started at time %0t\n", next_burst_length, $time);
            `else
            next_sync                  = prbs[15:8];
            next_empty                 = prbs[4:0];
            `endif

          end

          TG_ACTIVE_BURST: begin


	    gen_valid 			= (stall_tg_d == 1'b0) ? 1'b1 : 1'b0;
            tgen_words_count_enable	= (stall_tg_d == 1'b0) ? 1'b1 : 1'b0;


            `ifdef SIMULATION
            next_sync                  = $urandom_range(256, 0);
            next_empty                 = $urandom_range(32, 0);
            next_tx_is_usr_cmd         = $urandom_range(1, 0);
            next_tx_num_valid_bytes_eob    = 4'd0;
            `else
            next_sync                  = prbs[7:0];
            next_empty                 = prbs[12:8];
            `endif

            next_tg_burst_state_count  = tg_burst_state_count + 8'd1;
            next_burst_length = burst_length; 
            next_inter_burst_count     = 8'd0;
            next_tx_is_usr_cmd         = 1'b0;



              `ifdef SIMULATION
            if ((tg_burst_state_count       == burst_length - 1)   ||  (tg_words_transferred == TOTAL_SAMPLES_TO_TRANSFER_M1  - 64'd1) || (sync_tg_enable == 1'b0)) begin
              `else
            if ((tg_burst_state_count       == burst_length - 1)   ||  (sync_tg_enable == 1'b0)) begin
              `endif

                next_tg_state              	= TG_ACTIVE_COMPLETE ; //TG_IDLE;	 
		next_tx_num_valid_bytes_eob    	= 4'd8;
                next_start_of_burst        	= 1'b0;
                next_end_of_burst          	= 1'b1;
              end else begin
                next_tg_state              	= TG_ACTIVE_BURST;
		next_tx_num_valid_bytes_eob    	= 4'd0;
                next_start_of_burst        	= 1'b0;
                next_end_of_burst          	= 1'b0;
              end

          end

          TG_ACTIVE_CONT: begin
            next_tg_burst_state_count = 8'd0;
            next_tx_is_usr_cmd         = 1'b0;


            gen_valid                  = (stall_tg_d == 1'b0) ? 1'b1 : 1'b0;
            tgen_words_count_enable    = (stall_tg_d == 1'b0) ? 1'b1 : 1'b0;    


            `ifdef SIMULATION
            next_sync                  = $urandom_range(256, 0);
            next_empty                 = $urandom_range(32, 0);
            next_tx_num_valid_bytes_eob    = 4'd8;

            `else
            next_sync                  = prbs[7:0];
            next_empty                 = prbs[12:8];
            `endif

            next_burst_length          = burst_length;
            next_inter_burst_count     = 8'd0;

 
              `ifdef SIMULATION
             if ((gen_tg_words_transferred == TOTAL_SAMPLES_TO_TRANSFER_M1 - 64'd1) || (sync_tg_enable == 1'b0)) begin
              `else
             if ((sync_tg_enable == 1'b0)) begin
              `endif
              next_tg_state              = TG_ACTIVE_COMPLETE;
              next_start_of_burst        = 1'b0;
              next_end_of_burst          = 1'b1;
              next_tx_num_valid_bytes_eob    = 4'd8;

            end else begin
              next_tx_num_valid_bytes_eob   = 4'd0;
              next_tg_state              = TG_ACTIVE_CONT;
              next_start_of_burst        = 1'b0;
              next_end_of_burst          = 1'b0;
            end 


          end


          TG_ACTIVE_COMPLETE: begin

            next_tg_state              = TG_IDLE;

            gen_valid                  = (stall_tg_d == 1'b0) ? 1'b1 : 1'b0;
            tgen_words_count_enable    = (stall_tg_d == 1'b0) ? 1'b1 : 1'b0; 


            next_start_of_burst        = 1'b0;
            next_end_of_burst          = 1'b0;
            next_tx_num_valid_bytes_eob = 4'd0;
            next_tx_is_usr_cmd         = 1'b0;

            `ifdef SIMULATION
            next_sync                  = $urandom_range(256, 0);
            next_empty                 = $urandom_range(32, 0);
            `else
            next_sync                  = prbs[7:0];
            next_empty                 = prbs[12:8];
            `endif
            next_tg_burst_state_count = 8'd0;
            next_burst_length         = burst_length;


            if(sync_tg_test_mode == `CONTINUOUS)
              begin
                next_inter_burst_count     = 8'd0;
              end 
            else begin

              `ifdef SIMULATION
              next_inter_burst_count     = $urandom_range(max_inter_burst_gap, MIN_INTER_BURST_GAP) - 8'd1;
              `else
              next_inter_burst_count = {prbs[15:9],synth_min_gap};
              `endif
            end

          end


          default: begin

            gen_valid                         = 1'b0;
            next_tg_state                 = TG_IDLE;
            next_start_of_burst           = 1'b0;
            next_end_of_burst             = 1'b0;
            next_sync                     = 8'd0;
            next_empty                    = 5'd0;
            next_tx_num_valid_bytes_eob   = 4'd0;
            next_tx_is_usr_cmd         = 1'b0;
            next_tg_burst_state_count  = 8'd0;
            next_burst_length          = 8'd0;
            next_inter_burst_count     = 8'd0;
      tgen_words_count_enable    = 1'b0;
          end

        endcase

      end

      always @(posedge user_clock) begin

        if (user_clock_reset) begin
		  valid_p              	 	  <= 1'b0;
          start_of_burst_p            <= 1'b0;
          end_of_burst_p              <= 1'b0;
          valid              		  <= 1'b0;
          start_of_burst              <= 1'b0;
          end_of_burst                <= 1'b0;
          sync                        <= 8'd0;
          tx_num_valid_bytes_eob      <= 4'd0;
          empty                       <= 5'd0;
          tx_is_usr_cmd            	<= 1'b0;
		  stall_tg_d            	<= 1'b0;
		  stall_tg_d1            	<= 1'b0;
		  tgen_words_count_enable_d	<= 	1'b0;
          tg_words_transferred        <= 64'd0;
        end else begin
		    valid_p              	 <= gen_valid;
            start_of_burst_p         <= gen_start_of_burst;
            end_of_burst_p           <= gen_end_of_burst;
			valid_p1              	 <= valid_p;
            start_of_burst_p1        <= start_of_burst_p;
            end_of_burst_p1          <= end_of_burst_p;
			valid              	     <= valid_p1;
            start_of_burst           <= start_of_burst_p1;
            end_of_burst             <= end_of_burst_p1;
            sync                     <= gen_sync;
            empty                    <= gen_empty;
            tx_num_valid_bytes_eob   <= gen_tx_num_valid_bytes_eob;
            tx_is_usr_cmd            <= gen_tx_is_usr_cmd;
		    stall_tg_d            	<= stall_tg;
		    stall_tg_d1            	<= stall_tg_d;
		  tgen_words_count_enable_d	<= 	tgen_words_count_enable;
            tg_words_transferred     <= gen_tg_words_transferred;

          end
		 end
      //*********************************************************************
      //
      // Burst counter
      //
      //*********************************************************************
wire [26:0] tg_burst_count_inc;
wire [26:0] tg_burst_count_inc_out;

cascaded_counter #
(
.NUM_ADD_BLOCK  (3),
.ADD_BLOCK_BITS     (9)
) tg_burst_count_inst (
.reset (user_clock_reset),
.count_enable (start_of_burst_p),
.clock (user_clock),
.data_in (tg_burst_count),
.data_out (tg_burst_count_inc_out)

);

generate
if (STREAM == 1) begin
assign tg_burst_count_inc = tg_burst_count_inc_out;
end 
else if (STREAM == 0) begin
assign tg_burst_count_inc = 27'd0;
end 

endgenerate

always @(posedge user_clock) begin
if (user_clock_reset) begin
  tg_burst_count <= 27'd0;
end else begin
  tg_burst_count   <= tg_burst_count_inc;
end
end


`ifdef SIMULATION

      //*********************************************************************
      //
      // Transaction enqueue logic
      //
      //*********************************************************************

      reg      [63:0]   unbuf_burst_descr, next_unbuf_burst_descr;
      reg               unbuf_burst_descr_wr, next_unbuf_burst_descr_wr;
      reg      [1:0]    last_stall_state;


      wire     [7:0]             unbuf_burst_descr_type;
      wire     [7:0]             burst_descr_type;

      reg      [0:255]           unbuf_burst_descr_type_encode;
      reg      [0:255]           burst_descr_type_encode;


      assign unbuf_burst_descr_type = unbuf_burst_descr[63:56];
      assign burst_descr_type       = burst_descr[63:56];


      always @* begin

        case (unbuf_burst_descr_type)

          `BURST_DESCR_START_END: unbuf_burst_descr_type_encode   <= "START_END"; 
          `BURST_DESCR_START:     unbuf_burst_descr_type_encode   <= "START"; 
          `BURST_DESCR_END:       unbuf_burst_descr_type_encode   <= "END"; 
          `BURST_DESCR_STALL:     unbuf_burst_descr_type_encode   <= "STALL"; 
          default:                unbuf_burst_descr_type_encode   <= "reserved"; 

        endcase

      end


      always @* begin

        case (burst_descr_type)

          `BURST_DESCR_START_END: burst_descr_type_encode   <= "START_END"; 
          `BURST_DESCR_START:     burst_descr_type_encode   <= "START"; 
          `BURST_DESCR_END:       burst_descr_type_encode   <= "END"; 
          `BURST_DESCR_STALL:     burst_descr_type_encode   <= "STALL"; 
          default:                burst_descr_type_encode   <= "reserved"; 

        endcase

      end


      //
      // Transaction enqueue logic storage
      //
      always @(posedge user_clock or posedge user_clock_reset) begin

        if (user_clock_reset == 1'b1) begin

          unbuf_burst_descr       <= 64'd0;
          unbuf_burst_descr_wr    <= 1'b0;

        end else begin

          unbuf_burst_descr       <= next_unbuf_burst_descr;
          unbuf_burst_descr_wr    <= next_unbuf_burst_descr_wr;

        end


      end


      //
      // Transaction enqueue logic decode
      //
      always @* begin
        if ((start_of_burst == 1'b1) && (end_of_burst == 1'b1) && (valid == 1'b1)) begin
          next_unbuf_burst_descr     = {`BURST_DESCR_START_END, next_inter_burst_count, sync, empty, 3'b0, data[31:0]};
          next_unbuf_burst_descr_wr  = 1'b1;
        end else if ((start_of_burst == 1'b1) && (valid == 1'b1)) begin
			next_unbuf_burst_descr     = {`BURST_DESCR_START, 8'd0, sync, empty, 3'b0, data[31:0]};
			  next_unbuf_burst_descr_wr  = 1'b1;

        end else if ((end_of_burst == 1'b1) && (valid == 1'b1)) begin
			next_unbuf_burst_descr     = {`BURST_DESCR_END, next_inter_burst_count, sync, empty, 3'b0, data[31:0]};
          next_unbuf_burst_descr_wr  = 1'b1;

          //      end else if ((stall_state == STALLING) && (last_stall_state != STALLING)) begin

          //         next_unbuf_burst_descr     = {`BURST_DESCR_STALL, {5'd0, stall_beats}, 16'd0, next_tg_words_transferred[31:0]};
          //         next_unbuf_burst_descr_wr  = 1'b1;

        end else begin

          next_unbuf_burst_descr     = 64'd0;
          next_unbuf_burst_descr_wr  = 1'b0;

        end

      end

      `endif


      //*********************************************************************
      //
      // Per-word datapath logic
      //
      //*********************************************************************
		  	wire [words-1:0] user_clock_per_gen;
			reg [words-1:0] user_clock_reset_per_gen; /* synthesis preserve_syn_only */
			reg [words-1:0] next_start_of_burst_per_gen; /* synthesis preserve_syn_only */
			reg [words-1:0] valid_per_gen; /* synthesis preserve_syn_only */
			reg [words-1:0] prbs_en_data_per_gen; /* synthesis preserve_syn_only */

      genvar            word;

      generate

        for (word = 0; word < words; word = word + 1) begin : word_generator_inst

          /* synthesis translate_off */

          // These wires are used for simulation debug
          wire [63:0] word_data         = data[((words*64)-1)-(64*word):(words*64)-((word+1)*64)];
          wire [4:0]  word_num          = word_data[63:59];
          wire [26:0] word_burst_num    = word_data[58:32];
          wire [31:0] word_sample_num   = word_data[31:0];

          /* synthesis translate_on */
			assign user_clock_per_gen[word] = user_clock;

      always @(posedge user_clock) begin
			if (~tg_valid) begin
				user_clock_reset_per_gen[word] <= 1'b1;
				next_start_of_burst_per_gen[word] <= 1'b0;
				valid_per_gen[word] <= 1'b0;
				prbs_en_data_per_gen[word] <= 1'b0;
			end else begin
				user_clock_reset_per_gen[word] <= 1'b0;
				// next_start_of_burst_per_gen[word] <= ((stall_tg == 1'b0) ? next_start_of_burst : 1'b0);
				next_start_of_burst_per_gen[word] <= ((stall_tg_d1 == 1'b0) ? start_of_burst_p : 1'b0);
				valid_per_gen[word] <= valid_p1;
				// valid_per_gen[word] <= gen_valid;
				prbs_en_data_per_gen[word] <= sync_prbs_en_data;
			end
		end
		
          word_traffic_gen #
          (
            .word_id(word),
      .STREAM(STREAM)
          )
          word_generator
          (
            .clk(user_clock_per_gen[word]),
            .reset(user_clock_reset_per_gen[word]),
            .start_of_burst(next_start_of_burst_per_gen[word]),
            .burst_enable(valid_per_gen[word]),
            .word_data(data[((words<<6)-1)-(word<<6):(words<<6)-((word+1)<<6)]),
            .prbs_en_data(prbs_en_data_per_gen[word])
          );

        end

      endgenerate


      `ifdef SIMULATION

      //*********************************************************************
      //
      // Transaction FIFO
      //
      //*********************************************************************

      localparam  trans_fifo_depth = 16;
      localparam  trans_fifo_width = 64;

      wire        trans_fifo_empty;

      assign burst_descr_ready = ~trans_fifo_empty;


      dcfifo #
      (
        .intended_device_family("Stratix V"),
        .lpm_numwords(trans_fifo_depth),
        .lpm_showahead("ON"),
        .lpm_type("dcfifo"),
        .lpm_width(trans_fifo_width),
        .lpm_widthu($clog2(trans_fifo_depth)),
        .overflow_checking("ON"),
        .rdsync_delaypipe(5),
        .read_aclr_synch("ON"),
        .underflow_checking("ON"),
        .use_eab("ON"),
        .write_aclr_synch("ON"),
        .wrsync_delaypipe(5)
      )
      dcfifo_component
      (
        .aclr(user_clock_reset),

        .wrclk(user_clock),
        .data(unbuf_burst_descr),
        .wrreq(unbuf_burst_descr_wr),
        .wrfull(),
        .wrempty(),
        .wrusedw(),

        .rdclk(burst_descr_read_clk),
        .q(burst_descr),
        .rdreq(burst_descr_read),
        .rdempty(trans_fifo_empty),
        .rdfull(),
        .rdusedw()
      );

      `endif

      endmodule


      module word_traffic_gen #
        (
          parameter            word_id = 0,
    parameter            STREAM = 1
        )
        (
          input                clk,
          input                reset,

          input                start_of_burst,
          input                burst_enable,

          output reg  [63:0]   word_data,

          input prbs_en_data
        );

        wire         [31:0]   word_id_reg32  = word_id;
        wire         [4:0]    word_id_reg    = word_id_reg32[4:0];

wire         [26:0]   burst_count_inc;
wire         [31:0]   byte_count_inc; 
wire         [26:0]   burst_count_inc_out;
wire         [31:0]   byte_count_inc_out; 
wire         [63:0]   prbs_data; 
reg word_gen_valid;

always @(posedge clk) begin
if (reset) 
word_gen_valid <= 1'b0;
else 
word_gen_valid <= 1'b1;
end

        prbs_generator #
        (
          .PRBS_INITIAL_VALUE(97),
          .DATA_WIDTH(64),
          .PRBS(23)
        ) 
        prbs_data_generator_inst_2
        (
          .clk(clk),
          .nreset(~reset),
          .xaui_word_align(1'b1),
          .dout(prbs_data),
          .pause(~burst_enable),
          .insert_error(1'b0)
        );

cascaded_counter #
(
.NUM_ADD_BLOCK  (3),
.ADD_BLOCK_BITS     (9)
) burst_count_inst (
.reset (reset),
.count_enable (start_of_burst),
.clock (clk),
.data_in (word_data[58:32]),
.data_out (burst_count_inc_out)

);
cascaded_counter #
(
.NUM_ADD_BLOCK  (4),
.ADD_BLOCK_BITS     (8)
) byte_count_inst (
.reset (reset),
.count_enable (burst_enable),
.clock (clk),
.data_in (byte_count_inc),
.data_out (byte_count_inc_out)

);
assign byte_count_inc  = word_data[31:0]; 

  generate
    if (STREAM == 1) begin
	assign burst_count_inc = burst_count_inc_out;
    end 
    else if (STREAM == 0) begin
          assign burst_count_inc = 27'd0;
    end 
  endgenerate

//         Traffic generator word storage
        
         always @(posedge clk) begin

           if (~word_gen_valid) begin

             if (prbs_en_data == 1'b0)
               word_data   <= {word_id_reg, 27'd0, 32'd1};
             else if (prbs_en_data == 1'b1)
               word_data   <= 32'd1;

           end else begin

				if (prbs_en_data == 1'b0)
								word_data   <= (start_of_burst == 1'b1) ? {word_id_reg, burst_count_inc,  word_data[31:0]        } :
				(burst_enable   == 1'b1) ? {word_id_reg, word_data[58:32], byte_count_inc_out} : word_data;
				else if (prbs_en_data == 1'b1)
				word_data   <= (start_of_burst == 1'b1) ? word_data: (burst_enable   == 1'b1) ? prbs_data : word_data;

           end

        end
    
    

      endmodule


